#include "action.h"
#include "math.h"
#include "bin_path.h"

#include "data_debug_print.h"

static int is_end = 0;

int Action_create(Action * action, Bone bone)
{
	Action res = NULL;
	if((res = (Action)malloc(sizeof(struct _Action))) == NULL)
		return 0;
		
	res->bone = bone;
	res->vector = 0;
	res->steps = NULL;
	res->next_bone = NULL;
	
	*action = res;
	return 1;
}

void Action_free(Action * action)
{
	if(action == NULL || *action == NULL)
		return;
	Action_free(&((*action)->next_bone));
	ActionStep_free(&((*action)->steps));
	free(*action);
	*action = NULL;
}

int Action_append(Action * action, Bone bone)
{
	Action ajout = NULL;
	if(action == NULL)
		return 0;
	if(Action_find(*action, bone) != NULL)
		return 0;
	if(! Action_create(&ajout, bone))
		return 0;
	ajout->next_bone = *action;
	*action = ajout;
	return 1;
}

int Action_delete(Action * action, Bone bone)
{
	if(*action == NULL)
		return 0;
	if((*action)->bone == bone)
	{
		Action del = *action;
		*action = (*action)->next_bone;
		del->next_bone = NULL;
		Action_free(&del);
		return 1;
	}
	return Action_delete(&((*action)->next_bone), bone);
}

Action Action_find(Action action, Bone bone)
{
	for(; action != NULL; action = action->next_bone)
	{
		if(action->bone == bone)
			return action;
	}
	return NULL;
}

int Action_save(FILE * flow, Action action)
{
	unsigned int version = 2;
	unsigned char is_action;
	if(flow == NULL || action == NULL)
		return 0;
	version <<= 8;
	version |= 'a';
	version <<= 8;
	version |= (action->next_bone != NULL);
	if(! fwrite(&version, sizeof(unsigned int), 1, flow))
	{
		fprintf(stderr, "Erreur sauvegarde action version\n");
		return 0;
	}
	if(! BinPath_save(flow, action->bone->id))
	{
		fprintf(stderr, "Erreur sauvegarde action chemin binaire\n");
		return 0;
	}
	if(! fwrite(&(action->vector), sizeof(double), 1, flow))
	{
		fprintf(stderr, "Erreur sauvegarde action vector\n");
		return 0;
	}
	is_action = action->steps != NULL;
	if(! fwrite(&is_action, sizeof(unsigned char), 1, flow))/* ajout version 2 */
		return 0;
	if(is_action)
	{
		if(! ActionStep_save(flow, action->steps))
		{
			fprintf(stderr, "Erreur sauvegarde action temps\n");
			return 0;
		}
	}
	if(action->next_bone == NULL)
		return 1;
	return Action_save(flow, action->next_bone);
}

int Action_load(FILE * flow, Action * action, Bone arma)
{
	unsigned int version = 0;
	unsigned char next_bone = 0;
	unsigned char is_action = 1;
	BinPath id = NULL;
	Bone bone = NULL;
	Action res = NULL;
	if(flow == NULL || action == NULL || arma == NULL)
		return 0;
	if(! fread(&version, sizeof(unsigned int), 1, flow))
	{
		fprintf(stderr, "Erreur chargement action version\n");
		return 0;
	}
	next_bone = (version & 0xff);
	version >>= 8;
	if((version & 0xff) != 'a')
	{
		fprintf(stderr, "Erreur chargement action entete id, trouvé '%c' attendu 'a'\n", version & 0xff);
		return 0;
	}
	version >>= 8;
	if((version & 0xff) == 0 || (version & 0xff) > 2)
	{
		fprintf(stderr, "Erreur chargement action version %d non reconnue\n", version & 0xff);
		return 0;
	}
	if(! BinPath_load(flow, &id))
	{
		fprintf(stderr, "Erreur chargement action id binaire de l'os\n");
		return 0;
	}
	if((bone = BinPath_get_bone(id, arma)) == NULL)
	{
		fprintf(stderr, "Erreur chargement action rechercher os connaissant id binaire\n");
		BinPath_free(&id);
		return 0;
	}
	BinPath_free(&id);
	if(! Action_create(&res, bone))
	{
		fprintf(stderr, "Erreur chargement action création\n");
		return 0;
	}
	if(! fread(&(res->vector), sizeof(double), 1, flow))
	{
		fprintf(stderr, "Erreur chargement action vector\n");
		return 0;
	}
	if((version & 0xff) == 2)
	{
		if(! fread(&is_action, sizeof(unsigned char), 1, flow))
		{
			fprintf(stderr, "Erreur chargement action is_action\n");
			Action_free(&res);
			return 0;
		}
	}
	if(is_action)
	{
		if(! ActionStep_load(flow, &(res->steps)))
		{
			fprintf(stderr, "Erreur chargement action step\n");
			Action_free(&res);
			return 0;
		}
	}
	if(! next_bone)
	{
		*action = res;
		return 1;
	}
	if(! Action_load(flow, &(res->next_bone), arma))
	{
		fprintf(stderr, "Erreur chargement action next_bone\n");
		Action_free(&res);
		return 0;
	}
	*action = res;
	return 1;
}

/* ActionSteps */

int ActionStep_create(ActionStep * step, double target_angle, int start_time)
{
	ActionStep res = NULL;
	if((res = (ActionStep)malloc(sizeof(struct _ActionStep))) == NULL)
		return 0;
	res->target_angle = target_angle;
	res->start_time = start_time;
	res->next_time = NULL;
	res->last_time = NULL;
	
	res->duration = 0;
	res->real_start_time = start_time;
	
	*step = res;
	return 1;
}

void ActionStep_free(ActionStep * step)
{
	if(step == NULL || *step == NULL)
		return;
	ActionStep_free(&((*step)->next_time));
	free(*step);
	*step = NULL;
}

static int ActionStep_append_aux(ActionStep * step, ActionStep prec, Bone bone, double target_angle, int start_time)
{
	if(*step == NULL || start_time < (*step)->start_time)
	{
		ActionStep ajout = NULL;
		if(target_angle == ((prec == NULL) ? bone->start_angle : prec->target_angle))
			return 0;
		if(! ActionStep_create(&ajout, correct_degre_angle(target_angle), start_time))
			return 0;
			
		ajout->last_time = prec;
		ajout->next_time = *step;
		if(*step != NULL)
			(*step)->last_time = ajout;
		if(prec != NULL)
			prec->next_time = ajout;
		
		
		*step = ajout;
		return 1;
	}
	if(start_time == (*step)->start_time)
		return 0;
	return ActionStep_append_aux(&((*step)->next_time), *step, bone, target_angle, start_time);
}

int ActionStep_append(ActionStep * step, Bone bone, double target_angle, int start_time)
{
	return ActionStep_append_aux(step, (*step != NULL) ? (*step)->last_time : NULL, bone, target_angle, start_time);
}

int ActionStep_delete(ActionStep * step, int start_time)
{
	if(*step == NULL || start_time < (*step)->start_time)
		return 0;
	if(start_time == (*step)->start_time)
	{
		ActionStep del = NULL;
		del = *step;
		if(del->next_time != NULL)
			del->next_time->last_time = del->last_time;
		if(del->last_time != NULL)
			del->last_time->next_time = del->next_time;
		*step = del->next_time;
		del->next_time = NULL;
		del->last_time = NULL;
		ActionStep_free(&del);
		return 1;
	}
	return ActionStep_delete(&((*step)->next_time), start_time);
}

ActionStep ActionStep_find(ActionStep action, int start_time)
{
	if(action == NULL || start_time > action->start_time)
		return NULL;
	if(start_time == action->start_time)
		return action;
	return ActionStep_find(action->next_time, start_time);
}

ActionStep ActionStep_find_realtime(ActionStep action, int time, int only_real_start_time)
{
	int condi;
	if(action == NULL)
		return NULL;
	if(time >= action->real_start_time)
	{
		if(only_real_start_time)
			condi = time == action->real_start_time;
		else
			condi = time < action->real_start_time + action->duration;
		if(condi)
			return action;
	}
	return ActionStep_find_realtime(action->next_time, time, only_real_start_time);
}

static void ActionStep_calculate_move(ActionStep action, Bone bone, double vector)
{
	double start_angle = (action->last_time == NULL) ? bone->start_angle : action->last_time->target_angle;
	action->vector_sign = Bone_vector_sign(bone, start_angle, action->target_angle);
	if(action->vector_sign > 0)
	{
		action->duration = (action->target_angle + ((action->target_angle > start_angle)? 0 : 360) - start_angle) / vector;
		/*printf("%g %g => %d\n", action->target_angle + ((action->target_angle > start_angle)? 0 : 360), start_angle, action->duration);*/
	}
	else
	{
		action->duration = (start_angle + ((start_angle > action->target_angle)? 0 : 360) - action->target_angle) / vector;
		/*printf("%g %g => %d\n", start_angle + ((start_angle > action->target_angle)? 0 : 360), action->target_angle, action->duration);*/
	}
}

static void ActionStep_calculate_aux(ActionStep action, Bone bone, double vector)
{
	int last_time_end = 0;
	
	for( ; action != NULL; action = action->next_time)
	{
		ActionStep_calculate_move(action, bone, vector);
		if(last_time_end > action->start_time - action->duration)
			action->real_start_time = last_time_end;
		else
			action->real_start_time = action->start_time - action->duration;
		last_time_end = action->real_start_time + action->duration;
		/*printf("bone (%p): %d -> %d (%d) | %g\n", bone, action->start_time, action->real_start_time, action->duration, action->vector_sign * vector);*/
	}
}

int ActionStep_calculate(ActionStep action, Bone bone, double vector)
{
	if(action == NULL)
		return 0;
	for( ; action->last_time != NULL; action = action->last_time);
	ActionStep_calculate_aux(action, bone, vector);
	return 1;
}

int ActionStep_make_action(ActionStep action, Bone bone, double vector, int time)
{
	double last_angle;
	ActionStep first = action;
	for( ; action != NULL && 	(time < action->real_start_time
					|| (time >= action->real_start_time + action->duration
					&& action->next_time != NULL));
					
					action = action->next_time);
	if(action == NULL)
	{
		if(first == NULL)
			return 0;
		if(time < first->real_start_time)
		{
			Bone_set_angle(bone, bone->start_angle);
			return 0;
		}
		for(action = first; action->next_time != NULL && time < action->real_start_time; action = action->next_time);
		Bone_set_angle(bone, action->target_angle);
		return 0;
	}
	if(time - action->real_start_time >= action->duration)
	{
		if(! is_end)
		{
			if(ActionStep_make_end(action, bone, vector, time))
				return 1;
		}
		Bone_set_angle(bone, action->target_angle);
		return 1;
	}
	last_angle = (action->last_time == NULL) ? bone->start_angle : action->last_time->target_angle;
	Bone_set_angle(bone, last_angle + (time - action->real_start_time) * vector * action->vector_sign);

	return 1;
}

int ActionStep_make_end(ActionStep last, Bone bone, double vector, int time)
{
	struct _ActionStep step;
	int retour = 0;
	if(last == NULL)
		return 0;
	for(; last->next_time != NULL; last = last->next_time);
	step = ActionStep_last_step(last, bone->start_angle, last->real_start_time + last->duration);
	last->next_time = &step;
	ActionStep_calculate(&step, bone, vector);
	is_end = 1;
	retour = ActionStep_make_action(&step, bone, vector, time);
	last->next_time = NULL;
	is_end = 0;
	return retour;
}

static int ActionStep_exists_end(ActionStep last, Bone bone, double vector, int time)
{
	struct _ActionStep step;
	if(last == NULL)
		return 0;
	for(; last->next_time != NULL; last = last->next_time);
	step = ActionStep_last_step(last, bone->start_angle, last->real_start_time + last->duration);
	last->next_time = &step;
	ActionStep_calculate(&step, bone, vector);
	last->next_time = NULL;
	return time <= step.real_start_time + step.duration;
}

int ActionStep_exists_action_after(ActionStep action, int time)
{
	if(action == NULL)
		return 0;
	for( ; action->next_time != NULL; action = action->next_time)
	{
		if(time < action->real_start_time)
			return 1;
	}
	if(time <= action->real_start_time + action->duration)
		return 1;
	return 0;
}

int ActionStep_save(FILE * flow, ActionStep action)
{
	unsigned int version = 1;
	unsigned char is_next = 0;
	if(flow == NULL || action == NULL)
	{
		fprintf(stderr, "Erreur sauvegarde step is NULL\n");
		return 0;
	}
	is_next = action->next_time != NULL;
	version <<= 8;
	version |= (action->next_time != NULL);
	version <<= 8;
	version |= 's';
	if(! fwrite(&version, sizeof(unsigned int), 1, flow))
	{
		fprintf(stderr, "Erreur sauvegarde step version\n");
		return 0;
	}
	if(! fwrite(&(action->target_angle), sizeof(double), 1, flow)
	|| ! fwrite(&(action->start_time), sizeof(int), 1, flow)
	|| ! fwrite(&is_next, sizeof(unsigned char), 1, flow))
	{
		fprintf(stderr, "Erreur sauvegarde step data\n");
		return 0;
	}
	
	if(action->next_time == NULL)
		return 1;
	return ActionStep_save(flow, action->next_time);
}

int ActionStep_load(FILE * flow, ActionStep * action)
{
	unsigned int version = 0;
	unsigned char is_next = 0;
	ActionStep res = NULL;
	if(flow == NULL || action == NULL)
		return 0;
	if(! fread(&version, sizeof(unsigned int), 1, flow))
	{
		fprintf(stderr, "Erreur chargement step version\n");
		return 0;
	}
	if((version & 0xff) != 's')
	{
		fprintf(stderr, "Erreur chargement step entete id, trouvé '%c', attendu 's'\n", version & 0xff);
		return 0;
	}
	version >>= 16;
	if((version & 0xff) != 1)
	{
		fprintf(stderr, "Erreur chargement step version %d non reconnue\n", version & 0xff);
		return 0;
	}
	if(! ActionStep_create(&res, 0, 0))
	{
		fprintf(stderr, "Erreur chargement step echec de création\n");
		return 0;
	}
	if(! fread(&(res->target_angle), sizeof(double), 1, flow)
	|| ! fread(&(res->start_time), sizeof(int), 1, flow)
	|| ! fread(&is_next, sizeof(unsigned char), 1, flow))
	{
		fprintf(stderr, "Erreur chargement step data\n");
		ActionStep_free(&res);
		return 0;
	}
	if(! is_next)
	{
		*action = res;
		return 1;
	}
	if(! ActionStep_load(flow, &(res->next_time)))
	{
		fprintf(stderr, "Erreur chargement step next_time\n");
		ActionStep_free(&res);
		return 0;
	}
	res->next_time->last_time = res;
	
	*action = res;
	return 1;
}

struct _ActionStep ActionStep_last_step(ActionStep last, double target_angle, int start_time)
{
	struct _ActionStep step;
	step.target_angle = target_angle;
	step.start_time = start_time;
	step.last_time = last;
	step.next_time = NULL;
	
	return step;
}

void ActionStep_reverse_angle(ActionStep action)
{
	if(action == NULL)
		return;
	action->target_angle = correct_degre_angle(-action->target_angle);
	ActionStep_reverse_angle(action->next_time);
}

int ActionStep_copy(ActionStep * copy, ActionStep action)
{
	ActionStep res = NULL;
	if(action == NULL)
	{
		*copy = NULL;
		return 1;
	}
	if(! ActionStep_create(&res, action->target_angle, action->start_time))
	{
		fprintf(stderr, "Erreur action_steps copie allocation\n");
		return 0;
	}
	res->duration = action->duration;
	res->real_start_time = action->real_start_time;
	res->vector_sign = action->vector_sign;
	if(! ActionStep_copy(&(res->next_time), action->next_time))
	{
		ActionStep_free(&res);
		return 0;
	}
	if(res->next_time != NULL)
		res->next_time->last_time = res;
	*copy = res;
	return 1;
}

/* Action Rotate actions */

int Action_append_rotate(Action * action, Bone bone, double target_angle, int start_time)
{
	Action action_bone = NULL;
	if(action == NULL)
		return 0;
	if((action_bone = Action_find(*action, bone)) == NULL)
	{
		if(! Action_append(action, bone))
			return 0;
		if((action_bone = Action_find(*action, bone)) == NULL)
			return 0;
	}
	if(! ActionStep_append(&(action_bone->steps), bone, target_angle, start_time))
		return 0;
	Action_calculate_rotate(action_bone);
	return 1;
}

int Action_delete_rotate(Action * action, Bone bone, int start_time)
{
	Action action_bone = NULL;
	ActionStep step = NULL;
	if(action == NULL)
		return 0;
	if((action_bone = Action_find(*action, bone)) == NULL)
		return 0;
	if((step = ActionStep_find_realtime(action_bone->steps, start_time, 0)) == NULL)
		return 0;
	if(! ActionStep_delete(&(action_bone->steps), step->start_time))
		return 0;
	if(action_bone->steps == NULL)
	{
		if(! Action_delete(action, action_bone->bone))
			return 0;
	}
	Action_calculate_rotate(action_bone);
	return 1;
}

ActionStep Action_find_rotate(Action action, Bone bone, int start_time)
{
	Action action_bone = NULL;
	if((action_bone = Action_find(action, bone)) == NULL)
		return NULL;
	return ActionStep_find(action_bone->steps, start_time);
}

int Action_calculate_rotate(Action action)
{
	if(action == NULL)
		return 0;
	return ActionStep_calculate(action->steps, action->bone, action->vector = Bone_get_power(action->bone));
}

int Action_calculate_rotate_full(Action action)
{
	int retour = 1;
	for( ; action != NULL; action = action->next_bone)
	{
		retour &= Action_calculate_rotate(action);
	}
	return retour;
}

int Action_make_action_rotate(Action action, int time)
{
	if(action == NULL)
		return 0;
	return ActionStep_make_action(action->steps, action->bone, action->vector, time);
}

int Action_make_action_rotate_full(Action action, int time)
{
	int retour = 0;
	for( ; action != NULL; action = action->next_bone)
		retour |= Action_make_action_rotate(action, time);
	return retour;
}

int Action_exists_action_rotate_after(Action action, int time)
{
	if(action == NULL)
		return 0;
	if(ActionStep_exists_action_after(action->steps, time))
		return 1;
	return ActionStep_exists_end(action->steps, action->bone, action->vector, time);
}

int Action_exists_action_rotate_full_after(Action action, int time)
{
	int retour = 0;
	for( ; action != NULL; action = action->next_bone)
		retour |= Action_exists_action_rotate_after(action, time);
	return retour;
}

void Action_reverse_angle(Action action)
{
	if(action == NULL)
		return;
	ActionStep_reverse_angle(action->steps);
	Action_reverse_angle(action->next_bone);
}

int Action_copy(Action * copy, Action action, Bone copy_arma)
{
	Action res = NULL;
	if(action == NULL)
	{
		*copy = NULL;
		return 1;
	}
	if(! Action_create(&res, BinPath_get_bone(action->bone->id, copy_arma)))
	{
		fprintf(stderr, "Erreur action copie allocation\n");
		return 0;
	}
	res->vector = action->vector;
	if(! ActionStep_copy(&(res->steps), action->steps))
	{
		fprintf(stderr, "Erreur action copie steps\n");
		Action_free(&res);
		return 0;
	}
	if(! Action_copy(&(res->next_bone), action->next_bone, copy_arma))
	{
		Action_free(&res);
		return 0;
	}
	*copy = res;
	return 1;
}

int Action_move_step(Action action, int time, int offset)
{
	ActionStep step = NULL;
	if((step = ActionStep_find_realtime(action->steps, time, 0)) == NULL)
		return 0;
	return 1;
}

int Action_move_start(Action action, int time, int offset)
{
	ActionStep step = NULL;
	if((step = ActionStep_find_realtime(action->steps, time, 0)) == NULL)
		return 0;
	return 1;
}

int Action_move_end(Action action, int time, int offset)
{
	ActionStep step = NULL;
	if((step = ActionStep_find_realtime(action->steps, time, 0)) == NULL)
		return 0;
	return 1;
}