#ifndef DEF_HEADER_ACTION
#define DEF_HEADER_ACTION

#include "armature.h"

typedef struct _ActionStep * ActionStep;
struct _ActionStep
{
	double target_angle;
	int start_time;
	ActionStep next_time;
	ActionStep last_time;
	
	/* calculated */
	int duration;
	int real_start_time;
	int vector_sign;
};

typedef struct _Action * Action;
struct _Action
{
	Bone bone;
	double vector;
	ActionStep steps;
	Action next_bone;
};

/* Action */

int Action_create(Action * action, Bone bone);

void Action_free(Action * action);

int Action_append(Action * action, Bone bone);

int Action_delete(Action * action, Bone bone);

Action Action_find(Action action, Bone bone);

int Action_save(FILE * flow, Action action);

int Action_load(FILE * flow, Action * action, Bone arma);

/* ActionSteps */

int ActionStep_create(ActionStep * step, double target_angle, int start_time);

void ActionStep_free(ActionStep * step);

int ActionStep_append(ActionStep * step, Bone bone, double target_angle, int start_time);

int ActionStep_delete(ActionStep * step, int start_time);

ActionStep ActionStep_find(ActionStep action, int start_time);

ActionStep ActionStep_find_realtime(ActionStep action, int time, int only_real_start_time);

int ActionStep_calculate(ActionStep action, Bone bone, double vector);

int ActionStep_make_action(ActionStep action, Bone bone, double vector, int time);

int ActionStep_make_end(ActionStep last, Bone bone, double vector, int time);

int ActionStep_exists_action_after(ActionStep action, int time);

int ActionStep_save(FILE * flow, ActionStep action);

int ActionStep_load(FILE * flow, ActionStep * action);

struct _ActionStep ActionStep_last_step(ActionStep last, double target_angle, int start_time);

void ActionStep_reverse_angle(ActionStep action);

int ActionStep_copy(ActionStep * copy, ActionStep action);

/* Action Rotate actions */

int Action_append_rotate(Action * action, Bone bone, double target_angle, int start_time);

int Action_delete_rotate(Action * action, Bone bone, int start_time);

ActionStep Action_find_rotate(Action action, Bone bone, int start_time);

int Action_calculate_rotate(Action action);

int Action_calculate_rotate_full(Action action);

int Action_make_action_rotate(Action action, int time);

int Action_make_action_rotate_full(Action action, int time);

int Action_exists_action_rotate_after(Action action, int time);

int Action_exists_action_rotate_full_after(Action action, int time);

void Action_reverse_angle(Action action);

int Action_copy(Action * copy, Action action, Bone copy_arma);

int Action_move_step(Action action, int time, int offset);

int Action_move_start(Action action, int time, int offset);

int Action_move_end(Action action, int time, int offset);

#endif