#include "action_control.h"
#include "color.h"

int ActionControl_init(ActionControl * control, Action * action)
{
	control->action = action;
	control->time = 0;
	control->bone = NULL;
	if(action != NULL)
		Action_calculate_rotate_full(*action);
	return 1;
}

int ActionControl_select_bone(ActionControl * control, Bone bone)
{
	control->bone = bone;
	return 1;
}

int ActionControl_add_rotate(ActionControl * control)
{
	if(control->bone == NULL)
		return 0;
	if(! Action_append_rotate(control->action, control->bone, control->bone->angle, control->time))
		return 0;
	return 1;
}

int ActionControl_delete_rotate(ActionControl * control)
{
	if(control->bone == NULL)
		return 0;
	if(! Action_delete_rotate(control->action, control->bone, control->time))
		return 0;
	return 1;
}

int ActionControl_set_time(ActionControl * control, int time)
{
	control->time = time;
	ActionControl_play_frame(control);
	return 1;
}

int ActionControl_play_frame(ActionControl * control)
{
	if(control->action == NULL)
		return 0;
	Action_make_action_rotate_full(*(control->action), control->time);
	return 1;
}

int ActionControl_continue(ActionControl * control)
{
	if(control->action == NULL)
		return 0;
	if(*(control->action) == NULL)
		return 0;
	return Action_exists_action_rotate_full_after(*(control->action), (control->time)++);
}

/* ActionControlDrawer */

int ActionControlDrawer_init(ActionControlDrawer * drawer, ActionControl * control, int x, int y, int width, int height, int steps)
{
	drawer->control = control;
	drawer->dim.x = x;
	drawer->dim.y = y;
	drawer->dim.w = width;
	drawer->dim.h = height;
	drawer->steps = steps;
	drawer->is_mouse_clic = 0;
	return 1;
}

void ActionControlDrawer_draw(ActionControlDrawer * drawer, SDL_Surface * screen)
{
	SDL_Rect rect;
	SDL_Color color;
	Action action_get = NULL;
	ActionStep action = NULL;
	
	if(drawer->control->action == NULL)
		return;
	
	SDL_FillRect(screen, &(drawer->dim), SDL_MapRGB(screen->format, 127, 127, 127));
	rect = drawer->dim;
	if((action_get = Action_find(*(drawer->control->action), drawer->control->bone)) != NULL)
	{
		for(action = action_get->steps; action != NULL; action = action->next_time)
		{
			rect.x = drawer->dim.x + action->real_start_time * drawer->steps;
			rect.w = action->duration * drawer->steps;
			if(drawer->control->time >= action->real_start_time && drawer->control->time < action->real_start_time + action->duration)
				color = SDL_Color_create(45, 165, 91);
			else
				color = SDL_Color_create(165, 45, 45);
			SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, color.r, color.g, color.b));
			rect.w = 4;
			if(drawer->control->time >= action->real_start_time && drawer->control->time < action->real_start_time + action->duration)
				color = SDL_Color_create(21, 77, 42);
			else
				color = SDL_Color_create(77, 21, 21);
			SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, color.r, color.g, color.b));
		}
	}
	rect.x = drawer->dim.x + drawer->control->time * drawer->steps;
	rect.w = 2;
	SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, 196, 196, 196));
}

int ActionControlDrawer_gest(ActionControlDrawer * drawer, SDL_Event * event, int * mouse_used)
{
	int x = 0, y = 0;
	if(drawer->control->action == NULL)
		return 0;
	if(mouse_used != NULL)
		*mouse_used = 0;
	switch(event->type)
	{
		case SDL_KEYUP:
		switch(event->key.keysym.sym)
		{
			case SDLK_a:
			return ActionControl_add_rotate(drawer->control);
			break;
			
			case SDLK_DELETE:
			return ActionControl_delete_rotate(drawer->control);
			break;
			
			default:
			break;
		}
		break;
		
		case SDL_MOUSEBUTTONDOWN:
		switch(event->button.button)
		{
			case SDL_BUTTON_RIGHT:
			x = event->button.x;
			y = event->button.y;
			if(in_SDL_Rect(x, y, drawer->dim))
				drawer->is_mouse_clic = 1;
			break;
		}
		break;
		
		case SDL_MOUSEBUTTONUP:
		switch(event->button.button)
		{
			case SDL_BUTTON_RIGHT:
			x = event->button.x;
			y = event->button.y;
			if(drawer->is_mouse_clic)
			{
				drawer->is_mouse_clic = 0;
				if(mouse_used != NULL)
					*mouse_used = 1;
			}
			break;
			
			default:
			break;
		}
		break;
		
		case SDL_MOUSEMOTION:
		x = event->motion.x;
		y = event->motion.y;
		break;
		
		default:
		break;
	}
	if(drawer->is_mouse_clic)
	{
		if(in_SDL_Rect(x, y, drawer->dim))
		{
			if(mouse_used != NULL)
				*mouse_used = 1;
			ActionControl_set_time(drawer->control, (x - drawer->dim.x) / drawer->steps);
			return 1;
		}
	}
	return 0;
}