#include "action_edit.h"

#include "math.h"
#include "draw.h"
#include "window.h"
#include "dialog.h"

#ifdef DEBUG
#include <SDL/SDL_gfxPrimitives.h>
#endif
#include <string.h>

static void ActionEdit_Select_draw(ActionEdit_Select select)
{
	clear_screen();
	Edit_print_background(get_screen());
	if(select->action_stats != NULL)
	{
		MovingVector_draw(&(select->action_stats->vector), get_screen(), SDL_Color_create(196, 196, 196));
	}
	(*(select->print_arma))(*(select->arma));
	ActionEdit_Select_draw_selection(select);
	ActionList_draw(select->action_list, get_screen());
	ActionControlDrawer_draw(&(select->control_drawer), get_screen());
	refresh_screen();
}

static int ActionEdit_set_mini_menu_background(Mini_Menu menu)
{
	SDL_Surface * rectangle = NULL;
	SDL_Surface * rectangle_hover = NULL;
	int success = 1;
	if((rectangle = SDL_CreateRGBSurface(SDL_HWSURFACE, 225, 25, 32, 0, 0, 0, 0)) == NULL)
		return 0;
	if(! SDL_CopySurface(&rectangle_hover, rectangle))
		return 0;
	SDL_FillRect(rectangle, NULL, SDL_MapRGB(rectangle->format, 255, 216, 0));
	SDL_FillRect(rectangle_hover, NULL, SDL_MapRGB(rectangle_hover->format, 255, 236, 127));
	success &= Mini_Menu_set_button_background(menu, rectangle, rectangle_hover);
	SDL_FreeSurface(rectangle);
	SDL_FreeSurface(rectangle_hover);
	return success;
}

static int ActionEdit_Select_append_action(ActionEdit_Select select)
{
	char * new_action = NULL;
	int size;
	int id = -1;
	if(! Dialog_global_ask_text(get_screen(), "Nom de la nouvelle action:", &new_action))
		return 0;
	if(new_action == NULL)
		return 1;
	size = strlen(new_action);
	if(size <= 0)
	{
		free(new_action);
		return 1;
	}
	if(size > 25)
	{
		Dialog_global_print(get_screen(), "Evitez les noms aussi longs pour une action");
		free(new_action);
		return 1;
	}
	if(! ActionList_append(select->action_list, new_action, &id))
	{
		fprintf(stderr, "ActionEdit : echec ajout action\n");
		free(new_action);
		return 1;
	}
	free(new_action);
	ActionEdit_Select_def_action(select, ActionList_get_action_adress(select->action_list, id));
	ActionControl_set_time(&(select->control), 0);
	select->action_stats = ActionList_get_action_stats(select->action_list, id);
	return 1;
}

static int ActionEdit_Select_delete_action(ActionEdit_Select select)
{
	if(select->action_stats == NULL)
		return 1;
	if(! ActionList_delete(select->action_list, ActionList_get_id_from_action_stats(select->action_list, select->action_stats)))
	{
		fprintf(stderr, "ActionEdit : echec suppression action\n");
		return 1;
	}
	ActionEdit_Select_def_action(select, NULL);
	select->action_stats = NULL;
	return 1;
}

static int ActionEdit_Select_append_vector(ActionEdit_Select select)
{
	if(select->action_stats == NULL)
		return 1;
	ActionEdit_Select_draw(select);
	MovingVector_graphic_define(&(select->action_stats->vector), get_screen());
	return 1;
}

static int ActionEdit_Select_delete_vector(ActionEdit_Select select)
{
	if(select->action_stats == NULL)
		return 1;
	MovingVector_define_direction(&(select->action_stats->vector), MVC_NULL);
	return 1;
}

static int ActionEdit_Select_rename_action(ActionEdit_Select select)
{
	char * new_name = NULL;
	int size;
	if(select->action_stats == NULL)
		return 1;
	if(! Dialog_global_ask_text(get_screen(), "Remplacer \"%s\" par :", &new_name, select->action_stats->name))
		return 0;
	if(new_name == NULL)
		return 1;
	size = strlen(new_name);
	if(size <= 0)
	{
		free(new_name);
		return 1;
	}
	if(size > 25)
	{
		Dialog_global_print(get_screen(), "Evitez les noms aussi longs pour une action");
		free(new_name);
		return 1;
	}
	free(select->action_stats->name);
	select->action_stats->name = new_name;
	if(! ActionList_make_menu(select->action_list))
		return 0;
	return 1;
}

static int ActionEdit_Select_define_key(ActionEdit_Select select)
{
	int cont = 1;
	int success = 1;
	int need_refresh = 1;
	SDL_Event event;
	Image back_screen = NULL;
	SDL_Rect rect;
	
	if(select->action_stats == NULL)
		return 1;
	if(! Image_create_from_surface_copy(&back_screen, get_screen()))
		return 0;
	rect.x = 0;
	rect.y = get_screen()->h / 2 - 25;
	rect.w = get_screen()->w;
	rect.h = 50;
	while(cont)
	{
		if(need_refresh)
		{
			Image_print_on_surface(get_screen(), back_screen);
			SDL_FillRect(get_screen(), &rect, SDL_MapRGB(get_screen()->format, 18, 57, 125));
			print_text(get_screen(), SDL_Color_create(255, 255, 255), "Touche : %s", SDL_GetKeyName(select->action_stats->key));
			refresh_screen();
			need_refresh = 0;
		}
		SDL_WaitEvent(&event);
		switch(event.type)
		{
			case SDL_QUIT:
			success = 0;
			break;
			
			case SDL_KEYUP:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
				cont = 0;
				break;
				
				default:
				select->action_stats->key = event.key.keysym.sym;
				Key_Gest_key_clean(&(select->action_stats->key_mod));
				need_refresh = 1;
				break;
			}
			break;
			
			default:
			break;
		}
	}
	Image_free(&back_screen);
	return success;
}

static int ActionEdit_Select_append_move(ActionEdit_Select select)
{
	return ActionControl_add_rotate(&(select->control));
}

static int ActionEdit_Select_delete_move(ActionEdit_Select select)
{
	return ActionControl_delete_rotate(&(select->control));
}

static int ActionEdit_Select_define_step_start(ActionEdit_Select select)
{
	/* TODO */
	return 1;
}

static int ActionEdit_Select_define_step_end(ActionEdit_Select select)
{
	/* TODO */
	return 1;
}

static int ActionEdit_Select_move_step(ActionEdit_Select select)
{
	/* TODO */
	return 1;
}

static int ActionEdit_Select_clear_start_angle(ActionEdit_Select select)
{
	if(select->selected_bone == NULL)
		return 1;
	Bone_set_start_angle(select->selected_bone, 0);
	Bone_set_angle(select->selected_bone, 0);
	return 1;
}

static int ActionEdit_Select_clear_moves_bone(ActionEdit_Select select)
{
	if(select->selected_bone == NULL || select->action_stats == NULL)
		return 1;
	Action_delete(&(select->action_stats->action), select->selected_bone);
	return 1;
}

int ActionEdit_Select_create(ActionEdit_Select * select, Personnage * perso)
{
	ActionEdit_Select res = NULL;
	if((res = (ActionEdit_Select)malloc(sizeof(struct _ActionEdit_Select))) == NULL)
		return 0;
	res->arma = &((*perso)->armature);
	res->selected_bone = NULL;
	res->mouse.x = res->mouse.y = 0;
	res->angle_start = 0;
	res->mouse_angle_start = 0;
	res->move_mode = ACTIONEDIT_SELECT_MOVEMODE_NULL;
	res->start_mouse.x = res->start_mouse.y = 0;
	res->selected_bone_start.x = res->selected_bone_start.y = 0;
	
	res->print_arma = &draw_action_armature;
	
	ActionEdit_Select_def_action(res, NULL);
	res->action_list = (*perso)->actions;
	res->action_menu = NULL;
	res->step_menu = NULL;
	res->bone_menu = NULL;
	res->none_menu = NULL;
	if(! Mini_Menu_create(&(res->action_menu))
	|| ! ActionEdit_set_mini_menu_background(res->action_menu)
	|| ! Mini_Menu_configure(res->action_menu, "media/arial.ttf", 20, SDL_Color_create(63, 0, 0), 7,
				"Ajouter action", &ActionEdit_Select_append_action,
				"Supprimer action", &ActionEdit_Select_delete_action,
				"Ajouter deplacement", &ActionEdit_Select_append_vector,
				"Supprimer deplacement", &ActionEdit_Select_delete_vector,
				"Renommer action", &ActionEdit_Select_rename_action,
				"Définir touche", &ActionEdit_Select_define_key,
				"Annuler", NULL)
	|| ! Mini_Menu_create(&(res->step_menu))
	|| ! ActionEdit_set_mini_menu_background(res->step_menu)
	|| ! Mini_Menu_configure(res->step_menu, "media/arial.ttf", 20, SDL_Color_create(63, 0, 0), 7,
				"Ajouter mouvement", &ActionEdit_Select_append_move,
				"Supprimer mouvement", &ActionEdit_Select_delete_move,
				"Définir début", &ActionEdit_Select_define_step_start,
				"Définir fin", &ActionEdit_Select_define_step_end,
				"Deplacer mouvement", &ActionEdit_Select_move_step,
				"Nettoyer mouvements", ActionEdit_Select_clear_moves_bone,
				"Annuler", NULL)
	|| ! Mini_Menu_create(&(res->bone_menu))
	|| ! ActionEdit_set_mini_menu_background(res->bone_menu)
	|| ! Mini_Menu_configure(res->bone_menu, "media/arial.ttf", 20, SDL_Color_create(63, 0, 0), 2,
				"Supprimer angle début", ActionEdit_Select_clear_start_angle,
				"Annuler", NULL)
	|| ! Mini_Menu_create(&(res->none_menu))
	|| ! ActionEdit_set_mini_menu_background(res->none_menu)
	|| ! Mini_Menu_configure(res->none_menu, "media/arial.ttf", 20, SDL_Color_create(63, 0, 0), 2,
				"Ajouter action", &ActionEdit_Select_append_action,
				"Annuler", NULL))
	{
		fprintf(stderr, "In the Action Editor: error in configuration of mini_menu\n");
		ActionEdit_Select_free(&res);
		return 0;
	}
	
	*select = res;
	return 1;
}

void ActionEdit_Select_free(ActionEdit_Select * select)
{
	if(select == NULL || *select == NULL)
		return;
	/*Action_free(&((*select)->action));*/
	Mini_Menu_free(&((*select)->action_menu));
	Mini_Menu_free(&((*select)->step_menu));
	Mini_Menu_free(&((*select)->bone_menu));
	Mini_Menu_free(&((*select)->none_menu));
	free(*select);
	*select = NULL;
}

int Action_edit(Personnage * perso, ActionEdit_Select select, EditExit * exit_choice, File_Explorer file_exp, FileSaver * file_saver)
{
	int cont = 1;
	int need_refresh = 1;
	int last_refresh_time = SDL_GetTicks() - REFRESH_TIME - 1;
	int success = 1;
	int mouse_used = 0;
	SDL_Event event;
	Bone * arma = NULL;
	ActionStats action_select = NULL;
	int (*menu_fct)(ActionEdit_Select) = NULL;
	Mini_Menu mini_menu = NULL;
	Key_Gest key_gest = Key_Gest_create();
	
	ActionEdit_Select_cancel_selection(select);
	
	if(perso == NULL || *perso == NULL)
	{
		fprintf(stderr, "Action Editor needs a caracter to define actions\n");
		return 0;
	}
	
	arma = &((*perso)->armature);
	select->arma = arma;
	select->action_list = (*perso)->actions;
	if(! Personnage_prepare(*perso))
	{
		fprintf(stderr, "Action Editor erreur de preparation du personnage\n");
		return 0;
	}
	Bone_make_init(*arma);
	ActionEdit_Select_def_action(select, NULL);
	ActionControl_set_time(&(select->control), 0);
	
	while(cont && success)
	{
		if(need_refresh && last_refresh_time + REFRESH_TIME < SDL_GetTicks())
		{
			ActionEdit_Select_draw(select);
			need_refresh = 0;
			last_refresh_time = SDL_GetTicks();
		}
		SDL_WaitEvent(&event);
		need_refresh |= ActionControlDrawer_gest(&(select->control_drawer), &event, &mouse_used);
		if(ActionList_get_event(select->action_list, &event, &action_select))
		{
			ActionEdit_Select_def_action(select, &(action_select->action));
			select->action_stats = action_select;
			action_select = NULL;
			need_refresh = 1;
		}
		
		switch(event.type)
		{
			case SDL_QUIT:
			cont = 0;
			if(exit_choice != NULL)
				*exit_choice = EDIT_EXIT_QUIT;
			break;
			
			case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
				cont = 0;
				if(exit_choice != NULL)
					*exit_choice = EDIT_EXIT_QUIT;
				break;
				
				case SDLK_LEFT:
				cont = 0;
				if(exit_choice != NULL)
					*exit_choice = EDIT_EXIT_GOTO_SHAPE_EDITOR;
				break;
				
				case SDLK_RIGHT:
				cont = 0;
				if(exit_choice != NULL)
					*exit_choice = EDIT_EXIT_GOTO_ARENA;
				break;
				
				default:
				break;
			}
			break;
			
			case SDL_KEYUP:
			switch(event.key.keysym.sym)
			{
				case SDLK_SPACE:
				if(select->action != NULL)
				{
					if(select->selected_bone != NULL)
						mini_menu = select->step_menu;
					else
						mini_menu = select->action_menu;
				}
				else
				{
					if(select->selected_bone != NULL)
						mini_menu = select->bone_menu;
					else
						mini_menu = select->none_menu;
				}
				Mini_Menu_set_origin(mini_menu, 25, 25);
				success &= Mini_Menu_gest(mini_menu, get_screen(), &menu_fct, &key_gest);
				if(menu_fct != NULL)
					success &= (*menu_fct)(select);
				menu_fct = NULL;
				mini_menu = NULL;
				need_refresh = 1;
				break;
				
				case SDLK_p:
				Bone_make_init(*arma);
				ActionControl_set_time(&(select->control), 0);
				do
				{
					ActionControl_play_frame(&(select->control));
					ActionEdit_Select_draw(select);
					SDL_Delay(10);
				} while(ActionControl_continue(&(select->control)));
				break;
				
				case SDLK_v:
				if(select->print_arma == &draw_action_armature)
					select->print_arma = &draw_action_armature_real;
				else
					select->print_arma = &draw_action_armature;
				need_refresh = 1;
				break;
				
				case SDLK_m:
				Bone_make_init(*arma);
				ActionEdit_Select_def_action(select, NULL);
				ActionControl_set_time(&(select->control), 0);
				
				ActionList_show_menu(select->action_list);
				need_refresh = 1;
				break;
				
				case SDLK_s:
				if(FileSaver_save(file_saver, file_exp, &key_gest, *perso, &FileSaver_Personnage_save))
					need_refresh = 1;
				break;
				
				case SDLK_l:
				if(FileSaver_load(file_saver, file_exp, &key_gest, perso, &FileSaver_Personnage_load))
				{
					ActionEdit_Select_cancel_selection(select);
					arma = &((*perso)->armature);
					select->arma = arma;
					select->action_list = (*perso)->actions;
					if(! Personnage_prepare(*perso))
						return 0;
					Bone_make_init(*arma);
					ActionEdit_Select_def_action(select, NULL);
					ActionControl_set_time(&(select->control), 0);
					arma = &((*perso)->armature);
					need_refresh = 1;
				}
				break;
				
				default:
				break;
			}
			break;
			
			case SDL_MOUSEBUTTONDOWN:
			switch(event.button.button)
			{
				case SDL_BUTTON_LEFT:
				need_refresh |= ActionEdit_Select_start_mouse_move(select, event.button.x, event.button.y, ACTIONEDIT_SELECT_MOVEMODE_ROTATION);
				break;
				
				default:
				break;
			}
			break;
			
			case SDL_MOUSEBUTTONUP:
			switch(event.button.button)
			{
				case SDL_BUTTON_RIGHT:
				if(mouse_used)
					break;
				if(ActionEdit_Select_is_mouse_move(select))
					break;
				need_refresh |= ActionEdit_Select_make_selection(select, event.button.x, event.button.y);
				break;
				
				case SDL_BUTTON_LEFT:
				need_refresh |= ActionEdit_Select_end_mouse_move(select);
				break;
			}
			break;
			
			case SDL_MOUSEMOTION:
			ActionEdit_Select_set_mouse(select, event.motion.x, event.motion.y);
			if(ActionEdit_Select_is_mouse_move(select))
				need_refresh = ActionEdit_Select_mouse_move_selection(select, event.motion.x, event.motion.y);
			break;
			
			default:
			break;
		}
		Key_Gest_event(&key_gest, &event);
	}
	
	return success;
}

/* Selection */

void ActionEdit_Select_draw_selection(ActionEdit_Select select)
{
	draw_action_select(*(select->arma), select->selected_bone);
}

int ActionEdit_Select_is_selection(ActionEdit_Select select)
{
	if(select == NULL)
		return 0;
	return select->selected_bone != NULL;
}

void ActionEdit_Select_make_selection_aux(ActionEdit_Select select, Bone bone, int x, int y, int b_x, int b_y, double angle, int * min_dist, int range)
{
	int distance;
	SDL_Rect end;
	SDL_Rect start;
	SDL_Rect origin;
	double old_angle;
	origin.x = origin.y = 0;
	
	if(bone == NULL)
		return;
	start = get_SDL_Rect_by_rotation(bone->start, origin, degre_to_rad(angle));
	b_x += (start.x - bone->start.x);
	b_y += (start.y - bone->start.y);
	old_angle = angle;
	angle = correct_degre_angle(angle + bone->angle);
	end = get_SDL_Rect_by_rotation(bone->end, bone->start, degre_to_rad(angle));
	#ifdef DEBUG
	lineRGBA(get_screen(), bone->start.x + b_x, bone->start.y + b_y, b_x + end.x, b_y + end.y, 63, 255, 63, 255);
	SDL_Flip(get_screen());
	SDL_Delay(1);
	#endif
	if((distance = distance_carre_segment(x, y, b_x + bone->start.x, b_y + bone->start.y, b_x + end.x, b_y + end.y)) < carre(range))
	{
		if(*min_dist == -1 || distance < *min_dist)
		{
			select->selected_bone = bone;
			*min_dist = distance;
		}
	}
	ActionEdit_Select_make_selection_aux(select, bone->brother, x, y, b_x - (start.x - bone->start.x), b_y - (start.y - bone->start.y), old_angle, min_dist, range);
	ActionEdit_Select_make_selection_aux(select, bone->child, x, y, b_x + end.x, b_y + end.y, angle, min_dist, range);
}

int ActionEdit_Select_make_selection(ActionEdit_Select select, int x, int y)
{
	int min_dist = -1;
	int canceled = 0;
	if(select == NULL)
		return 0;
	canceled |= ActionEdit_Select_cancel_selection(select);
	ActionEdit_Select_make_selection_aux(select, *(select->arma), x, y, 0, 0, 0, &min_dist, ACTION_EDIT_RANGE);
	ActionControl_select_bone(&(select->control), select->selected_bone);
	return ActionEdit_Select_is_selection(select) || canceled;
}

int ActionEdit_Select_cancel_selection(ActionEdit_Select select)
{
	if(select == NULL)
		return 0;
	if(select->selected_bone == NULL)
		return 0;
	select->selected_bone = NULL;
	ActionControl_select_bone(&(select->control), NULL);
	return 1;
}

void ActionEdit_Select_def_action(ActionEdit_Select select, Action * action)
{
	if(select == NULL)
		return;
	select->action = action;
	if(action == NULL)
		select->action_stats = NULL;
	else
		select->action_stats = ActionList_get_action_stats(select->action_list, ActionList_get_id_from_action(select->action_list, *action));
	ActionControl_init(&(select->control), action);
	ActionControlDrawer_init(&(select->control_drawer), &(select->control), 0, get_screen()->h - 50, get_screen()->w, 50, 8);
	ActionControl_select_bone(&(select->control), select->selected_bone);
}

/* MouseMove */

int ActionEdit_Select_set_mouse(ActionEdit_Select select, int x, int y)
{
	if(select == NULL)
		return 0;
	if(x == select->mouse.x && y == select->mouse.y)
		return 0;
	select->mouse.x = x;
	select->mouse.y = y;
	return 1;
}

int ActionEdit_Select_start_mouse_move(ActionEdit_Select select, int x, int y, ActionEdit_Select_MoveMode mode)
{
	int sx, sy;
	SDL_Rect start;
	SDL_Rect origin;
	origin.x = origin.y = 0;
	if(select == NULL)
		return 0;
	if(select->selected_bone == NULL)
	{
		mode = ACTIONEDIT_SELECT_MOVEMODE_NULL;
		return 0;
	}
	select->move_mode = mode;
	select->start_mouse.x = x;
	select->start_mouse.y = y;
	select->last_mouse.x = x;
	select->last_mouse.y = y;
	Bone_getXY_of_parent_with_angles(select->selected_bone, *(select->arma), &sx, &sy);
	start = get_SDL_Rect_by_rotation(select->selected_bone->start, origin, degre_to_rad(Bone_angle_to_bone(*(select->arma), select->selected_bone)));
	sx += start.x;
	sy += start.y;
	select->selected_bone_start.x = sx;
	select->selected_bone_start.y = sy;
	select->mouse_angle_start = rad_to_degre(angle_segment(sx, sy, x, y));
	select->angle_start = select->selected_bone->angle;
	return 1;
}

int ActionEdit_Select_end_mouse_move(ActionEdit_Select select)
{
	if(select == NULL)
		return 0;
	if(! ActionEdit_Select_is_mouse_move(select))
		return 0;
	select->move_mode = ACTIONEDIT_SELECT_MOVEMODE_NULL;
	return 1;
}

int ActionEdit_Select_is_mouse_move(ActionEdit_Select select)
{
	if(select == NULL)
		return 0;
	return select->move_mode != ACTIONEDIT_SELECT_MOVEMODE_NULL;
}

int ActionEdit_Select_mouse_move_selection(ActionEdit_Select select, int x, int y)
{
	double angle;
	if(! ActionEdit_Select_is_mouse_move(select))
		return 0;
	if(select->last_mouse.x == x && select->last_mouse.y == y)
		return 0;
	#ifdef DEBUG
	circleRGBA(get_screen(), select->selected_bone_start.x, select->selected_bone_start.y, 5, 63, 255, 63, 255);
	SDL_Flip(get_screen());
	#endif
	select->last_mouse.x = x;
	select->last_mouse.y = y;
	angle = correct_degre_angle(rad_to_degre(angle_segment(select->selected_bone_start.x, select->selected_bone_start.y, x, y)) - select->mouse_angle_start + select->angle_start);
	if(select->action == NULL)
		Bone_set_start_angle(select->selected_bone, angle);
	else
		Bone_set_angle(select->selected_bone, angle);
	return 1;
}