#ifndef DEF_HEADER_ACTION_LIST
#define DEF_HEADER_ACTION_LIST

#include <SDL/SDL.h>
#include "key_gest.h"
#include "action.h"
#include "menu.h"
#include "menu_list.h"
#include "moving_vector.h"

typedef struct _ActionStats * ActionStats;
struct _ActionStats
{
	Action action;
	char * name;
	SDLKey key;
	Key_Gest key_mod;
	MovingVector vector;
	int press_key;
};

typedef struct _ActionList * ActionList;
struct _ActionList
{
	ActionStats * actions;
	int nb_actions;
	int max_nb_actions;
	Menu menu;
	int show_menu;
};

/* ActionStats */

int ActionStats_create(ActionStats * stats, const char * name);

void ActionStats_free(ActionStats * stats);

void ActionStats_set_key(ActionStats stats, SDLKey key);

void ActionStats_set_key_mod(ActionStats stats, Key_Gest_Key key_mod);

void ActionStats_set_vector_direction(ActionStats stats, MovingVectorCoord direction);

int ActionStats_save(FILE * flow, ActionStats stats);

int ActionStats_load(FILE * flow, ActionStats * stats, Bone arma);

int ActionStats_copy(ActionStats * copy, ActionStats stats, Bone copy_arma);

/* ActionList */

int ActionList_create(ActionList * list);

void ActionList_free(ActionList * list);

void ActionList_clean(ActionList list);

int ActionList_make_menu(ActionList list);

int ActionList_append(ActionList list, const char * name, int * last_id);

int ActionList_delete(ActionList list, int id);

int ActionList_delete_bone(ActionList list, Bone bone);

Action * ActionList_get_action_adress(ActionList list, int id);

ActionStats ActionList_get_action_stats(ActionList list, int id);

int ActionList_get_id_from_action(ActionList list, Action action);

int ActionList_get_id_from_action_stats(ActionList list, ActionStats action);

int ActionList_get_id_from_name(ActionList list, const char * name);

int ActionList_get_id_from_keys(ActionList list, SDLKey key, Key_Gest * key_mod);

int ActionList_set_key(ActionList list, int id, SDLKey key);

int ActionList_set_key_mod(ActionList list, int id, Key_Gest_Key key_mod);

int ActionList_set_vector_direction(ActionList list, int id, MovingVectorCoord direction);

void ActionList_show_menu(ActionList list);

int ActionList_get_event(ActionList list, SDL_Event * event, ActionStats * get_action);

void ActionList_draw(ActionList list, SDL_Surface * screen);

int ActionList_save(FILE * flow, ActionList list);

int ActionList_load(FILE * flow, ActionList list, Bone arma);

int ActionList_save_in_file(const char * path, ActionList list);

int ActionList_load_in_file(const char * path, ActionList list, Bone arma);

int ActionList_add_default_actions(ActionList list);

int ActionList_calculate_rotate(ActionList list);

void ActionList_reverse_angle(ActionList list);

int ActionList_copy(ActionList * copy, ActionList list, Bone copy_arma);

#endif