#include "animation.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "image.h"

int Animation_create(Animation * anim)
{
	Animation res = NULL;
	
	if((res = (Animation)malloc(sizeof(struct _Animation))) == NULL)
		return 0;
	res->nb_elements = 0;
	res->types = NULL;
	res->gest_types = NULL;
	res->elems = NULL;
	res->print_functions = NULL;
	res->set_pos = NULL;
	res->translation = NULL;
	res->start_x = res->start_y = NULL;
	res->end_x = res->end_y = NULL;
	
	*anim = res;
	
	return 1;
}

int Animation_clear_elements(Animation anim)
{
	if(anim->nb_elements <= 0)
		return 0;
		
	free(anim->types);
	anim->types = NULL;
	free(anim->gest_types);
	anim->gest_types = NULL;
	free(anim->elems);
	anim->elems = NULL;
	free(anim->print_functions);
	anim->print_functions = NULL;
	free(anim->set_pos);
	anim->set_pos = NULL;
	free(anim->translation);
	anim->translation = NULL;
	free(anim->start_x);
	anim->start_x = NULL;
	free(anim->start_y);
	anim->start_y = NULL;
	free(anim->end_x);
	anim->end_x = NULL;
	free(anim->end_y);
	anim->end_y = NULL;
	
	anim->nb_elements = 0;
	
	return 1;
}

void Animation_free(Animation * anim)
{
	if(anim == NULL || *anim == NULL)
		return;
	Animation_clear_elements(*anim);
	free(*anim);
	*anim = NULL;
}

static int Animation_def_nb_elements(Animation anim, int nb_elements)
{
	if(anim == NULL)
		return 0;
	Animation_clear_elements(anim);
	
	if((anim->types = (Animation_Type *)malloc(nb_elements * sizeof(Animation_Type))) == NULL
	|| (anim->gest_types = (Animation_Gest_Type *)malloc(nb_elements * sizeof(Animation_Gest_Type))) == NULL
	|| (anim->elems = (void *)malloc(nb_elements * sizeof(void *))) == NULL
	|| (anim->print_functions = (void *)malloc(nb_elements * sizeof(void *))) == NULL
	|| (anim->set_pos = (void (**)(void *, int, int))malloc(nb_elements * sizeof(void (*)(void *, int, int)))) == NULL
	|| (anim->translation = (int (**)(int, int, int, int, int, int, int *, int *))malloc(nb_elements * sizeof(int (*)(int, int, int, int, int, int, int *, int *)))) == NULL
	|| (anim->start_x = (int *)calloc(nb_elements, sizeof(int))) == NULL
	|| (anim->start_y = (int *)calloc(nb_elements, sizeof(int))) == NULL
	|| (anim->end_x = (int *)calloc(nb_elements, sizeof(int))) == NULL
	|| (anim->end_y = (int *)calloc(nb_elements, sizeof(int))) == NULL)
	{
		Animation_clear_elements(anim);
		return 0;
	}
	anim->nb_elements = nb_elements;
	
	return 1;
}

static int Animation_define_va(Animation anim, int nb_elements, va_list ap)
{
	int i;
	int read_start;
	int read_end;
	int read_translation;
	Animation_Type type;
	Animation_Gest_Type gest_type;
	
	if(! Animation_def_nb_elements(anim, nb_elements))
		return 0;
	for(i = 0; i < nb_elements; ++i)
	{
		type = *(anim->types + i) = (Animation_Type)va_arg(ap, Animation_Type);
		gest_type = *(anim->gest_types + i) = (Animation_Gest_Type)va_arg(ap, Animation_Gest_Type);
		*(((char **)(anim->elems)) + i) = (char *)va_arg(ap, void *);
		*(((char **)(anim->print_functions)) + i) = (char *)va_arg(ap, void *);
		
		read_start = 0;
		read_end = 0;
		read_translation = 0;
		switch(type)
		{
			case ANIMATION_NULL:
			switch(gest_type)
			{
				case ANIMATION_GEST_INCLUDED_DATA:
				break;
				
				case ANIMATION_GEST_EXTERNAL_DATA:
				read_start = 1;
				break;
				
				default:
				break;
			}
			break;
			
			case ANIMATION_TRANSLATION:
			switch(gest_type)
			{
				case ANIMATION_GEST_INCLUDED_DATA:
				*(anim->set_pos + i) = (void (*)(void *, int, int))va_arg(ap, void (*)(void *, int, int));
				break;
				
				case ANIMATION_GEST_EXTERNAL_DATA:
				break;
				
				default:
				break;
			}
			read_start = 1;
			read_end = 1;
			read_translation = 1;
			break;
			
			default:
			return 0;
		}
		if(read_start)
		{
			*(anim->start_x + i) = (int)va_arg(ap, int);
			*(anim->start_y + i) = (int)va_arg(ap, int);
		}
		if(read_end)
		{
			*(anim->end_x + i) = (int)va_arg(ap, int);
			*(anim->end_y + i) = (int)va_arg(ap, int);
		}
		if(read_translation)
			*(anim->translation + i) = (int (*)(int, int, int, int, int, int, int *, int *))va_arg(ap, int (*)(int, int, int, int, int, int, int *, int *));
	}
	return 1;
}

int Animation_define(Animation anim, int nb_elements, ...)
{
	va_list ap;
	int retour;
	
	va_start(ap, nb_elements);
	retour = Animation_define_va(anim, nb_elements, ap);
	va_end(ap);
	return retour;
}

int Animation_gest_defined(SDL_Surface * screen, SDL_Surface * background, int nb_frames, int frame_time, Animation anim)
{
	SDL_Rect back_pos;
	int frame;
	int start_time;
	int i;
	int x, y;
	
	if(nb_frames <= 0)
		return 0;
	
	back_pos.x = back_pos.y = 0;
	
	for(frame = 0; frame <= nb_frames; ++frame)
	{
		start_time = SDL_GetTicks();
		
		if(background != NULL)
			SDL_BlitSurface(background, NULL, screen, &back_pos);
		
		for(i = 0; i < anim->nb_elements; ++i)
		{
			switch(*(anim->types + i))
			{
				case ANIMATION_NULL:
				switch(*(anim->gest_types + i))
				{
					case ANIMATION_GEST_INCLUDED_DATA:
					(**((void (**)(void *))(anim->print_functions) + i))((void *)(*(((char **)(anim->elems)) + i)));
					break;
					
					case ANIMATION_GEST_EXTERNAL_DATA:
					(**((void (**)(void *, int, int))(anim->print_functions) + i))((void *)(*(((char **)(anim->elems)) + i)), *(anim->start_x + i), *(anim->start_y + i));
					break;
					
					default:
					break;
				}
				break;
				
				case ANIMATION_TRANSLATION:
				if(*(anim->translation + i) == NULL || ! (**(anim->translation + i))(nb_frames, frame, *(anim->start_x + i), *(anim->start_y + i), *(anim->end_x + i), *(anim->end_y + i), &x, &y))
				{
					x = ((*(anim->start_x + i) * (nb_frames - frame)) + (*(anim->end_x + i) * frame)) / nb_frames;
					y = ((*(anim->start_y + i) * (nb_frames - frame)) + (*(anim->end_y + i) * frame)) / nb_frames;
				}
				switch(*(anim->gest_types + i))
				{
					case ANIMATION_GEST_INCLUDED_DATA:
					(**(anim->set_pos + i))((void *)(*(((char **)(anim->elems)) + i)), x, y);
					(**((void (**)(void *))(anim->print_functions) + i))((void *)(*(((char **)(anim->elems)) + i)));
					break;
					
					case ANIMATION_GEST_EXTERNAL_DATA:
					(**((void (**)(void *, int, int))(anim->print_functions) + i))((void *)(*(((char **)(anim->elems)) + i)), x, y);
					break;
					
					default:
					break;
				}
				break;
				
				default:
				break;
			}
		}
		
		SDL_Flip(screen);
		
		start_time = frame_time - (SDL_GetTicks() - start_time);
		if(start_time > 0)
			SDL_Delay(start_time);
	}
	
	return 1;
}

int Animation_gest(SDL_Surface * screen, SDL_Surface * background, int nb_frames, int frame_time, int nb_elements, ...)
{
	va_list ap;
	int retour;
	Animation anim = NULL;
	
	if(! Animation_create(&anim))
		return 0;
	
	va_start(ap, nb_elements);
	retour = Animation_define_va(anim, nb_elements, ap);
	va_end(ap);
	
	if(! retour)
		return 0;
	retour = Animation_gest_defined(screen, background, nb_frames, frame_time, anim);
	Animation_free(&anim);
	
	return retour;
}

int Animation_redefine_field(Animation anim, int elem_id, void * elem)
{
	if(anim == NULL || elem_id < 0 || elem_id >= anim->nb_elements)
		return 0;
	*(((char **)(anim->elems)) + elem_id) = (char *)elem;
	return 1;
}