#ifndef DEF_HEADER_ANIMATION
#define DEF_HEADER_ANIMATION

#include <SDL/SDL.h>

typedef enum
{
	ANIMATION_NULL,/* No animation, just print element */
	ANIMATION_TRANSLATION,
	NB_ANIMATION_TYPE
} Animation_Type;

typedef enum
{
	ANIMATION_GEST_INCLUDED_DATA,
	ANIMATION_GEST_EXTERNAL_DATA,
	NB_ANIMATION_GEST_TYPE
} Animation_Gest_Type;

typedef struct _Animation * Animation;
struct _Animation
{
	int nb_elements;
	Animation_Type * types;
	Animation_Gest_Type * gest_types;
	void * elems;
	void * print_functions;
	void (**set_pos)(void *, int x, int y);
	int (**translation)(int nb_frames, int current_frame, int start_x, int start_y, int end_x, int end_y, int * res_x, int * res_y);
	int * start_x;
	int * start_y;
	int * end_x;
	int * end_y;
};

int Animation_create(Animation * anim);

int Animation_clear_elements(Animation anim);

void Animation_free(Animation * anim);

/*
	each element start by its type, followed by its gest_type if needed, end by translation for no linear translation
	
	ANIMATION_NULL, ANIMATION_GEST_INCLUDED_DATA, void * elem,
	void (*print_elem)(void *)
	
	ANIMATION_NULL, ANIMATION_GEST_EXTERNAL_DATA, void * elem,
	void (*print_elem)(void *, int x, int y), int x, int y
	
	ANIMATION_TRANSLATION, ANIMATION_GEST_INCLUDED_DATA, void * elem,
	void (*print_elem)(void *), void (*set_pos)(void *, int x, int y),
	int start_x, int start_y, int end_x, int end_y,
	int (*translation)(int nb_frames, int current_frame, int start_x, int start_y, int end_x, int end_y, int * res_x, int * res_y)
	
	ANIMATION_TRANSLATION, ANIMATION_GEST_EXTERNAL_DATA, void * elem,
	void (*print_elem)(void *, int x, int y),
	int start_x, int start_y, int end_x, int end_y,
	int (*translation)(int nb_frames, int current_frame, int start_x, int start_y, int end_x, int end_y, int * res_x, int * res_y)
*/
int Animation_define(Animation anim, int nb_elements, ...);

int Animation_gest_defined(SDL_Surface * screen, SDL_Surface * background, int nb_frames, int frame_time, Animation anim);

/*
	Combine Animation_gest_defined with Animation_define
*/
int Animation_gest(SDL_Surface * screen, SDL_Surface * background, int nb_frames, int frame_time, int nb_elements, ...);

int Animation_redefine_field(Animation anim, int elem_id, void * elem);

#endif