#include "arena.h"

#include "draw.h"
#include "image.h"
#include "math.h"
#include <time.h>
#include <stdio.h>
#include <SDL/SDL_gfxPrimitives.h>

Image background = NULL;

int Arena_init_test(Arena * arena, Personnage player)
{
	if(player == NULL)
		return 0;
	arena->player = NULL;
	arena->adv = NULL;
	BoneTable_init(&(arena->player_bones), NULL);
	BoneTable_init(&(arena->adv_bones), NULL);
	if(! Personnage_copy(&(arena->player), player))
		return 0;
	return 1;
}

int Arena_init_fight(Arena * arena, Personnage player, Personnage adv)
{
	if(player == NULL || adv == NULL)
		return 0;
	arena->player = NULL;
	arena->adv = NULL;
	BoneTable_init(&(arena->player_bones), NULL);
	BoneTable_init(&(arena->adv_bones), NULL);
	if(! Personnage_copy(&(arena->player), player))
		return 0;
	if(! Personnage_copy(&(arena->adv), adv))
	{
		Arena_quit(arena);
		return 0;
	}
	if(! BoneTable_init(&(arena->player_bones), arena->player->armature)
	|| ! BoneTable_init(&(arena->adv_bones), arena->adv->armature))
	{
		Arena_quit(arena);
		return 0;
	}
	return 1;
}

void Arena_quit(Arena * arena)
{
	Personnage_free(&(arena->player));
	Personnage_free(&(arena->adv));
	BoneTable_quit(&(arena->player_bones));
	BoneTable_quit(&(arena->adv_bones));
}

void Arena_draw_test(Arena * arena)
{
	clear_screen();
	Edit_print_background(get_screen());
	Personnage_draw(arena->player);
	refresh_screen();
}

void Arena_draw_fight(Arena * arena)
{
	clear_screen();
	/*Edit_print_background(get_screen());*/
	Image_print_on_surface(get_screen(), background);
	Personnage_draw(arena->player);
	Personnage_draw(arena->adv);
	Arena_gest_collision(arena);
	refresh_screen();
}

int Arena_play_test(Arena * arena, EditExit * exit_choice)
{
	int cont = 1;
	int need_refresh = 1;
	int last_refresh_time = SDL_GetTicks() - REFRESH_TIME - 1;
	int success = 1;
	int ground;
	Key_Gest key_gest = Key_Gest_create();
	
	SDL_Event event;
	
	ground = get_screen()->h;
	Personnage_scale(arena->player, 0.5);
	Personnage_put_on_ground(arena->player, ground);
	
	while(cont && success)
	{
		if(need_refresh && last_refresh_time + REFRESH_TIME < SDL_GetTicks())
		{
			Arena_draw_test(arena);
			need_refresh = 0;
			last_refresh_time = SDL_GetTicks();
		}
		SDL_PollEvent(&event);
		switch(event.type)
		{
			case SDL_QUIT:
			cont = 0;
			if(exit_choice != NULL)
				*exit_choice = EDIT_EXIT_QUIT;
			break;
			
			case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
				default:
				need_refresh |= Personnage_start_action(arena->player, event.key.keysym.sym, &key_gest);
				break;
			}
			break;
			
			case SDL_KEYUP:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
				cont = 0;
				if(exit_choice != NULL)
					*exit_choice = EDIT_EXIT_GOTO_ACTION_EDITOR;
				break;
				
				default:
				need_refresh |= Personnage_end_action(arena->player, event.key.keysym.sym, &key_gest);
				break;
			}
			break;
			
			default:
			break;
		}
		need_refresh |= Personnage_play_action(arena->player);
		if(Personnage_check_ground(arena->player, ground))
			need_refresh |= Personnage_put_on_ground(arena->player, ground);
		Key_Gest_event(&key_gest, &event);
		SDL_Delay(1);
	}
	
	return 1;
}

int Arena_play_fight(Arena * arena)
{
	int cont = 1;
	int need_refresh = 1;
	int last_refresh_time = SDL_GetTicks() - REFRESH_TIME - 1;
	int success = 1;
	int ground;
	Key_Gest key_gest = Key_Gest_create();
	
	SDL_Event event;
	
	ground = get_screen()->h - GROUND;
	Personnage_put_on_ground(arena->player, ground);
	Personnage_put_on_ground(arena->adv, ground);
	
	while(cont && success)
	{
		if(need_refresh && last_refresh_time + REFRESH_TIME < SDL_GetTicks())
		{
			Arena_draw_fight(arena);
			need_refresh = 0;
			last_refresh_time = SDL_GetTicks();
		}
		SDL_PollEvent(&event);
		switch(event.type)
		{
			case SDL_QUIT:
			success = 0;
			break;
			
			case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
				default:
				need_refresh |= Personnage_start_action(arena->player, event.key.keysym.sym, &key_gest);
				need_refresh |= Personnage_start_action(arena->adv, event.key.keysym.sym, &key_gest);
				break;
			}
			break;
			
			case SDL_KEYUP:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
				cont = 0;
				break;
				
				default:
				need_refresh |= Personnage_end_action(arena->player, event.key.keysym.sym, &key_gest);
				need_refresh |= Personnage_end_action(arena->adv, event.key.keysym.sym, &key_gest);
				break;
			}
			break;
			
			default:
			break;
		}
		need_refresh |= Personnage_play_action(arena->player);
		need_refresh |= Personnage_play_action(arena->adv);
		if(Personnage_check_ground(arena->player, ground))
			need_refresh |= Personnage_put_on_ground(arena->player, ground);
		if(Personnage_check_ground(arena->adv, ground))
			need_refresh |= Personnage_put_on_ground(arena->adv, ground);
		BoneTable_gest(&(arena->player_bones));
		BoneTable_gest(&(arena->adv_bones));
		Key_Gest_event(&key_gest, &event);
		SDL_Delay(1);
	}
	
	return success;
}

static int Make_impact(Personnage perso, BoneTable * table)
{
	Bone hit = NULL;
	int i;
	for(i = 0; BoneTable_get(table, &hit, i); ++i)
	{
		if(Personnage_make_impacted(perso, hit))
		{
			/*printf("impact on %d\n", i);*/
			return 1;
		}
	}
	return 0;
}

int Arena_gest_collision(Arena * arena)
{
	SDL_Rect * player_dim = NULL;
	SDL_Rect * adv_dim = NULL;
	
	player_dim = &(arena->player->dims);
	adv_dim = &(arena->adv->dims);
	if(! box_collision(	player_dim->x, player_dim->y, player_dim->x + player_dim->w, player_dim->y + player_dim->h,
				adv_dim->x, adv_dim->y, adv_dim->x + adv_dim->w, adv_dim->y + adv_dim->h))
	{
		/*printf("No dim collision\n");*/
		return 0;
	}
	/*printf("dim collision\n");*/
	if(Personnage_can_hit(arena->player))
	{
		Make_impact(arena->adv, &(arena->player_bones));
		/*printf("Adv hit %d\n", SDL_GetTicks())*/
	}
	if(Personnage_can_hit(arena->adv))
	{
		Make_impact(arena->player, &(arena->adv_bones));
		/*printf("Player hit %d\n", SDL_GetTicks())*/
	}
	return 1;
}

static void make_right_pil(Sint16 x[20], Sint16 y[20])
{
	int i;
	x[0] = 100;				y[0] = get_screen()->h - GROUND;
	x[19] = 0;				y[19] = get_screen()->h - GROUND;
	for(i = 1; i < 10; ++i)
	{
		x[i] = 100 + i * 10;		y[i] = parabole_y(x[i], 100, 100 + 200, -get_screen()->h / 2) + get_screen()->h - 2 * GROUND;
		x[19 - i] = i * 20;		y[19 - i] = parabole_y(x[19 - i], 0, 400, -get_screen()->h / 2) + get_screen()->h - 2 * GROUND;
		x[i] += ((rand() % 10) - 5);
		x[19 - i] += ((rand() % 10) - 5);
		y[i] += ((rand() % 10) - 5);
		y[19 - i] += ((rand() % 10) - 5);
	}
	x[10] = 200;				y[10] = get_screen()->h - 2 * GROUND - get_screen()->h / 2;
}

int Arena_generate_background()
{
	Sint16 x[20];
	Sint16 y[20];
	int i;
	Arena_free_background();
	if(! Image_create(&background))
		return 0;
	if(! SDL_GetRectangleRGB(&(background->surface), get_screen()->w, get_screen()->h, 0, 0, 0))
	{
		Arena_free_background();
		return 0;
	}
	Image_fillHorizontalDegrade(background, 0, get_screen()->h, 28, 15, 10, 207, 78, 30, NULL);
	Image_fillCircleDegrade(background, get_screen()->w / 2, get_screen()->h / 3, get_screen()->h / 4, 228, 158, 17);
	
	srand(time(NULL));
	for(i = 0; i < 16; ++i)
	{
		x[i] = 100 + ((get_screen()->w - 200) * i) / 16 + ((rand() % 20) - 10);
		y[i] = get_screen()->h - 2 * GROUND + ((rand() % 20) - 10);
	}
	x[16] = get_screen()->w;		y[16] = get_screen()->h - 2 * GROUND;
	x[17] = get_screen()->w;		y[17] = get_screen()->h;
	x[18] = 0;				y[18] = get_screen()->h;
	x[19] = 0;				y[19] = get_screen()->h - 2 * GROUND;
	
	filledPolygonRGBA(background->surface, x, y, 20, 77, 36, 26, 255);
	
	make_right_pil(x, y);
	filledPolygonRGBA(background->surface, x, y, 20, 77, 36, 26, 255);
	
	make_right_pil(x, y);
	for(i = 0; i < 20; ++i)
		x[i] = get_screen()->w - x[i];
	filledPolygonRGBA(background->surface, x, y, 20, 77, 36, 26, 255);
	
	return 1;
}

void Arena_free_background()
{
	Image_free(&background);
}