#ifndef DEF_HEADER_ARMATURE
#define DEF_HEADER_ARMATURE

#include <SDL/SDL.h>
#include "chain_list.h"
#include "shape.h"
#include "bin_path.h"
#include "image.h"
#include "hitbox.h"

#define MINIMUM_DENY_ANGLE_AREA 30
#define HIT_DELAY 20

/*
	if bone is the main bone, coordinates are absolute
	else coordinates are relatives to parent end
*/
typedef struct _Bone * Bone;
struct _Bone
{
	/* Main data */
	
	SDL_Rect start;/* Start coords of the bone */
	SDL_Rect end;/* End coords of the bone */
	double angle;/* Current angle of the bone */
	double start_angle;/* Defined default angle to start from */
	Bone child;/* adress of child bone */
	Bone brother;/* adress of brother bone */
	double angle_min;/* value of minimum angle */
	double deny_angle_area;/* size of not allow area angle under angle_min*/
	BinPath id;/* id of the bone in its armature*/
	ChainList prop_list;
	ShapeSet shape;
	int in_front_of_parent;
	
	/* Calculated data */
	Image shape_image;
	HitBox hitbox;
	
	SDL_Rect real_end;
	SDL_Rect last_real_end;
	SDL_Rect real_start;
	double real_angle;
	
	int hit_delay;
};

typedef enum
{
	BONEPROP_MIRROR,
	NB_BONEPROP
} BoneProp_Type;

typedef struct _BonePropriety BoneProp;
struct _BonePropriety
{
	BoneProp_Type type;
	union
	{
		Bone mirror_bone;
	};
};

/* Creation functions */

int Bone_create(Bone * bone);

void Bone_free(Bone * bone);

/* Edition functions */

int Bone_set_start(Bone bone, int x, int y);

int Bone_set_end(Bone bone, int x, int y);

int Bone_add_start(Bone bone, int vx, int vy);

int Bone_add_end(Bone bone, int vx, int vy);

int Bone_move_to(Bone bone, int x, int y);

int Bone_set_start_with_mirror(Bone bone, int x, int y);

int Bone_set_end_with_mirror(Bone bone, int x, int y);

int Bone_add_start_with_mirror(Bone bone, int vx, int vy);

int Bone_add_end_with_mirror(Bone bone, int vx, int vy);

void Bone_set_real_end(Bone bone, int x, int y);

void Bone_set_real_start(Bone bone, int x, int y);

void Bone_set_real_angle(Bone bone, double angle);

double Bone_creation_angle(Bone bone);

int Bone_in_deny_angle_area(Bone bone, double angle);

int Bone_set_angle(Bone bone, double angle);

int Bone_set_start_angle(Bone bone, double angle);

int Bone_vector_sign(Bone bone, double start_angle, double target_angle);

int Bone_set_angle_min(Bone bone, double angle_min);

int Bone_set_deny_angle_area(Bone bone, double deny_angle_area);

int Bone_append(Bone * bone, Bone * appened);

int Bone_delete(Bone * bone, Bone del);

int Bone_getXY_of_parent(Bone bone, Bone arma, int * x, int * y);

int Bone_getXY_of_parent_with_angles(Bone bone, Bone arma, int * x, int * y);

double Bone_angle_to_bone(Bone arma, Bone target);

int Bone_get_parent(Bone bone, Bone arma, Bone * parent);

Bone * Bone_adress_in_armature(Bone bone, Bone * arma);

double correct_area(double area);

int Bone_goto_front(Bone bone, Bone * arma);

int Bone_goto_back(Bone bone, Bone * arma);

int Bone_reparent(Bone * arma, Bone bone, Bone new_parent);

int Bone_scale(Bone bone, double factor);

int Bone_get_dim(Bone bone, int * x_min, int * y_min, int * x_max, int * y_max);

void Bone_reverse(Bone bone);

int Bone_copy(Bone * copy, Bone bone);

/* Save functions */

int Armature_save(FILE * flow, const Bone arma);

int Armature_load(FILE * flow, Bone * arma);

int Armature_save_in_file(const char * path, const Bone arma);

int Armature_load_from_file(const char * path, Bone * arma);

/* BinPath */

int Bone_include_in(Bone bone, Bone arma);

int BinPath_calculate(Bone arma);

int BinPath_calculate_from(Bone arma);

Bone BinPath_get_bone(BinPath id, Bone arma);

/*
	current is supposed to be on the path given by id
*/
Bone BinPath_get_next(BinPath id, Bone current);

Bone Bone_get_next(Bone target, Bone current);

void Bone_aff_debug_id(Bone bone);

int Bone_cmp(const void * a, const void * b);

/* BoneProp */

int ChainList_create_BoneProp(ChainList * list);

int BoneProp_create(BoneProp * prop, BoneProp_Type type, void * value);

int BoneProp_create_forlist(void * elem_adress, void * value);

void BoneProp_free_forlist(void * elem_adress);

int BoneProp_affect_forlist(void * dest_adress, void * src);

int BoneProp_eq_forlist(void * in_list, void * elem);

int BoneProp_eq_is_mirror(void * in_list, void * elem);

/* BoneProp use */

Bone Bone_get_mirror(Bone bone);

int Bone_make_mirror(Bone bone, Bone mirror);

int Bone_break_mirror(Bone bone);

int Bone_add_mirror(Bone bone, Bone parent);

/* Fight functions */

void Bone_make_init(Bone bone);

double Bone_get_power(Bone bone);

double Bone_get_vector_distance_carre(Bone bone);

int Bone_make_hitbox(Bone bone);

void Bone_rotate_hitbox(Bone bone);

int Bone_get_fight_dimensions(Bone bone, int * x_min, int * y_min, int * x_max, int * y_max);

int Bone_make_impacted(Bone bone, Bone hit);

#endif