#include "armature_editor.h"

#include "armature.h"
#include "window.h"
#include "draw.h"
#include "mini_menu.h"
#include "color.h"
#include "image.h"
#include "key_gest.h"
#include "math.h"
#include "menu.h"
#include "file_explorer.h"
#include "menu_list.h"
#include "dialog.h"
#include "data_debug_print.h"

#define switch_values(tmp, a, b)	(tmp) = (a);\
					(a) = (b);\
					(b) = (tmp);

static SDL_Surface * last_screen = NULL;
static SDL_Surface * screen_background = NULL;

static void Armature_edit_draw(ArmaEdit_Select * selector, Bone arma)
{
	clear_screen();
	Edit_print_background(get_screen());
	draw_armature(arma);
	ArmaEdit_Select_draw(selector, arma);
	refresh_screen();
}

static int Mini_Menu_set_armaedit_background(Mini_Menu menu)
{
	SDL_Surface * rectangle = NULL;
	SDL_Surface * rectangle_hover = NULL;
	int success = 1;
	if((rectangle = SDL_CreateRGBSurface(SDL_HWSURFACE, 100, 25, 32, 0, 0, 0, 0)) == NULL)
		return 0;
	if(! SDL_CopySurface(&rectangle_hover, rectangle))
		return 0;
	SDL_FillRect(rectangle, NULL, SDL_MapRGB(rectangle->format, 255, 216, 0));
	SDL_FillRect(rectangle_hover, NULL, SDL_MapRGB(rectangle_hover->format, 255, 236, 127));
	success &= Mini_Menu_set_button_background(menu, rectangle, rectangle_hover);
	SDL_FreeSurface(rectangle);
	SDL_FreeSurface(rectangle_hover);
	return success;
}

static int Menu_armaedit_define(Menu menu)
{
	SDL_Surface * rectangle = NULL;
	SDL_Surface * rectangle_hover = NULL;
	int success = 1;
	if((rectangle = SDL_CreateRGBSurface(SDL_HWSURFACE, 300, 50, 32, 0, 0, 0, 0)) == NULL)
		return 0;
	if(! SDL_CopySurface(&rectangle_hover, rectangle))
		return 0;
	if(screen_background != NULL)
		SDL_FreeSurface(screen_background);
	if((screen_background = SDL_CreateRGBSurface(SDL_HWSURFACE, get_screen()->w, get_screen()->h, 32, 0, 0, 0, 0)) == NULL)
		return 0;
	SDL_FillRect(rectangle, NULL, SDL_MapRGB(rectangle->format, 255, 216, 0));
	SDL_FillRect(rectangle_hover, NULL, SDL_MapRGB(rectangle_hover->format, 255, 236, 127));
	SDL_FillRect(screen_background, NULL, SDL_MapRGB(screen_background->format, 0, 0, 0));
	Menu_set_origin(menu, get_screen()->w, 0);
	success &= Menu_define_Menu_List(menu, "media/arial.ttf", 25, SDL_Color_create(21, 21, 21), rectangle, rectangle_hover);
	SDL_FreeSurface(rectangle);
	SDL_FreeSurface(rectangle_hover);
	return success;
}

static int ArmaEdit_Select_getXY(const ArmaEdit_Select * select, const Bone arma, int * x, int * y)
{
	if(select == NULL)
		return 0;
	if(! Bone_getXY_of_parent(select->selected_bone, arma, x, y))
		return 0;
	switch(select->select_type)
	{
		case ARMAEDIT_SELECT_START:
		if(x != NULL)
			*x += select->selected_bone->start.x;
		if(y != NULL)
			*y += select->selected_bone->start.y;
		return 1;
		
		case ARMAEDIT_SELECT_END:
		if(x != NULL)
			*x += select->selected_bone->end.x;
		if(y != NULL)
			*y += select->selected_bone->end.y;
		return 1;
		
		case ARMAEDIT_SELECT_BONE:
		if(x != NULL)
			*x += ((select->selected_bone->start.x + select->selected_bone->end.x) / 2);
		if(y != NULL)
			*y += ((select->selected_bone->start.y + select->selected_bone->end.y) / 2);
		return 1;
		
		default:
		break;
	}
	
	return 0;
}

static int ArmaEdit_Select_getX(const ArmaEdit_Select * select, const Bone arma)
{
	int x = 0;
	if(! ArmaEdit_Select_getXY(select, arma, &x, NULL))
		return 0;
	return x;
}

static int ArmaEdit_Select_getY(const ArmaEdit_Select * select, const Bone arma)
{
	int y = 0;
	if(! ArmaEdit_Select_getXY(select, arma, NULL, &y))
		return 0;
	return y;
}

static int ArmaEdit_Select_get_startXY(ArmaEdit_Select * select, const Bone arma, int * x, int * y)
{
	ArmaEdit_Select_Type type;
	int retour = 0;
	if(select == NULL)
		return 0;
	type = select->select_type;
	select->select_type = ARMAEDIT_SELECT_START;
	retour = ArmaEdit_Select_getXY(select, arma, x, y);
	select->select_type = type;
	return retour;
}

static int ArmaEdit_Select_get_startX(ArmaEdit_Select * select, const Bone arma)
{
	int x = 0;
	if(! ArmaEdit_Select_get_startXY(select, arma, &x, NULL))
		return 0;
	return x;
}

static int ArmaEdit_Select_get_startY(ArmaEdit_Select * select, const Bone arma)
{
	int y = 0;
	if(! ArmaEdit_Select_get_startXY(select, arma, NULL, &y))
		return 0;
	return y;
}

static int ArmaEdit_Select_reparent_bone(ArmaEdit_Select * select, Bone * arma, ArmaEdit_MouseMove * mouse_move, Key_Gest * key_gest)
{
	if(! ArmaEdit_Select_is_selection(select))
		return 0;
	if(! ArmaEdit_Select_is_selection(select->next))
	{
		Dialog_global_print(get_screen(), "Selectionnez l'os sur lequel reparenter");
		return 1;
	}
	if(! Bone_reparent(arma, select->selected_bone, select->next->selected_bone))
	{
		if(Bone_include_in(select->next->selected_bone, select->selected_bone))
		{
			Dialog_global_print(get_screen(), "Pas possible de reparenter l'os avec un de ses enfants");
			return 1;
		}
		return 0;
	}
	return 1;
}

static int ArmaEdit_Select_make_mirror(ArmaEdit_Select * select, Bone * arma, ArmaEdit_MouseMove * mouse_move, Key_Gest * key_gest)
{
	Bone parent = NULL;
	Bone tmp = NULL;
	
	if(! ArmaEdit_Select_is_selection(select))
		return 0;
	if(select->next == NULL)
	{
		if(select->selected_bone == *arma)
			return 1;
		if(Bone_get_mirror(select->selected_bone) != NULL)
		{
			return Bone_break_mirror(select->selected_bone);
		}
		if(! Bone_get_parent(select->selected_bone, *arma, &parent))
			return 0;
		if(! Key_Gest_key_in(key_gest, KGK_CTRL) && (tmp = Bone_get_mirror(parent)) != NULL)
			parent = tmp;
		if(! Bone_add_mirror(select->selected_bone, parent))
			return 0;
	}
	else
	{
		Bone first = select->selected_bone;
		Bone second = select->next->selected_bone;
		Bone mirror = NULL;
		if(first == *arma || second == *arma)
			return 1;
		if((mirror = Bone_get_mirror(first)) != NULL)
		{
			if(mirror == second)
				return Bone_break_mirror(first);
			if(! Bone_break_mirror(first))
				return 0;
		}
		if(Bone_get_mirror(second) != NULL)
		{
			if(! Bone_break_mirror(second))
				return 0;
		}
		if(! Bone_make_mirror(first, second))
			return 0;
	}
	return 1;
}

static int ArmaEdit_Select_append_bone_to_select(ArmaEdit_Select * select, Bone * arma, ArmaEdit_MouseMove * mouse_move, Key_Gest * key_gest)
{	
	if(! ArmaEdit_Select_is_selection(select))
		return 0;
	if(select->select_type != ARMAEDIT_SELECT_START)
	{
		Bone new_selected_bone = NULL;
		Bone * select_adress = NULL;
	
		if((select_adress = Bone_adress_in_armature(select->selected_bone, arma)) == NULL)
			return 0;
		if(! Bone_append(select_adress, &new_selected_bone))
			return 0;
			
		select->selected_bone = new_selected_bone;
		BinPath_calculate(*arma);
	}
	else
	{
		Bone * bone = NULL;
		Bone link = NULL;
		if((bone = Bone_adress_in_armature(select->selected_bone, arma)) == NULL)
			return 0;
		if(! Bone_create(&link))
			return 0;
		Bone_set_start(link, (*bone)->start.x, (*bone)->start.y);
		Bone_set_end(link, (*bone)->start.x, (*bone)->start.y);
		Bone_add_end(select->selected_bone, - (*bone)->start.x, - (*bone)->start.y);
		Bone_set_start(select->selected_bone, 0, 0);
		link->brother = (*bone)->brother;
		link->child = *bone;
		(*bone)->brother = NULL;
		*bone = link;
		BinPath_calculate(*arma);
		
		select->selected_bone = link;
	}
	select->select_type = ARMAEDIT_SELECT_END;
	/*
		Ajout extrude adaptée à selection
	*/
	if(! ArmaEdit_Select_make_ordered_selection(select))
		return 0;
	ArmaEdit_MouseMove_set_start_selection(mouse_move, select, mouse_move->mouse.x, mouse_move->mouse.y);
	return 1;
}

static int ArmaEdit_Select_delete_selected_bone(ArmaEdit_Select * select, Bone * arma, ArmaEdit_MouseMove * mouse_move, Key_Gest * key_gest)
{
	if(! ArmaEdit_Select_is_selection(select))
		return 0;
	if(select->selected_bone == *arma)
	{
		Dialog_global_print(get_screen(), "L'os de départ ne peut pas être supprimé");
		return 1;
	}
	if(! ActionList_delete_bone((*(select->perso))->actions, select->selected_bone))
		return 0;
	if(! Bone_delete(arma, select->selected_bone))
		return 0;
	ArmaEdit_MouseMove_stop(mouse_move);
	ArmaEdit_Select_cancel_selection(select);
	return 1;
}

static int Bone_apply_link_to_parent(Bone bone, Bone arma, Key_Gest * key_gest)
{
	int start_x, start_y, end_x, end_y;
	start_x = bone->start.x;
	start_y = bone->start.y;
	end_x = bone->end.x;
	end_y = bone->end.y;
	if(Key_Gest_key_in(key_gest, KGK_SHIFT))
	{
		Bone parent = NULL;
		if(! Bone_get_parent(bone, arma, &parent))
			return 0;
		if(! Key_Gest_key_in(key_gest, KGK_CTRL))
		{
			end_x = (parent->start.x - parent->end.x) + (bone->end.x - bone->start.x);
			end_y = (parent->start.y - parent->end.y) + (bone->end.y - bone->start.y);
		}
		start_x = parent->start.x - parent->end.x;
		start_y = parent->start.y - parent->end.y;
	}
	else
	{
		if(! Key_Gest_key_in(key_gest, KGK_CTRL))
		{
			end_x -= bone->start.x;
			end_y -= bone->start.y;
		}
		start_x = 0;
		start_y = 0;
	}
	Bone_set_start(bone, start_x, start_y);
	Bone_set_end(bone, end_x, end_y);
	
	return 1;
}

static int ArmaEdit_Select_link_selected_bone_to_parent(ArmaEdit_Select * select, Bone * arma, ArmaEdit_MouseMove * mouse_move, Key_Gest * key_gest)
{
	Bone * tab = NULL;
	int size;
	int i;
	if(! ArmaEdit_Select_is_selection(select))
		return 0;
	
	if(select->next == NULL)
	{
		tab = &(select->selected_bone);
		size = 1;
	}
	else
	{
		tab = select->ordered_bones;
		size = select->nb_ordered_bones;
	}
	for(i = 0; i < size; ++i)
	{
		if(*(tab + i) == *arma)
			continue;
		if(! Bone_apply_link_to_parent(*(tab + i), *arma, key_gest))
			return 0;
		if(! Key_Gest_key_in(key_gest, KGK_ALT))
		{
			Bone mirror = NULL;
			if((mirror = Bone_get_mirror(*(tab + i))) != NULL)
			{
				if(! Bone_apply_link_to_parent(mirror, *arma, key_gest))
					return 0;
			}
		}
	}
	/*ArmaEdit_MouseMove_stop(mouse_move);
	ArmaEdit_Select_cancel_selection(select);*/
	
	return 1;
}



static int ArmaEdit_Select_move_angle_area(ArmaEdit_Select * select, Bone * arma, ArmaEdit_MouseMove * mouse_move, Key_Gest * key_gest)
{
	int cont = 1;
	int need_refresh = 1;
	int last_refresh_time = SDL_GetTicks() - REFRESH_TIME - 1;
	SDL_Event event;
	
	if(! ArmaEdit_Select_is_selection(select))
		return 0;
	while(cont)
	{
		if(need_refresh && last_refresh_time + REFRESH_TIME < SDL_GetTicks())
		{
			clear_screen();
			Edit_print_background(get_screen());
			draw_armature(*arma);
			ArmaEdit_Select_draw_angle(select, *arma);
			refresh_screen();
			need_refresh = 0;
			last_refresh_time = SDL_GetTicks();
		}
		SDL_WaitEvent(&event);
		switch(event.type)
		{
			case SDL_QUIT:
			goto exit_zone;
			
			case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
				goto exit_zone;
				
				default:
				break;
			}
			break;
			
			case SDL_MOUSEBUTTONUP:
			switch(event.button.button)
			{
				case SDL_BUTTON_RIGHT:
				cont = 0;
				break;
				
				case SDL_BUTTON_LEFT:
				need_refresh = 1;
				ArmaEdit_MouseMove_stop(mouse_move);
				ArmaEdit_Select_cancel_angle_selection(select);
				break;
			}
			break;
			
			case SDL_MOUSEBUTTONDOWN:
			switch(event.button.button)
			{
				case SDL_BUTTON_LEFT:
				if(ArmaEdit_MouseMove_is_active(mouse_move))
					break;
				need_refresh = 1;
				ArmaEdit_Select_cancel_angle_selection(select);
				if(! ArmaEdit_Select_make_angle_selection(select, event.button.x, event.button.y, SELECTION_RANGE))
					break;
				ArmaEdit_MouseMove_angle_set_start(mouse_move, select, ArmaEdit_Select_get_startX(select, *arma), ArmaEdit_Select_get_startY(select, *arma), event.button.x, event.button.y);
				break;
			}
			break;
			
			case SDL_MOUSEMOTION:
			if(ArmaEdit_MouseMove_is_active(mouse_move))
			{
				need_refresh |= ArmaEdit_MouseMove_move_angle_selection(mouse_move, select, key_gest, event.motion.x, event.motion.y);
				break;
			}
			ArmaEdit_MouseMove_set_mouse(mouse_move, event.motion.x, event.motion.y);
			break;
		}
		Key_Gest_event(key_gest, &event);
	}
	
	ArmaEdit_Select_cancel_angle_selection(select);
	ArmaEdit_MouseMove_stop(mouse_move);
		
	return 1;
	
	exit_zone:;
	
	return 0;
}

static int ArmaEdit_Select_bone_go_front(ArmaEdit_Select * select, Bone * arma, ArmaEdit_MouseMove * mouse_move, Key_Gest * key_gest)
{
	if(! ArmaEdit_Select_is_selection(select))
		return 1;
	if(! Bone_goto_front(select->selected_bone, arma))
		Dialog_global_print(get_screen(), "L'os est déjà devant ses frères et son parent");
	return 1;
}

static int ArmaEdit_Select_bone_go_back(ArmaEdit_Select * select, Bone * arma, ArmaEdit_MouseMove * mouse_move, Key_Gest * key_gest)
{
	if(! ArmaEdit_Select_is_selection(select))
		return 1;
	if(! Bone_goto_back(select->selected_bone, arma))
		Dialog_global_print(get_screen(), "L'os est déjà derrière ses frères et son parent");
	return 1;
}

int (*get_edit_function(SDLKey key))(ArmaEdit_Select *, Bone *, ArmaEdit_MouseMove *, Key_Gest *)
{
	switch(key)
	{
		case SDLK_e:
		return &ArmaEdit_Select_append_bone_to_select;
		
		case SDLK_DELETE:
		case SDLK_BACKSPACE:
		return &ArmaEdit_Select_delete_selected_bone;
		
		case SDLK_p:
		return &ArmaEdit_Select_link_selected_bone_to_parent;
		
		case SDLK_g:
		return &ArmaEdit_Select_move_angle_area;
		
		case SDLK_m:
		return &ArmaEdit_Select_make_mirror;
		
		case SDLK_r:
		return &ArmaEdit_Select_reparent_bone;
		
		case SDLK_UP:
		return &ArmaEdit_Select_bone_go_front;
		
		case SDLK_DOWN:
		return &ArmaEdit_Select_bone_go_back;
		
		default:
		break;
	}
	return NULL;
}

int ArmaEdit_create(ArmaEdit * data, Bone * arma)
{
	if(data == NULL || arma == NULL)
		return 0;
	data->selector = ArmaEdit_Select_create(arma);
	data->mouse_move = ArmaEdit_MouseMove_create();
	data->mini_menu = NULL;
	data->menu = NULL;
	data->anims = NULL;
	
	if(! Mini_Menu_create(&(data->mini_menu)))
	{
		fprintf(stderr, "In the Armature Editor: error in creation of mini_menu\n");
		goto free_zone;
	}
	if(! Menu_create(&(data->menu)))
	{
		fprintf(stderr, "In the Armature Editor: error in creation of menu\n");
		goto free_zone;
	}
	
	if(! Menu_armaedit_define(data->menu)
	|| ! Menu_configure(data->menu, 3,
				NULL, MENU_ELEMENT_TEXT, "Save", NULL,
				NULL, MENU_ELEMENT_TEXT, "Save as", NULL,
				NULL, MENU_ELEMENT_TEXT, "Load", NULL))
	{
		fprintf(stderr, "In the Armature Editor: error in configuration of menu\n");
		goto free_zone;
	}
	
	if(! Mini_Menu_set_armaedit_background(data->mini_menu)
	|| ! Mini_Menu_configure(data->mini_menu, "media/arial.ttf", 20, SDL_Color_create(63, 0, 0), 8,
				"Ajouter", &ArmaEdit_Select_append_bone_to_select,
				"Supprimer", &ArmaEdit_Select_delete_selected_bone,
				"Relier", &ArmaEdit_Select_link_selected_bone_to_parent,
				"Zone angle", &ArmaEdit_Select_move_angle_area,
				"Miroir", &ArmaEdit_Select_make_mirror,
				"Avancer", &ArmaEdit_Select_bone_go_front,
				"Reculer", &ArmaEdit_Select_bone_go_back,
				"Reparenter", &ArmaEdit_Select_reparent_bone,
				"Annuler", NULL))
	{
		fprintf(stderr, "In the Armature Editor: error in configuration of mini_menu\n");
		goto free_zone;
	}
	if(! ArmaEdit_define_animations(&(data->anims), data->menu))
	{
		fprintf(stderr, "In the Armature Editor: error in definition of animations\n");
		goto free_zone;
	}
	return 1;
	
	free_zone:;
	ArmaEdit_destroy(data);
	return 0;
}

void ArmaEdit_destroy(ArmaEdit * data)
{
	if(data == NULL)
		return;
	Mini_Menu_free(&(data->mini_menu));
	Menu_free(&(data->menu));
	ArmaEdit_delete_animations(&(data->anims));
	ArmaEdit_Select_destroy(&(data->selector));
}

int Armature_edit(Personnage * perso, ArmaEdit * data, EditExit * exit_choice, File_Explorer file_exp, FileSaver * file_saver)
{
	int cont = 1;
	int success = 1;
	int need_refresh = 1;
	int last_refresh_time = SDL_GetTicks() - REFRESH_TIME - 1;
	SDL_Event event;
	ArmaEdit_Select selector;
	ArmaEdit_MouseMove mouse_move;
	Mini_Menu mini_menu = NULL;
	int (*mini_menu_function)(ArmaEdit_Select *, Bone *, ArmaEdit_MouseMove *, Key_Gest *) = NULL;
	Key_Gest key_gest = Key_Gest_create();
	#if 0
	void * menu_function = NULL;
	Menu menu = NULL;
	Animation * anims = NULL;
	#endif
	ArmaEdit alternative;
	Bone * arma = NULL;
	
	last_screen = NULL;
	
	if(perso == NULL || *perso == NULL)
	{
		fprintf(stderr, "Armature Editor need a target caracter to edit it\n");
		return 0;
	}
	arma = &((*perso)->armature);
	
	if(data == NULL)
	{
		if(! ArmaEdit_create(&alternative, arma))
			goto free_zone;
		data = &alternative;
	}
	selector = data->selector;
	selector.perso = perso;
	mouse_move = data->mouse_move;
	mini_menu = data->mini_menu;
	#if 0
	menu = data->menu;
	anims = data->anims;
	#endif
	ArmaEdit_Select_cancel_selection(&selector);
	
	while(cont && success)
	{
		if(need_refresh && last_refresh_time + REFRESH_TIME < SDL_GetTicks())
		{
			Armature_edit_draw(&selector, *arma);
			need_refresh = 0;
			last_refresh_time = SDL_GetTicks();
		}
		
		SDL_WaitEvent(&event);
		switch(event.type)
		{
			case SDL_QUIT:
			if(exit_choice != NULL)
				*exit_choice = EDIT_EXIT_QUIT;
			cont = 0;
			break;
			
			case SDL_KEYUP:
			switch(event.key.keysym.sym)
			{
				case SDLK_SPACE:
				if(! ArmaEdit_Select_is_selection(&selector))
				{
					#if 0
					int id;
					if(! ArmaEdit_play_animation(anims, ARMAEDIT_ANIMATION_MENU_START))
					{
						fprintf(stderr, "In the Armature Editor: animation for menu start cannot be played\n");
						success = 0;
						break;
					}
					if(Menu_gest(menu, get_screen(), &menu_function, &key_gest, MENU_OPTION_GET_ID, &id, MENU_OPTION_END))
					{
						#ifdef DEBUG
						printf("Menu choice: %d\n", id);
						#endif
						
					}
					if(id == -1)
					{
						if(! ArmaEdit_play_animation(anims, ARMAEDIT_ANIMATION_MENU_QUIT))
						{
							fprintf(stderr, "In the Armature Editor: animation for menu quit cannot be played\n");
							success = 0;
							break;
						}
					}
					need_refresh = 1;
					#endif
					break;
				}
				if(ArmaEdit_MouseMove_is_active(&mouse_move))
					break;
				Mini_Menu_set_origin_on_screen(mini_menu, ArmaEdit_Select_getX(&selector, *arma), ArmaEdit_Select_getY(&selector, *arma), get_screen());
				success &= Mini_Menu_gest(mini_menu, get_screen(), &mini_menu_function, &key_gest);
				if(mini_menu_function != NULL)
					success &= (*mini_menu_function)(&selector, arma, &mouse_move, &key_gest);
				mini_menu_function = NULL;
				need_refresh = 1;
				break;
				
				case SDLK_s:
				/*if(Key_Gest_key_in(&key_gest, KGK_CTRL))
				{
					if(file_name == NULL || Key_Gest_key_in(&key_gest, KGK_SHIFT))
						File_Explorer_gest_event(file_exp, get_screen(), FILE_EXPLORER_MODE_SAVE, &file_name);
					if(file_name != NULL)
					{
						if(! Personnage_save_in_file(file_name, *perso))
							Dialog_global_print(get_screen(), "Echec de sauvegarde du personnage %s", file_name);
					}
					need_refresh = 1;
					Key_Gest_key_clean(&key_gest);
				}*/
				/*if(Armature_save_in_file("media/arma.arma", *arma))
					printf("Save: success\n");*/
				need_refresh |= FileSaver_save(file_saver, file_exp, &key_gest, *perso, &FileSaver_Personnage_save);
				/*DP_Personnage(perso);*/
				break;
				
				case SDLK_l:
				/*if(Key_Gest_key_in(&key_gest, KGK_CTRL))
				{
					file_name = NULL;
					File_Explorer_gest_event(file_exp, get_screen(), FILE_EXPLORER_MODE_LOAD, &file_name);
					if(file_name != NULL)
					{
						ArmaEdit_Select_cancel_angle_selection(&selector);
						if(! Personnage_load_from_file(file_name, perso))
							Dialog_global_print(get_screen(), "Echec du chargement du personnage %s", file_name);
						else
						{
							need_refresh = 1;
							arma = &((*perso)->armature);
						}
					}
					need_refresh = 1;
					Key_Gest_key_clean(&key_gest);
				}*/
				if(FileSaver_load(file_saver, file_exp, &key_gest, perso, &FileSaver_Personnage_load))
				{
					ArmaEdit_Select_cancel_angle_selection(&selector);
					selector.arma = arma = &((*perso)->armature);
					need_refresh = 1;
				}
				/*DP_Personnage(perso);*/
				/*if(need_refresh |= Armature_load_from_file("media/arma.arma", arma))
					printf("Load: success\n");*/
				break;
				
				case SDLK_z:
				Bone_aff_debug_id(*arma);
				break;
				
				case SDLK_a:
				need_refresh |= ArmaEdit_Select_select_all(&selector, arma);
				break;
				
				default:
				if((mini_menu_function = get_edit_function(event.key.keysym.sym)) != NULL)
				{
					if(! ArmaEdit_Select_is_selection(&selector))
						break;
					if(ArmaEdit_MouseMove_is_active(&mouse_move))
						break;
					success &= (*mini_menu_function)(&selector, arma, &mouse_move, &key_gest);
					mini_menu_function = NULL;
					need_refresh = 1;
				}
				break;
			}
			break;
			
			case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
				if(exit_choice != NULL)
					*exit_choice = EDIT_EXIT_QUIT;
				cont = 0;
				break;
				
				case SDLK_RIGHT:
				cont = 0;
				if(exit_choice != NULL)
					*exit_choice = EDIT_EXIT_GOTO_SHAPE_EDITOR;
				break;
				
				case SDLK_LEFT:
				cont = 0;
				if(exit_choice != NULL)
					*exit_choice = EDIT_EXIT_GOTO_GAME;
				break;
				
				default:
				break;
			}
			break;
			
			case SDL_MOUSEBUTTONUP:
			ArmaEdit_MouseMove_set_mouse(&mouse_move, event.button.x, event.button.y);
			switch(event.button.button)
			{
				case SDL_BUTTON_RIGHT:
				if(ArmaEdit_MouseMove_is_active(&mouse_move))
					break;
				if(Key_Gest_key_in(&key_gest, KGK_CTRL))
				{
					need_refresh |= ArmaEdit_Select_add_selection(&selector, *arma, event.button.x, event.button.y, SELECTION_RANGE);
					break;
				}
				if(ArmaEdit_Select_is_selection(&selector))
					need_refresh |= ArmaEdit_Select_cancel_selection(&selector);
				need_refresh |= ArmaEdit_Select_make_selection(&selector, *arma, event.button.x, event.button.y, SELECTION_RANGE);
				break;
				
				case SDL_BUTTON_LEFT:
				ArmaEdit_MouseMove_stop(&mouse_move);
				break;
			}
			break;
			
			case SDL_MOUSEBUTTONDOWN:
			ArmaEdit_MouseMove_set_mouse(&mouse_move, event.button.x, event.button.y);
			switch(event.button.button)
			{
				case SDL_BUTTON_LEFT:
				if(ArmaEdit_MouseMove_is_active(&mouse_move))
					break;
				if(! ArmaEdit_Select_is_selection(&selector))
					break;
				ArmaEdit_MouseMove_set_start_selection(&mouse_move, &selector, event.button.x, event.button.y);
				break;
			}
			break;
			
			case SDL_MOUSEMOTION:
			if(ArmaEdit_MouseMove_is_active(&mouse_move))
			{
				need_refresh |= ArmaEdit_MouseMove_move_selection(&mouse_move, &selector, &key_gest, event.motion.x, event.motion.y);
				break;
			}
			ArmaEdit_MouseMove_set_mouse(&mouse_move, event.motion.x, event.motion.y);
			break;
			
			default:
			break;
		}
		Key_Gest_event(&key_gest, &event);
	}
	free_zone:;
	
	data->selector = selector;
	data->mouse_move = mouse_move;
	if(data == &alternative)
		ArmaEdit_destroy(data);
	if(last_screen != NULL)
		SDL_FreeSurface(last_screen);
	last_screen = NULL;
	if(screen_background != NULL)
		SDL_FreeSurface(screen_background);
	screen_background = NULL;
	
	return success;
}

/* Selection */

ArmaEdit_Select ArmaEdit_Select_create(Bone * arma)
{
	ArmaEdit_Select selector = {arma, NULL, ARMAEDIT_SELECT_NULL, ARMAEDIT_SELECT_ANGLE_NULL, NULL, NULL, 0};
	return selector;
}

int ArmaEdit_Select_createAllocated(ArmaEdit_Select ** select, ArmaEdit_Select val)
{
	ArmaEdit_Select * res = NULL;
	if(select == NULL)
		return 0;
	if((res = (ArmaEdit_Select *)malloc(sizeof(ArmaEdit_Select))) == NULL)
		return 0;
	*res = val;
	res->ordered_bones = NULL;
	res->nb_ordered_bones = 0;
	*select = res;
	return 1;
}

void ArmaEdit_Select_destroy(ArmaEdit_Select * select)
{
	if(select == NULL)
		return;
		
	select->selected_bone = NULL;
	ArmaEdit_Select_clear_ordered_selection(select);
	ArmaEdit_Select_free(&(select->next));
}

void ArmaEdit_Select_free(ArmaEdit_Select ** select)
{
	if(select == NULL || *select == NULL)
		return;
	
	ArmaEdit_Select_clear_ordered_selection(*select);	
	ArmaEdit_Select_free(&((*select)->next));
	free(*select);
	*select = NULL;
}

static ArmaDraw_Part convert_ArmaEdit_Select_Type_in_ArmaDraw_Part(ArmaEdit_Select_Type type)
{
	switch(type)
	{
		case ARMAEDIT_SELECT_NULL:
		return ARMADRAW_PART_NULL;
		
		case ARMAEDIT_SELECT_START:
		return ARMADRAW_PART_START;
		
		case ARMAEDIT_SELECT_END:
		return ARMADRAW_PART_END;
		
		case ARMAEDIT_SELECT_BONE:
		return ARMADRAW_PART_BONE;
		
		case NB_ARMAEDIT_SELECT:
		return NB_ARMADRAW_PART;
	}
	return ARMADRAW_PART_NULL;
}

static ArmaDraw_Part convert_ArmaEdit_SelectAngle_Type_in_ArmaDraw_Part(ArmaEdit_SelectAngle_Type type)
{
	switch(type)
	{
		case ARMAEDIT_SELECT_ANGLE_NULL:
		return ARMADRAW_PART_NULL;
		
		case ARMAEDIT_SELECT_ANGLE_START:
		return ARMADRAW_PART_START;
		
		case ARMAEDIT_SELECT_ANGLE_END:
		return ARMADRAW_PART_END;
		
		case ARMAEDIT_SELECT_ANGLE_AREA:
		return ARMADRAW_PART_BONE;
		
		case NB_ARMAEDIT_SELECT_ANGLE:
		return NB_ARMADRAW_PART;
	}
	return ARMADRAW_PART_NULL;
}

void ArmaEdit_Select_draw(const ArmaEdit_Select * selector, Bone arma)
{
	const ArmaEdit_Select * start = selector;
	Bone mirror = NULL;
	if(selector == NULL || arma == NULL || selector->selected_bone == NULL)
		return;
	if((mirror = Bone_get_mirror(start->selected_bone)) != NULL)
		draw_in_armature_color(arma, mirror, convert_ArmaEdit_Select_Type_in_ArmaDraw_Part(start->select_type), convert_ArmaEdit_SelectAngle_Type_in_ArmaDraw_Part(start->angle_type), SDL_Color_create(63, 157, 95), SDL_Color_create(149, 44, 38), SDL_Color_create(149, 44, 38));
	for(selector = selector->next; selector != NULL; selector = selector->next)
		draw_in_armature_color(arma, selector->selected_bone, convert_ArmaEdit_Select_Type_in_ArmaDraw_Part(selector->select_type), convert_ArmaEdit_SelectAngle_Type_in_ArmaDraw_Part(selector->angle_type), SDL_Color_create(255, 108, 0), SDL_Color_create(149, 44, 38), SDL_Color_create(149, 44, 38));
	draw_in_armature(arma, start->selected_bone, convert_ArmaEdit_Select_Type_in_ArmaDraw_Part(start->select_type), convert_ArmaEdit_SelectAngle_Type_in_ArmaDraw_Part(start->angle_type));
}

void ArmaEdit_Select_draw_angle(const ArmaEdit_Select * selector, Bone arma)
{
	const ArmaEdit_Select * start = selector;
	Bone mirror = NULL;
	if(selector == NULL || arma == NULL || selector->selected_bone == NULL)
		return;
	if((mirror = Bone_get_mirror(start->selected_bone)) != NULL)
		draw_in_armature_color(arma, mirror, convert_ArmaEdit_Select_Type_in_ArmaDraw_Part(start->select_type), convert_ArmaEdit_SelectAngle_Type_in_ArmaDraw_Part(start->angle_type), SDL_Color_create(63, 157, 95), SDL_Color_create(149, 44, 38), SDL_Color_create(149, 44, 38));
	for(selector = selector->next; selector != NULL; selector = selector->next)
		draw_in_armature_color(arma, selector->selected_bone, convert_ArmaEdit_Select_Type_in_ArmaDraw_Part(selector->select_type), convert_ArmaEdit_SelectAngle_Type_in_ArmaDraw_Part(selector->angle_type), SDL_Color_create(127, 54, 0), SDL_Color_create(38, 76, 134), SDL_Color_create(38, 76, 134));
	draw_in_armature_color(arma, start->selected_bone, convert_ArmaEdit_Select_Type_in_ArmaDraw_Part(start->select_type), convert_ArmaEdit_SelectAngle_Type_in_ArmaDraw_Part(start->angle_type), SDL_Color_create(255, 108, 0), SDL_Color_create(17, 97, 217), SDL_Color_create(159, 189, 235));
}

int ArmaEdit_Select_is_selection(const ArmaEdit_Select * selector)
{
	if(selector == NULL)
		return 0;
	return selector->selected_bone != NULL && selector->select_type != ARMAEDIT_SELECT_NULL;
}

int ArmaEdit_Select_cancel_selection(ArmaEdit_Select * selector)
{
	if(selector == NULL || selector->selected_bone == NULL)
		return 0;
	selector->selected_bone = NULL;
	selector->select_type = ARMAEDIT_SELECT_NULL;
	ArmaEdit_Select_clear_ordered_selection(selector);
	ArmaEdit_Select_free(&(selector->next));
	return 1;
}

static void Select_make_selection_aux(int * min_dist, Bone * nearest_bone, ArmaEdit_Select_Type * select_type, Bone arma, int e_x, int e_y, int x, int y, int range)
{
	int distance = -1;
	if(arma == NULL)
		return;
	
	if((distance = distance_carre(x, y, arma->end.x + e_x, arma->end.y + e_y)) < carre(range)) /* priority is given to the end of bone because it moves its children */
	{
		if(*min_dist == -1 || distance < *min_dist)
		{
			*select_type = ARMAEDIT_SELECT_END;
			*min_dist = distance;
			*nearest_bone = arma;
		}
	}
	if((distance = distance_carre(x, y, arma->start.x + e_x, arma->start.y + e_y)) < carre(range))
	{
		if(*min_dist == -1 || distance < *min_dist)
		{
			*select_type = ARMAEDIT_SELECT_START;
			*min_dist = distance;
			*nearest_bone = arma;
		}
	}
	if(*min_dist == -1 || (*select_type != ARMAEDIT_SELECT_START && *select_type != ARMAEDIT_SELECT_END)) /* priority is given to start or end instead of whole bone */
	{
		if((distance = distance_carre_segment(x, y, arma->start.x + e_x, arma->start.y + e_y, arma->end.x + e_x, arma->end.y + e_y)) < carre(range))
		{
			if(*min_dist == -1 || distance < *min_dist)
			{
				*select_type = ARMAEDIT_SELECT_BONE;
				*min_dist = distance;
				*nearest_bone = arma;
			}
		}
	}
	
	Select_make_selection_aux(min_dist, nearest_bone, select_type, arma->child, e_x + arma->end.x, e_y + arma->end.y, x, y, range);
	Select_make_selection_aux(min_dist, nearest_bone, select_type, arma->brother, e_x, e_y, x, y, range);
}

int ArmaEdit_Select_make_selection(ArmaEdit_Select * selector, Bone arma, int x, int y, int range)
{
	int min_dist = -1;
	Bone nearest_bone = NULL;
	ArmaEdit_Select_Type select_type;
	
	if(selector == NULL)
		return  0;
	
	selector->angle_type = ARMAEDIT_SELECT_ANGLE_NULL;	
	Select_make_selection_aux(&min_dist, &nearest_bone, &select_type, arma, 0, 0, x, y, range);
	
	if(nearest_bone == NULL)
		return 0;
		
	selector->selected_bone = nearest_bone;
	selector->select_type = select_type;
	if(! ArmaEdit_Select_make_ordered_selection(selector))
		return 0;
	
	return 1;
}

static void ArmaEdit_Select_switch(ArmaEdit_Select * a, ArmaEdit_Select * b)
{
	Bone * tmp_arma = NULL;
	Bone tmp_bone = NULL;
	ArmaEdit_Select_Type tmp_select_type;
	ArmaEdit_SelectAngle_Type tmp_angle_type;
	if(a == NULL || b == NULL)
		return;
	switch_values(tmp_arma, a->arma, b->arma);
	switch_values(tmp_bone, a->selected_bone, b->selected_bone);
	switch_values(tmp_select_type, a->select_type, b->select_type);
	switch_values(tmp_angle_type, a->angle_type, b->angle_type);
}

int ArmaEdit_Select_add_selection(ArmaEdit_Select * selector, Bone arma, int x, int y, int range)
{
	ArmaEdit_Select res;
	ArmaEdit_Select * start = NULL;
	ArmaEdit_Select * has_next = NULL;
	if(selector == NULL)
		return 0;
	start = selector;
	if(! ArmaEdit_Select_is_selection(selector))
		return ArmaEdit_Select_make_selection(selector, arma, x, y, range);
	res = ArmaEdit_Select_create(selector->arma);
	if(! ArmaEdit_Select_make_selection(&res, arma, x, y, range))
		goto fail_zone;
	for(; selector != NULL; selector = selector->next)
	{
		has_next = selector;
		if(res.selected_bone == selector->selected_bone)
		{
			if(selector != start && selector->select_type == res.select_type)
			{
				ArmaEdit_Select_switch(selector, start);
				goto success_zone;
			}
			if((selector->select_type | res.select_type) != ARMAEDIT_SELECT_BONE)
				goto fail_zone;
			selector->select_type = ARMAEDIT_SELECT_BONE;
			goto success_zone;
		}
	}
	if(! ArmaEdit_Select_createAllocated(&(has_next->next), res))
		goto fail_zone;
	if(! ArmaEdit_Select_make_ordered_selection(start))
		goto fail_zone;
	
	success_zone:;
	
	ArmaEdit_Select_destroy(&res);
	return 1;
	
	fail_zone:;
	
	ArmaEdit_Select_destroy(&res);
	return 0;
}

int ArmaEdit_Select_get_ordered_selection(ArmaEdit_Select * select, Bone ** tab, int * size)
{
	int taille;
	int i;
	ArmaEdit_Select * start = select;
	Bone * res = NULL;
	
	for(taille = 0; select != NULL; select = select->next, ++taille);
	
	if(taille == 0)
	{
		if(tab != NULL)
			*tab = NULL;
		if(size != NULL)
			*size = 0;
		return 1;
	}
	if((res = (Bone *)malloc(taille * sizeof(Bone))) == NULL)
		return 0;
	for(i = 0, select = start; select != NULL; select = select->next, ++i)
		*(res + i) = select->selected_bone;
	qsort(res, taille, sizeof(Bone), &Bone_cmp);
	if(tab != NULL)
		*tab = res;
	else
		free(res);
	if(size != NULL)
		*size = taille;
	return 1; 
}

int ArmaEdit_Select_select_all_aux(Bone cur, ArmaEdit_Select ** select_cur, Bone * arma)
{
	ArmaEdit_Select val = ArmaEdit_Select_create(arma);
	if(cur == NULL)
		return 1;
	val.select_type = ARMAEDIT_SELECT_BONE;
	val.selected_bone = cur;
	if(! ArmaEdit_Select_createAllocated(select_cur, val))
		return 0;
	if(! ArmaEdit_Select_select_all_aux(cur->brother, &((*select_cur)->next), arma))
		return 0;
	for(; *select_cur != NULL; select_cur = &((*select_cur)->next));
	if(! ArmaEdit_Select_select_all_aux(cur->child, select_cur, arma))
		return 0;
	return 1;
}

int ArmaEdit_Select_select_all(ArmaEdit_Select * selector, Bone * arma)
{
	if(ArmaEdit_Select_is_selection(selector))
		return ArmaEdit_Select_cancel_selection(selector);
	selector->select_type = ARMAEDIT_SELECT_BONE;
	selector->selected_bone = *arma;
	if(! ArmaEdit_Select_select_all_aux((*arma)->brother, &(selector->next), arma))
		return 0;
	for(; selector->next != NULL; selector = selector->next);
	if(! ArmaEdit_Select_select_all_aux((*arma)->child, &(selector->next), arma))
		return 0;
	if(! ArmaEdit_Select_make_ordered_selection(selector))
		return 0;
	return 1;
}

int ArmaEdit_Select_make_ordered_selection(ArmaEdit_Select * select)
{
	if(select == NULL)
		return 0;
	ArmaEdit_Select_clear_ordered_selection(select);
	return ArmaEdit_Select_get_ordered_selection(select, &(select->ordered_bones), &(select->nb_ordered_bones));
}

void ArmaEdit_Select_clear_ordered_selection(ArmaEdit_Select * select)
{
	if(select == NULL)
		return;
	if(select->ordered_bones != NULL)
	{
		free(select->ordered_bones);
		select->ordered_bones = NULL;
	}
	select->nb_ordered_bones = 0;
}

/* Angle Selection */

int ArmaEdit_Select_is_angle_selection(const ArmaEdit_Select * selector)
{
	if(selector == NULL)
		return 0;
	return selector->selected_bone != NULL && selector->angle_type != ARMAEDIT_SELECT_ANGLE_NULL;
}

int ArmaEdit_Select_cancel_angle_selection(ArmaEdit_Select * selector)
{
	if(selector == NULL || selector->selected_bone == NULL || selector->angle_type == ARMAEDIT_SELECT_ANGLE_NULL)
		return 0;
	selector->angle_type = ARMAEDIT_SELECT_ANGLE_NULL;
	return 1;
}

int ArmaEdit_Select_make_angle_selection(ArmaEdit_Select * selector, int x, int y, int range)
{
	double angle_clic;
	double other_area;
	
	if(selector == NULL || selector->selected_bone == NULL)
		return 0;
		
	angle_clic = correct_degre_angle(rad_to_degre(angle_segment(ArmaEdit_Select_get_startX(selector, *(selector->arma)), ArmaEdit_Select_get_startY(selector, *(selector->arma)), x, y)) - selector->selected_bone->angle_min);
	other_area = 360 - selector->selected_bone->deny_angle_area;
	
	if(angle_clic < selector->selected_bone->deny_angle_area)
		selector->angle_type = ARMAEDIT_SELECT_ANGLE_AREA;
	else if(angle_clic < selector->selected_bone->deny_angle_area + (other_area / 2))
		selector->angle_type = ARMAEDIT_SELECT_ANGLE_END;
	else
		selector->angle_type = ARMAEDIT_SELECT_ANGLE_START;
	return selector->angle_type != ARMAEDIT_SELECT_ANGLE_NULL;
}

/* Translation Actions */

ArmaEdit_MouseMove ArmaEdit_MouseMove_create()
{
	ArmaEdit_MouseMove mouse_move;
	mouse_move.active = 0;
	return mouse_move;
}

int ArmaEdit_MouseMove_set_start_selection(ArmaEdit_MouseMove * mouse_move, ArmaEdit_Select * selector, int mouse_x, int mouse_y)
{
	if(mouse_move == NULL || ! ArmaEdit_Select_is_selection(selector))
		return 0;
	switch(selector->select_type)
	{
		case ARMAEDIT_SELECT_START:
		return ArmaEdit_MouseMove_set_start(mouse_move, selector->selected_bone->start.x, selector->selected_bone->start.y, mouse_x, mouse_y);
		
		case ARMAEDIT_SELECT_END:
		return ArmaEdit_MouseMove_set_start(mouse_move, selector->selected_bone->end.x, selector->selected_bone->end.y, mouse_x, mouse_y);
		
		case ARMAEDIT_SELECT_BONE:
		return ArmaEdit_MouseMove_set_start(mouse_move, selector->selected_bone->start.x, selector->selected_bone->start.y, mouse_x, mouse_y);
		
		default:
		return 0;
	}
	return 1;
}

int ArmaEdit_MouseMove_set_start(ArmaEdit_MouseMove * mouse_move, int target_x, int target_y, int mouse_x, int mouse_y)
{
	if(mouse_move == NULL)
		return 0;
	mouse_move->start_target.x = target_x;
	mouse_move->start_target.y = target_y;
	mouse_move->start_mouse.x = mouse_move->last_mouse.x = mouse_x;
	mouse_move->start_mouse.y = mouse_move->last_mouse.y = mouse_y;
	mouse_move->active = 1;
	return 1;
}

int ArmaEdit_MouseMove_angle_set_start(ArmaEdit_MouseMove * mouse_move, ArmaEdit_Select * selector, int target_x, int target_y, int mouse_x, int mouse_y)
{
	if(! ArmaEdit_MouseMove_set_start(mouse_move, target_x, target_y, mouse_x, mouse_y))
		return 0;
	mouse_move->start_angle = selector->selected_bone->angle_min;
	mouse_move->start_angle_area = selector->selected_bone->deny_angle_area;
	mouse_move->start_mouse_angle = correct_degre_angle(rad_to_degre(angle_segment(target_x, target_y, mouse_x, mouse_y)));
	return 1;
}

int ArmaEdit_MouseMove_is_active(const ArmaEdit_MouseMove * mouse_move)
{
	if(mouse_move == NULL)
		return 0;
	return mouse_move->active;
}

int ArmaEdit_MouseMove_stop(ArmaEdit_MouseMove * mouse_move)
{
	if(mouse_move == NULL || ! mouse_move->active)
		return 0;
	mouse_move->active = 0;
	return 1;
}

static int Bone_has_mirror_brother(Bone bone, Bone brother_start)
{
	Bone mirror = NULL;
	if((mirror = Bone_get_mirror(bone)) == NULL)
		return 0;
	for(; brother_start != NULL; brother_start = brother_start->brother)
	{
		if(brother_start == mirror)
			return 1;
	}
	return 0;
}

static ArmaEdit_Select * Bone_has_parent_in_ArmaEdit_Select(Bone bone, ArmaEdit_Select * select)
{
	if(select == NULL)
		return NULL;
	if(select->selected_bone != bone)
	{
		if(Bone_include_in(bone, select->selected_bone))
			return select;
	}
	return Bone_has_parent_in_ArmaEdit_Select(bone, select->next);
}

static ArmaEdit_Select * Bone_has_direct_parent_in_ArmaEdit_Select(Bone bone, ArmaEdit_Select * select)
{
	Bone cur = NULL;
	if(select == NULL)
		return NULL;
	if(select->selected_bone != bone)
	{
		if((cur = select->selected_bone)->child != NULL)
		{
			for(cur = cur->child; cur != NULL; cur = cur->brother)
			{
				if(bone == cur)
					return select;
			}
		}
	}
	return Bone_has_direct_parent_in_ArmaEdit_Select(bone, select->next);
}

int ArmaEdit_MouseMove_move_selection(ArmaEdit_MouseMove * mouse_move, ArmaEdit_Select * selector, Key_Gest * key, int mouse_x, int mouse_y)
{
	int start_x, start_y, end_x, end_y, vx, vy;
	ArmaEdit_Select * start = selector;
	ArmaEdit_Select * cur = NULL;
	int cont;
	if(! ArmaEdit_MouseMove_set_mouse(mouse_move, mouse_x, mouse_y))
		return 0;
	if(! ArmaEdit_MouseMove_is_active(mouse_move))
		return 0;
	if(! ArmaEdit_Select_is_selection(selector))
		return 0;
	if(mouse_move->last_mouse.x == mouse_x && mouse_move->last_mouse.y == mouse_y)
		return 0;
		
	vx = mouse_move->last_mouse.x - mouse_x;
	vy = mouse_move->last_mouse.y - mouse_y;
	
	for(; selector != NULL; selector = selector->next)
	{
		start_x = selector->selected_bone->start.x;
		start_y = selector->selected_bone->start.y;
		end_x = selector->selected_bone->end.x;
		end_y = selector->selected_bone->end.y;
		if((selector->select_type & ARMAEDIT_SELECT_END) != 0 && (Key_Gest_key_in(key, KGK_CTRL) && ! Key_Gest_key_in(key, KGK_SHIFT)))
		{
			Bone cur = NULL;
			if(selector->selected_bone == NULL)
				return 0;
			for(cur = selector->selected_bone->child; cur != NULL; cur = cur->brother)
			{
				if(Bone_has_mirror_brother(cur, selector->selected_bone->child))
					Bone_add_end(cur, vx, vy);
				else
					Bone_add_end_with_mirror(cur, vx, vy);
			}
		}
		if((selector->select_type & ARMAEDIT_SELECT_END) != 0 && (Key_Gest_key_in(key, KGK_SHIFT) && ! Key_Gest_key_in(key, KGK_CTRL)))
		{
			Bone cur = NULL;
			if(selector->selected_bone == NULL)
				return 0;
			for(cur = selector->selected_bone->child; cur != NULL; cur = cur->brother)
			{
				if(Bone_has_mirror_brother(cur, selector->selected_bone->child))
				{
					Bone_add_end(cur, vx, vy);
					Bone_add_start(cur, vx, vy);
				}
				else
				{
					Bone_add_end_with_mirror(cur, vx, vy);
					Bone_add_start_with_mirror(cur, vx, vy);
				}
			}
		}
		if((! Key_Gest_key_in(key, KGK_CTRL) && ! Key_Gest_key_in(key, KGK_SHIFT)))
		{
			cont = 1;
			for(cur = Bone_has_parent_in_ArmaEdit_Select(selector->selected_bone, start); cur != NULL; cur = Bone_has_parent_in_ArmaEdit_Select(selector->selected_bone, cur->next))
			{
				if(cur->select_type == ARMAEDIT_SELECT_START)
					continue;
				cont = 0;
				break;
			}
			if(! cont)
				continue;
		}
		else if(Key_Gest_key_in(key, KGK_CTRL) && ! Key_Gest_key_in(key, KGK_SHIFT))
		{
			cont = 1;
			for(cur = Bone_has_direct_parent_in_ArmaEdit_Select(selector->selected_bone, start); cur != NULL; cur = Bone_has_direct_parent_in_ArmaEdit_Select(selector->selected_bone, cur->next))
			{
				if(cur->select_type == ARMAEDIT_SELECT_START)
					continue;
				if(selector->select_type != ARMAEDIT_SELECT_START)
				{
					Bone_set_end_with_mirror(selector->selected_bone, selector->selected_bone->end.x - vx, selector->selected_bone->end.y - vy);
				}
				cont = 0;
				break;
			}
			if(! cont)
				continue;
		}
		switch(selector->select_type)
		{
			case ARMAEDIT_SELECT_START:
			start_x = selector->selected_bone->start.x - vx;
			start_y = selector->selected_bone->start.y - vy;
			ShapeSet_add_center(selector->selected_bone->shape, vx, vy);
			break;
		
			case ARMAEDIT_SELECT_END:
			end_x = selector->selected_bone->end.x - vx;
			end_y = selector->selected_bone->end.y - vy;
			break;
		
			case ARMAEDIT_SELECT_BONE:
			end_x = selector->selected_bone->end.x - vx;
			end_y = selector->selected_bone->end.y - vy;
			start_x = selector->selected_bone->start.x - vx;
			start_y = selector->selected_bone->start.y - vy;
			break;
		
			default:
			return 0;
		}
		if(Key_Gest_key_in(key, KGK_SHIFT) && Key_Gest_key_in(key, KGK_CTRL))
		{
			Bone_set_start(selector->selected_bone, start_x, start_y);
			Bone_set_end(selector->selected_bone, end_x, end_y);
		}
		else
		{
			Bone_set_start_with_mirror(selector->selected_bone, start_x, start_y);
			Bone_set_end_with_mirror(selector->selected_bone, end_x, end_y);
		}
	}
	mouse_move->last_mouse.x = mouse_x;
	mouse_move->last_mouse.y = mouse_y;
	
	return 1;
}

int ArmaEdit_MouseMove_move_angle_selection(ArmaEdit_MouseMove * mouse_move, ArmaEdit_Select * selector, Key_Gest * key, int mouse_x, int mouse_y)
{
	double new_angle;
	if(! ArmaEdit_MouseMove_set_mouse(mouse_move, mouse_x, mouse_y))
		return 0;
	if(! ArmaEdit_MouseMove_is_active(mouse_move))
		return 0;
	if(! ArmaEdit_Select_is_angle_selection(selector))
		return 0;
	if(mouse_move->last_mouse.x == mouse_x && mouse_move->last_mouse.y == mouse_y)
		return 0;
	new_angle = rad_to_degre(angle_segment(mouse_move->start_target.x, mouse_move->start_target.y, mouse_x, mouse_y));
	switch(selector->angle_type)
	{
		case ARMAEDIT_SELECT_ANGLE_START:
		selector->selected_bone->angle_min = correct_degre_angle(mouse_move->start_angle + new_angle - mouse_move->start_mouse_angle);
		selector->selected_bone->deny_angle_area = correct_area(correct_degre_angle(mouse_move->start_angle_area - new_angle + mouse_move->start_mouse_angle));
		break;
		
		case ARMAEDIT_SELECT_ANGLE_END:
		selector->selected_bone->deny_angle_area = correct_area(correct_degre_angle(mouse_move->start_angle_area + new_angle - mouse_move->start_mouse_angle));
		break;
		
		case ARMAEDIT_SELECT_ANGLE_AREA:
		selector->selected_bone->angle_min = correct_degre_angle(mouse_move->start_angle + new_angle - mouse_move->start_mouse_angle);
		break;
		
		default:
		return 0;
	}
	
	mouse_move->last_mouse.x = mouse_x;
	mouse_move->last_mouse.y = mouse_y;
	
	return 1;
}

int ArmaEdit_MouseMove_set_mouse(ArmaEdit_MouseMove * mouse_move, int x, int y)
{
	if(mouse_move == NULL)
		return 0;
	if(mouse_move->mouse.x == x && mouse_move->mouse.y == y)
		return 0;
	mouse_move->mouse.x = x;
	mouse_move->mouse.y = y;
	return 1;
}

static void Menu_print_anim(void * menu)
{
	Menu_draw((Menu)menu, get_screen());
}

static void Menu_set_pos_anim(void * menu, int x, int y)
{
	Menu_set_origin((Menu)menu, x, y);
}

int ArmaEdit_define_animations(Animation ** animations, Menu menu)
{
	Animation * anims = NULL;
	Menu_List user_data = NULL;
	int i;
	
	if(animations == NULL || menu == NULL)
		return 0;
	if((user_data = (Menu_List)(menu->user_data)) == NULL
	|| user_data->background == NULL)
		return 0;
	
	if((anims = (Animation *)calloc(NB_ARMAEDIT_ANIMATION, sizeof(Animation))) == NULL)
		return 0;
	for(i = 0; i < NB_ARMAEDIT_ANIMATION; ++i)
	{
		if(! Animation_create(anims + i))
			goto free_zone;
		switch(i)
		{
			case ARMAEDIT_ANIMATION_MENU_START:
			if(! Animation_define(*(anims + i), 2,
						ANIMATION_TRANSLATION, ANIMATION_GEST_EXTERNAL_DATA, get_screen(), &SDL_Surface_print_generic, 0, 0, - (user_data->background->w / 2), 0, NULL,
						ANIMATION_TRANSLATION, ANIMATION_GEST_INCLUDED_DATA, menu, &Menu_print_anim, &Menu_set_pos_anim, menu->origin.x, menu->origin.y, menu->origin.x - user_data->background->w, menu->origin.y, NULL))
				goto free_zone;
			break;
			
			case ARMAEDIT_ANIMATION_MENU_QUIT:
			if(! Animation_define(*(anims + i), 2,
						ANIMATION_TRANSLATION, ANIMATION_GEST_EXTERNAL_DATA, get_screen(), &SDL_Surface_print_generic, - (user_data->background->w / 2), 0, 0, 0, NULL,
						ANIMATION_TRANSLATION, ANIMATION_GEST_INCLUDED_DATA, menu, &Menu_print_anim, &Menu_set_pos_anim, menu->origin.x - user_data->background->w, menu->origin.y, menu->origin.x, menu->origin.y, NULL))
				goto free_zone;
			break;
			
			default:
			break;
		}
	}
	
	*animations = anims;
	
	return 1;
	
	free_zone:;
	
	for(i = 0; i < NB_ARMAEDIT_ANIMATION; ++i)
	{
		if(*(anims + i) == NULL)
			break;
		Animation_free(anims + i);
	}
	free(anims);
	
	return 0;
}

int ArmaEdit_play_animation(Animation * animations, ArmaEdit_Animation_Type anim_id)
{
	int success = 0;
	if(animations == NULL)
		return 0;
	switch(anim_id)
	{
		case ARMAEDIT_ANIMATION_MENU_START:
		if(last_screen != NULL)
			SDL_FreeSurface(last_screen);
		last_screen = NULL;
		if(! SDL_CopySurface(&last_screen, get_screen()))
			return 0;
		break;
		
		case ARMAEDIT_ANIMATION_MENU_QUIT:
		break;
		
		default:
		break;
	}
	if(! Animation_redefine_field(*(animations + anim_id), 0, last_screen))
		goto end_zone;
	if(! Animation_gest_defined(get_screen(), screen_background, 15, 10, *(animations + anim_id)))
		goto end_zone;
	success = 1;
	end_zone:;
	Animation_redefine_field(*(animations + anim_id), 0, get_screen());
	return success;
}

void ArmaEdit_delete_animations(Animation ** animations)
{
	int i;
	if(animations == NULL || *animations == NULL)
		return;
	for(i = 0; i < NB_ARMAEDIT_ANIMATION; ++i)
		Animation_free((*animations) + i);
	free(*animations);
	*animations = NULL;
}