#ifndef DEF_HEADER_ARMATURE_EDITOR
#define DEF_HEADER_ARMATURE_EDITOR

#include "armature.h"
#include "key_gest.h"
#include "animation.h"
#include "menu.h"
#include "mini_menu.h"
#include "edit.h"
#include "file_explorer.h"
#include "personnage.h"
#include "file_saver.h"

#define SELECTION_RANGE 15

typedef enum
{
	ARMAEDIT_SELECT_NULL = 0x0,
	ARMAEDIT_SELECT_START = 0x1,
	ARMAEDIT_SELECT_END = 0x2,
	ARMAEDIT_SELECT_BONE = 0x3,
	NB_ARMAEDIT_SELECT
} ArmaEdit_Select_Type;

typedef enum
{
	ARMAEDIT_SELECT_ANGLE_NULL,
	ARMAEDIT_SELECT_ANGLE_START,
	ARMAEDIT_SELECT_ANGLE_END,
	ARMAEDIT_SELECT_ANGLE_AREA,
	NB_ARMAEDIT_SELECT_ANGLE
} ArmaEdit_SelectAngle_Type;

typedef enum
{
	ARMAEDIT_ANIMATION_MENU_START,
	ARMAEDIT_ANIMATION_MENU_QUIT,
	NB_ARMAEDIT_ANIMATION
} ArmaEdit_Animation_Type;

typedef struct _ArmaEdit_Select ArmaEdit_Select;
struct _ArmaEdit_Select
{
	Bone * arma;
	Bone selected_bone;
	ArmaEdit_Select_Type select_type;
	ArmaEdit_SelectAngle_Type angle_type;
	ArmaEdit_Select * next;
	Bone * ordered_bones;
	int nb_ordered_bones;
	Personnage * perso;
};

typedef struct _ArmaEdit_MouseMove ArmaEdit_MouseMove;
struct _ArmaEdit_MouseMove
{
	int active;
	SDL_Rect start_target;
	SDL_Rect start_mouse;
	SDL_Rect last_mouse;
	SDL_Rect mouse;
	double start_angle;
	double start_mouse_angle;
	double start_angle_area;
};

typedef struct _ArmaEdit ArmaEdit;
struct _ArmaEdit
{
	ArmaEdit_Select selector;
	ArmaEdit_MouseMove mouse_move;
	Mini_Menu mini_menu;
	Menu menu;
	Animation * anims;
};

int Armature_edit(Personnage * perso, ArmaEdit * data, EditExit * exit_choice, File_Explorer file_exp, FileSaver * file_saver);

/* Global data */

int ArmaEdit_create(ArmaEdit * data, Bone * arma);

void ArmaEdit_destroy(ArmaEdit * data);

/* Selection */

ArmaEdit_Select ArmaEdit_Select_create(Bone * arma);

int ArmaEdit_Select_createAllocated(ArmaEdit_Select ** select, ArmaEdit_Select val);

void ArmaEdit_Select_destroy(ArmaEdit_Select * select);

void ArmaEdit_Select_free(ArmaEdit_Select ** select);

void ArmaEdit_Select_draw(const ArmaEdit_Select * selector, Bone arma);

int ArmaEdit_Select_is_selection(const ArmaEdit_Select * selector);

int ArmaEdit_Select_cancel_selection(ArmaEdit_Select * selector);

int ArmaEdit_Select_make_selection(ArmaEdit_Select * selector, Bone arma, int x, int y, int range);

int ArmaEdit_Select_add_selection(ArmaEdit_Select * selector, Bone arma, int x, int y, int range);

int ArmaEdit_Select_get_ordered_selection(ArmaEdit_Select * select, Bone ** tab, int * size);

int ArmaEdit_Select_select_all(ArmaEdit_Select * selector, Bone * arma);

int ArmaEdit_Select_make_ordered_selection(ArmaEdit_Select * select);

void ArmaEdit_Select_clear_ordered_selection(ArmaEdit_Select * select);

/* Angle Selection */

void ArmaEdit_Select_draw_angle(const ArmaEdit_Select * selector, Bone arma);

int ArmaEdit_Select_is_angle_selection(const ArmaEdit_Select * selector);

int ArmaEdit_Select_cancel_angle_selection(ArmaEdit_Select * selector);

int ArmaEdit_Select_make_angle_selection(ArmaEdit_Select * selector, int x, int y, int range);

/* Translation actions */

ArmaEdit_MouseMove ArmaEdit_MouseMove_create();

int ArmaEdit_MouseMove_set_start_selection(ArmaEdit_MouseMove * mouse_move, ArmaEdit_Select * selector, int mouse_x, int mouse_y);

int ArmaEdit_MouseMove_set_start(ArmaEdit_MouseMove * mouse_move, int target_x, int target_y, int mouse_x, int mouse_y);

int ArmaEdit_MouseMove_angle_set_start(ArmaEdit_MouseMove * mouse_move, ArmaEdit_Select * selector, int target_x, int target_y, int mouse_x, int mouse_y);

int ArmaEdit_MouseMove_is_active(const ArmaEdit_MouseMove * mouse_move);

int ArmaEdit_MouseMove_stop(ArmaEdit_MouseMove * mouse_move);

int ArmaEdit_MouseMove_move_selection(ArmaEdit_MouseMove * mouse_move, ArmaEdit_Select * selector, Key_Gest * key, int mouse_x, int mouse_y);

int ArmaEdit_MouseMove_move_angle_selection(ArmaEdit_MouseMove * mouse_move, ArmaEdit_Select * selector, Key_Gest * key, int mouse_x, int mouse_y);

int ArmaEdit_MouseMove_set_mouse(ArmaEdit_MouseMove * mouse_move, int x, int y);

/* Animations */

int ArmaEdit_define_animations(Animation ** animations, Menu menu);

int ArmaEdit_play_animation(Animation * animations, ArmaEdit_Animation_Type anim_id);

void ArmaEdit_delete_animations(Animation ** animations);

#endif