#include "bin_path.h"

#include <stdlib.h>
#include <stdio.h>

int BinPath_create(BinPath * id)
{
	BinPath res = NULL;
	if((res = (BinPath)malloc(sizeof(struct _BinPath))) == NULL)
		return 0;
	res->length = 0;
	res->path = NULL;
	*id = res;
	return 1;
}

void BinPath_free(BinPath * id)
{
	if(id == NULL || *id == NULL)
		return;
	BinPath_clear(*id);
	free(*id);
	*id = NULL;
}

void BinPath_clear(BinPath id)
{
	if(id == NULL)
		return;
	if(id->path != NULL)
	{
		free(id->path);
		id->path = NULL;
	}
	id->length = 0;
}

int BinPath_affect(BinPath id, int bit, int val)
{
	unsigned char * path = NULL;
	if(id == NULL || bit < 0 || bit >= id->length)
		return 0;
	val = (val != 0);
	for(path = id->path; bit >= 8; ++path, bit -= 8);
	*path &= ~((unsigned char)1 << bit);
	*path |= ((unsigned char)val << bit);
	return 1;
}

int BinPath_affect_prefixe(BinPath dest, BinPath src, int nb_pref)
{
	int i;
	if(src == NULL || src->length == 0)
		return 1;
	if(dest == NULL)
		return 0;
	if(nb_pref < 0)
		nb_pref = src->length;
	if(nb_pref > src->length)
		return 0;
	for(i = 0; i < nb_pref / 8; ++i)
		*(dest->path + i) = *(src->path + i);
	nb_pref %= 8;
	*(dest->path + i) &= ~((unsigned char)0xFF >> (8 - nb_pref));
	*(dest->path + i) |= (*(src->path + i) & ((unsigned char)0xFF >> (8 - nb_pref)));
	return 1;
}

int BinPath_is_prefixe(BinPath pref, BinPath id)
{
	int i;
	if(pref == NULL || pref->length == 0)
		return 1;
	if(id == NULL)
		return 0;
	if(id->length < pref->length)
		return 0;
	for(i = 0; i < (pref->length / 8); ++i)
	{
		if(*(pref->path + i) != *(id->path + i))
			return 0;
	}
	return ((*(pref->path + i) ^ *(id->path + i)) & ((unsigned char)0xFF >> (8 - (pref->length % 8)))) == 0;
}

int BinPath_get(BinPath id, int bit, int * val)
{
	unsigned char * path = NULL;
	if(id == NULL || id->path == NULL || bit < 0 || bit >= id->length)
		return 0;
	for(path = id->path; bit >= 8; ++path, bit -= 8);
	if(val != NULL)
		*val = (*path & ((unsigned char)1 << bit)) != 0;
	return 1;
}

int BinPath_copy(BinPath * copy, BinPath id)
{
	BinPath res = NULL;
	int i;
	if(id == NULL)
	{
		*copy = NULL;
		return 1;
	}
	if(! BinPath_create(&res))
		return 0;
	res->length = id->length;
	if(! (res->length == 0 || (res->path = (unsigned char *)calloc((res->length / 8) + ((res->length % 8) > 0), sizeof(unsigned char))) != NULL))
	{
		BinPath_free(&res);
		return 0;
	}
	for(i = 0; i < (res->length / 8) + ((res->length % 8) > 0); ++i)
		*(res->path + i) = *(id->path + i);
	*copy = res;
	return 1;
}

void BinPath_aff_debug(BinPath id)
{
	int i;
	int val = 0;
	if(id == NULL)
	{
		printf("(nil)\n");
		return;
	}
	if(id->length == 0)
	{
		printf("(sqr)\n");
		return;
	}
	for(i = 0; BinPath_get(id, i, &val); ++i)
		printf("%d", val);
	printf("\n");
}

int BinPath_save(FILE * flow, const BinPath id)
{
	if(id == NULL)
		return 0;
	if(! fwrite(&(id->length), sizeof(int), 1, flow)
	|| fwrite(id->path, sizeof(unsigned char), (id->length / 8) + ((id->length % 8) > 0), flow) != (id->length / 8) + ((id->length % 8) > 0))
		return 0;
		
	return 1;
}

int BinPath_load(FILE * flow, BinPath * id)
{
	BinPath res = NULL;
	if(id == NULL)
		return 0;
	if(! BinPath_create(&res)
	|| ! fread(&(res->length), sizeof(int), 1, flow)
	|| ! (res->length == 0 || (res->path = (unsigned char *)calloc((res->length / 8) + ((res->length % 8) > 0), sizeof(unsigned char))) != NULL)
	|| fread(res->path, sizeof(unsigned char), (res->length / 8) + ((res->length % 8) > 0), flow) != (res->length / 8) + ((res->length % 8) > 0))
	{
		BinPath_free(&res);
		return 0;
	}
	
	*id = res;
	return 1;
}