#include "bone_table.h"

static void BoneTable_init_count(Bone bone, int * c)
{
	if(bone == NULL)
		return;
	++(*c);
	BoneTable_init_count(bone->child, c);
	BoneTable_init_count(bone->brother, c);
	return;
}

static void BoneTable_init_aux(Bone bone, int * id, Bone * tab)
{
	if(bone == NULL)
		return;
	*(tab + (*id)) = bone;
	++(*id);
	BoneTable_init_aux(bone->child, id, tab);
	BoneTable_init_aux(bone->brother, id, tab);
}

int BoneTable_init(BoneTable * table, Bone bone)
{
	int i;
	table->tab = NULL;
	table->size = 0;
	BoneTable_init_count(bone, &(table->size));
	if(table->size == 0)
		return 1;
	if((table->tab = (Bone *)malloc(table->size * sizeof(Bone))) == NULL)
	{
		table->size = 0;
		return 0;
	}
	i = 0;
	BoneTable_init_aux(bone, &i, table->tab);
	return 1;
}

void BoneTable_quit(BoneTable * table)
{
	if(table->tab != NULL)
	{
		free(table->tab);
		table->tab = NULL;
	}
	table->size = 0;
}

static int cmp_bone(const void * a, const void * b)
{
	return Bone_get_vector_distance_carre(*((Bone *)b)) - Bone_get_vector_distance_carre(*((Bone *)a));
}

void BoneTable_sort(BoneTable * table)
{
	qsort(table->tab, table->size, sizeof(Bone), &cmp_bone);
}

int BoneTable_correct(BoneTable * table)
{
	int i;
	for(i = 1; i < table->size; ++i)
	{
		if(cmp_bone(table->tab + (i - 1), table->tab + i) > 0)
			return 0;
	}
	return 1;
}

void BoneTable_gest(BoneTable * table)
{
	if(BoneTable_correct(table))
		return;
	BoneTable_sort(table);
}

int BoneTable_get(BoneTable * table, Bone * bone, int id)
{
	if(id < 0 || id >= table->size)
		return 0;
	if(Bone_get_vector_distance_carre(*(table->tab + id)) <= 0)
		return 0;
	*bone = *(table->tab + id);
	return 1;
}