#include "chain_list.h"

#include <stdlib.h>

#include "armature.h"

static int eq_adress(void * a, void * b)
{
	return a == b;
}

static int affect_adress(void * dest_adress, void * value)
{
	if(dest_adress == NULL)
		return 0;
	*((char **)dest_adress) = (char *)value;
	return 1;
}

static int ChainListElem_create(ChainListElem * list, int (*create_elem)(void *, void *), void * value)
{
	ChainListElem res = NULL;
	if((res = (ChainListElem)malloc(sizeof(struct _ChainListElem))) == NULL)
		return 0;
	res->elem = NULL;
	res->next = NULL;
	if(! (*create_elem)(&(res->elem), value))
	{
		free(res);
		return 0;
	}
	*list = res;
	return 1;
}

static void ChainListElem_free(ChainListElem * list, void (*free_elem)(void *))
{
	if(list == NULL || *list == NULL)
		return;
	ChainListElem_free(&((*list)->next), free_elem);
	if(free_elem != NULL)
		(*free_elem)(&((*list)->elem));
	free(*list);
	*list = NULL;
}

static int ChainListElem_find(ChainListElem list, int (*eq_elem)(void *, void *), void * value)
{
	for(; list != NULL; list = list->next)
	{
		if((*eq_elem)(list->elem, value))
			return 1;
	}
	return 0;
}

static int ChainListElem_get(ChainListElem list, int (*eq_elem)(void *, void *), int (*affect_elem)(void * dest_adress, void * src), void * data, void * get)
{
	for(; list != NULL; list = list->next)
	{
		if((*eq_elem)(list->elem, data))
		{
			if(! (*affect_elem)(get, list->elem))
				return 0;
			return 1;
		}
	}
	return 0;
}

static int ChainListElem_append_head(ChainListElem * list, int (*create_elem)(void *, void *), void * value)
{
	ChainListElem add = NULL;
	if(! ChainListElem_create(&add, create_elem, value))
		return 0;
	add->next = *list;
	*list = add;
	return 1;
}

static int ChainListElem_append_tail(ChainListElem * list, int (*create_elem)(void *, void *), void * value)
{
	for(; *list != NULL; list = &((*list)->next));
	return ChainListElem_append_head(list, create_elem, value);
}

static int ChainListElem_append_if_not_exists(ChainListElem * list, int (*create_elem)(void *, void *), int (*eq_elem)(void *, void *), void * value)
{
	for(; *list != NULL; list = &((*list)->next))
	{
		if((*eq_elem)((*list)->elem, value))
			return 0;
	}
	return ChainListElem_append_head(list, create_elem, value);
}

static int ChainListElem_delete_head(ChainListElem * list, void (*free_elem)(void *), int (*affect_elem)(void * dest_adress, void * src), void * pop_adress)
{
	ChainListElem del = NULL;
	int success = 1;
	if(*list == NULL)
		return 0;
	del = *list;
	*list = (*list)->next;
	del->next = NULL;
	if(pop_adress != NULL)
	{
		if(affect_elem != NULL)
			success &= (*affect_elem)(pop_adress, del->elem);
		else
			success = 0;
	}
	ChainListElem_free(&del, free_elem);
	return success;
}

static int ChainListElem_delete_tail(ChainListElem * list, void (*free_elem)(void *), int (*affect_elem)(void * dest_adress, void * src), void * pop_adress)
{
	if(*list == NULL)
		return 0;
	for(; (*list)->next != NULL; list = &((*list)->next));
	return ChainListElem_delete_head(list, free_elem, affect_elem, pop_adress);
}

static int ChainListElem_delete_if_exists(ChainListElem * list, void (*free_elem)(void *), int (*eq_elem)(void *, void *), void * value)
{
	if(*list == NULL)
		return 0;
	for(; *list != NULL; list = &((*list)->next))
	{
		if((*eq_elem)((*list)->elem, value))
		{
			return ChainListElem_delete_head(list, free_elem, NULL, NULL);
		}
	}
	return 0;
}

int ChainList_create(ChainList * list,
					int (*create_elem)(void * elem_adress, void * value),
					void (*free_elem)(void * elem_adress),
					int (*affect_elem)(void * dest_adress, void * src),
					int (*eq_elem)(void * in_list, void * elem))
{
	ChainList res = NULL;
	
	if(create_elem == NULL)
		create_elem = &affect_adress;
	if(eq_elem == NULL)
		eq_elem = &eq_adress;
	if(affect_elem == NULL)
		affect_elem = &affect_adress;
		
	if((res = (ChainList)malloc(sizeof(struct _ChainList))) == NULL)
		return 0;
	res->list = NULL;
	res->cursor = NULL;
	res->create_elem = create_elem;
	res->free_elem = free_elem;
	res->affect_elem = affect_elem;
	res->eq_elem = eq_elem;
	
	*list = res;
	return 1;
}

void ChainList_free(ChainList * list)
{
	if(list == NULL || *list == NULL)
		return;
	ChainListElem_free(&((*list)->list), (*list)->free_elem);
	free(*list);
	*list = NULL;
}

int ChainList_find(ChainList list, void * elem)
{
	if(list == NULL)
		return 0;
	return ChainList_find_with_function(list, list->eq_elem, elem);
}

int ChainList_find_with_function(ChainList list, int (*eq_elem)(void *, void *), void * elem)
{
	if(list == NULL)
		return 0;
	return ChainListElem_find(list->list, eq_elem, elem);
}

int ChainList_get_with_function(ChainList list, int (*eq_elem)(void *, void *), void * data, void * get)
{
	if(list == NULL)
		return 0;
	return ChainListElem_get(list->list, eq_elem, list->affect_elem, data, get);
}

int ChainList_append_head(ChainList list, void * elem)
{
	if(list == NULL)
		return 0;
	return ChainListElem_append_head(&(list->list), list->create_elem, elem);
}

int ChainList_append_tail(ChainList list, void * elem)
{
	if(list == NULL)
		return 0;
	return ChainListElem_append_tail(&(list->list), list->create_elem, elem);
}

int ChainList_append_if_not_exists(ChainList list, void * elem)
{
	if(list == NULL)
		return 0;
	return ChainList_append_if_not_exists_with_function(list, list->eq_elem, elem);
}

int ChainList_append_if_not_exists_with_function(ChainList list, int (*eq_elem)(void *, void *), void * elem)
{
	if(list == NULL)
		return 0;
	return ChainListElem_append_if_not_exists(&(list->list), list->create_elem, eq_elem, elem);
}

int ChainList_delete_head(ChainList list, void * pop_adress)
{
	if(list == NULL)
		return 0;
	return ChainListElem_delete_head(&(list->list), list->free_elem, list->affect_elem, pop_adress);
}

int ChainList_delete_tail(ChainList list, void * pop_adress)
{
	if(list == NULL)
		return 0;
	return ChainListElem_delete_tail(&(list->list), list->free_elem, list->affect_elem, pop_adress);
}

int ChainList_delete_if_exists(ChainList list, void * elem)
{
	if(list == NULL)
		return 0;
	return ChainList_delete_if_exists_with_function(list, list->eq_elem, elem);
}

int ChainList_delete_if_exists_with_function(ChainList list, int (*eq_elem)(void *, void *), void * elem)
{
	if(list == NULL)
		return 0;
	return ChainListElem_delete_if_exists(&(list->list), list->free_elem, eq_elem, elem);
}

void ChainList_cursor_to_start(ChainList list)
{
	if(list == NULL)
		return;
	list->cursor = list->list;
}

int ChainList_cursor_goto_next(ChainList list, void * get_cursor)
{
	if(list == NULL)
		return 0;
	if(list->cursor == NULL)
		return 0;
	if(! (*(list->affect_elem))(get_cursor, list->cursor->elem))
		return 0;
	list->cursor = list->cursor->next;
	return 1;
}