#include "color.h"
#include "math.h"

#include <stdlib.h>
#include <math.h>
#include <stdarg.h>
#include "edit.h"

static SDL_Surface * cercle_teinte = NULL;
SDL_Color last_color = {0, 0, 0};

Uint32 SDL_Color_to_Uint32(SDL_Color color)
{
	Uint32 res = 0;
	Uint32_set_values(&res, color.r, color.g, color.b, 255);
	return res;
}

SDL_Color Uint32_to_SDL_Color(Uint32 color)
{
	int r, g, b;
	Uint32_get_values(color, &r, &g, &b, NULL);
	return SDL_Color_create(r, g, b);
}

void Uint32_get_values(Uint32 color, int * r, int * g, int * b, int * a)
{
	if(r != NULL)
		*r = ((color & MASK_RED) >> DECAL_RED);
	if(g != NULL)
		*g = ((color & MASK_GREEN) >> DECAL_GREEN);
	if(b != NULL)
		*b = ((color & MASK_BLUE) >> DECAL_BLUE);
	if(a != NULL)
		*a = ((color & MASK_ALPHA) >> DECAL_ALPHA);
}

void Uint32_set_values(Uint32 * color, int r, int g, int b, int a)
{
	*color = ((Uint32)r << DECAL_RED) | ((Uint32)g << DECAL_GREEN) | ((Uint32)b << DECAL_BLUE) | ((Uint32)a << DECAL_ALPHA);
}

void Uint32_get_from_surface(Uint32 * color, SDL_Surface * surface, int x, int y)
{
	*color = *(((Uint32 *) surface->pixels)+ ((y * surface->pitch) / 4) + x);
}

void Uint32_set_on_surface(Uint32 color, SDL_Surface * surface, int x, int y)
{
	*(((Uint32 *) surface->pixels)+ ((y * surface->pitch) / 4) + x) = color;
}

void SDL_Color_get_from_surface(SDL_Color * color, SDL_Surface * surface, int x, int y)
{
	Uint32 get_color = 0;
	Uint32_get_from_surface(&get_color, surface, x, y);
	*color = Uint32_to_SDL_Color(get_color);
}

void SDL_Color_set_on_surface(SDL_Color color, SDL_Surface * surface, int x, int y)
{
	Uint32_set_on_surface(SDL_Color_to_Uint32(color), surface, x, y);
}

Uint32 Uint32_by_teinte(int teinte)
{
	Uint32 color;
	teinte %= 360;
	teinte = (teinte < 0) ? teinte + 360 : teinte;
	if(teinte < 120)
		Uint32_set_values(&color, ((120 - teinte) * 255) / 120, (teinte * 255) / 120, 0, 255);
	else if(teinte < 240)
		Uint32_set_values(&color, 0, ((240 - teinte) * 255) / 120, ((teinte - 120) * 255) / 120, 255);
	else
		Uint32_set_values(&color, ((teinte - 240) * 255) / 120, 0, ((360 - teinte) * 255) / 120, 255);
	return color;
}

SDL_Color SDL_Color_get_max_teinte(int r, int g, int b)
{
	int max_value, min_value;
	if(r == g && g == b)
		return SDL_Color_create(255, 0, 0);
	max_value = max_int(3, r, g, b);
	/*max_value = (r > g) ? ((b > r) ? b : r) : ((b > g) ? b : g);*/
	r = (r * 255) / max_value;
	g = (g * 255) / max_value;
	b = (b * 255) / max_value;
	min_value = min_int(3, r, g, b);
	/*min_value = (r < g) ? ((b < r) ? b : r) : ((b < g) ? b : g);*/
	if(min_value > 0)
	{
		r = (((r - min_value) * 255) / (255 - min_value));
		g = (((g - min_value) * 255) / (255 - min_value));
		b = (((b - min_value) * 255) / (255 - min_value));
	}
	return SDL_Color_create(r, g, b);
}

int teinte_by_rgb(int r, int g, int b)
{
	SDL_Color color = SDL_Color_get_max_teinte(r, g, b);
	if(color.r == 255)
		return ((color.g * 60) / 255) + ((color.b > 0) * (360 - ((color.b * 60) / 255)));
	else if(color.g == 255)
		return 120 + ((color.b * 60) / 255) - ((color.r * 60) / 255);
	else if(color.b == 255)
		return 240 - ((color.g * 60) / 255) + ((color.r * 60) / 255);
	return 0;
}

SDL_Color SDL_Color_create(int r, int g, int b)
{
	SDL_Color color;
	color.r = r;
	color.g = g;
	color.b = b;
	return color;
}

SDL_Color SDL_Color_less(SDL_Color color, int intensity)
{
	return SDL_Color_create((color.r * intensity) / 255,
				(color.g * intensity) / 255,
				(color.b * intensity) / 255);
}

SDL_Color SDL_Color_more(SDL_Color color, int add)
{
	return SDL_Color_create(((color.r * (255 - add)) / 255) + add,
				((color.g * (255 - add)) / 255) + add,
				((color.b * (255 - add)) / 255) + add);
}

SDL_Color SDL_Color_invert(SDL_Color color)
{
	return SDL_Color_create((255 - color.r),
				(255 - color.g),
				(255 - color.b));
}

int SDL_Color_get_intensity(SDL_Color color)
{
	return max_int(3, color.r, color.g, color.b);
}

int SDL_Color_get_add(SDL_Color color)
{
	return min_int(3, color.r, color.g, color.b);
}

int SDL_Color_equal(SDL_Color color, SDL_Color other)
{
	return 	color.r == other.r
		&& color.g == other.g
		&& color.b == other.b;
}

static SDL_Color color_trig_teinte(SDL_Color base_color, int distance_base, int distance_black, int distance_white)
{
	/*int max_distance = (distance_base > distance_black) ? ((distance_base > distance_white) ? distance_base : distance_white) : ((distance_black > distance_white) ? distance_black : distance_white);*/
	int max_distance = max_int(3, distance_base, distance_black, distance_white);
	distance_base = (distance_base <= 0) ? 1 : distance_base;
	distance_black = (distance_black <= 0) ? 1 : distance_black;
	distance_white = (distance_white <= 0) ? 1 : distance_white;
	return SDL_Color_by_color_distances(3, base_color, (max_distance * 255) / distance_base, SDL_Color_create(0, 0, 0), (max_distance * 255) / distance_black, SDL_Color_create(255, 255, 255), (max_distance * 255) / distance_white);
}

static int Color_calculer_cercle_teinte(SDL_Color base_color, int size)
{
	SDL_Surface * image = NULL;
	SDL_Rect color_coord, white_coord, black_coord;
	int i, j;
	double angle = 0;
	Uint32 color;
	SDL_Color triangle_color;
	
	if((image = SDL_CreateRGBSurface(SDL_HWSURFACE, size, size, 32, MASK_RED, MASK_GREEN, MASK_BLUE, 0)) == NULL)
		return 0;
	SDL_FillRect(image, NULL, SDL_MapRGB(image->format, 0, 0, 0));
	base_color = SDL_Color_get_max_teinte(base_color.r, base_color.g, base_color.b);
	angle = degre_to_rad(teinte_by_rgb(base_color.r, base_color.g, base_color.b) + 180);
	color_coord = coord_in_circle(size / 2, size / 2, (size * 3) / 8, angle);
	black_coord = coord_in_circle(size / 2, size / 2, (size * 3) / 8, angle + (2 * M_PI) / 3);
	white_coord = coord_in_circle(size / 2, size / 2, (size * 3) / 8, angle + (4 * M_PI) / 3);
	for(i = 0; i < size; ++i)
	{
		for(j = 0; j < size; ++j)
		{
			if(distance_carre(i, j, size / 2, size / 2) <= carre(size / 2)
			&& distance_carre(i, j, size / 2, size / 2) >= carre((size * 3) / 8))
				Uint32_set_on_surface(Uint32_by_teinte(rad_to_degre(angle_segment(i, j, size / 2, size / 2))), image, i, j);
			else if(in_triangle(i, j, color_coord.x, color_coord.y, black_coord.x, black_coord.y, white_coord.x, white_coord.y))
			{
				triangle_color = color_trig_teinte(base_color, 	distance_carre(i, j, color_coord.x, color_coord.y),
										distance_carre(i, j, black_coord.x, black_coord.y),
										distance_carre(i, j, white_coord.x, white_coord.y));
				Uint32_set_values(&color, triangle_color.r, triangle_color.g, triangle_color.b, 255);
				Uint32_set_on_surface(color, image, i, j);
			}
		}
	}
	SDL_SetColorKey(image, SDL_SRCCOLORKEY, SDL_MapRGB(image->format, 0, 0, 0));
	if(cercle_teinte != NULL)
		SDL_FreeSurface(cercle_teinte);
	cercle_teinte = image;
	return 1;
}

static int recalcul_cercle_teinte(SDL_Color * color, int size)
{
	if(cercle_teinte == NULL || cercle_teinte->w != size || ! SDL_Color_equal(SDL_Color_get_max_teinte(color->r, color->g, color->b), last_color))
	{
		last_color = SDL_Color_get_max_teinte(color->r, color->g, color->b);
		if(! Color_calculer_cercle_teinte(last_color, size))
			return 0;
	}
	return 1;
}

void SDL_Color_draw_triangle(SDL_Surface * screen, SDL_Color color, int x, int y, int size)
{
	SDL_Rect pos;
	if(cercle_teinte == NULL)
		return;
	pos.x = x;
	pos.y = y;
	SDL_BlitSurface(cercle_teinte, NULL, screen, &pos);
}

int SDL_Color_get_in_triangle(SDL_Color * color, int vx, int vy, int size)
{
	Uint32 color_get;
	int r, g, b, a;
	if(vx < 0 || vy < 0 || vx >= size || vy >= size)
		return 0;
	Uint32_get_from_surface(&color_get, cercle_teinte, vx, vy);
	Uint32_get_values(color_get, &r, &g, &b, &a);
	if(a <= 0)
		return 0;
	*color = SDL_Color_create(r, g, b);
	if(distance_carre(size / 2, size / 2, vx, vy) >= carre((size * 3) / 8))
	{
		if(! recalcul_cercle_teinte(color, size))
			return 0;
	}
	return 1;
}

void Color_quit()
{
	if(cercle_teinte != NULL)
	{
		SDL_FreeSurface(cercle_teinte);
		cercle_teinte = NULL;
	}
}

SDL_Color SDL_Color_by_color_distances(int nb_colors, ...)
{
	int i;
	int distances[nb_colors];
	int total = 0;
	SDL_Color colors[nb_colors];
	SDL_Color res;
	va_list ap;
	
	res = SDL_Color_create(0, 0, 0);
	
	va_start(ap, nb_colors);
	for(i = 0; i < nb_colors; ++i)
	{
		colors[i] = va_arg(ap, SDL_Color);
		total += (distances[i] = va_arg(ap, int));
	}
	va_end(ap);
	
	if(total <= 0)
		return res;
	
	for(i = 0; i < nb_colors; ++i)
	{
		res.r += (distances[i] * colors[i].r) / total;
		res.g += (distances[i] * colors[i].g) / total;
		res.b += (distances[i] * colors[i].b) / total;
	}
	return res;
}

int SDL_Color_window_gest(SDL_Color * color, SDL_Surface * screen, int x, int y, int size, void (*draw)(void * data), void * data)
{
	SDL_Surface * back = NULL;
	SDL_Event event;
	SDL_Rect origin;
	int cont = 1;
	int success = 1;
	int need_refresh = 1;
	int last_refresh_time = SDL_GetTicks() - REFRESH_TIME - 1;
	int last_press_time;
	int is_press = 0;
	
	origin.x = origin.y = 0;
	if((back = SDL_CreateRGBSurface(SDL_HWSURFACE, screen->w, screen->h, 32, 0, 0, 0, 0)) == NULL)
		return 0;
	SDL_BlitSurface(screen, NULL, back, &origin);
	if(! recalcul_cercle_teinte(color, size))
		return 0;
	while(cont && success)
	{
		if(need_refresh && last_refresh_time + REFRESH_TIME < SDL_GetTicks())
		{
			SDL_BlitSurface(back, NULL, screen, &origin);
			(*draw)(data);
			SDL_Color_draw_triangle(screen, *color, x, y, size);
			SDL_Flip(screen);
			need_refresh = 0;
			last_refresh_time = SDL_GetTicks();
		}
		SDL_WaitEvent(&event);
		switch(event.type)
		{
			case SDL_QUIT:
			success = 0;
			break;
			
			case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
				success = 0;
				break;
				
				default:
				break;
			}
			break;
			
			case SDL_MOUSEBUTTONDOWN:
			switch(event.button.button)
			{
				case SDL_BUTTON_LEFT:
				case SDL_BUTTON_RIGHT:
				is_press = 1;
				last_press_time = SDL_GetTicks() - REFRESH_TIME - 1;
				break;
				
				default:
				break;
			}
			break;
			
			case SDL_MOUSEMOTION:
			if(is_press && last_press_time + REFRESH_TIME < SDL_GetTicks())
			{
				need_refresh |= SDL_Color_get_in_triangle(color, event.motion.x - x, event.motion.y - y, size);
				last_press_time = SDL_GetTicks();
			}
			break;
			
			case SDL_MOUSEBUTTONUP:
			switch(event.button.button)
			{
				case SDL_BUTTON_LEFT:
				case SDL_BUTTON_RIGHT:
				cont &= SDL_Color_get_in_triangle(color, event.button.x - x, event.button.y - y, size);
				need_refresh = 1;
				is_press = 0;
				break;
				
				default:
				break;
			}
			break;
			
			default:
			break;
		}
	}
	SDL_FreeSurface(back);
	return success;
}

/*int SDL_Surface_set_color_key(SDL_Surface * surface, int r, int g, int b)
{
	Uint32 color_key = 0;
	Uint32 color = 0;
	Uint32 replace = 0;
	int x, y;
	Uint32_set_values(&color_key, r, g, b, 0);
	Uint32_set_values(&replace, r, g, b, 255);
	for(x = 0; x < surface->w; ++x)
	{
		for(y = 0; y < surface->h; ++y)
		{
			Uint32_get_from_surface(&color, surface, x, y);
			if(color == color_key)
				Uint32_set_on_surface(replace, surface, x, y);
		}
	}
	return 1;
}*/