#include "draw.h"

#include <SDL/SDL_gfxPrimitives.h>
#include "math.h"
#include "image.h"

void draw_bone_part_startRGBA(SDL_Surface * screen, int start_x, int start_y, int end_x, int end_y, int r, int g, int b, int a)
{
	filledCircleRGBA(screen, start_x, start_y, BONE_RAD, r, g, b, a);
}

void draw_bone_part_endRGBA(SDL_Surface * screen, int start_x, int start_y, int end_x, int end_y, int r, int g, int b, int a)
{
	filledCircleRGBA(screen, end_x, end_y, BONE_RAD, r, g, b, a);
}

void draw_bone_part_boneRGBA(SDL_Surface * screen, int start_x, int start_y, int end_x, int end_y, int r, int g, int b, int a)
{
	double angle = correct_rad_angle(angle_segment(start_x, start_y, end_x, end_y) - (M_PI / 2));
	/*double angle2 = correct_rad_angle(angle + M_PI);*/
	double distance = sqrt((double)distance_carre(start_x, start_y, end_x, end_y));
	double dist_angle = correct_rad_angle(angle + (M_PI / 4));
	double dist_angle2 = correct_rad_angle(dist_angle + (M_PI / 2));
	{
		Sint16 x[] = {start_x, start_x + (cos(dist_angle) * (distance / 10)), end_x, start_x + (cos(dist_angle2) * (distance / 10))};
		Sint16 y[] = {start_y, start_y + (sin(dist_angle) * (distance / 10)), end_y, start_y + (sin(dist_angle2) * (distance / 10))};
		filledPolygonRGBA(screen, x, y, 4, (r * 5) / 8, (g * 5) / 8, (b * 5) / 8, a);
		polygonRGBA(screen, x, y, 4, r, g, b, a);
		lineRGBA(screen, *(x + 1), *(y + 1), *(x + 3), *(y + 3), r, g, b, a);
	}
	{
		/*Sint16 x[] = {start_x + (cos(angle) * BONE_RAD), end_x + (cos(angle) * BONE_RAD), end_x + (cos(angle2) * BONE_RAD), start_x + (cos(angle2) * BONE_RAD)};
		Sint16 y[] = {start_y + (sin(angle) * BONE_RAD), end_y + (sin(angle) * BONE_RAD), end_y + (sin(angle2) * BONE_RAD), start_y + (sin(angle2) * BONE_RAD)};
		filledPolygonRGBA(screen, x, y, 4, r, g, b, a);*/
	}
	lineRGBA(screen, start_x, start_y, end_x, end_y, r, g, b, a);
}

static void draw_angle_area(Bone cur, int x, int y, double angle_add, ArmaDraw_Part angle_part, SDL_Color * angle_color, SDL_Color * angle_select_color, int alpha)
{
	/*SDL_Rect origin, start;*/
	double min_angle, max_angle;
	/*origin.x = origin.y = 0;*/
	min_angle = correct_degre_angle(cur->angle_min + angle_add);
	max_angle = correct_degre_angle(cur->angle_min + cur->deny_angle_area + angle_add);
	/*start = get_SDL_Rect_by_rotation(cur->start, origin, degre_to_rad(angle_add));*/
	draw_arcRGBA(get_screen(), x, y, ANGLE_RAD, min_angle, max_angle,
		(angle_color->r * (angle_part != ARMADRAW_PART_BONE)) + (angle_select_color->r * (angle_part == ARMADRAW_PART_BONE)),
		(angle_color->g * (angle_part != ARMADRAW_PART_BONE)) + (angle_select_color->g * (angle_part == ARMADRAW_PART_BONE)),
		(angle_color->b * (angle_part != ARMADRAW_PART_BONE)) + (angle_select_color->b * (angle_part == ARMADRAW_PART_BONE)), alpha);
	lineRGBA(get_screen(), x, y, x + (cos(degre_to_rad(min_angle)) * ANGLE_RAD), y + (sin(degre_to_rad(min_angle)) * ANGLE_RAD),
		(angle_color->r * (angle_part != ARMADRAW_PART_START)) + (angle_select_color->r * (angle_part == ARMADRAW_PART_START)),
		(angle_color->g * (angle_part != ARMADRAW_PART_START)) + (angle_select_color->g * (angle_part == ARMADRAW_PART_START)),
		(angle_color->b * (angle_part != ARMADRAW_PART_START)) + (angle_select_color->b * (angle_part == ARMADRAW_PART_START)), alpha);
	lineRGBA(get_screen(), x, y, x + (cos(degre_to_rad(max_angle)) * ANGLE_RAD), y + (sin(degre_to_rad(max_angle)) * ANGLE_RAD),
		(angle_color->r * (angle_part != ARMADRAW_PART_END)) + (angle_select_color->r * (angle_part == ARMADRAW_PART_END)),
		(angle_color->g * (angle_part != ARMADRAW_PART_END)) + (angle_select_color->g * (angle_part == ARMADRAW_PART_END)),
		(angle_color->b * (angle_part != ARMADRAW_PART_END)) + (angle_select_color->b * (angle_part == ARMADRAW_PART_END)), alpha);
}

static void draw_angle_area_action(Bone cur, int x, int y, double angle_add, int alpha)
{
	SDL_Color angle_select_color, angle_color;
	angle_select_color = SDL_Color_create(0, 0, 0);
	angle_color = SDL_Color_create(255, 0, 0);
	draw_angle_area(cur, x, y, angle_add, ARMADRAW_PART_NULL, &angle_color, &angle_select_color, alpha);
}

static void draw_armature_aux_bones(SDL_Surface * screen, int start_x, int start_y, int end_x, int end_y, int a)
{
	draw_bone_part_startRGBA(screen, start_x, start_y, end_x, end_y, 127, 127, 127, a);
	draw_bone_part_boneRGBA(screen, start_x, start_y, end_x, end_y, 100, 100, 100, a);
	draw_bone_part_endRGBA(screen, start_x, start_y, end_x, end_y, 127, 127, 127, a);
}

static int draw_shape_armature_aux(Bone cur, int x, int y)
{
	Bone cur_link = NULL;
	Bone child = NULL;
	int retour = 1;
	
	if(cur == NULL)
		return 0;
	
	for(cur_link = cur->child; cur_link != NULL; cur_link = cur_link->brother)
		lineRGBA(get_screen(), x + cur->end.x, y + cur->end.y, x + cur->end.x + cur_link->start.x, y + cur->end.y + cur_link->start.y, 127, 127, 127, 63);
		
	/*retour = 1 + draw_shape_armature_aux(cur->child, x + cur->end.x, y + cur->end.y);
	retour += draw_shape_armature_aux(cur->brother, x, y);*/
	
	for(child = cur->child; child != NULL && ! child->in_front_of_parent; child = child->brother)
		retour += draw_shape_armature_aux(child, x + cur->end.x, y + cur->end.y);
	draw_shape_set(cur->shape, x + cur->start.x, y + cur->start.y);
	for(; child != NULL; child = child->brother)
		retour += draw_shape_armature_aux(child, x + cur->end.x, y + cur->end.y);
	
	return retour;
}

static int draw_armature_aux(Bone cur, int x, int y)
{
	int retour = 1;
	Bone child = NULL;
	
	if(cur == NULL)
		return 0;
		
	for(child = cur->child; child != NULL && ! child->in_front_of_parent; child = child->brother)
		retour += draw_armature_aux(child, x + cur->end.x, y + cur->end.y);
	draw_armature_aux_bones(get_screen(), x + cur->start.x, y + cur->start.y, x + cur->end.x, y + cur->end.y, 255);
	for(; child != NULL; child = child->brother)
		retour += draw_armature_aux(child, x + cur->end.x, y + cur->end.y);
	
	/*retour = 1 + draw_armature_aux(cur->child, x + cur->end.x, y + cur->end.y);
	retour += draw_armature_aux(cur->brother, x, y);*/
	
	return retour;
}

int draw_armature(Bone start)
{
	int retour;
	if(start == NULL)
		return 0;
	retour = draw_shape_armature_aux(start, 0, 0);
	retour += draw_armature_aux(start, 0, 0);
	return retour;
}

int draw_shape_armature(Bone start)
{
	int retour;
	if(start == NULL)
		return 0;
	retour = draw_armature_aux(start, 0, 0);
	retour += draw_shape_armature_aux(start, 0, 0);
	return retour;
}

int draw_action_armature(Bone start)
{
	return draw_caracter(start, &Bone_draw_bone_shape, NULL, NULL)
	+ draw_caracter(start, &Bone_draw_bone_arma, NULL, NULL);
}

int draw_action_armature_real(Bone start)
{
	return draw_caracter(start, &Bone_draw_bone_shape, NULL, NULL);
}

int draw_action_select(Bone start, Bone select)
{
	return draw_caracter(start, &Bone_draw_selected_bone, select, &Bone_condition_equal_bone);
}

void Bone_draw_bone_color(SDL_Surface * screen, Bone bone, int x, int y, double angle, int r, int g, int b, int a)
{
	SDL_Rect end;
	end = get_SDL_Rect_by_rotation(bone->end, bone->start, degre_to_rad(angle + bone->angle));
	end.x += x;
	end.y += y;
	draw_bone_part_startRGBA(screen, bone->start.x + x, bone->start.y + y, end.x, end.y, r, g, b, a);
	draw_bone_part_boneRGBA(screen, bone->start.x + x, bone->start.y + y, end.x, end.y, (r * 4) / 5, (g * 4) / 5, (b * 4) / 5, a);
	draw_bone_part_endRGBA(screen, bone->start.x + x, bone->start.y + y, end.x, end.y, r, g, b, a);
}

void Bone_draw_bone(SDL_Surface * screen, Bone bone, int x, int y, double angle)
{
	Bone_draw_bone_shape(screen, bone, x, y, angle);
	Bone_draw_bone_arma(screen, bone, x, y, angle);
}

void Bone_draw_bone_arma(SDL_Surface * screen, Bone bone, int x, int y, double angle)
{
	Bone_draw_bone_color(screen, bone, x, y, angle, 127, 127, 127, 255);
	/*draw_hitbox(bone->hitbox, x + bone->start.x, y + bone->start.y);*/
}

void Bone_draw_bone_shape(SDL_Surface * screen, Bone bone, int x, int y, double angle)
{
	SDL_Rect end;
	end = get_SDL_Rect_by_rotation(bone->end, bone->start, degre_to_rad(angle + bone->angle));
	Image_print_rotated_on_surface(screen, bone->shape_image, x + bone->start.x, y + bone->start.y, degre_to_rad(angle + bone->angle), x + bone->start.x, y + bone->start.y);
	Bone_set_real_start(bone, x + bone->start.x, y + bone->start.y);
	Bone_set_real_angle(bone, degre_to_rad(angle + bone->angle));
	Bone_set_real_end(bone, x + end.x, y + end.y);
	if(bone->hit_delay > 0)
		--(bone->hit_delay);
	/*draw_bone_part_boneRGBA(screen, bone->real_start.x, bone->real_start.y, bone->real_end.x, bone->real_end.y, 255, 0, 0, 255);*/
}

void Bone_draw_selected_bone(SDL_Surface * screen, Bone bone, int x, int y, double angle)
{
	SDL_Rect end;
	end = get_SDL_Rect_by_rotation(bone->end, bone->start, degre_to_rad(angle + bone->angle));
	end.x += x;
	end.y += y;
	Bone_draw_bone_color(screen, bone, x, y, angle, 255, 127, 0, 255);
	lineRGBA(screen, x + bone->start.x, y + bone->start.y, end.x + (end.x - bone->start.x - x) / 2, end.y + (end.y - bone->start.y - y) / 2, 46, 168, 82, 255);
	draw_angle_area_action(bone, x + bone->start.x, y + bone->start.y, angle, 255);
}

int Bone_condition_equal_bone(Bone bone, void * equal_bone)
{
	return bone == (Bone)equal_bone;
}

static int draw_caracter_aux(SDL_Surface * screen, Bone bone, int x, int y, double angle, void (*bone_draw)(SDL_Surface *, Bone, int x, int y, double angle), void * data, int (*condition)(Bone bone, void * data))
{
	SDL_Rect next, start, origin;
	int retour = 0;
	Bone child = NULL;
	double old_angle;
	
	origin.x = origin.y = 0;
	if(bone == NULL)
		return 0;
	start = get_SDL_Rect_by_rotation(bone->start, origin, degre_to_rad(angle));
	x += (start.x - bone->start.x);
	y += (start.y - bone->start.y);
	old_angle = angle;
	angle = correct_degre_angle(angle + bone->angle);
	next = get_SDL_Rect_by_rotation(bone->end, bone->start, degre_to_rad(angle));
	
	for(child = bone->child; child != NULL && ! child->in_front_of_parent; child = child->brother)
		retour += draw_caracter_aux(screen, child, x + next.x, y + next.y, angle, bone_draw, data, condition);
	if(condition == NULL || (*condition)(bone, data))
		(*bone_draw)(screen, bone, x, y, old_angle);
	for(; child != NULL && child->in_front_of_parent; child = child->brother)
		retour += draw_caracter_aux(screen, child, x + next.x, y + next.y, angle, bone_draw, data, condition);
	
	return 1;
	/*+ draw_caracter_aux(screen, bone->child, x + next.x, y + next.y, angle, bone_draw, data, condition)
	+ draw_caracter_aux(screen, bone->brother, x - (start.x - bone->start.x), y - (start.y - bone->start.y), correct_degre_angle(angle - bone->angle), bone_draw, data, condition);*/
}

int draw_caracter(Bone start, void (*bone_draw)(SDL_Surface *, Bone, int x, int y, double angle), void * data, int (*condition)(Bone bone, void * data))
{
	return draw_caracter_aux(get_screen(), start, 0, 0, 0, bone_draw, data, condition);
}

/*static int draw_in_armature_aux(Bone cur, Bone bone, ArmaDraw_Part part, ArmaDraw_Part angle_part, int x, int y)
{
	if(cur == NULL)
		return 0;
		
	if(cur == bone)
	{
		switch(part)
		{
			case ARMADRAW_PART_BONE:
			case ARMADRAW_PART_START:
			circleRGBA(get_screen(), x + cur->start.x, y + cur->start.y, 5, 255, 216, 0, 255);
			if(part != ARMADRAW_PART_BONE)
				break;
			lineRGBA(get_screen(), x + cur->start.x, y + cur->start.y, x + cur->end.x, y + cur->end.y, 255, 216, 0, 255);
			
			case ARMADRAW_PART_END:
			circleRGBA(get_screen(), x + cur->end.x, y + cur->end.y, 5, 255, 216, 0, 255);
			break;
			
			default:
			break;
		}
		draw_arcRGBA(get_screen(), x + cur->start.x, y + cur->start.y, 25, cur->angle_min, cur->angle_min + cur->deny_angle_area, 255, 216 * (angle_part == ARMADRAW_PART_BONE), 0, 255);
		lineRGBA(get_screen(), x + cur->start.x, y + cur->start.y, x + cur->start.x + (cos(degre_to_rad(cur->angle_min)) * 25), y + cur->start.y + (sin(degre_to_rad(cur->angle_min)) * 25), 255, 216 * (angle_part == ARMADRAW_PART_START), 0, 255);
		lineRGBA(get_screen(), x + cur->start.x, y + cur->start.y, x + cur->start.x + (cos(degre_to_rad(cur->angle_min + cur->deny_angle_area)) * 25), y + cur->start.y + (sin(degre_to_rad(cur->angle_min + cur->deny_angle_area)) * 25), 255, 216 * (angle_part == ARMADRAW_PART_END), 0, 255);
		return 1;
	}
	
	if(draw_in_armature_aux(cur->child, bone, part, angle_part, x + cur->end.x, y + cur->end.y))
		return 1;
	if(draw_in_armature_aux(cur->brother, bone, part, angle_part, x, y))
		return 1;
	return 0;
}*/

int draw_in_armature(Bone start, Bone bone, ArmaDraw_Part part, ArmaDraw_Part angle_part)
{
	if(start == NULL || bone == NULL)
		return 0;
		
	return draw_in_armature_color(start, bone, part, angle_part, SDL_Color_create(255, 216, 0), SDL_Color_create(255, 0, 0), SDL_Color_create(255, 216, 0));
}

static int draw_in_armature_color_aux(Bone cur, Bone bone, ArmaDraw_Part part, ArmaDraw_Part angle_part, SDL_Color * bone_color, SDL_Color * angle_color, SDL_Color * angle_select_color, int x, int y, int alpha)
{
	Bone next = NULL;
	if(cur == NULL)
		return 0;
		
	if(cur == bone)
	{
		switch(part)
		{
			case ARMADRAW_PART_BONE:
			case ARMADRAW_PART_START:
			draw_bone_part_startRGBA(get_screen(), x + cur->start.x, y + cur->start.y, x + cur->end.x, y + cur->end.y, bone_color->r, bone_color->g, bone_color->b, alpha);
			if(part != ARMADRAW_PART_BONE)
				break;
			draw_bone_part_boneRGBA(get_screen(), x + cur->start.x, y + cur->start.y, x + cur->end.x, y + cur->end.y, bone_color->r, bone_color->g, bone_color->b, alpha);
			
			case ARMADRAW_PART_END:
			draw_bone_part_endRGBA(get_screen(), x + cur->start.x, y + cur->start.y, x + cur->end.x, y + cur->end.y, bone_color->r, bone_color->g, bone_color->b, alpha);
			break;
			
			default:
			break;
		}
		if(angle_part != ARMADRAW_PART_IGNORE)
		{
			draw_angle_area(cur, x + cur->start.x, y + cur->start.y, 0, angle_part, angle_color, angle_select_color, alpha);
			/*draw_arcRGBA(get_screen(), x + cur->start.x, y + cur->start.y, ANGLE_RAD, cur->angle_min, cur->angle_min + cur->deny_angle_area,
				(angle_color->r * (angle_part != ARMADRAW_PART_BONE)) + (angle_select_color->r * (angle_part == ARMADRAW_PART_BONE)),
				(angle_color->g * (angle_part != ARMADRAW_PART_BONE)) + (angle_select_color->g * (angle_part == ARMADRAW_PART_BONE)),
				(angle_color->b * (angle_part != ARMADRAW_PART_BONE)) + (angle_select_color->b * (angle_part == ARMADRAW_PART_BONE)), alpha);
			lineRGBA(get_screen(), x + cur->start.x, y + cur->start.y, x + cur->start.x + (cos(degre_to_rad(cur->angle_min)) * ANGLE_RAD), y + cur->start.y + (sin(degre_to_rad(cur->angle_min)) * ANGLE_RAD),
				(angle_color->r * (angle_part != ARMADRAW_PART_START)) + (angle_select_color->r * (angle_part == ARMADRAW_PART_START)),
				(angle_color->g * (angle_part != ARMADRAW_PART_START)) + (angle_select_color->g * (angle_part == ARMADRAW_PART_START)),
				(angle_color->b * (angle_part != ARMADRAW_PART_START)) + (angle_select_color->b * (angle_part == ARMADRAW_PART_START)), alpha);
			lineRGBA(get_screen(), x + cur->start.x, y + cur->start.y, x + cur->start.x + (cos(degre_to_rad(cur->angle_min + cur->deny_angle_area)) * ANGLE_RAD), y + cur->start.y + (sin(degre_to_rad(cur->angle_min + cur->deny_angle_area)) * ANGLE_RAD),
				(angle_color->r * (angle_part != ARMADRAW_PART_END)) + (angle_select_color->r * (angle_part == ARMADRAW_PART_END)),
				(angle_color->g * (angle_part != ARMADRAW_PART_END)) + (angle_select_color->g * (angle_part == ARMADRAW_PART_END)),
				(angle_color->b * (angle_part != ARMADRAW_PART_END)) + (angle_select_color->b * (angle_part == ARMADRAW_PART_END)), alpha);*/
		}
		return 1;
	}
	if((next = Bone_get_next(bone, cur)) == NULL)
	{
		if(draw_in_armature_color_aux(cur->child, bone, part, angle_part, bone_color, angle_color, angle_select_color, x + cur->end.x, y + cur->end.y, alpha))
			return 1;
		if(draw_in_armature_color_aux(cur->brother, bone, part, angle_part, bone_color, angle_color, angle_select_color, x, y, alpha))
			return 1;
	}
	if(next == cur->child)
	{
		x += cur->end.x;
		y += cur->end.y;
	}
	if(draw_in_armature_color_aux(next, bone, part, angle_part, bone_color, angle_color, angle_select_color, x, y, alpha))
		return 1;
	return 0;
}

int draw_in_armature_color(Bone start, Bone bone, ArmaDraw_Part part, ArmaDraw_Part angle_part, SDL_Color bone_color, SDL_Color angle_color, SDL_Color angle_select_color)
{
	if(start == NULL || bone == NULL)
		return 0;
		
	return draw_in_armature_color_aux(start, bone, part, angle_part, &bone_color, &angle_color, &angle_select_color, 0, 0, 255);
}

int draw_in_armature_color_with_alpha(Bone start, Bone bone, ArmaDraw_Part part, ArmaDraw_Part angle_part, SDL_Color bone_color, SDL_Color angle_color, SDL_Color angle_select_color, int a)
{
	if(start == NULL || bone == NULL)
		return 0;
		
	return draw_in_armature_color_aux(start, bone, part, angle_part, &bone_color, &angle_color, &angle_select_color, 0, 0, a);
}

void draw_arcRGBA(SDL_Surface * dest, int x, int y, int rad, double start, double end, int r, int g, int b, int a)
{
	if(end < 360)
	{
		arcRGBA(dest, x, y, rad, start, end, r, g, b, a);
		return;
	}
	arcRGBA(dest, x, y, rad, start, 360, r, g, b, a);
	arcRGBA(dest, x, y, rad, 0, correct_degre_angle(end), r, g, b, a);
}

void SDL_Surface_print_generic(void * surface, int x, int y)
{
	SDL_Rect pos;
	if(surface == NULL)
		return;
	pos.x = x;
	pos.y = y;
	SDL_BlitSurface((SDL_Surface *)surface, NULL, get_screen(), &pos);
}

static void draw_shape_aux(SDL_Surface * screen, Shape shape, SDL_Color * color, SDL_Color * border_color, int x, int y)
{
	Shape cur = NULL;
	Sint16 vpx[4];
	Sint16 vpy[4];
	int px[4];
	int py[4];
	int i;
	SDL_Rect shape_center;
	SDL_Rect child_center;
	if(shape == NULL)
		return;
	shape_center.x = x + shape->center.x;
	shape_center.y = y + shape->center.y;
	if(shape->border > 0)
		filledCircleRGBA(screen, shape_center.x, shape_center.y, shape->radius + shape->border, border_color->r, border_color->g, border_color->b, 255);
	filledCircleRGBA(screen, shape_center.x, shape_center.y, shape->radius, color->r, color->g, color->b, 255);
	
	for(cur = shape->child; cur != NULL; cur = cur->sister)
	{
		child_center.x = x + cur->center.x;
		child_center.y = y + cur->center.y;
		if(! get_external_tangents_from_circles(shape->radius, shape_center, cur->radius, child_center, px + 0, py + 0, px + 3, py + 3, px + 1, py + 1, px + 2, py + 2))
			continue;
		for(i = 0; i < 4; ++i)
		{
			vpx[i] = px[i];
			vpy[i] = py[i];
		}
		filledPolygonRGBA(screen, vpx, vpy, 4, color->r, color->g, color->b, 255);
	}
	
	draw_shape_aux(screen, shape->child, color, border_color, x, y);
	draw_shape_aux(screen, shape->sister, color, border_color, x, y);
}

void draw_shape(Shape shape, SDL_Color color, SDL_Color border_color, int x, int y)
{
	draw_shape_aux(get_screen(), shape, &color, &border_color, x, y);
}

static void draw_shape_set_aux(SDL_Surface * screen, ShapeSet shape, int x, int y)
{
	int i;
	for(i = 0; i < shape->nb_elems; ++i)
		draw_shape_aux(screen, (*(shape->elems + i))->shape, &((*(shape->elems + i))->color), &((*(shape->elems + i))->border_color), x + shape->center.x, y + shape->center.y);
	#ifdef DEBUG
	circleRGBA(screen, x + shape->center.x, y + shape->center.y, 3, 255, 0, 0, 255);
	#endif
}

void draw_shape_set(ShapeSet shape, int x, int y)
{
	draw_shape_set_aux(get_screen(), shape, x, y);
}

void draw_hitbox(HitBox hitbox, int x, int y)
{
	HitBox cur = NULL;
	Sint16 vpx[4];
	Sint16 vpy[4];
	int px[4];
	int py[4];
	int i;
	SDL_Rect hitbox_center;
	SDL_Rect child_center;
	if(hitbox == NULL)
		return;
	hitbox_center.x = x + hitbox->rotated_center.x;
	hitbox_center.y = y + hitbox->rotated_center.y;
	filledCircleRGBA(get_screen(), hitbox_center.x, hitbox_center.y, hitbox->radius, 255, 0, 0, 63);
	circleRGBA(get_screen(), hitbox_center.x, hitbox_center.y, hitbox->radius, 255, 0, 0, 255);
	
	for(cur = hitbox->child; cur != NULL; cur = cur->brother)
	{
		child_center.x = x + cur->rotated_center.x;
		child_center.y = y + cur->rotated_center.y;
		if(! get_external_tangents_from_circles((hitbox->radius + cur->radius) / 2, hitbox_center, (hitbox->radius + cur->radius) / 2, child_center, px + 0, py + 0, px + 3, py + 3, px + 1, py + 1, px + 2, py + 2))
			continue;
		for(i = 0; i < 4; ++i)
		{
			vpx[i] = px[i];
			vpy[i] = py[i];
		}
		filledPolygonRGBA(get_screen(), vpx, vpy, 4, 255, 0, 0, 63);
		polygonRGBA(get_screen(), vpx, vpy, 4, 255, 0, 0, 255);
	}
	
	draw_hitbox(hitbox->child, x, y);
	draw_hitbox(hitbox->brother, x, y);
}

static void draw_select_shape_aux_explore(Shape select, SDL_Color * color, int x, int y, int recur)
{
	if(select == NULL)
		return;
	if(recur == 0) /* permet de tout explorer si valeur négative */
		return;
	circleRGBA(get_screen(), x + select->center.x, y + select->center.y, select->radius + select->border, color->r, color->g, color->b, 255);
	draw_select_shape_aux_explore(select->child, color, x, y, recur - 1);
	draw_select_shape_aux_explore(select->sister, color, x, y, recur - 1);
}

static void draw_select_shape_aux(Bone arma, Bone bone, Shape select, SDL_Color color, int recur)
{
	int x, y;
	if(! Bone_getXY_of_parent(bone, arma, &x, &y))
		return;
	x += bone->start.x + bone->shape->center.x;
	y += bone->start.y + bone->shape->center.y;
	draw_select_shape_aux_explore(select, &color, x, y, recur);
}

void draw_select_shape(Bone arma, Bone bone, Shape select, SDL_Color color)
{
	draw_select_shape_aux(arma, bone, select, color, 1);
}

void draw_select_shape_elem(Bone arma, Bone bone, Shape select, SDL_Color color)
{
	draw_select_shape_aux(arma, bone, select, color, -1); /* pas de limite de répétition on fait toute la forme */
}

static int cmp_color(const void * a, const void * b)
{
	SDL_Color * un = (SDL_Color *)a;
	SDL_Color * deux = (SDL_Color *)b;
	if(un->r != deux->r)
		return un->r - deux->r;
	if(un->g != deux->g)
		return un->g - deux->g;
	if(un->b != deux->b)
		return un->b - deux->b;
	return 0;
}

static SDL_Color ShapeSet_find_color_key(ShapeSet shape)
{
	SDL_Color res = SDL_Color_create(0, 0, 0);
	SDL_Color * tab = NULL;
	int i;
	if((tab = (SDL_Color *)malloc(2 * shape->nb_elems * sizeof(SDL_Color))) == NULL)
		return SDL_Color_create(0, 255, 0);
	for(i = 0; i < shape->nb_elems; ++i)
	{
		*(tab + i) = (*(shape->elems + i))->color;
		*(tab + i + shape->nb_elems) = (*(shape->elems + i))->border_color;
	}
	qsort(tab, 2 * shape->nb_elems, sizeof(SDL_Color), cmp_color);
	for(i = 0; i < 2 * shape->nb_elems; ++i)
	{
		if(cmp_color(&res, tab + i) < -1)
		{
			free(tab);
			return res;
		}
		if(res.r == 255 && res.g == 255 && res.b == 255)
		{
			free(tab);
			return SDL_Color_create(0, 255, 0);
		}
		if(res.b == 255)
		{
			res.b = 0;
			if(res.g == 255)
			{
				res.g = 0;
				++(res.r);
			}
			else
				++(res.g);
		}
		else
			++(res.b);
	}
	free(tab);
	return res;
}

int ShapeSet_make_image(Image * image, ShapeSet shape)
{
	int x = 0, y = 0, w = 0, h = 0;
	SDL_Color color_key;
	SDL_Surface * res = NULL;
	if(image == NULL)
		return 0;
	if(! ShapeSet_get_dimensions(shape, &x, &y, &w, &h))
	{
		fprintf(stderr, "Erreur récupération dimension ShapeSet\n");
		return 0;
	}
	w -= x;
	h -= y;
	x += shape->center.x;
	y += shape->center.y;
	color_key = ShapeSet_find_color_key(shape);
	if(! SDL_GetRectangleRGB(&res, w + 1, h + 1, color_key.r, color_key.g, color_key.b))
	{
		fprintf(stderr, "Erreur création rectangle ShapeSet (%d %d) (%d %d %d)\n", w, h, color_key.r, color_key.g, color_key.b);
		return 0;
	}
	draw_shape_set_aux(res, shape, -x, -y);
	Image_create_from_surface_copy(image, res);
	SDL_FreeSurface(res);
	Image_set_coord(*image, x, y);
	Image_set_color_key(*image, color_key);
	return 1;
}

int Armature_calculate_images(Bone arma)
{
	if(arma == NULL)
		return 1;
	Image_free(&(arma->shape_image));
	if(arma->shape->nb_elems > 0)
	{
		if(! ShapeSet_make_image(&(arma->shape_image), arma->shape))
		{
			fprintf(stderr, "ShapetSet echec creation image\n");
			return 0;
		}
	}
	if(! Armature_calculate_images(arma->child))
		return 0;
	if(! Armature_calculate_images(arma->brother))
		return 0;
	return 1;
}

int Personnage_draw(Personnage perso)
{
	/*SDL_FillRect(get_screen(), &(perso->dims), SDL_MapRGB(get_screen()->format, 0, 255, 0));*/
	if(perso == NULL)
		return 0;
	return draw_action_armature_real(perso->armature);
}