#include "file_explorer.h"

#include <stdio.h>
#include <stdlib.h>

#include "menu_list.h"
#include "color.h"
#include "image.h"

static int end_char(int car)
{
	return car == EOF || car == '\n' || car == '\0';
}

static int get_line(FILE * flow, char ** chaine)
{
	char * res = NULL;
	int taille = 0;
	int max_taille = 0;
	int car = '0';
	if(chaine == NULL)
		return 0;
	while(! end_char(car = fgetc(flow)))
	{
		if(taille == max_taille)
		{
			max_taille = (2 * max_taille) + 1;
			if((res = (char *)realloc(res, max_taille * sizeof(char))) == NULL)
				return 0;
		}
		*(res + taille) = car;
		++taille;
	}
	if(res == NULL)
		return 0;
	if((res = (char *)realloc(res, (taille + 1) * sizeof(char))) == NULL)
		return 0;
	*(res + taille) = '\0';
	*chaine = res;
	return 1;
}

int is_repertory(const char * path)
{
	char * cmd = NULL;
	char * format = "test -d %s";
	int res = 0;
	if(path == NULL)
		return 0;
	if((cmd = (char *)malloc((strlen(path) + (strlen(format) - 2) + 1) * sizeof(char))) == NULL)
		return 0;
	sprintf(cmd, format, path);
	res = ! system(cmd);
	free(cmd);
	return res;
}

int is_file_or_repertory(const char * path)
{
	FILE * lec = NULL;
	if(path == NULL)
		return 0;
	if((lec = fopen(path, "r")) == NULL)
		return 0;
	fclose(lec);
	return 1;
}

int is_file(const char * path)
{
	return is_file_or_repertory(path) && ! is_repertory(path);
}

static int get_back_folder(const char * path, char ** back_folder)
{
	int taille = 0;
	if(path == NULL)
		return 0;
	for(taille = strlen(path) - 1; taille >= 0; --taille)
	{
		if(*(path + taille) == '/')
			break;
	}
	if(taille < 0)
		return 0;
	if(strcmp(path, "/") == 0 || taille == 0)
	{
		if((*back_folder = (char *)malloc(2 * sizeof(char))) == NULL)
			return 0;
		strcpy(*back_folder, "/");
		return 1;
	}
	if((*back_folder = (char *)malloc((taille + 1) * sizeof(char))) == NULL)
		return 0;
	for(*(*back_folder + (taille--)) = '\0'; taille >= 0; --taille)
		*(*back_folder + taille) = *(path + taille);
	return 1;
}

int File_Explorer_create(File_Explorer * exp)
{
	File_Explorer res = NULL;
	
	if((res = (File_Explorer)malloc(sizeof(struct _File_Explorer))) == NULL)
		return 0;
	
	res->path = NULL;
	res->content = NULL;
	res->back_input = NULL;
	
	if(! Input_create(&(res->path))
	|| ! Menu_create(&(res->content))
	|| ! Input_set_font(res->path, "media/arial.ttf", 14, SDL_Color_create(255, 216, 0)))
	{
		File_Explorer_free(&res);
		return 0;
	}
	
	*exp = res;
	return 1;
}

void File_Explorer_free(File_Explorer * exp)
{
	if(exp == NULL || *exp == NULL)
		return;
	if((*exp)->back_input != NULL)
		SDL_FreeSurface((*exp)->back_input);
	Input_free(&((*exp)->path));
	Menu_free(&((*exp)->content));
	free(*exp);
	*exp = NULL;
}

int File_Explorer_menu_define(File_Explorer exp)
{
	FILE * cmd = NULL;
	char * rep_text = NULL;
	char * cmd_text = NULL;
	char * format_text = "ls -1 %s 2>/dev/null | wc -l";
	char * line = NULL;
	int nb_elems = 0;
	int not_racine = 1;
	int i;
	
	if(exp == NULL || exp->path == NULL || exp->content == NULL)
		return 0;
	if(! Input_get_text(exp->path, &rep_text))
		return 0;
	if(strcmp(rep_text, "/") == 0)
		not_racine = 0;
	if((cmd_text = (char *)malloc((strlen(rep_text) + (strlen(format_text) - 2) + 1) * sizeof(char))) == NULL)
		return 0;
	sprintf(cmd_text, format_text, rep_text);
	if((cmd = popen(cmd_text, "r")) == NULL)
	{
		free(cmd_text);
		return 0;
	}
	if(fscanf(cmd, "%d", &nb_elems) != 1)
	{
		free(cmd_text);
		pclose(cmd);
		return 0;
	}
	free(cmd_text);
	pclose(cmd);
	if(! Menu_alloc_elems(exp->content, nb_elems + not_racine))
		return 0;
	format_text = "ls -1 %s 2>/dev/null";
	if((cmd_text = (char *)malloc((strlen(rep_text) + (strlen(format_text) - 2) + 1) * sizeof(char))) == NULL)
		return 0;
	sprintf(cmd_text, format_text, rep_text);
	if((cmd = popen(cmd_text, "r")) == NULL)
	{
		free(cmd_text);
		return 0;
	}
	if(not_racine)
	{
		if(! Menu_define_elem(exp->content, 0, NULL, MENU_ELEMENT_TEXT, "..", NULL))
			return 0;
	}
	for(i = not_racine; get_line(cmd, &line); ++i)
	{
		if(! Menu_define_elem(exp->content, i, NULL, MENU_ELEMENT_TEXT, line, NULL))
		{
			free(line);
			return 0;
		}
		free(line);
	}
	free(cmd_text);
	pclose(cmd);
	Menu_List_reset_scroll(exp->content);
	if(! Menu_build_user_data(exp->content))
		return 0;
	return 1;
}

int File_Explorer_define(File_Explorer exp, SDL_Surface * screen)
{
	FILE * cmd = NULL;
	char * pwd = NULL;
	SDL_Surface * back = NULL;
	SDL_Surface * back_hover = NULL;
	if(screen == NULL)
		return 0;
	if(exp == NULL || exp->path == NULL || exp->content == NULL)
		return 0;
	if((cmd = popen("pwd", "r")) == NULL)
		return 0;
	if(! get_line(cmd, &pwd))
	{
		pclose(cmd);
		return 0;
	}
	pclose(cmd);
	if(! Input_redefine(exp->path, pwd))
	{
		free(pwd);
		return 0;
	}
	free(pwd);
	if(! SDL_GetRectangleRGB(&back, screen->w, 16, 64, 64, 64))
		return 0;
	if(! SDL_GetRectangleRGB(&back_hover, screen->w, 16, 96, 96, 96))
	{
		SDL_FreeSurface(back);
		return 0;
	}
	if(! Menu_define_Menu_List(exp->content, "media/arial.ttf", 14, SDL_Color_create(255, 255, 255), back, back_hover)
	|| ! Input_set_box(exp->path, back))
	{
		SDL_FreeSurface(back);
		SDL_FreeSurface(back_hover);
		return 0;
	}
	Menu_set_origin(exp->content, 0, back->h * 2 + 20);
	Menu_List_set_align(exp->content, MENU_LIST_ALIGN_LEFT);
	Menu_List_reset_scroll(exp->content);
	Input_set_origin(exp->path, 0, 5);
	
	SDL_FreeSurface(back);
	SDL_FreeSurface(back_hover);
	if(exp->back_input != NULL)
		SDL_FreeSurface(exp->back_input);
	if(! SDL_GetRectangleRGB(&(exp->back_input), screen->w, back->h * 2 + 20, 32, 32, 32))
		return 0;
	if(! File_Explorer_menu_define(exp))
		return 0;
	return 1;
}

int File_Explorer_gest_event(File_Explorer exp, SDL_Surface * screen, File_Explorer_Mode mode, char ** file_name)
{
	int need_refresh = 1;
	int exit_explorer = 0;
	int is_select_input = 0;
	char * last_input_text = NULL;
	SDL_Event event;
	char * name = NULL;
	SDL_Rect back_pos;
	back_pos.x = back_pos.y = 0;
	
	if(exp == NULL || exp->path == NULL || exp->content == NULL || screen == NULL)
		return 0;
	is_select_input = exp->path->is_select;
	
	if(! Input_get_text(exp->path, &name))
		goto error_zone;
	if(name != NULL)
	{
		if(is_file(name))
		{
			char * res = NULL;
			if(! get_back_folder(name, &res))
				goto error_zone;
			if(! Input_redefine(exp->path, res))
			{
				free(res);
				goto error_zone;
			}
			free(res);
		}
		name = NULL;
		if(! File_Explorer_menu_define(exp))
			goto error_zone;
	}
	
	while(! exit_explorer)
	{
		if(need_refresh)
		{
			SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 32, 32, 32));
			Menu_draw(exp->content, screen);
			SDL_BlitSurface(exp->back_input, NULL, screen, &back_pos);
			Input_draw(exp->path, screen);
			SDL_Flip(screen);
			need_refresh = 0;
		}
		SDL_WaitEvent(&event);
		if(! exp->path->is_select)
			need_refresh |= Menu_event(exp->content, &event, NULL, NULL, MENU_OPTION_GET_TEXT, &name);
		need_refresh |= Input_event(exp->path, &event, NULL);
		if(exp->path->is_select != is_select_input)
		{
			if(is_select_input)
			{
				char * tmp = NULL;
				if(! Input_get_text(exp->path, &tmp))
					goto error_zone;
				if(! is_file_or_repertory(tmp))
				{
					if(mode == FILE_EXPLORER_MODE_SAVE)
					{
						if(! Input_get_text(exp->path, &name))
							goto error_zone;
						if(file_name != NULL)
							*file_name = name;
						goto success_zone;
					}
					else
					{
						if(! Input_redefine(exp->path, last_input_text))
							goto error_zone;
					}
				}
				if(*(tmp + (strlen(tmp) - 1)) == '/' && strcmp(tmp, "/") != 0)
				{
					Text_Input_set_cursor_at_end(exp->path->text);
					Text_Input_del_char_before(exp->path->text);
					Input_surface_calcul(exp->path);
				}
				if(! File_Explorer_menu_define(exp))
					goto error_zone;
			}
			else
			{
				if(last_input_text != NULL)
				{
					free(last_input_text);
					last_input_text = NULL;
				}
				if(! Input_get_allocated_text(exp->path, &last_input_text))
					goto error_zone;
			}
			is_select_input = exp->path->is_select;
		}
		if(name != NULL)
		{
			if(strcmp(name, "..") == 0)
			{
				char * res = NULL;
				if(! Input_get_text(exp->path, &name))
					goto error_zone;
				if(! get_back_folder(name, &res))
					goto error_zone;
				if(! Input_redefine(exp->path, res))
				{
					free(res);
					goto error_zone;
				}
				free(res);
			}
			else
			{
				char * tmp = NULL;
				if(! Input_get_text(exp->path, &tmp))
					goto error_zone;
				if(strcmp(tmp, "/") != 0)
				{
					if(! Text_Input_add_char(exp->path->text, '/'))
						return 0;
				}
				if(! Input_concat_text(exp->path, name))
					goto error_zone;
				if(! Input_get_text(exp->path, &name))
					goto error_zone;
				if(is_file(name))
				{
					if(file_name != NULL)
						*file_name = name;
					#ifdef DEBUG
					printf("select file : \"%s\"\n", name);
					#endif
					goto success_zone;
				}
			}
			if(! File_Explorer_menu_define(exp))
				goto error_zone;
			name = NULL;
		}
		switch(event.type)
		{
			case SDL_QUIT:
			exit_explorer = 1;
			break;
			
			case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
				exit_explorer = 1;
				break;
				
				default:
				break;
			}
			break;
		}
	}
	
	success_zone:;
	
	if(last_input_text != NULL)
		free(last_input_text);
	
	return 1;
	
	error_zone:;
	
	if(last_input_text != NULL)
		free(last_input_text);
	
	return 0;
}