#include "game.h"

#include "window.h"
#include "armature.h"
#include "draw.h"
#include "math.h"
#include "text_input.h"
#include "file_explorer.h"
#include "gest.h"
#include "test.h"
#include "dialog.h"
#include "personnage.h"
#include "edit.h"
#include "file_saver.h"
#include "arena.h"

static File_Explorer file_explorer = NULL;
static FileSaver file_saver;

int Game_play()
{
	Game game;
	int success = 1;
		
	if(! Game_init(&game))
	{
		fprintf(stderr, "Error in game initialisation\n");
		return 0;
	}
	
	success &= Game_gest_menu(&game);
	
	Game_quit(&game);
	
	/*Gest_editor(&perso);*/
	
	return 1;
}

static int Game_def_menu_back(Game * game)
{
	SDL_Surface * rectangle = NULL;
	SDL_Surface * rectangle_hover = NULL;
	int success = 1;
	
	if(! SDL_GetMecaRectangleRGB(&rectangle, 250, 50, 19, 123, 188))
		return 0;
	if(! SDL_GetMecaRectangleRGB(&rectangle_hover, 250, 50, 23, 151, 230))
	{
		SDL_FreeSurface(rectangle);
		return 0;
	}
	success &= Menu_define_Menu_List(game->player_menu, "media/arial.ttf", 25, SDL_Color_create(255, 255, 255), rectangle, rectangle_hover);
	success &= Menu_define_Menu_List(game->adv_menu, "media/arial.ttf", 25, SDL_Color_create(255, 255, 255), rectangle, rectangle_hover);
	success &= Menu_define_Menu_List(game->main_menu, "media/arial.ttf", 25, SDL_Color_create(255, 255, 255), rectangle, rectangle_hover);
	Menu_set_origin(game->player_menu, 100, 50);
	Menu_set_origin(game->adv_menu, get_screen()->w - rectangle->w - 100, 50);
	Menu_set_origin(game->main_menu, get_screen()->w / 2 - rectangle->w / 2, (get_screen()->h * 3) / 4);
	SDL_FreeSurface(rectangle);
	SDL_FreeSurface(rectangle_hover);
	return success;
}

int Game_init(Game * game)
{
	if(game == NULL)
		return 0;
	game->player = NULL;
	game->adv = NULL;
	game->player_fighter = NULL;
	game->adv_fighter = NULL;
	game->player_menu = NULL;
	game->adv_menu = NULL;
	game->main_menu = NULL;
	
	if(! Edit_init())
		return 0;
	if(! FileSaver_create(&file_saver))
		return 0;
	if(! File_Explorer_create(&file_explorer)
	|| ! File_Explorer_define(file_explorer, get_screen()))
	{
		fprintf(stderr, "Error of starting file explorer\n");
		Game_quit(game);
		return 0;
	}
	if(! Arena_generate_background())
	{
		fprintf(stderr, "Error no arena background created\n");
		Game_quit(game);
		return 0;
	}
	
	if(! Personnage_create(&(game->player))
	|| ! Personnage_create(&(game->adv)))
	{
		fprintf(stderr, "Starting caracter can't be created\n");
		Game_quit(game);
		return 0;
	}
	Personnage_set_coord(game->player, get_screen()->w / 2, (get_screen()->h * 2) / 3);
	Personnage_set_coord(game->adv, get_screen()->w / 2, (get_screen()->h * 2) / 3);
	
	if(! Menu_create(&(game->player_menu))
	|| ! Menu_create(&(game->adv_menu))
	|| ! Menu_create(&(game->main_menu)))
	{
		fprintf(stderr, "Error in Game menu creation\n");
		Game_quit(game);
		return 0;
	}
	if(! Game_def_menu_back(game)
	|| ! Menu_configure(game->player_menu, 3,
				&Game_create_perso, MENU_ELEMENT_TEXT, "Creer", NULL,
				&Game_load_perso, MENU_ELEMENT_TEXT, "Charger", NULL,
				&Game_edit_perso, MENU_ELEMENT_TEXT, "Editer", NULL)
	|| ! Menu_configure(game->adv_menu, 3,
				&Game_create_perso, MENU_ELEMENT_TEXT, "Creer", NULL,
				&Game_load_perso, MENU_ELEMENT_TEXT, "Charger", NULL,
				&Game_edit_perso, MENU_ELEMENT_TEXT, "Editer", NULL)
	|| ! Menu_configure(game->main_menu, 1,
				&Game_start_fight, MENU_ELEMENT_TEXT, "Fight !", NULL))
	{
		fprintf(stderr, "Error in Game menu configuration\n");
		Game_quit(game);
		return 0;
	}
	return 1;	
}

void Game_quit(Game * game)
{
	if(game == NULL)
		return;
	Personnage_free(&(game->player));
	Personnage_free(&(game->adv));
	Personnage_free(&(game->player_fighter));
	Personnage_free(&(game->adv_fighter));
	Menu_free(&(game->player_menu));
	Menu_free(&(game->adv_menu));
	Menu_free(&(game->main_menu));
	Arena_free_background();
	File_Explorer_free(&file_explorer);
	Edit_quit();
}

int Game_gest_menu(Game * game)
{
	int cont = 1;
	int success = 1;
	int need_refresh = 1;
	int last_refresh_time = SDL_GetTicks() - REFRESH_TIME - 1;
	int (*perso_function)(Game *, Personnage *, GamePersoId) = NULL;
	int (*main_function)(Game *) = NULL;
	SDL_Event event;
	
	while(cont && success)
	{
		if(need_refresh && last_refresh_time + REFRESH_TIME < SDL_GetTicks())
		{
			clear_screen();
			Edit_print_background(get_screen());
			Personnage_draw(game->player_fighter);
			Personnage_draw(game->adv_fighter);
			Menu_draw(game->main_menu, get_screen());
			Menu_draw(game->player_menu, get_screen());
			Menu_draw(game->adv_menu, get_screen());
			refresh_screen();
			
			need_refresh = 0;
			last_refresh_time = SDL_GetTicks();
		}
		
		SDL_WaitEvent(&event);
		
		switch(event.type)
		{
			case SDL_QUIT:
			cont = 0;
			break;
			
			case SDL_KEYUP:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
				cont = 0;
				break;
				
				default:
				break;
			}
			break;
			
			default:
			break;
		}
		main_function = NULL;
		if(Menu_event(game->main_menu, &event, &main_function, NULL, MENU_OPTION_END))
		{
			if(main_function != NULL)
				cont &= (*main_function)(game);
			need_refresh = 1;
		}
		perso_function = NULL;
		if(Menu_event(game->player_menu, &event, &perso_function, NULL, MENU_OPTION_END))
		{
			if(perso_function != NULL)
				cont &= (*perso_function)(game, &(game->player), GPI_PLAYER);
			need_refresh = 1;
		}
		perso_function = NULL;
		if(Menu_event(game->adv_menu, &event, &perso_function, NULL, MENU_OPTION_END))
		{
			if(perso_function != NULL)
				cont &= (*perso_function)(game, &(game->adv), GPI_ADV);
			need_refresh = 1;
		}
	}
	
	return success;
}

int Game_create_perso(Game * game, Personnage * perso, GamePersoId id)
{
	Personnage new_perso = NULL;
	if(! Personnage_create(&new_perso))
	{
		fprintf(stderr, "Error no caracter created for edition\n");
		return 0;
	}
	Personnage_set_coord(new_perso, get_screen()->w / 2, (get_screen()->h * 2) / 3);
	if(! Gest_editor(&new_perso, file_explorer, &file_saver))
	{
		Personnage_free(&new_perso);
		return 0;
	}
	Personnage_free(perso);
	*perso = new_perso;
	Game_make_fighter(game, id);
	return 1;
}

int Game_load_perso(Game * game, Personnage * perso, GamePersoId id)
{
	Personnage loaded = NULL;
	file_saver.file_name = NULL;
	File_Explorer_gest_event(file_explorer, get_screen(), FILE_EXPLORER_MODE_LOAD, &(file_saver.file_name));
	if(file_saver.file_name != NULL)
	{
		if(! Personnage_load_from_file(file_saver.file_name, &loaded))
		{
			Dialog_global_print(get_screen(), "Echec du chargement du personnage");
			Personnage_free(&loaded);
			return 1;
		}
		Personnage_free(perso);
		*perso = loaded;
		Game_make_fighter(game, id);
	}
	return 1;
}

int Game_edit_perso(Game * game, Personnage * perso, GamePersoId id)
{
	if(! Gest_editor(perso, file_explorer, &file_saver))
		return 0;
	Game_make_fighter(game, id);
	return 1;
}

int Game_start_fight(Game * game)
{
	Arena arena;
	int success = 1;
	
	if(! Personnage_able_to_fight(game->player_fighter))
	{
		Dialog_global_print(get_screen(), "Le personnage joueur n'est pas autorisé à se battre");
		return 1;
	}
	if(! Personnage_able_to_fight(game->adv_fighter))
	{
		Dialog_global_print(get_screen(), "Le personnage adverse n'est pas autorisé à se battre");
		return 1;
	}
	
	Arena_init_fight(&arena, game->player_fighter, game->adv_fighter);
	success &= Arena_play_fight(&arena);
	Arena_quit(&arena);
	
	return success;
}

int Game_make_fighter(Game * game, GamePersoId id)
{
	Personnage mini = NULL;
	if(id == GPI_NULL)
		return 0;
	if(! Personnage_copy(&mini, (id == GPI_PLAYER) ? (game->player) : (game->adv)))
		return 0;
	Personnage_scale(mini, 0.5);
	if(id == GPI_ADV)
		Personnage_reverse(mini);
	Personnage_set_coord(mini, (get_screen()->w * ((id == GPI_PLAYER) ? 1 : 7)) / 8, get_screen()->h);
	Personnage_prepare(mini);
	Personnage_put_on_ground(mini, get_screen()->h);
	switch(id)
	{
		case GPI_PLAYER:
		Personnage_free(&(game->player_fighter));
		game->player_fighter = mini;
		break;
		
		case GPI_ADV:
		Personnage_free(&(game->adv_fighter));
		game->adv_fighter = mini;
		break;
		
		default:
		Personnage_free(&mini);
		return 0;
	}
	return 1;
}