#include "gest.h"

#include "window.h"
#include "armature_editor.h"
#include "shape_editor.h"
#include "file_explorer.h"
#include "action_edit.h"
#include "file_saver.h"
#include "arena.h"

int Gest_editor(Personnage * perso, File_Explorer file_explorer, FileSaver * file_saver)
{
	EditExit choice = EDIT_EXIT_GOTO_ARMA_EDITOR;
	ArmaEdit arma_edit_data;
	ShapeEdit shape_edit_data;
	ActionEdit_Select action_edit_select;
	Bone * arma = NULL;
	Arena arena;
	
	int retour = 1;
	
	if(perso == NULL || *perso == NULL)
		return 0;
	arma = &((*perso)->armature);
	
	if(! ArmaEdit_create(&arma_edit_data, arma))
		return 0;
	if(! ShapeEdit_create(&shape_edit_data, arma))
		return 0;
	if(! ActionEdit_Select_create(&action_edit_select, perso))
		return 0;
	
	if(is_file(LAST_SHAPE_LIST))
	{
		if(! ShapeList_load_from_file(LAST_SHAPE_LIST, shape_edit_data.select->shape_list))
			fprintf(stderr, "Error while loading \"%s\" when starting\n", LAST_SHAPE_LIST);
	}
	while(choice != EDIT_EXIT_QUIT)
	{
		switch(choice)
		{
			case EDIT_EXIT_GOTO_ARMA_EDITOR:
			if(! Armature_edit(perso, &arma_edit_data, &choice, file_explorer, file_saver))
				goto exit_zone;
			break;
			
			case EDIT_EXIT_GOTO_SHAPE_EDITOR:
			if(! Shape_edit(perso, &shape_edit_data, &choice, file_explorer, file_saver))
				goto exit_zone;
			break;
			
			case EDIT_EXIT_GOTO_ACTION_EDITOR:
			if(! Action_edit(perso, action_edit_select, &choice, file_explorer, file_saver))
				goto exit_zone;
			break;
			
			case EDIT_EXIT_GOTO_ARENA:
			if(! Arena_init_test(&arena, *perso))
				goto exit_zone;
			if(! Arena_play_test(&arena, &choice))
				goto exit_zone;
			Arena_quit(&arena);
			break;
			
			case EDIT_EXIT_GOTO_GAME:
			goto exit_zone;
			break;
			
			default:
			return 0;
		}
	}
	exit_zone:;
	
	if(! ShapeList_save_in_file(LAST_SHAPE_LIST, shape_edit_data.select->shape_list))
		fprintf(stderr, "Error while saving \"%s\" at exit\n", LAST_SHAPE_LIST);
	
	ArmaEdit_destroy(&arma_edit_data);
	ShapeEdit_destroy(&shape_edit_data);
	ActionEdit_Select_free(&action_edit_select);
	Color_quit();
	
	return retour;
}