#include "hitbox.h"

#include <stdio.h>
#include <stdlib.h>

#include "generic_meca.h"
#include "math.h"

int HitBox_create(HitBox * hitbox, int x, int y, int radius)
{
	HitBox res = NULL;
	
	if((res = (HitBox)malloc(sizeof(struct _HitBox))) == NULL)
		return 0;
	res->center.x = x;
	res->center.y = y;
	res->radius = radius;
	res->brother = NULL;
	res->child = NULL;
	
	res->rotated_center = res->center;
	
	*hitbox = res;
	return 1;
}

void HitBox_free(HitBox * hitbox)
{
	if(hitbox == NULL || *hitbox == NULL)
		return;
	HitBox_free(&((*hitbox)->child));
	HitBox_free(&((*hitbox)->brother));
	free(*hitbox);
	*hitbox = NULL;
}

int HitBox_set_center(HitBox hitbox, int x, int y)
{
	if(hitbox == NULL)
		return 0;
	if(hitbox->center.x == x && hitbox->center.y == y)
		return 0;
	hitbox->center.x = x;
	hitbox->center.y = y;
	hitbox->rotated_center = hitbox->center;
	return 1;
}

int HitBox_set_radius(HitBox hitbox, int radius)
{
	if(hitbox == NULL)
		return 0;
	if(hitbox->radius == radius)
		return 0;
	hitbox->radius = radius;
	return 1;
}

void HitBox_rotate(HitBox hitbox, double angle)
{
	SDL_Rect origin;
	if(hitbox == NULL)
		return;
	origin.x = origin.y = 0;
	hitbox->rotated_center = get_SDL_Rect_by_rotation(hitbox->center, origin, degre_to_rad(angle));
	HitBox_rotate(hitbox->child, angle);
	HitBox_rotate(hitbox->brother, angle);
}

static int HitBox_generic_create(void * hitbox)
{
	return HitBox_create((HitBox *)hitbox, 0, 0, 0);
}

int HitBox_append(HitBox * hitbox, HitBox * appened, int x, int y, int radius)
{
	HitBox res = NULL;
	if(! TreeChildBrother_generic_append(hitbox, &HitBox_generic_create, get_id_in_structure(child, *hitbox), get_id_in_structure(brother, *hitbox), &res))
		return 0;
	if(res == NULL)
		return 0;
	HitBox_set_center(res, x, y);
	HitBox_set_radius(res, radius);
	if(appened != NULL)
		*appened = res;
	return 1;
}

void HitBox_generic_free(void * hitbox)
{
	HitBox_free((HitBox *)hitbox);
}

int HitBox_delete(HitBox * hitbox, HitBox del)
{
	if(! TreeChildBrother_generic_delete(hitbox, del, &HitBox_generic_free, get_id_in_structure(child, *hitbox), get_id_in_structure(brother, *hitbox), NULL))
		return 0;
	return 1;
}

int HitBox_copy(HitBox * copy, HitBox hitbox)
{
	HitBox res = NULL;
	if(hitbox == NULL)
	{
		*copy = NULL;
		return 1;
	}
	if(! HitBox_create(&res, hitbox->center.x, hitbox->center.y, hitbox->radius))
		return 0;
	res->rotated_center = hitbox->rotated_center;
	if(! HitBox_copy(&(res->child), hitbox->child)
	|| ! HitBox_copy(&(res->brother), hitbox->brother))
	{
		HitBox_free(&res);
		return 0;
	}
	*copy = res;
	return 1;
}

static void HitBox_get_nearest_aux(HitBox * hitbox, HitBox ** nearest, int x, int y, int * min_distance)
{
	int distance;
	if(hitbox == NULL || *hitbox == NULL)
		return;
	if((distance = distance_carre((*hitbox)->center.x, (*hitbox)->center.y, x, y)) < *min_distance || *min_distance == -1)
	{
		*nearest = hitbox;
		*min_distance = distance;
	}
	HitBox_get_nearest_aux(&((*hitbox)->child), nearest, x, y, min_distance);
	HitBox_get_nearest_aux(&((*hitbox)->brother), nearest, x, y, min_distance);
}

static int HitBox_get_nearest(HitBox * hitbox, HitBox ** nearest, int x, int y)
{
	int distance = -1;
	HitBox_get_nearest_aux(hitbox, nearest, x, y, &distance);
	return distance != -1;
}

static int HitBox_get_included(HitBox hitbox, HitBox * included, int x, int y, int radius)
{
	if(hitbox == NULL)
		return 0;
	if(circle_in_circle(x, y, radius, hitbox->center.x, hitbox->center.y, hitbox->radius))
	{
		*included = hitbox;
		return 1;
	}
	if(HitBox_get_included(hitbox->child, included, x, y, radius))
		return 1;
	if(HitBox_get_included(hitbox->brother, included, x, y, radius))
		return 1;
	return 0;
}

static int HitBox_generate_on_shape(HitBox * hitbox, Shape shape, int add_x, int add_y)
{
	HitBox * near = NULL;
	if(shape == NULL)
		return 1;
	if(! HitBox_get_nearest(hitbox, &near, shape->center.x + add_x, shape->center.y + add_y))
		return 0;
	if(circle_in_circle((*near)->center.x, (*near)->center.y, (*near)->radius, shape->center.x + add_x, shape->center.y + add_y, shape->radius))
	{
		if(shape->radius > (*near)->radius)
		{
			(*near)->center.x = shape->center.x + add_x;
			(*near)->center.y = shape->center.y + add_y;
			(*near)->radius = shape->radius;
		}
	}
	else
	{
		HitBox included = NULL;
		if(HitBox_get_included(*hitbox, &included, shape->center.x, shape->center.y, shape->radius))
		{
			if(included->radius < shape->radius)
			{
				included->center.x = shape->center.x + add_x;
				included->center.y = shape->center.y + add_y;
				included->radius = shape->radius;
			}
		}
		else if(! HitBox_append(near, NULL, shape->center.x + add_x, shape->center.y + add_y, shape->radius))
			return 0;
	}
	HitBox_generate_on_shape(hitbox, shape->child, add_x, add_y);
	HitBox_generate_on_shape(hitbox, shape->sister, add_x, add_y);
	return 1;
}

static void HitBox_correct_next(HitBox hitbox)
{
	if(hitbox != NULL)
	{
		if(hitbox->child == NULL)
		{
			hitbox->child = hitbox->brother;
			hitbox->brother = NULL;
		}
	}
}

static int HitBox_simplify_aux(HitBox * hitbox, HitBox * cur)
{
	int simpl = 0;
	if(cur == NULL || *cur == NULL)
		return 1;
	if(hitbox == NULL || *hitbox == NULL)
		return 1;
	if(*cur != *hitbox && (*cur)->radius < (*hitbox)->radius)
	{
		if((*cur)->radius < HB_GROW_LIM)
			simpl = 1;
		else if(circle_in_circle((*cur)->center.x, (*cur)->center.y, (*cur)->radius + HB_GROW_LIM * (((*cur)->radius < (*hitbox)->radius) ? (-1) : (1)),
					(*hitbox)->center.x, (*hitbox)->center.y, (*hitbox)->radius + HB_GROW_LIM * (((*cur)->radius >= (*hitbox)->radius) ? (-1) : (1))))
			simpl = 1;
		else
		{
			HitBox next = NULL;
			for(next = (*hitbox)->child; next != NULL; next = next->brother)
			{
				if(next == *cur)
					continue;
				if(distance_carre_segment((*cur)->center.x, (*cur)->center.y, (*hitbox)->center.x, (*hitbox)->center.y, next->center.x, next->center.y) < carre(((*hitbox)->radius + next->radius) / 2 - (*cur)->radius + HB_GROW_LIM))
					simpl = 1;
			}
		}
	}
	if(simpl)
	{
		if(! HitBox_delete(cur, *cur))
			return 0;
		HitBox_correct_next(*cur);
		return HitBox_simplify_aux(hitbox, cur);
	}
	if(! HitBox_simplify_aux(hitbox, &((*cur)->brother)))
		return 0;
	if(! HitBox_simplify_aux(hitbox, &((*cur)->child)))
		return 0;
	return 1;
}

static int HitBox_simplify_pre_aux(HitBox * racine, HitBox * cur)
{
	if(cur == NULL || *cur == NULL)
		return 1;
	if(! HitBox_simplify_aux(cur, racine))
		return 0;
	if(*cur != NULL)
	{
		if(! HitBox_simplify_pre_aux(racine, &((*cur)->child)))
			return 0;
	}
	if(*cur != NULL)
	{
		if(! HitBox_simplify_pre_aux(racine, &((*cur)->brother)))
			return 0;
	}
	return 1;	
}

static int HitBox_simplify(HitBox * hitbox)
{
	if(hitbox == NULL || *hitbox == NULL)
		return 1;
	if(! HitBox_simplify_pre_aux(hitbox, hitbox))
		return 0;
	return 1;
}

int HitBox_generate(HitBox * hitbox, SDL_Rect bone_start, SDL_Rect bone_end, ShapeSet shape)
{
	HitBox res = NULL;
	int i;
	if(! HitBox_append(&res, NULL, 0, 0, BONE_HITBOX)
	|| ! HitBox_append(&res, NULL, bone_end.x - bone_start.x, bone_end.y - bone_start.y, BONE_HITBOX))
	{
		HitBox_free(&res);
		return 0;
	}
	for(i = 0; i < shape->nb_elems; ++i)
	{
		if(! HitBox_generate_on_shape(&res, (*(shape->elems + i))->shape, shape->center.x, shape->center.y))
		{
			HitBox_free(&res);
			return 0;
		}
	}
	if(! HitBox_simplify(&res))
	{
		HitBox_free(&res);
		return 0;
	}
	*hitbox = res;
	return 1;
}

int HitBox_dimensions_aux(HitBox hitbox, int * x_min, int * y_min, int * x_max, int * y_max, int x, int y)
{
	if(hitbox == NULL)
		return 1;
	if(x_min != NULL && hitbox->rotated_center.x + x < *x_min)
		*x_min = hitbox->rotated_center.x + x - hitbox->radius;
	if(y_min != NULL && hitbox->rotated_center.y + y < *y_min)
		*y_min = hitbox->rotated_center.y + y - hitbox->radius;
	if(x_max != NULL && hitbox->rotated_center.x + x > *x_max)
		*x_max = hitbox->rotated_center.x + x + hitbox->radius;
	if(y_max != NULL && hitbox->rotated_center.y + y > *y_max)
		*y_max = hitbox->rotated_center.y + y + hitbox->radius;
	if(! HitBox_dimensions_aux(hitbox->child, x_min, y_min, x_max, y_max, x, y))
		return 0;
	if(! HitBox_dimensions_aux(hitbox->brother, x_min, y_min, x_max, y_max, x, y))
		return 0;
	return 1;
}

int HitBox_dimensions(HitBox hitbox, int * x_min, int * y_min, int * x_max, int * y_max)
{
	if(hitbox == NULL)
		return 0;
	if(x_min != NULL)
		*x_min = hitbox->rotated_center.x;
	if(y_min != NULL)
		*y_min = hitbox->rotated_center.y;
	if(x_max != NULL)
		*x_max = hitbox->rotated_center.x;
	if(y_max != NULL)
		*y_max = hitbox->rotated_center.y;
	return HitBox_dimensions_aux(hitbox, x_min, y_min, x_max, y_max, 0, 0);
}

int HitBox_check_collision(HitBox first, HitBox second, int first_x, int first_y, int second_x, int second_y)
{
	return 1;
}