#ifndef DEF_HEADER_IMAGE
#define DEF_HEADER_IMAGE

#include <SDL/SDL.h>

#define IMPACT_SIZE 20

typedef struct _Image * Image;
struct _Image
{
	SDL_Surface * surface;
	SDL_Rect coord;
	int use_color_key;
	SDL_Color color_key;
	
	SDL_Surface * last_rotated;
	double last_angle;
	int is_angle;
	SDL_Rect last_rotated_coord;
};

int SDL_CopySurface(SDL_Surface ** dest, SDL_Surface * src);

int in_surface_area(int x, int y, SDL_Rect origin, SDL_Surface * surface);

int in_area(int x, int y, int ox, int oy, int fx, int fy);

int in_SDL_Rect(int x, int y, SDL_Rect rect);

void draw_rectangle(SDL_Surface * surface, int x, int y, int w, int h, int r, int g, int b);

int SDL_GetRectangleRGB(SDL_Surface ** surface, int width, int height, int r, int g, int b);

int SDL_GetMecaRectangleRGB(SDL_Surface ** surface, int width, int height, int r, int g, int b);

void SDL_GetPixelOnSurface(SDL_Surface * surface, int x, int y, SDL_Color * color);

void SDL_SetPixelOnSurface(SDL_Surface * surface, int x, int y, SDL_Color color);

void SDL_SetPixelOnSurfaceIfNotColorKey(SDL_Surface * surface, int x, int y, int intensity, SDL_Color color, SDL_Color color_key, SDL_Color (*affect)(SDL_Color, SDL_Color, int));

/* Image creation */

int Image_create(Image * image);

int Image_create_from_surface_copy(Image * image, SDL_Surface * surface);

void Image_free(Image * image);

int Image_copy(Image * copy, Image image);

/* Image edition */

int Image_replace_surface_by_copy(Image image, SDL_Surface * surface);

int Image_set_coord(Image image, int x, int y);

int Image_clear(Image image);

int Image_set_color_key(Image image, SDL_Color color_key);

void Image_fillRect(Image image, SDL_Rect * coord, int r, int g, int b);

void Image_fillHorizontalDegrade(Image image, int start, int end, int start_r, int start_g, int start_b, int end_r, int end_g, int end_b, int (*transition)(int time, int start_time, int end_time, int start_data, int end_data));

int transition_linear(int time, int start_time, int end_time, int start_data, int end_data);

void Image_fillCircleDegrade(Image image, int x, int y, int rad, int r, int g, int b);

void Image_draw_impact(Image image);

void Image_clear_rotate(Image image);

/* Image print */

int Image_print_on_surface(SDL_Surface * dest, Image image);

int Image_print_on_surface_with_add(SDL_Surface * dest, Image image, int add_x, int add_y);

int Image_print_rotated_on_surface(SDL_Surface * dest, Image image, int add_x, int add_y, double rad_angle, int axe_x, int axe_y);

/* Image info */

int Image_exists(Image image);

int Image_get_width(Image image);

int Image_get_height(Image image);

SDL_Surface * Image_get_surface(Image image);

#endif