#include "key_gest.h"

#include <stdio.h>
#include <stdlib.h>

Key_Gest Key_Gest_create()
{
	Key_Gest gest;
	gest.key = KGK_NULL;
	return gest;
}

int Key_Gest_key_in(Key_Gest * gest, Key_Gest_Key key)
{
	if(gest == NULL)
		return 0;
	return (gest->key & key) != 0;
}

int Key_Gest_key_append(Key_Gest * gest, Key_Gest_Key key)
{
	if(gest == NULL)
		return 0;
	if(Key_Gest_key_in(gest, key))
		return 0;
	(gest->key) |= key;
	return 1;
}

int Key_Gest_key_delete(Key_Gest * gest, Key_Gest_Key key)
{
	if(gest == NULL)
		return 0;
	if(! Key_Gest_key_in(gest, key))
		return 0;
	(gest->key) &= ~key;
	return 1;
}

int Key_Gest_key_clean(Key_Gest * gest)
{
	if(gest == NULL)
		return 0;
	(gest->key) = 0;
	return 1;
}

int Key_Gest_equal(const Key_Gest * self, const Key_Gest * other)
{
	if(self == other)
		return 1;
	if(self == NULL || other == NULL)
		return 0;
	return self->key == other->key;
}

int Key_Gest_event(Key_Gest * gest, SDL_Event * event)
{
	if(gest == NULL)
		return 0;
	switch(event->type)
	{
		case SDL_KEYDOWN:
		switch(event->key.keysym.sym)
		{
			case SDLK_RCTRL:
			return Key_Gest_key_append(gest, KGK_RCTRL);
			
			case SDLK_LCTRL:
			return Key_Gest_key_append(gest, KGK_LCTRL);
			
			case SDLK_RSHIFT:
			return Key_Gest_key_append(gest, KGK_RSHIFT);
			
			case SDLK_LSHIFT:
			return Key_Gest_key_append(gest, KGK_LSHIFT);
			
			case SDLK_RALT:
			return Key_Gest_key_append(gest, KGK_RALT);
			
			case SDLK_LALT:
			return Key_Gest_key_append(gest, KGK_LALT);
			
			default:
			return 0;
		}
		break;
		
		case SDL_KEYUP:
		switch(event->key.keysym.sym)
		{
			case SDLK_RCTRL:
			return Key_Gest_key_delete(gest, KGK_RCTRL);
			
			case SDLK_LCTRL:
			return Key_Gest_key_delete(gest, KGK_LCTRL);
			
			case SDLK_RSHIFT:
			return Key_Gest_key_delete(gest, KGK_RSHIFT);
			
			case SDLK_LSHIFT:
			return Key_Gest_key_delete(gest, KGK_LSHIFT);
			
			case SDLK_RALT:
			return Key_Gest_key_delete(gest, KGK_RALT);
			
			case SDLK_LALT:
			return Key_Gest_key_delete(gest, KGK_LALT);
			
			default:
			return 0;
		}
		break;
		
		default:
		break;
	}
	
	return 0;
}

int Key_Gest_save(FILE * flow, const Key_Gest * gest)
{
	if(flow == NULL || gest == NULL)
		return 0;
	return fwrite(&(gest->key), sizeof(Key_Gest_Key), 1, flow);
}

int Key_Gest_load(FILE * flow, Key_Gest * gest)
{
	if(flow == NULL || gest == NULL)
		return 0;
	return fread(&(gest->key), sizeof(Key_Gest_Key), 1, flow);
}