#include "math.h"

#include <stdarg.h>

double carre_float(double x)
{
	return x * x;
}

double abs_float(double x)
{
	if(x < 0)
		return -x;
	return x;
}

double min_float(double a, double b)
{
	if(a < b)
		return a;
	return b;
}

int carre(int x)
{
	return x * x;
}

int distance_carre(int x, int y, int target_x, int target_y)
{
	return carre(x - target_x) + carre(y - target_y);
}

int distance_carre_droite(int x, int y, int t1x, int t1y, int t2x, int t2y)
{
	double a, b, c;
	if(t1x == t2x && t1y == t2y)
		return -1;
	if(t1x == t2x)
	{
		a = 1;
		b = 0;
		c = -t1x;
	}
	else
	{
		a = ((double)(t1y - t2y)) / ((double)(t1x - t2x));
		b = -1;
		c = t1y - (a * (double)t1x);
	}
	return (int)(carre_float(a * x + b * y + c) / (carre_float(a) + carre_float(b)));
}

int distance_carre_segment(int x, int y, int t1x, int t1y, int t2x, int t2y)
{
	int dt1, dt2, dt1t2;
	dt1t2 = distance_carre(t1x, t1y, t2x, t2y);
	dt1 = distance_carre_droite(x, y, t1x, t1y, t1x - (t2y - t1y), t1y + (t2x - t1x));
	dt2 = distance_carre_droite(x, y, t2x, t2y, t2x - (t1y - t2y), t2y + (t1x - t2x));
	
	if(dt1 < dt1t2 && dt2 < dt1t2)
		return distance_carre_droite(x, y, t1x, t1y, t2x, t2y);
	else if(dt1 < dt2)
		return distance_carre(x, y, t1x, t1y);
	else
		return distance_carre(x, y, t2x, t2y);
}

double angle_segment(int x1, int y1, int x2, int y2)
{
	int vx = x2 - x1;
	int vy = y2 - y1;
	double res;
	if(vx == 0)
	{
		if(vy < 0)
			return (3 * M_PI) / 2;
		else if(vy > 0)
			return M_PI / 2;
		else
			return 0;
	}
	res = atan((double)vy / (double)vx);
	if(vx < 0)
		res += M_PI;
	return correct_rad_angle(res);
}

double degre_to_rad(double degre)
{
	return (degre * M_PI) / 180;
}

double rad_to_degre(double rad)
{
	return (rad * 180) / M_PI;
}

double correct_rad_angle(double degre)
{
	while(degre < 0)
		degre += (2 * M_PI);
	while(degre >= (2 * M_PI))
		degre -= (2 * M_PI);
	return degre;
}

double correct_degre_angle(double degre)
{
	while(degre < 0)
		degre += 360;
	while(degre >= 360)
		degre -= 360;
	return degre;
}

int in_angle_area(double area_start, double area_width, double deg_angle)
{
	if(area_width < 0)
		return in_angle_area(area_start + area_width, -area_width, deg_angle);
	area_width = correct_degre_angle(area_width);
	deg_angle = correct_degre_angle(deg_angle - area_start);
	return deg_angle <= area_width;
}

double distance_angles(double degre, double value)
{
	degre = correct_degre_angle(degre);
	value = correct_degre_angle(value);
	return min_float(correct_degre_angle(degre - value), correct_degre_angle(value - degre));
}

int get_external_tangents_from_circles(int a_radius, SDL_Rect a_center, int b_radius, SDL_Rect b_center, int * pa1_x, int * pa1_y, int * pa2_x, int * pa2_y, int * pb1_x, int * pb1_y, int * pb2_x, int * pb2_y)
{
	if(a_radius < 0)
		a_radius *= -1;
	if(b_radius < 0)
		b_radius *= -1;
	if(a_radius < b_radius)/* To get the pa1, pa2, pb1, pb2 in the right order we work on a_radius >= b_radius */
		return get_external_tangents_from_circles(b_radius, b_center, a_radius, a_center, pb2_x, pb2_y, pb1_x, pb1_y, pa2_x, pa2_y, pa1_x, pa1_y);
	if(pow((double)(a_center.x - b_center.x), 2) + pow((double)(a_center.y - b_center.y), 2) <= pow((double)(a_radius - b_radius), 2))/* If the b circle is in the a circle, no external tangents */
		return 0;
	double angle1 = 0;
	double angle2 = 0;
	if(a_radius != b_radius)
	{
		double a = ((double)(a_center.x - b_center.x)) / ((double)(a_radius - b_radius));
		double b = ((double)(a_center.y - b_center.y)) / ((double)(a_radius - b_radius));
		if(a != -1)
		{
			angle1 = 2 * atan((b + sqrt(pow(a, 2) + pow(b, 2) - 1)) / (a + 1)) + M_PI;
			angle2 = 2 * atan((b - sqrt(pow(a, 2) + pow(b, 2) - 1)) / (a + 1)) + M_PI;
		}
		else
		{
			if(b > 0)/* angle1 is already 0 */
				angle2 = 2 * atan(1 / b) + M_PI;
			else if(b < 0)/* angle2 is already 0 */
				angle1 = 2 * atan(1 / b) + M_PI;
			/* else both have 0 as value */
		}
	}
	else
	{
		double a = (double)(a_radius * (a_center.y - b_center.y));
		double b = (double)(-a_radius * (a_center.x - b_center.x));
		if(a != 0)
		{
			angle1 = 2 * atan((b - sqrt(pow(a, 2) + pow(b, 2))) / a) + (M_PI / 2);
			angle2 = 2 * atan((b + sqrt(pow(a, 2) + pow(b, 2))) / a) + (M_PI / 2);
		}
		else
		{
			if(b > 0)
			{
				angle1 = M_PI / 2;
				angle2 = -M_PI / 2;
			}
			else if(b < 0)
			{
				angle1 = -M_PI / 2;
				angle2 = M_PI / 2;
			}
			/* Case a=0 et b=0, b is in a, so no tangents to calculate */
		}
	}
	if(pa1_x != NULL)
		*pa1_x = (int)((a_radius * cos(angle1)) + a_center.x);
	if(pa1_y != NULL)
		*pa1_y = (int)((a_radius * sin(angle1)) + a_center.y);
		
	if(pa2_x != NULL)
		*pa2_x = (int)((a_radius * cos(angle2)) + a_center.x);
	if(pa2_y != NULL)
		*pa2_y = (int)((a_radius * sin(angle2)) + a_center.y);
		
	if(pb1_x != NULL)
		*pb1_x = (int)((b_radius * cos(angle1)) + b_center.x);
	if(pb1_y != NULL)
		*pb1_y = (int)((b_radius * sin(angle1)) + b_center.y);
		
	if(pb2_x != NULL)
		*pb2_x = (int)((b_radius * cos(angle2)) + b_center.x);
	if(pb2_y != NULL)
		*pb2_y = (int)((b_radius * sin(angle2)) + b_center.y);
		
	return 1;
}

int in_triangle(int x, int y, int x1, int y1, int x2, int y2, int x3, int y3)
{
	int vx = x - x1;
	int vy = y - y1;
	int val = (x2 - x1) * vy - (y2 - y1) * vx > 0;
	if(((x3 - x1) * vy - (y3 - y1) * vx > 0) == val)
		return 0;
	if(((x3 - x2) * (y - y2) - (y3 - y2) * (x - x2) > 0) != val)
		return 0;
	return 1;
}

int coordX_in_circle(int x, int y, int radius, double angle)
{
	return (int)(cos(angle) * radius) + x;
}

int coordY_in_circle(int x, int y, int radius, double angle)
{
	return (int)(sin(angle) * radius) + y;
}

SDL_Rect coord_in_circle(int x, int y, int radius, double angle)
{
	SDL_Rect coord;
	coord.x = coordX_in_circle(x, y, radius, angle);
	coord.y = coordY_in_circle(x, y, radius, angle);
	return coord;
}

static int choose_max(int elem, int new_elem)
{
	return new_elem > elem;
}

static int choose_min(int elem, int new_elem)
{
	return new_elem < elem;
}

static int minmax_by_choice(int nb, va_list ap, int (*to_choose)(int elem, int new_elem))
{
	int i;
	int get_val;
	int final = 0;
	for(i = 0; i < nb; ++i)
	{
		get_val = va_arg(ap, int);
		if(i == 0 || (*to_choose)(final, get_val))
			final = get_val;
	}
	return final;
}

int max_int(int nb, ...)
{
	va_list ap;
	int val;
	va_start(ap, nb);
	val = minmax_by_choice(nb, ap, &choose_max);
	va_end(ap);
	return val;
}

int min_int(int nb, ...)
{
	va_list ap;
	int val;
	va_start(ap, nb);
	val = minmax_by_choice(nb, ap, &choose_min);
	va_end(ap);
	return val;
}

void get_coord_by_rotation(int * nx, int * ny, int x, int y, int center_x, int center_y, double rad_angle)
{
	if(nx != NULL)
		*nx = center_x + (x - center_x) * cos(rad_angle) - (y - center_y) * sin(rad_angle);
	if(ny != NULL)
		*ny = center_y + (x - center_x) * sin(rad_angle) + (y - center_y) * cos(rad_angle);
}

SDL_Rect get_SDL_Rect_by_rotation(SDL_Rect coord, SDL_Rect center, double rad_angle)
{
	int gx, gy;
	SDL_Rect res;
	get_coord_by_rotation(&gx, &gy, coord.x, coord.y, center.x, center.y, rad_angle);
	res.x = gx;
	res.y = gy;
	return res;
}

int circle_in_circle(int a_x, int a_y, int a_r, int b_x, int b_y, int b_r)
{
	if(pow((double)(a_x - b_x), 2) + pow((double)(a_y - b_y), 2) <= pow((double)(a_r - b_r), 2))
		return 1;
	return 0;
}

int degre_same(double first, double second)
{
	double diff = first - second;
	if(diff > 360)
		diff -= 360;
	if(diff < -360)
		diff += 360;
	return abs_float(diff) < 1;
}

int rad_same(double first, double second)
{
	return degre_same(rad_to_degre(first), rad_to_degre(second));
}

float parabole_y(float x, float x1, float x2, float h)
{
	if(x1 == x2)
		return 0;
	return (-4 * h * (x - x1) * (x - x2)) / carre(x1 - x2);
}

int box_collision(int ax1, int ay1, int ax2, int ay2, int bx1, int by1, int bx2, int by2)
{
	return (bx2 >= ax1 && bx1 <= ax2) && (by2 >= ay1 && by1 <= ay2);
}

int circle_collision(int x1, int y1, int rad1, int x2, int y2, int rad2)
{
	return carre(x2 - x1) + carre(y2 - y1) <= carre(rad2 - rad1);
}

int speed_circle_collision_box(int x, int y, int rad, int x1, int y1, int x2, int y2)
{
	return box_collision(x - rad, y - rad, x + rad, y + rad, x1, y1, x2, y2);
}

int circle_collision_segment(int x, int y, int rad, int x1, int y1, int x2, int y2)
{
	return distance_carre_segment(x, y, x1, y1, x2, y2) < carre(rad);
}