#include "menu.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

#include "image.h"

int Menu_create(Menu * menu)
{
	Menu res = NULL;
	
	if((res = (Menu)malloc(sizeof(struct _Menu))) == NULL)
		return 0;
		
	res->origin.x = res->origin.y = 0;
	res->elems = NULL;
	res->types = NULL;
	res->functions = NULL;
	res->complements = NULL;
	res->options_count = 0;
	res->user_data = NULL;
	res->last_hover = -1;
	
	res->get_id_byXY = NULL;
	res->getXY_by_id = NULL;
	
	res->print_by_type = NULL;
	res->free_by_type = Menu_free_by_type;
	res->user_data_free = NULL;
	res->build_user_data = NULL;
	res->gest_event = NULL;
	
	*menu = res;
	
	return 1;
}

void Menu_clear(Menu menu)
{
	int i;
	void (*free_fct)(void *) = NULL;
	if(menu == NULL)
		return;
		
	if(menu->free_by_type != NULL)
	{
		for(i = 0; i < menu->options_count; ++i)
		{
			if((free_fct = (*(menu->free_by_type))(*(menu->types + i))) == NULL)
				continue;
			(*free_fct)(((char **)(menu->elems)) + i);
		}
	}
	if(menu->elems != NULL)
		free(menu->elems);
	if(menu->functions != NULL)
		free(menu->functions);
	if(menu->types != NULL)
		free(menu->types);
	if(menu->complements != NULL)
		free(menu->complements);
}

void Menu_free(Menu * menu)
{
	if(menu == NULL)
		return;
	Menu_clear(*menu);
	if((*menu)->user_data_free != NULL)
		(*((*menu)->user_data_free))(&((*menu)->user_data));
	free(*menu);
	*menu = NULL;
}

int Menu_define_controls(Menu menu,
			int (*get_id_byXY)(Menu, int x, int y, int * id),
			int (*getXY_by_id)(Menu, int id, int * x, int * y),
			void (*((*print_by_type)(Menu_Element_Type)))(Menu, SDL_Surface *, int id),
			void (*((*free_by_type)(Menu_Element_Type)))(void *),
			void (*user_data_free)(void * user_data_adress),
			int (*build_user_data)(Menu),
			int (*gest_event)(Menu, SDL_Event *))
{
	if(get_id_byXY == NULL
	|| getXY_by_id == NULL
	|| print_by_type == NULL)
		return 0;
	menu->get_id_byXY = get_id_byXY;
	menu->getXY_by_id = getXY_by_id;
	menu->print_by_type = print_by_type;
	menu->free_by_type = free_by_type;
	menu->user_data_free = user_data_free;
	menu->build_user_data = build_user_data;
	menu->gest_event = gest_event;
	return 1;
}
			
int Menu_set_user_data(Menu menu, void * user_data)
{
	if(menu == NULL)
		return 0;
	menu->user_data = user_data;
	return 1;
}

int Menu_set_origin(Menu menu, int x, int y)
{
	if(menu == NULL)
		return 0;
	menu->origin.x = x;
	menu->origin.y = y;
	return 1;
}

int Menu_alloc_elems(Menu menu, int count)
{
	if(menu == NULL)
		return 0;
	if(count <= 0)
		return 0;
	Menu_clear(menu);
	if((menu->elems = (void *)calloc(count, sizeof(void *))) == NULL)
		return 0;
	if((menu->types = (Menu_Element_Type *)calloc(count, sizeof(Menu_Element_Type))) == NULL
	|| (menu->complements = (void *)calloc(count, sizeof(void *))) == NULL
	|| (menu->functions = (void *)calloc(count, sizeof(void *))) == NULL)
	{
		Menu_clear(menu);
		return 0;
	}
	menu->options_count = count;
	return 1;
}

static int Menu_define_elem_va(Menu menu, int i, va_list ap)
{
	if(menu == NULL || i < 0 || i >= menu->options_count)
		return 0;
	*(((char **)(menu->functions)) + i) = (char *)va_arg(ap, void *);
	switch((*(menu->types + i) = va_arg(ap, Menu_Element_Type)))
	{
		case MENU_ELEMENT_TEXT:
		{
			char * get_chaine = (char *)va_arg(ap, const char *);
			if((*(((char **)(menu->elems)) + i) = (char *)malloc((strlen(get_chaine) + 1) * sizeof(char))) == NULL)
				return 0;
			strcpy(*(((char **)(menu->elems)) + i), get_chaine);
		}
		break;
		
		default:
		break;
	}
	*(((char **)(menu->complements)) + i) = (char *)va_arg(ap, void *);
	return 1;
}

int Menu_define_elem(Menu menu, int id, ...)
{
	va_list ap;
	int retour;
	va_start(ap, id);
	retour = Menu_define_elem_va(menu, id, ap);
	va_end(ap);
	if(! Menu_build_user_data(menu))
		retour = 0;
	
	return retour;
}

int Menu_build_user_data(Menu menu)
{
	if(menu == NULL)
		return 0;
	if(menu->build_user_data != NULL)
	{
		if(! (*(menu->build_user_data))(menu))
			return 0;
	}
	return 1;
}

int Menu_configure(Menu menu, int count, ...)
{
	int i;
	va_list ap;
	if(menu == NULL)
		return 0;
	if(count <= 0)
		return 0;
		
	if(! Menu_alloc_elems(menu, count))
		return 0;
	
	va_start(ap, count);
	for(i = 0; i < count; ++i)
	{
		if(! Menu_define_elem_va(menu, i, ap))
		{
			va_end(ap);
			goto free_zone;
		}
	}
	va_end(ap);
	
	if(! Menu_build_user_data(menu))
		goto free_zone;
	
	return 1;
	
	free_zone:;
	
	Menu_clear(menu);
	
	return 0;
}

void Menu_draw(Menu menu, SDL_Surface * screen)
{
	int i;
	void (*print_fct)(Menu, SDL_Surface *, int id) = NULL;
	if(menu == NULL || screen == NULL)
		return;
	
	for(i = 0; i < menu->options_count; ++i)
	{
		if((print_fct = (*(menu->print_by_type))(*(menu->types + i))) != NULL)
			(*print_fct)(menu, screen, i);
	}
}

static int Menu_event_va(Menu menu, SDL_Event * event, void * get_function, int * exit_menu, va_list ap)
{
	Menu_Option option = MENU_OPTION_END;
	int * option_id = NULL;
	char ** option_text = NULL;
	int need_refresh = 0;
	int get_id = -1;
	if(menu->gest_event != NULL)
	{
		need_refresh |= (*(menu->gest_event))(menu, event);
	}
	switch(event->type)
	{
		case SDL_MOUSEMOTION:
		if((*(menu->get_id_byXY))(menu, event->motion.x, event->motion.y, &get_id))
		{
			if(get_id != menu->last_hover)
			{
				menu->last_hover = get_id;
				need_refresh = 1;
			}
		}
		break;
		
		case SDL_MOUSEBUTTONUP:
		switch(event->button.button)
		{
			case SDL_BUTTON_LEFT:
			if(menu->last_hover >= 0 && menu->last_hover < menu->options_count)
			{
				if(get_function != NULL)
					*((char **)get_function) = *(((char **)(menu->functions)) + menu->last_hover);
				
			}
			if(exit_menu != NULL)
				*exit_menu = 1;
			while((option = (Menu_Option)va_arg(ap, Menu_Option)) != MENU_OPTION_END)
			{
				switch(option)
				{
					case MENU_OPTION_GET_ID:
					option_id = (int *)va_arg(ap, int *);
					if(option_id == NULL)
						break;
					*option_id = menu->last_hover;
					option_id = NULL;
					break;
					
					case MENU_OPTION_GET_TEXT:
					option_text = (char **)va_arg(ap, char **);
					if(option_text == NULL)
						break;
					if(menu->last_hover < 0 || menu->last_hover >= menu->options_count)
					{
						*option_text = NULL;
						break;
					}
					switch(*(menu->types + menu->last_hover))
					{
						case MENU_ELEMENT_TEXT:
						*option_text = *(((char **)(menu->elems)) + menu->last_hover);
						break;
						
						default:
						break;
					}
					option_text = NULL;
					break;
					
					default:
					break;
				}
			}
			menu->last_hover = -1;
			need_refresh = 1;
			break;
		}
		break;
	}
	return need_refresh;
}

int Menu_event(Menu menu, SDL_Event * event, void * get_function, int * exit_menu, ...)
{
	int retour;
	va_list ap;
	
	va_start(ap, exit_menu);
	retour = Menu_event_va(menu, event, get_function, exit_menu, ap);
	va_end(ap);
	
	return retour;
}

int Menu_gest(Menu menu, SDL_Surface * screen, void * get_function, Key_Gest * key, ...)
{
	int need_refresh = 1;
	int exit_menu = 0;
	SDL_Event event;
	va_list ap;
	
	if(menu == NULL || screen == NULL || key == NULL)
		return 0;
	
	menu->last_hover = -1;
	while(! exit_menu)
	{
		if(need_refresh)
		{
			Menu_draw(menu, screen);
			SDL_Flip(screen);
			need_refresh = 0;
		}
		SDL_WaitEvent(&event);
		va_start(ap, key);
		need_refresh = Menu_event_va(menu, &event, get_function, &exit_menu, ap);
		va_end(ap);
		switch(event.type)
		{
			case SDL_QUIT:
			return 0;
			
			case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
				return 0;
				
				default:
				break;
			}
			break;
			
			/*case SDL_MOUSEBUTTONUP:
			switch(event.button.button)
			{
				case SDL_BUTTON_LEFT:
				break;
				
				default:
				exit_menu = 1;
				break;
			}
			break;*/
		}
		Key_Gest_event(key, &event);
	}
	return 1;
}

void Menu_Element_Text_free(void * text)
{
	char ** val = NULL;
	if((val = (char **)text) == NULL)
		return;
	if(*val == NULL)
		return;
	free(*val);
	*val = NULL;
}

void (*Menu_free_by_type(Menu_Element_Type type))(void *)
{
	switch(type)
	{
		case MENU_ELEMENT_TEXT:
		return &Menu_Element_Text_free;
		
		default:
		break;
	}
	return NULL;
}