#ifndef DEF_HEADER_MENU
#define DEF_HEADER_MENU

#include <SDL/SDL.h>
#include "key_gest.h"

typedef enum
{
	MENU_ELEMENT_NULL,
	MENU_ELEMENT_TEXT,
	NB_MENU_ELEMENT
} Menu_Element_Type;

typedef enum
{
	MENU_OPTION_END,
	MENU_OPTION_GET_ID,
	MENU_OPTION_GET_TEXT,
	NB_MENU_OPTION
} Menu_Option;

typedef struct _Menu * Menu;
struct _Menu
{
	SDL_Rect origin;/* Symbolic origin of the Menu */
	void * elems;/* values of diffentes Menu choices */
	Menu_Element_Type * types;/* types associate to each value of elems */
	void * complements;/* data complement adress for each elem, should be referenced in user_data*/
	void * functions;/* functions associate to each elems value */
	int options_count;/* number of elems values */
	void * user_data;/* data complement, for example SDL_Surfaces or other usefull informations used by controls functions */
	int last_hover;
	
	int (*get_id_byXY)(Menu, int x, int y, int * id);
	int (*getXY_by_id)(Menu, int id, int * x, int * y);
	
	void (*((*print_by_type)(Menu_Element_Type)))(Menu, SDL_Surface *, int id);/* print the id element from Menu->elems according to its type, allow user to choose different prints for menu he use */
	void (*((*free_by_type)(Menu_Element_Type)))(void *);
	void (*user_data_free)(void * user_data_adress);
	int (*build_user_data)(Menu);
	int (*gest_event)(Menu, SDL_Event *);
};

/* Menu functions */

int Menu_create(Menu * menu);

void Menu_free(Menu * menu);

void Menu_clear(Menu menu);

int Menu_define_controls(Menu menu,
			int (*get_id_byXY)(Menu, int x, int y, int * id),
			int (*getXY_by_id)(Menu, int id, int * x, int * y),
			void (*((*print_by_type)(Menu_Element_Type)))(Menu, SDL_Surface *, int id),
			void (*((*free_by_type)(Menu_Element_Type)))(void *),
			void (*user_data_free)(void * user_data_adress),
			int (*build_user_data)(Menu),
			int (*gest_event)(Menu, SDL_Event *));
			
int Menu_set_user_data(Menu menu, void * user_data);

int Menu_set_origin(Menu menu, int x, int y);

/*
	For each element, the first argument is the function associate, the second is a Menu_Element_Type:
	for each void * data_complement, a good idea can be to have it in user_data
	NULL, MENU_ELEMENT_NULL, void * data_complement
	&function, MENU_ELEMENT_TEXT, const char * name, void * data_complement
*/
int Menu_configure(Menu menu, int count, ...);

/* Configuration sub functions */
int Menu_alloc_elems(Menu menu, int count);

int Menu_define_elem(Menu menu, int id, ...);

int Menu_build_user_data(Menu menu);

/* SDL gesttion */
void Menu_draw(Menu menu, SDL_Surface * screen);

/*
	va_list of Menu_Option:
	MENU_OPTION_END => define the end of the menu
	MENU_OPTION_GET_ID, int * id => give the id of the element in the menu
	MENU_OPTION_GET_TEXT, char ** text => give the text associate to the menu choice
*/
int Menu_event(Menu menu, SDL_Event * event, void * get_function, int * exit_menu, ...);

int Menu_gest(Menu menu, SDL_Surface * screen, void * get_function, Key_Gest * key, ...);

void (*Menu_free_by_type(Menu_Element_Type type))(void *);

void Menu_Element_Text_free(void * text);

#endif