#include "menu_list.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

#include "image.h"
#include "window.h"

static int Menu_List_create(Menu_List * user_data)
{
	Menu_List res = NULL;
	
	if((res = (Menu_List)malloc(sizeof(struct _Menu_List))) == NULL)
		return 0;
	
	res->background = res->background_hover = NULL;
	res->text_images = NULL;
	res->nb_text_images = 0;
	res->font = NULL;
	res->font_color.r = res->font_color.g = res->font_color.b = 0;
	res->align = MENU_LIST_ALIGN_CENTER;
	res->scroll = 0;
	
	*user_data = res;
	return 1;
}

static void Menu_List_free(void * user_data_adress)
{
	Menu_List * liste = NULL;
	if((liste = (Menu_List *)user_data_adress) == NULL)
		return;
		
	if(*liste == NULL)
		return;
	if((*liste)->background != NULL)
		SDL_FreeSurface((*liste)->background);
	if((*liste)->background_hover != NULL)
		SDL_FreeSurface((*liste)->background_hover);
	if((*liste)->text_images != NULL)
	{
		for(--((*liste)->nb_text_images); (*liste)->nb_text_images >= 0; --((*liste)->nb_text_images))
		{
			if(*((*liste)->text_images + (*liste)->nb_text_images) != NULL)
				SDL_FreeSurface(*((*liste)->text_images + (*liste)->nb_text_images));
		}
		free((*liste)->text_images);
	}
	if((*liste)->font != NULL)
	{
		TTF_CloseFont((*liste)->font);
		(*liste)->font = NULL;
	}
	free(*liste);
	*liste = NULL;
}

static int Menu_List_get_id_byXY(Menu menu, int x, int y, int * id)
{
	Menu_List user_data = NULL;
	if(menu == NULL)
		return 0;
	if((user_data = (Menu_List)(menu->user_data)) == NULL)
		return 0;
	if(user_data->background == NULL && user_data->background->h > 0)
		return 0;
	if(x <= menu->origin.x
	|| y <= menu->origin.y
	|| x > menu->origin.x + user_data->background->w
	|| y > menu->origin.y + (user_data->background->h * menu->options_count))
	{
		if(id == NULL)
			return 0;
		*id = -1;
		return 1;
	}
	if(id != NULL)
		*id = (y + user_data->scroll - menu->origin.y) / user_data->background->h;
	return 1;
}

static int Menu_List_getXY_by_id(Menu menu, int id, int * x, int * y)
{
	Menu_List user_data = NULL;
	if(menu == NULL)
		return 0;
	if(id < 0 || id >= menu->options_count)
		return 0;
	if((user_data = (Menu_List)(menu->user_data)) == NULL)
		return 0;
	if(user_data->background == NULL && user_data->background->h > 0)
		return 0;
	if(x != NULL)
		*x = menu->origin.x;
	if(y != NULL)
		*y = menu->origin.y + user_data->scroll + (user_data->background->h * id);
	return 1;
}

static int Menu_List_build_user_data(Menu menu)
{
	Menu_List user_data = NULL;
	int success = 1;
	SDL_Surface ** text_images = NULL;
	int i = 0;
	
	if(menu == NULL)
		return 0;
	if((user_data = (Menu_List)(menu->user_data)) == NULL)
		return 0;
	if((text_images = (SDL_Surface **)calloc(menu->options_count, sizeof(SDL_Surface *))) == NULL)
		goto free_zone;
	
	for(i = 0; i < menu->options_count; ++i)
	{
		if(*(menu->types + i) == MENU_ELEMENT_TEXT)
		{
			if((*(text_images + i) = TTF_RenderUTF8_Blended(user_data->font, *((char **)(menu->elems) + i), user_data->font_color)) == NULL)
				goto free_zone;
		}
	}
	
	if(user_data->text_images != NULL)
	{
		for(i = 0; i < user_data->nb_text_images; ++i)
		{
			if(*(user_data->text_images + i) != NULL)
				SDL_FreeSurface(*(user_data->text_images + i));
			*(user_data->text_images + i) = NULL;
		}
		free(user_data->text_images);
		user_data->text_images = NULL;
		i = menu->options_count;
	}
	user_data->text_images = text_images;
	user_data->nb_text_images = menu->options_count;
	
	goto success_zone;
	
	free_zone:;
	
	if(text_images != NULL)
	{
		for(--i; i >= 0; --i)
		{
			if(*(text_images + i) != NULL)
				SDL_FreeSurface(*(text_images + i));
		}
		free(text_images);
	}
	
	success = 0;
	
	success_zone:;
	
	return success;
}

static void Menu_List_Element_Text_print(Menu menu, SDL_Surface * screen, int id)
{
	Menu_List user_data = NULL;
	SDL_Rect pos;
	SDL_Surface * surface = NULL;
	if((user_data = (Menu_List)(menu->user_data)) == NULL)
		return;
	if(user_data->background == NULL)
		return;
	pos.x = menu->origin.x;
	pos.y = menu->origin.y + (user_data->background->h * id) - user_data->scroll;
	if(id == menu->last_hover && user_data->background_hover != NULL)
		surface = user_data->background_hover;
	else
		surface = user_data->background;
	SDL_BlitSurface(surface, NULL, screen, &pos);
	if((surface = *(user_data->text_images + id)) != NULL)
	{
		switch(user_data->align)
		{
			case MENU_LIST_ALIGN_LEFT:
			pos.x += 2;
			pos.y += ((user_data->background->h - surface->h) / 2);
			break;
			
			case MENU_LIST_ALIGN_CENTER:
			pos.x += ((user_data->background->w - surface->w) / 2);
			pos.y += ((user_data->background->h - surface->h) / 2);
			break;
			
			case MENU_LIST_ALIGN_RIGHT:
			pos.x += (user_data->background->w - surface->w) - 2;
			pos.y += ((user_data->background->h - surface->h) / 2);
			break;
			
			default:
			break;
		}
		SDL_BlitSurface(surface, NULL, screen, &pos);
	}
}

static void (*Menu_List_print_by_type(Menu_Element_Type type))(Menu, SDL_Surface *, int id)
{
	switch(type)
	{
		case MENU_ELEMENT_TEXT:
		return &Menu_List_Element_Text_print;
		
		default:
		break;
	}
	
	return NULL;
}

static int Menu_List_gest_event(Menu menu, SDL_Event * event)
{
	Menu_List user_data = NULL;
	int modif = 0;
	if((user_data = (Menu_List)(menu->user_data)) == NULL)
		return 0;
	switch(event->type)
	{
		case SDL_MOUSEBUTTONDOWN:
		switch(event->button.button)
		{
			case SDL_BUTTON_WHEELUP:
			(user_data->scroll) -= ML_SCROLL_ADD;
			if(user_data->scroll < 0)
				user_data->scroll = 0;
			modif = 1;
			break;
			
			case SDL_BUTTON_WHEELDOWN:
			if((menu->options_count * user_data->background->h) < get_screen()->h)
				break;
			(user_data->scroll) += ML_SCROLL_ADD;
			if((menu->options_count * user_data->background->h) - user_data->scroll + menu->origin.y < get_screen()->h)
				user_data->scroll = (menu->options_count * user_data->background->h) - get_screen()->h + menu->origin.y;
			modif = 1;
			break;
			
			default:
			break;
		}
		break;
		
		default:
		break;
	}
	return modif;
}

int Menu_define_Menu_List(Menu menu, const char * font_path, int font_size, SDL_Color font_color, SDL_Surface * background, SDL_Surface * background_hover)
{
	Menu_List user_data = NULL;
	if(font_path == NULL || font_size <= 0 || background == NULL)
		return 0;
	if(! Menu_List_create(&user_data))
		return 0;
	if((user_data->font = TTF_OpenFont(font_path, font_size)) == NULL)
	{
		fprintf(stderr, "\"%s\" font not found\n", font_path);
		goto free_zone;
	}
	if(! SDL_CopySurface(&(user_data->background), background))
		goto free_zone;
	if(! SDL_CopySurface(&(user_data->background_hover), background_hover))
		goto free_zone;
	user_data->font_color = font_color;
	
	if(! Menu_set_user_data(menu, user_data))
		goto free_zone;
	if(! Menu_define_controls(menu,
				&Menu_List_get_id_byXY,
				&Menu_List_getXY_by_id,
				&Menu_List_print_by_type,
				&Menu_free_by_type,
				&Menu_List_free,
				&Menu_List_build_user_data,
				&Menu_List_gest_event))
		goto free_zone;
	
	return 1;
	
	free_zone:;
	
	Menu_List_free(&user_data);
	
	return 0;
}

int Menu_List_set_align(Menu menu, Menu_List_Align align)
{
	Menu_List user_data = NULL;
	if(menu == NULL)
		return 0;
	if((user_data = (Menu_List)(menu->user_data)) == NULL)
		return 0;
	if(user_data->align == align)
		return 0;
	user_data->align = align;
	return 1;
}

void Menu_List_reset_scroll(Menu menu)
{
	Menu_List user_data = NULL;
	if((user_data = (Menu_List)(menu->user_data)) == NULL)
		return;
	user_data->scroll = 0;
}