#ifndef DEF_HEADER_MOVING_VECTOR
#define DEF_HEADER_MOVING_VECTOR

#include <SDL/SDL.h>

typedef enum
{
	MVC_NULL = 0x0,
	MVC_UP = 0x1,
	MVC_DOWN = 0x2,
	MVC_RIGHT = 0x4,
	MVC_LEFT = 0x8
} MovingVectorCoord;

typedef enum
{
	MVT_NULL = 0x0,
	MVT_STATIC = 0x1,
	MVT_CONTINUOUS = 0x2
} MovingVectorType;

typedef struct _MovingVector MovingVector;
struct _MovingVector
{
	MovingVectorCoord direction;
	int power;
	
	/* Calculated data */
	int time;
};

typedef struct _MovingVectorKey MovingVectorKey;
struct _MovingVectorKey
{
	SDLKey key;
	int press_key;
	MovingVector vector;
};

typedef struct _MovingVectorGest MovingVectorGest;
struct _MovingVectorGest
{
	MovingVector vector;
	MovingVectorKey static_vector;
	MovingVectorKey continuous_vector;
};

/* MovingVector */

int MovingVector_create(MovingVector * vector, MovingVectorCoord direction, int power);

int MovingVector_define_direction(MovingVector * vector, MovingVectorCoord direction);

int MovingVector_define_direction_by_coord(MovingVector * vector, int center_x, int center_y, int x, int y);

int MovingVector_graphic_define(MovingVector * vector, SDL_Surface * screen);

int MovingVector_copy(MovingVector * dest, MovingVector * src);

void MovingVector_draw(MovingVector * vector, SDL_Surface * screen, SDL_Color color);

int MovingVector_get_next_move_add(MovingVector * vector, int * x, int * y);

int MovingVector_start_move(MovingVector * vector);

int MovingVector_continue_after_anim(MovingVector * vector);

MovingVectorType MovingVector_type(MovingVector * vector);

int MovingVector_save(FILE * flow, MovingVector * vector);

int MovingVector_load(FILE * flow, MovingVector * vector);

int MovingVector_combinable(MovingVector * first, MovingVector * second);

int MovingVector_combine(MovingVector * res, MovingVector * first, MovingVector * second);

int MovingVector_delete_direction(MovingVector * vector, MovingVectorCoord direction);

int MovingVector_delete_continue_after_anim(MovingVector * vector);

int MovingVector_has_direction(MovingVector * vector, MovingVectorCoord direction);

void MovingVector_reverse(MovingVector * vector);

/* MovingVectorKey */

int MovingVectorKey_create(MovingVectorKey * key);

int MovingVectorKey_add_press(MovingVectorKey * mkey, SDLKey key, MovingVector * vector);

int MovingVectorKey_del_press(MovingVectorKey * key);

/* MovingVectorGest */

int MovingVectorGest_create(MovingVectorGest * gest, int power);

MovingVectorType MovingVectorGest_add_press_key(MovingVectorGest * gest, SDLKey key, MovingVector * vector);

MovingVectorType MovingVectorGest_del_press_key(MovingVectorGest * gest, SDLKey key);

MovingVectorType MovingVectorGest_del_continuous(MovingVectorGest * gest);

#endif