#include "personnage.h"

#include "data_debug_print.h"
#include "draw.h"

int Personnage_create(Personnage * perso)
{
	Personnage res = NULL;
	
	if((res = (Personnage)malloc(sizeof(struct _Personnage))) == NULL)
		return 0;
	res->armature = NULL;
	res->actions = NULL;
	res->current_action = NULL;
	res->static_action = NULL;
	res->continuous_action = NULL;
	res->fight_action = NULL;
	
	if(! Bone_create(&(res->armature))
	|| ! ActionList_create(&(res->actions))
	|| ! ActionList_add_default_actions(res->actions)
	|| ! MovingVectorGest_create(&(res->vector), 5))
	{
		Personnage_free(&res);
		return 0;
	}
	Bone_set_end(res->armature, 0, -200);
	ActionControl_init(&(res->control), NULL);
	res->last_time = 0;
	
	*perso = res;
	return 1;
}

void Personnage_free(Personnage * perso)
{
	if(perso == NULL || *perso == NULL)
		return;
	Bone_free(&((*perso)->armature));
	ActionList_free(&((*perso)->actions));
	free(*perso);
	*perso = NULL;
}

int Personnage_set_coord(Personnage perso, int x, int y)
{
	if(perso == NULL)
		return 0;
	return Bone_move_to(perso->armature, x, y);
}

static void Personnage_init_anim(Personnage perso)
{
	if(perso->current_action == NULL)
		return;
	Action_calculate_rotate_full(perso->current_action->action);
	ActionControl_init(&(perso->control), &(perso->current_action->action));
	ActionControl_set_time(&(perso->control), 0);
	if(perso->current_action->action != NULL)
		perso->last_time = SDL_GetTicks() - PERSONNAGE_DELAY_FRAME;
	MovingVector_start_move(&(perso->vector.vector));
}

int Personnage_prepare(Personnage perso)
{
	Bone_make_init(perso->armature);
	Personnage_init_anim(perso);
	if(! Armature_calculate_images(perso->armature))
		return 0;
	if(! Bone_make_hitbox(perso->armature))
		return 0;
	Bone_rotate_hitbox(perso->armature);
	Personnage_calculate_dimensions(perso);
	return 1;
}

int Personnage_scale(Personnage perso, double factor)
{
	if(! Bone_scale(perso->armature, factor))
		return 0;
	if(! Personnage_prepare(perso))
		return 0;
	return 1;
}

int Personnage_save(FILE * flow, Personnage perso)
{
	unsigned char type;
	unsigned int head_file;
	if(flow == NULL || perso == NULL)
		return 0;
	type = 'p';
	head_file = PERSONNAGE_VERSION;
	if(! fwrite(&type, sizeof(unsigned char), 1, flow)
	|| ! fwrite(&head_file, sizeof(unsigned int), 1, flow))
	{
		fprintf(stderr, "Erreur sauvegarde perso entete\n");
		return 0;
	}
	if(! Armature_save(flow, perso->armature))
	{
		fprintf(stderr, "Erreur sauvegarde perso bones\n");
		return 0;
	}
	if(! ActionList_save(flow, perso->actions))
	{
		fprintf(stderr, "Erreur sauvegarde perso actions\n");
		return 0;
	}
	return 1;
}

int Personnage_load(FILE * flow, Personnage * perso)
{
	unsigned char type;
	unsigned int head_file = 0;
	Bone armature = NULL;
	Personnage res = NULL;
	
	if(flow == NULL || perso == NULL)
		return 0;
	if(! fread(&type, sizeof(unsigned char), 1, flow))
	{
		fprintf(stderr, "Erreur chargement perso entete id\n");
		return 0;
	}
	if(type != 'p')
	{
		if(type == 'b')
		{
			fseek(flow, -1, SEEK_CUR);
			if(! Armature_load(flow, &armature))
			{
				fprintf(stderr, "Erreur chargement perso(%c) chargement armature\n", type);
				return 0;
			}
			Personnage_free(perso);
			if(! Personnage_create(perso))
			{
				fprintf(stderr, "Erreur chargement perso(%c) création perso\n", type);
				Bone_free(&armature);
				return 0;
			}
			Bone_free(&((*perso)->armature));
			(*perso)->armature = armature;
			return 1;
		}
		fprintf(stderr, "Erreur chargement perso entete id, trouvé '%c' attendu 'p' (personnage) ou 'b' (armature)\n", type);
		return 0;
	}
	if(! fread(&head_file, sizeof(unsigned int), 1, flow))
	{
		fprintf(stderr, "Erreur chargement perso entete\n");
		return 0;
	}
	if(head_file != PERSONNAGE_VERSION)
	{
		fprintf(stderr, "Erreur chargement perso version %d non reconnue\n", head_file);
		return 0;
	}
	if(! Armature_load(flow, &armature))
	{
		fprintf(stderr, "Erreur chargement perso armature\n");
		return 0;
	}
	if(! Personnage_create(&res))
	{
		fprintf(stderr, "Erreur chargement perso creation\n");
		Bone_free(&armature);
		return 0;
	}
	if(! ActionList_load(flow, res->actions, armature))
	{
		fprintf(stderr, "Erreur chargement perso action_list\n");
		Bone_free(&armature);
		Personnage_free(&res);
		return 0;
	}
	Bone_free(&(res->armature));
	res->armature = armature;
	if(! ActionList_calculate_rotate(res->actions))
	{
		fprintf(stderr, "Erreur chargement perso calcul des actions\n");
		Personnage_free(&res);
		return 0;
	}
	
	Personnage_free(perso);
	*perso = res;
	return 1;
}

int Personnage_save_in_file(const char * path, Personnage perso)
{
	FILE * ecr = NULL;
	int retour = 1;
	if((ecr = fopen(path, "w+")) == NULL)
		return 0;
	retour &= Personnage_save(ecr, perso);
	fclose(ecr);
	return retour;
}

int Personnage_load_from_file(const char * path, Personnage * perso)
{
	FILE * lec = NULL;
	int retour = 1;
	if((lec = fopen(path, "r")) == NULL)
		return 0;
	retour &= Personnage_load(lec, perso);
	fclose(lec);
	return retour;
}

int Personnage_copy(Personnage * copy, Personnage perso)
{
	Personnage res = NULL;
	if(! Personnage_create(&res))
		return 0;
	Bone_free(&(res->armature));
	ActionList_free(&(res->actions));
	if(! Bone_copy(&(res->armature), perso->armature))
	{
		fprintf(stderr, "Erreur perso copie armature\n");
		Personnage_free(&res);
		return 0;
	}
	if(! ActionList_copy(&(res->actions), perso->actions, res->armature))
	{
		fprintf(stderr, "Erreur perso copie actions\n");
		Personnage_free(&res);
		return 0;
	}
	*copy = res;
	return 1;
}

/* Fighting actions */

int Personnage_start_action(Personnage perso, SDLKey key, Key_Gest * key_mod)
{
	MovingVectorType type = MVT_NULL;
	ActionStats get_action = NULL;
	ActionStats last_current = NULL;
	if((get_action = ActionList_get_action_stats(perso->actions, ActionList_get_id_from_keys(perso->actions, key, key_mod))) == NULL)
		return 0;
	if((type = MovingVectorGest_add_press_key(&(perso->vector), key, &(get_action->vector))) == MVT_NULL)
	{
		if(perso->fight_action != NULL)
			return 0;
		if(perso->continuous_action != NULL)
			return 0;
		perso->fight_action = get_action;
		perso->current_action = perso->fight_action;
		Bone_make_init(perso->armature);
		Personnage_init_anim(perso);
		return 1;
	}
	last_current = perso->current_action;
	switch(type)
	{
		case MVT_CONTINUOUS:
		if(MovingVector_type(&(get_action->vector)) == MVT_CONTINUOUS)
			perso->continuous_action = get_action;
		else
			perso->static_action = get_action;
		perso->fight_action = NULL;
		perso->current_action = perso->continuous_action;
		break;
		
		case MVT_STATIC:
		perso->static_action = get_action;
		perso->fight_action = NULL;
		perso->current_action = perso->static_action;
		break;
		
		default:
		break;
	}
	if(last_current == perso->current_action)
		return 0;
	Bone_make_init(perso->armature);
	Personnage_init_anim(perso);
	return 1;
	/*if(perso->current_action != NULL)
	{
		if(get_action == perso->current_action)
		{
			perso->current_action->press_key = 1;
			return 1;
		}
		else if(MovingVector_continue_after_anim(&(perso->vector)))
			return MovingVector_combine(&(perso->vector), &(perso->vector), &(get_action->vector));
		else if(MovingVector_continue_after_anim(&(get_action->vector)))
			MovingVector_combine(&(perso->vector), &(perso->vector), &(get_action->vector));
		else
			MovingVector_copy(&(perso->vector), &(get_action->vector));
	}
	else
		MovingVector_copy(&(perso->vector), &(get_action->vector));
	if((perso->current_action = get_action) == NULL)
		return 0;
	
	perso->current_action->press_key = 1;
	Action_calculate_rotate_full(perso->current_action->action);
	ActionControl_init(&(perso->control), &(perso->current_action->action));
	ActionControl_set_time(&(perso->control), 0);
	if(perso->current_action->action != NULL)
		perso->last_time = SDL_GetTicks() - PERSONNAGE_DELAY_FRAME;
	MovingVector_start_move(&(perso->vector));
	return 1;*/
}

int Personnage_end_action(Personnage perso, SDLKey key, Key_Gest * key_mod)
{
	MovingVectorType type = MVT_NULL;
	if((type = MovingVectorGest_del_press_key(&(perso->vector), key)) == MVT_CONTINUOUS)
		return 0;
	if(perso->fight_action != NULL)
		return 0;
	switch(type)
	{
		case MVT_NULL:
		Bone_make_init(perso->armature);
		perso->current_action = NULL;
		perso->static_action = NULL;
		perso->continuous_action = NULL;
		break;
		
		case MVT_STATIC:
		/*Bone_make_init(perso->armature);*/
		perso->continuous_action = NULL;
		perso->current_action = perso->static_action;
		/*Personnage_init_anim(perso);*/
		break;
		
		default:
		break;
	}
	Bone_rotate_hitbox(perso->armature);
	Personnage_calculate_dimensions(perso);
	return 1;
	/*ActionStats get_action;
	if(perso->current_action == NULL)
		return 0;
	if(! MovingVector_continue_after_anim(&(perso->vector)))
		MovingVector_define_direction(&(perso->vector), MVC_NULL);
	if((get_action = ActionList_get_action_stats(perso->actions, ActionList_get_id_from_keys(perso->actions, key, key_mod))) != NULL)
	{
		if(! MovingVector_continue_after_anim(&(perso->vector)))
			MovingVector_delete_direction(&(perso->vector), get_action->vector.direction);
		get_action->press_key = 0;
	}
	return 1;*/
}

static void Personnage_del_continuous(Personnage perso)
{
	MovingVectorType type;
	if((type = MovingVectorGest_del_continuous(&(perso->vector))) == MVT_STATIC)
	{
		perso->continuous_action = NULL;
		perso->current_action = perso->static_action;
		Bone_make_init(perso->armature);
		Personnage_init_anim(perso);
	}
	else
	{
		perso->continuous_action = NULL;
		perso->static_action = NULL;
		perso->current_action = NULL;
		Bone_make_init(perso->armature);
	}
}

int Personnage_play_action(Personnage perso)
{
	int add_x = 0, add_y = 0;

	if(perso->current_action == NULL)
		return 0;

	if(perso->last_time + PERSONNAGE_DELAY_FRAME > SDL_GetTicks())
		return 0;

	perso->last_time = SDL_GetTicks();
	ActionControl_play_frame(&(perso->control));
	/*if(perso->current_action->press_key || MovingVector_continue_after_anim(&(perso->vector.vector)))
	{*/
	if(MovingVector_get_next_move_add(&(perso->vector.vector), &add_x, &add_y))
	{
		/*printf("MOVE %d %d\n", add_x, add_y);*/
		Bone_add_start(perso->armature, add_x, add_y);
		Bone_add_end(perso->armature, add_x, add_y);
	}
	else if(perso->continuous_action != NULL)
	{
		Personnage_del_continuous(perso);
		MovingVector_define_direction(&(perso->vector.vector), MVC_NULL);
	}

	/*}*/
	if(! ActionControl_continue(&(perso->control)))
	{
		if(perso->continuous_action == NULL)
		{
			if(MovingVector_type(&(perso->vector.vector)) == MVT_STATIC)
			{
				Bone_make_init(perso->armature);
				perso->current_action = perso->static_action;
				perso->fight_action = NULL;
				Personnage_init_anim(perso);
			}
			else
			{
				Bone_make_init(perso->armature);
				perso->current_action = NULL;
				perso->static_action = NULL;
				perso->fight_action = NULL;
			}
		}
	}
	Bone_rotate_hitbox(perso->armature);
	Personnage_calculate_dimensions(perso);
	return 1;
}

int Personnage_check_ground(Personnage perso, int ground)
{
	int bot = 0;
	/*if(! Bone_get_dim(perso->armature, NULL, NULL, NULL, &bot))
		return 0;*/
	bot = perso->dims.y + perso->dims.h;
	/*printf("%d %d\n", bot, ground);*/
	if(bot < ground && ! MovingVector_has_direction(&(perso->vector.vector), MVC_UP))
		return 1;
	if(bot <= ground)
		return 0;
	return 1;
}

int Personnage_put_on_ground(Personnage perso, int ground)
{
	int bot = 0;
	/*if(! Bone_get_dim(perso->armature, NULL, NULL, NULL, &bot))
		return 0;*/
	bot = perso->dims.y + perso->dims.h;
	Bone_add_start(perso->armature, 0, ground - bot);
	Bone_add_end(perso->armature, 0, ground - bot);
	if(bot > ground && MovingVector_has_direction(&(perso->vector.vector), MVC_UP))
		Personnage_del_continuous(perso);
	Personnage_calculate_dimensions(perso);
	
	/*MovingVector_delete_direction(&(perso->vector), MVC_UP);*/
	return 1;
}

void Personnage_reverse(Personnage perso)
{
	if(perso == NULL)
		return;
	Bone_reverse(perso->armature);
	ActionList_reverse_angle(perso->actions);
	Personnage_prepare(perso);
}

int Personnage_calculate_dimensions(Personnage perso)
{
	int top, bot, left, right;
	if(perso == NULL)
		return 0;
	if(! Bone_get_fight_dimensions(perso->armature, &left, &top, &right, &bot))
		return 0;
	perso->dims.x = left;
	perso->dims.y = top;
	perso->dims.w = right - left;
	perso->dims.h = bot - top;
	return 1;
}

int Personnage_able_to_fight(Personnage perso)
{
	if(perso == NULL)
		return 0;
	return 1;
}

int Personnage_can_hit(Personnage perso)
{
	if(perso == NULL)
		return 0;
	return perso->current_action != NULL && (perso->current_action == perso->fight_action || perso->current_action == perso->continuous_action);
}

int Personnage_make_impacted(Personnage perso, Bone hit)
{
	if(perso == NULL || hit == NULL)
		return 0;
	return Bone_make_impacted(perso->armature, hit);
}