#ifndef DEF_HEADER_PERSONNAGE
#define DEF_HEADER_PERSONNAGE

#include "armature.h"
#include "action_list.h"
#include "action_control.h"
#include "key_gest.h"

#define PERSONNAGE_VERSION 1
#define PERSONNAGE_DELAY_FRAME 10

typedef struct _Personnage * Personnage;
struct _Personnage
{
	Bone armature;
	ActionList actions;
	
	/* In game data */
	ActionStats current_action;
	ActionStats static_action;
	ActionStats continuous_action;
	ActionStats fight_action;
	
	ActionControl control;
	Uint32 last_time;
	MovingVectorGest vector;
	SDL_Rect dims;
};

int Personnage_create(Personnage * perso);

void Personnage_free(Personnage * perso);

int Personnage_set_coord(Personnage perso, int x, int y);

int Personnage_prepare(Personnage perso);

int Personnage_scale(Personnage perso, double factor);

int Personnage_save(FILE * flow, Personnage perso);

/*
	Autofree de *perso
	*perso doit être NULL si Personnage non défini
*/
int Personnage_load(FILE * flow, Personnage * perso);

int Personnage_save_in_file(const char * path, Personnage perso);

int Personnage_load_from_file(const char * path, Personnage * perso);

int Personnage_copy(Personnage * copy, Personnage perso);

/* Fighting actions */

int Personnage_start_action(Personnage perso, SDLKey key, Key_Gest * key_mod);

int Personnage_end_action(Personnage perso, SDLKey key, Key_Gest * key_mod);

int Personnage_play_action(Personnage perso);

int Personnage_check_ground(Personnage perso, int ground);

int Personnage_put_on_ground(Personnage perso, int ground);

void Personnage_reverse(Personnage perso);

int Personnage_calculate_dimensions(Personnage perso);

int Personnage_able_to_fight(Personnage perso);

int Personnage_can_hit(Personnage perso);

int Personnage_make_impacted(Personnage perso, Bone hit);

#endif