#include "shape.h"

#include <stdlib.h>
#include "generic_meca.h"
#include "math.h"

int Shape_create(Shape * shape, SDL_Rect center, int radius)
{
	Shape res = NULL;
	if((res = (Shape)malloc(sizeof(struct _Shape))) == NULL)
		return 0;
		
	res->center = center;
	res->radius = radius;
	res->border = 0;
	res->child = NULL;
	res->sister = NULL;
	
	*shape = res;
	return 1;
}

void Shape_free(Shape * shape)
{
	if(shape == NULL || *shape == NULL)
		return;
	Shape_free(&((*shape)->child));
	Shape_free(&((*shape)->sister));
	free(*shape);
	*shape = NULL;
}

int ShapeElem_create(ShapeElem * elem, SDL_Color color, SDL_Color border_color)
{
	ShapeElem res = NULL;
	if((res = (ShapeElem)malloc(sizeof(struct _ShapeElem))) == NULL)
		return 0;
		
	res->color = color;
	res->border_color = border_color;
	res->shape = NULL;
	
	*elem = res;
	return 1;
}

void ShapeElem_free(ShapeElem * elem)
{
	if(elem == NULL || *elem == NULL)
		return;
	Shape_free(&((*elem)->shape));
	free(*elem);
	*elem = NULL;
}

int ShapeSet_create(ShapeSet * set)
{
	ShapeSet res = NULL;
	if((res = (ShapeSet)malloc(sizeof(struct _ShapeSet))) == NULL)
		return 0;
		
	res->center.x = res->center.y = 0;
	res->elems = NULL;
	res->nb_elems = 0;
	res->maxi_elems = 0;
	
	*set = res;
	return 1;
}

void ShapeSet_free(ShapeSet * set)
{
	int i;
	if(set == NULL || *set == NULL)
		return;
	if((*set)->elems != NULL)
	{
		for(i = 0; i < (*set)->nb_elems; ++i)
			ShapeElem_free((*set)->elems + i);
		free((*set)->elems);
		(*set)->elems = NULL;
	}
	(*set)->nb_elems = 0;
	(*set)->maxi_elems = 0;
	free(*set);
	*set = NULL;
}

/* Shape */

Shape * Shape_adress(Shape * start, Shape shape)
{
	Shape * adr = NULL;
	if(start == NULL || *start == NULL)
		return NULL;
	if(*start == shape)
		return start;
	if((adr = Shape_adress(&((*start)->sister), shape)) != NULL)
		return adr;
	return Shape_adress(&((*start)->child), shape);
}

int Shape_set_center(Shape shape, int x, int y)
{
	if(shape == NULL)
		return 0;
	if(shape->center.x == x && shape->center.y == y)
		return 0;
	shape->center.x = x;
	shape->center.y = y;
	return 1;
}

int Shape_add_center(Shape shape, int vx, int vy)
{
	if(shape == NULL)
		return 0;
	return Shape_set_center(shape, shape->center.x + vx, shape->center.y + vy);
}

int Shape_set_radius(Shape shape, int radius)
{
	if(shape == NULL)
		return 0;
	if(radius < 0)
		return 0;
	if(shape->radius == radius)
		return 0;
	shape->radius = radius;
	return 1;
}

int Shape_add_radius(Shape shape, int add)
{
	if(shape == NULL)
		return 0;
	return Shape_set_radius(shape, shape->radius + add);
}

int Shape_set_border(Shape shape, int border)
{
	if(shape == NULL)
		return 0;
	if(border < 0)
		return 0;
	if(shape->border == border)
		return 0;
	shape->border = border;
	return 1;
}

static int Shape_generic_create(void * shape)
{
	SDL_Rect center = {0, 0};
	return Shape_create((Shape *)shape, center, 20);
}

int Shape_append(Shape * shape, Shape * appened)
{
	Shape app = NULL;
	if(! TreeChildBrother_generic_append(shape, &Shape_generic_create, get_id_in_structure(child, *shape), get_id_in_structure(sister, *shape), &app))
		return 0;
	Shape_set_center(app, (*shape)->center.x, (*shape)->center.y);
	Shape_set_radius(app, (*shape)->radius);
	Shape_set_border(app, (*shape)->border);
	if(appened != NULL)
		*appened = app;
	return 1;
}

static void Shape_generic_free(void * shape)
{
	Shape_free((Shape *)shape);
}

int Shape_delete(Shape * shape, Shape del)
{
	if(! TreeChildBrother_generic_delete(shape, del, &Shape_generic_free, get_id_in_structure(child, *shape), get_id_in_structure(sister, *shape), NULL))
		return 0;
	return 1;
}

int Shape_copy(Shape * dest, Shape src)
{
	Shape res = NULL;
	if(src == NULL)
	{
		*dest = NULL;
		return 1;
	}
	if(! Shape_create(&res, src->center, src->radius))
		return 0;
	res->border = src->border;
	if(! Shape_copy(&(res->child), src->child))
		return 0;
	if(! Shape_copy(&(res->sister), src->sister))
		return 0;
	*dest = res;
	return 1;
}

int Shape_field_copy(Shape * dest, Shape src)
{
	Shape res = NULL;
	if(dest == NULL || src == NULL)
		return 0;
	if(! Shape_create(&res, src->center, src->radius))
		return 0;
	res->border = src->border;
	*dest = res;
	return 1;
}

void Shape_average_center_aux(Shape cur, int * x, int * y, int * nb)
{
	if(cur == NULL)
		return;
	*x += cur->center.x;
	*y += cur->center.y;
	++(*nb);
	Shape_average_center_aux(cur->child, x, y, nb);
	Shape_average_center_aux(cur->sister, x, y, nb);
}

SDL_Rect Shape_average_center(Shape shape)
{
	SDL_Rect res;
	int x = 0;
	int y = 0;
	int nb = 0;
	
	res.x = res.y = 0;
	Shape_average_center_aux(shape, &x, &y, &nb);
	if(nb == 0)
		return res;
	res.x = (x / nb);
	res.y = (y / nb);
	return res;
}

int Shape_save(FILE * ecr, Shape shape)
{
	unsigned char next = 0;
	if(shape == NULL)
		return 1;
	next = (shape->child != NULL) | ((shape->sister != NULL) << 1);
	if(! fwrite(&next, sizeof(unsigned char), 1, ecr)
	|| ! fwrite(&(shape->center), sizeof(SDL_Rect), 1, ecr)
	|| ! fwrite(&(shape->radius), sizeof(int), 1, ecr)
	|| ! fwrite(&(shape->border), sizeof(int), 1, ecr))
	{
		fprintf(stderr, "Erreur sauvegarde shape data\n");
		return 0;
	}
	if(! Shape_save(ecr, shape->child))
	{
		fprintf(stderr, "Erreur sauvegarde shape child\n");
		return 0;
	}
	if(! Shape_save(ecr, shape->sister))
	{
		fprintf(stderr, "Erreur sauvegarde shape sister\n");
		return 0;
	}
	return 1;
}

int Shape_load(FILE * lec, Shape * shape)
{
	unsigned char next = 0;
	SDL_Rect center;
	int radius;
	int border;
	
	if(! fread(&next, sizeof(unsigned char), 1, lec)
	|| ! fread(&center, sizeof(SDL_Rect), 1, lec)
	|| ! fread(&radius, sizeof(int), 1, lec)
	|| ! fread(&border, sizeof(int), 1, lec))
	{
		fprintf(stderr, "Erreur chargement shape data\n");
		return 0;
	}
	if(! Shape_create(shape, center, radius))
	{
		fprintf(stderr, "Erreur chargement shape creation\n");
		return 0;
	}
	(*shape)->border = border;
	if(next & 1)
	{
		if(! Shape_load(lec, &((*shape)->child)))
		{
			fprintf(stderr, "Erreur chargement shape child\n");
			goto free_zone;
		}
	}
	if(next & 2)
	{
		if(! Shape_load(lec, &((*shape)->sister)))
		{
			fprintf(stderr, "Erreur chargement sister\n");
			goto free_zone;
		}
	}
	
	return 1;
	
	free_zone:;
	
	Shape_free(shape);
	return 0;
}

void Shape_reverse(Shape shape)
{
	if(shape == NULL)
		return;
	shape->center.x *= -1;
	Shape_reverse(shape->sister);
	Shape_reverse(shape->child);
}

void Shape_simplify(Shape * shape)
{
	if(shape == NULL || *shape == NULL)
		return;
	Shape_simplify(&((*shape)->child));
	Shape_simplify(&((*shape)->sister));
}

/* ShapeElem */

int ShapeElem_is_void(ShapeElem elem)
{
	if(elem == NULL)
		return 1;
	return elem->shape == NULL;
}

static int ShapeElem_add_center_aux(Shape shape, int vx, int vy)
{
	if(shape == NULL)
		return 1;
	if(! Shape_add_center(shape, vx, vy))
		return 0;
	return 	ShapeElem_add_center_aux(shape->child, vx, vy)
		&& ShapeElem_add_center_aux(shape->sister, vx, vy);
}

int ShapeElem_add_center(ShapeElem elem, int vx, int vy)
{
	if(elem == NULL)
		return 0;
	return ShapeElem_add_center_aux(elem->shape, vx, vy);
}

static int ShapeElem_add_radius_aux(Shape shape, int add)
{
	if(shape == NULL)
		return 1;
	Shape_add_radius(shape, add);
	return 	ShapeElem_add_radius_aux(shape->child, add)
		&& ShapeElem_add_radius_aux(shape->sister, add);
}

int ShapeElem_add_radius(ShapeElem elem, int add)
{
	if(elem == NULL)
		return 0;
	if(add == 0)
		return 0;
	return ShapeElem_add_radius_aux(elem->shape, add);
}

int ShapeElem_copy(ShapeElem * dest, ShapeElem src)
{
	ShapeElem res = NULL;
	Shape shape = NULL;
	if(dest == NULL || src == NULL)
		return 0;
	if(! Shape_copy(&shape, src->shape))
		return 0;
	if(! ShapeElem_create(&res, src->color, src->border_color))
	{
		Shape_free(&shape);
		return 0;
	}
	res->shape = shape;
	*dest = res;
	return 1;
}

SDL_Rect ShapeElem_average_center(ShapeElem elem)
{
	if(elem == NULL)
	{
		SDL_Rect res;
		res.x = res.y = 0;
		return res;
	}
	return Shape_average_center(elem->shape);
}

int ShapeElem_save(FILE * ecr, ShapeElem elem)
{
	if(elem == NULL)
		return 0;
	if(! fwrite(&(elem->color), sizeof(SDL_Color), 1, ecr)
	|| ! fwrite(&(elem->border_color), sizeof(SDL_Color), 1, ecr))
	{
		fprintf(stderr, "Erreur sauvegarde shape_elem data\n");
		return 0;
	}
	if(! Shape_save(ecr, elem->shape))
	{
		fprintf(stderr, "Erreur sauvegarde shape_elem shape\n");
		return 0;
	}
	return 1;
}

int ShapeElem_load(FILE * lec, ShapeElem * elem)
{
	SDL_Color color;
	SDL_Color border_color;
	if(! fread(&color, sizeof(SDL_Color), 1, lec)
	|| ! fread(&border_color, sizeof(SDL_Color), 1, lec))
	{
		fprintf(stderr, "Erreur chargement shape_elem data\n");
		return 0;
	}
	if(! ShapeElem_create(elem, color, border_color))
	{
		fprintf(stderr, "Erreur chargement shape_elem creation\n");
		return 0;
	}
	if(! Shape_load(lec, &((*elem)->shape)))
	{
		fprintf(stderr, "Erreur chargement shape_elem shape\n");
		ShapeElem_free(elem);
		return 0;
	}
	return 1;
}

int ShapeElem_version2_save(FILE * ecr, ShapeElem elem)
{
	unsigned int version = 1;
	unsigned int header = 0;
	if(elem != NULL)
		header = 1 | ((elem->shape != NULL) << 8);
	if(! fwrite(&version, sizeof(unsigned int), 1, ecr)
	|| ! fwrite(&header, sizeof(unsigned int), 1, ecr))
	{
		fprintf(stderr, "Erreur sauvegarde shape_elem entete\n");
		return 0;
	}
	if(elem == NULL)
		return 1;
	if(! fwrite(&(elem->color), sizeof(SDL_Color), 1, ecr)
	|| ! fwrite(&(elem->border_color), sizeof(SDL_Color), 1, ecr))
	{
		fprintf(stderr, "Erreur sauvegarde shape_elem data\n");
		return 0;
	}
	if(elem->shape != NULL)
	{
		if(! Shape_save(ecr, elem->shape))
		{
			fprintf(stderr, "Erreur sauvegarde shape_elem shape\n");
			return 0;
		}
	}
	return 1;
}

int ShapeElem_version2_load(FILE * lec, ShapeElem * elem)
{
	unsigned int version = 0;
	unsigned int header = 0;
	SDL_Color color;
	SDL_Color border_color;
	
	if(! fread(&version, sizeof(unsigned int), 1, lec)
	|| ! fread(&header, sizeof(unsigned int), 1, lec))
	{
		fprintf(stderr, "Erreur chargement shape_elem entete\n");
		return 0;
	}
	if(version != 1)
	{
		fprintf(stderr, "Erreur chargement shape_elem version non reconnue, trouvé version %d\n", version);
		return 0;
	}
	if(header == 0)
	{
		*elem = NULL;
		return 1;
	}
	header >>= 8;
	if(! fread(&color, sizeof(SDL_Color), 1, lec)
	|| ! fread(&border_color, sizeof(SDL_Color), 1, lec))
	{
		fprintf(stderr, "Erreur chargement shape_elem data\n");
		return 0;
	}
	if(! ShapeElem_create(elem, color, border_color))
	{
		fprintf(stderr, "Erreur chargement shape_elem creation\n");
		return 0;
	}
	if(header & 0xff)
	{
		if(! Shape_load(lec, &((*elem)->shape)))
		{
			fprintf(stderr, "Erreur chargement shape_elem shape\n");
			ShapeElem_free(elem);
			return 0;
		}
	}
	return 1;
}

/* ShapeSet */

int ShapeSet_is_void(ShapeSet shape)
{
	if(shape == NULL)
		return 1;
	return shape->nb_elems <= 0;
}

int ShapeSet_set_center(ShapeSet shape, int x, int y)
{
	if(shape == NULL)
		return 0;
	if(shape->center.x == x && shape->center.y == y)
		return 0;
	shape->center.x = x;
	shape->center.y = y;
	return 1;
}

int ShapeSet_add_center(ShapeSet shape, int vx, int vy)
{
	if(shape == NULL)
		return 0;
	return ShapeSet_set_center(shape, shape->center.x + vx, shape->center.y + vy);
}

int ShapeSet_add_center_elem(ShapeSet shape, int vx, int vy)
{
	int i;
	if(shape == NULL)
		return 0;
	if(vx == 0 && vy == 0)
		return 0;
	for(i = 0; i < shape->nb_elems; ++i)
		ShapeElem_add_center(*(shape->elems + i), vx, vy);
	return 1;
}

int ShapeSet_add_radius(ShapeSet shape, int add)
{
	int i;
	if(shape == NULL)
		return 0;
	if(add == 0)
		return 0;
	for(i = 0; i < shape->nb_elems; ++i)
		ShapeElem_add_radius(*(shape->elems + i), add);
	return 1;
}

static int ShapeSet_modif_maxi(ShapeSet shape, int new_size)
{
	int new_maxi = -1;
	ShapeElem * tmp = NULL;
	if(shape == NULL)
		return 0;
		
	if(new_size >= shape->maxi_elems)
		new_maxi = (shape->maxi_elems * 2) + 1;
	if(new_size < (shape->maxi_elems / 4) - 1)
		new_maxi = (shape->maxi_elems / 2);
	if(new_maxi == -1)
		return 1;
	if((tmp = (ShapeElem *)realloc(shape->elems, new_maxi * sizeof(ShapeElem))) == NULL)
		return 0;
	shape->elems = tmp;
	shape->maxi_elems = new_maxi;
	return 1;
}

int ShapeSet_append_elem(ShapeSet shape, SDL_Color color, SDL_Color border_color, int * appened_id)
{
	ShapeElem ajout = NULL;
	if(shape == NULL)
		return 0;
	if(! ShapeSet_modif_maxi(shape, shape->nb_elems + 1))
		return 0;
	if(! ShapeElem_create(&ajout, color, border_color))
		return 0;
	if(appened_id != NULL)
		*appened_id = shape->nb_elems;
	*(shape->elems + ((shape->nb_elems)++)) = ajout;
	return 1;
}

int ShapeSet_delete_elem(ShapeSet shape, int del_id)
{
	if(shape == NULL)
		return 0;
	if(del_id < 0 || del_id >= shape->nb_elems)
		return 0;
	ShapeElem_free(shape->elems + del_id);
	for(; del_id < shape->nb_elems - 1; ++del_id)
		*(shape->elems + del_id) = *(shape->elems + del_id + 1);
	--(shape->nb_elems);
	if(! ShapeSet_modif_maxi(shape, shape->nb_elems))
		return 0;
	return 1;
}

int ShapeSet_append(ShapeSet shape, SDL_Color color, SDL_Color border_color, int * appened_id, Shape * appened)
{
	int app_id;
	if(! ShapeSet_append_elem(shape, color, border_color, &app_id))
		return 0;
	if(! Shape_append(&((*(shape->elems + app_id))->shape), appened))
		return 0;
	if(appened_id != NULL)
		*appened_id = app_id;
	return 1;
}

int ShapeSet_delete(ShapeSet shape, SDL_Color color, SDL_Color border_color, int del_id, Shape del, int * destroyed)
{
	if(del_id < 0 || del_id >= shape->nb_elems)
		return 0;
	if(! Shape_delete(&((*(shape->elems + del_id))->shape), del))
		return 0;
	if(ShapeElem_is_void(*(shape->elems + del_id)))
	{
		if(! ShapeSet_delete_elem(shape, del_id))
			return 0;
		if(destroyed != NULL)
			*destroyed = 1;
	}
	else if(destroyed != NULL)
		*destroyed = 0;
	return 1;
}

int ShapeSet_clean(ShapeSet shape)
{
	if(shape == NULL)
		return 0;
	while(shape->nb_elems > 0)
	{
		if(! ShapeSet_delete_elem(shape, shape->nb_elems - 1))
			return 0;
	}
	return 1;
}

int ShapeSet_switch(ShapeSet shape, int first, int second)
{
	ShapeElem tmp = NULL;
	if(shape == NULL)
		return 0;
	if(first == second)
		return 0;
	if(first < 0 || first >= shape->nb_elems)
		return 0;
	if(second < 0 || second >= shape->nb_elems)
		return 0;
	tmp = *(shape->elems + first);
	*(shape->elems + first) = *(shape->elems + second);
	*(shape->elems + second) = tmp;
	return 1;
}

int ShapeSet_copy(ShapeSet * dest, ShapeSet src)
{
	ShapeSet res = NULL;
	if(dest == NULL || src == NULL)
		return 0;
	if(! ShapeSet_create(&res))
		return 0;
	res->maxi_elems = src->maxi_elems;
	if((res->elems = (ShapeElem *)calloc(src->maxi_elems, sizeof(ShapeElem))) == NULL)
		goto free_zone;
	res->center = src->center;
	
	for(res->nb_elems = 0; res->nb_elems < src->nb_elems; ++(res->nb_elems))
	{
		if(! ShapeElem_copy(res->elems + res->nb_elems, *(src->elems + res->nb_elems)))
			goto free_zone;
	}
	
	*dest = res;
	
	return 1;
	
	free_zone:;
	
	ShapeSet_free(&res);
	return 0;
}

int ShapeSet_join_shape(ShapeSet shape, SDL_Color color, SDL_Color border_color, Shape add)
{
	int app_id = -1;
	if(shape == NULL || add == NULL)
		return 0;
	if(! ShapeSet_append_elem(shape, color, border_color, &app_id))
		return 0;
	if(app_id == -1)
		return 0;
	if(! Shape_field_copy(&((*(shape->elems + app_id))->shape), add))
		return 0;
	return 1;
}

int ShapeSet_join_elem(ShapeSet shape, ShapeElem add)
{
	int app_id = -1;
	if(shape == NULL || add == NULL)
		return 0;
	if(! ShapeSet_append_elem(shape, add->color, add->border_color, &app_id))
		return 0;
	if(app_id == -1)
		return 0;
	if(! Shape_copy(&((*(shape->elems + app_id))->shape), add->shape))
		return 0;
	return 1;
}

int ShapeSet_join(ShapeSet shape, ShapeSet add)
{
	int i;
	if(shape == NULL || add == NULL)
		return 0;
	for(i = 0; i < add->nb_elems; ++i)
	{
		if(! ShapeSet_join_elem(shape, *(add->elems + i)))
			return 0;
	}
	return 1;
}

SDL_Rect ShapeSet_average_center(ShapeSet shape)
{
	SDL_Rect res;
	SDL_Rect tmp_res;
	int x = 0;
	int y = 0;
	int i;
	
	res.x = res.y = 0;
	if(shape == NULL)
		return res;
	for(i = 0; i < shape->nb_elems; ++i)
	{
		tmp_res = ShapeElem_average_center(*(shape->elems + i));
		x += tmp_res.x;
		y += tmp_res.y;
	}
	res.x = (x / shape->nb_elems);
	res.y = (y / shape->nb_elems);
	return res;
}

int ShapeSet_set_center_as_average(ShapeSet shape)
{
	SDL_Rect ave_center;
	if(shape == NULL)
		return 0;
	ave_center = ShapeSet_average_center(shape);
	ShapeSet_add_center_elem(shape, -ave_center.x, -ave_center.y);
	ShapeSet_add_center(shape, ave_center.x, ave_center.y);
	return 1;
}

int ShapeSet_get_dim_aux(Shape shape, int * x_min, int * y_min, int * x_max, int * y_max)
{
	int x_min_c, y_min_c, x_max_c, y_max_c;
	int x_min_s, y_min_s, x_max_s, y_max_s;
	int retour = 0;
	if(shape == NULL)
		return 0;
	x_min_c = x_min_s = *x_min = shape->center.x - shape->radius - shape->border;
	y_min_c = y_min_s = *y_min = shape->center.y - shape->radius - shape->border;
	x_max_c = x_max_s = *x_max = shape->center.x + shape->radius + shape->border;
	y_max_c = y_max_s = *y_max = shape->center.y + shape->radius + shape->border;
	retour += ShapeSet_get_dim_aux(shape->child, &x_min_c, &y_min_c, &x_max_c, &y_max_c);
	retour += ShapeSet_get_dim_aux(shape->sister, &x_min_s, &y_min_s, &x_max_s, &y_max_s);
	*x_min = min_int(3, *x_min, x_min_c, x_min_s);
	*y_min = min_int(3, *y_min, y_min_c, y_min_s);
	*x_max = max_int(3, *x_max, x_max_c, x_max_s);
	*y_max = max_int(3, *y_max, y_max_c, y_max_s);
	return 1;
}

int ShapeSet_get_dimensions(ShapeSet shape, int * x_min, int * y_min, int * x_max, int * y_max)
{
	int gx_min, gy_min, gx_max, gy_max;
	int i, somme = 0, tmp;
	if(shape == NULL)
		return 0;
	for(i = 0; i < shape->nb_elems; ++i)
	{
		if((tmp = ShapeSet_get_dim_aux((*(shape->elems + i))->shape, &gx_min, &gy_min, &gx_max, &gy_max)) == 0)
			continue;
		somme += tmp;
		if(somme == 0)
		{
			if(x_min != NULL)
				*x_min = gx_min;
			if(x_max != NULL)
				*x_max = gx_max;
			if(y_min != NULL)
				*y_min = gy_min;
			if(y_max != NULL)
				*y_max = gy_max;
			continue;
		}
		if(x_min != NULL)
			*x_min = min_int(2, *x_min, gx_min);
		if(y_min != NULL)
			*y_min = min_int(2, *y_min, gy_min);
		if(x_max != NULL)
			*x_max = max_int(2, *x_max, gx_max);
		if(y_max != NULL)
			*y_max = max_int(2, *y_max, gy_max);
	}
	if(somme == 0)
	{
		if(x_min != NULL)
			*x_min = shape->center.x;
		if(x_max != NULL)
			*x_max = shape->center.x;
		if(y_min != NULL)
			*y_min = shape->center.y;
		if(y_max != NULL)
			*y_max = shape->center.y;
	}	
	return 1;
}

static void Shape_scale(Shape shape, double scale)
{
	if(shape == NULL)
		return;
	shape->center.x *= scale;
	shape->center.y *= scale;
	shape->radius *= scale;
	
	Shape_scale(shape->child, scale);
	Shape_scale(shape->sister, scale);
}

void ShapeSet_scale(ShapeSet shape, double scale)
{
	int i;
	if(shape == NULL)
		return;
	for(i = 0; i < shape->nb_elems; ++i)
	{
		Shape_scale((*(shape->elems + i))->shape, scale);
	}
}

int ShapeSet_resize_in(ShapeSet shape, int x_min, int y_min, int x_max, int y_max)
{
	int sx_min, sy_min, sx_max, sy_max, tmp;
	double scale, scale_x, scale_y;
	if(x_min > x_max)
	{
		tmp = x_min;
		x_min = x_max;
		x_max = tmp;
	}
	if(y_min > y_max)
	{
		tmp = y_min;
		y_min = y_max;
		y_max = tmp;
	}
	if(! ShapeSet_get_dimensions(shape, &sx_min, &sy_min, &sx_max, &sy_max))
		return 0;
	scale_x = ((double)(x_max - x_min)) / ((double)(sx_max - sx_min));
	scale_y = ((double)(y_max - y_min)) / ((double)(sy_max - sy_min));
	scale = (scale_x < scale_y) ? scale_x : scale_y;
	ShapeSet_scale(shape, scale);
	ShapeSet_set_center(shape, (shape->center.x - ((sx_min + sx_max) / 2)) * scale + ((x_min + x_max) / 2), (shape->center.y - ((sy_min + sy_max) / 2)) * scale + ((y_min + y_max) / 2));
	return 1;
}

int ShapeSet_create_for_elem(ShapeSet * shape, ShapeElem elem)
{
	ShapeSet res = NULL;
	if(! ShapeSet_create(&res))
		return 0;
	if(! ShapeSet_join_elem(res, elem))
	{
		ShapeSet_free(&res);
		return 0;
	}
	*shape = res;
	return 1;
}

int ShapeSet_save(FILE * ecr, ShapeSet shape)
{
	int i;
	unsigned char type = 't';
	if(shape == NULL)
		return 0;
	if(! fwrite(&type, sizeof(unsigned char), 1, ecr)
	|| ! fwrite(&(shape->center), sizeof(SDL_Rect), 1, ecr)
	|| ! fwrite(&(shape->nb_elems), sizeof(int), 1, ecr)
	|| ! fwrite(&(shape->maxi_elems), sizeof(int), 1, ecr))
	{
		fprintf(stderr, "Erreur sauvegarde set data\n");
		return 0;
	}
	for(i = 0; i < shape->nb_elems; ++i)
	{
		if(! ShapeElem_version2_save(ecr, *(shape->elems + i)))
		{
			fprintf(stderr, "Erreur sauvegarde set shape_elem %d\n", i);
			return 0;
		}
	}
	return 1;
}

int ShapeSet_load(FILE * lec, ShapeSet * shape)
{
	ShapeSet res = NULL;
	if(shape == NULL)
		return 0;
	if(! ShapeSet_create(&res))
	{
		fprintf(stderr, "Erreur chargement set creation\n");
		return 0;
	}
	if(! ShapeSet_load_stats(lec, res))
	{
		fprintf(stderr, "Erreur chargement set stats\n");
		ShapeSet_free(&res);
		return 0;
	}
	*shape = res;
	return 1;
}

int ShapeSet_load_stats(FILE * lec, ShapeSet shape)
{
	unsigned char type;
	int i = 0;
	int (*load_shape_elem)(FILE *, ShapeElem *) = NULL;
	if(shape == NULL)
		return 0;
	if(! fread(&type, sizeof(unsigned char), 1, lec))
	{
		fprintf(stderr, "Erreur chargement set entete id\n");
		return 0;
	}
	if(type == 's')
		load_shape_elem = &ShapeElem_load;
	else if(type == 't')
		load_shape_elem = &ShapeElem_version2_load;
	else
	{
		fprintf(stderr, "Erreur chargement set entete id, trouvé '%c' attendu 's'\n", type);
		return 0;
	}
	if(! fread(&(shape->center), sizeof(SDL_Rect), 1, lec)
	|| ! fread(&(shape->nb_elems), sizeof(int), 1, lec)
	|| ! fread(&(shape->maxi_elems), sizeof(int), 1, lec))
	{
		fprintf(stderr, "Erreur chargement set data\n");
		return 0;
	}
	if(shape->elems != NULL)
	{
		free(shape->elems);
		shape->elems = NULL;
	}
	if(shape->maxi_elems > 0 && (shape->elems = (ShapeElem *)calloc(shape->maxi_elems, sizeof(ShapeElem))) == NULL)
	{
		fprintf(stderr, "Erreur chargement set allocation elems\n");
		goto free_zone;
	}
	for(i = 0; i < shape->nb_elems; ++i)
	{
		if(! (*load_shape_elem)(lec, shape->elems + i))
		{
			fprintf(stderr, "Erreur chargement set elem %d\n", i);
			goto free_zone;
		}
	}
	return 1;
	
	free_zone:;
	
	if(shape->elems != NULL)
	{
		for(--i; i >= 0; --i)
			ShapeElem_free(shape->elems + i);
		free(shape->elems);
		shape->elems = NULL;
	}
	shape->maxi_elems = 0;
	shape->nb_elems = 0;
	return 0;
}

int ShapeSet_real_scale(ShapeSet shape, double factor)
{
	int i;
	if(shape == NULL)
		return 0;
	shape->center.x = factor * shape->center.x;
	shape->center.y = factor * shape->center.y;
	for(i = 0; i < shape->nb_elems; ++i)
		Shape_scale((*(shape->elems + i))->shape, factor);
	return 1;
}

void ShapeSet_reverse(ShapeSet shape)
{
	int i;
	if(shape == NULL)
		return;
	shape->center.x *= -1;
	for(i = 0; i < shape->nb_elems; ++i)
		Shape_reverse((*(shape->elems + i))->shape);
}

void ShapeSet_simplify(ShapeSet shape)
{
	int i;
	if(shape == NULL)
		return;
	for(i = 0; i < shape->nb_elems; ++i)
		Shape_simplify(&((*(shape->elems + i))->shape));
}