#ifndef DEF_HEADER_SHAPE
#define DEF_HEADER_SHAPE

#include <SDL/SDL.h>

typedef struct _Shape * Shape;
struct _Shape
{
	SDL_Rect center;/* Relative to set */
	int radius;/* of circle */
	int border;/* additional border to circle */
	Shape child;
	Shape sister;
};

typedef struct _ShapeElem * ShapeElem;
struct _ShapeElem
{
	SDL_Color color;
	SDL_Color border_color;
	Shape shape;
};

typedef struct _ShapeSet * ShapeSet;
struct _ShapeSet
{
	SDL_Rect center;/* Relative to bone */
	ShapeElem * elems;
	int nb_elems;
	int maxi_elems;
};

int Shape_create(Shape * shape, SDL_Rect center, int radius);

void Shape_free(Shape * shape);

int ShapeElem_create(ShapeElem * elem, SDL_Color color, SDL_Color border_color);

void ShapeElem_free(ShapeElem * elem);

int ShapeSet_create(ShapeSet * set);

void ShapeSet_free(ShapeSet * set);

/* Shape */

Shape * Shape_adress(Shape * start, Shape shape);

int Shape_set_center(Shape shape, int x, int y);

int Shape_add_center(Shape shape, int vx, int vy);

int Shape_set_radius(Shape shape, int radius);

int Shape_add_radius(Shape shape, int add);

int Shape_set_border(Shape shape, int border);

int Shape_append(Shape * shape, Shape * appened);

int Shape_delete(Shape * shape, Shape del);

int Shape_copy(Shape * dest, Shape src);

int Shape_field_copy(Shape * dest, Shape src);

SDL_Rect Shape_average_center(Shape shape);

int Shape_save(FILE * ecr, Shape shape);

int Shape_load(FILE * lec, Shape * shape);

void Shape_reverse(Shape shape);

void Shape_simplify(Shape * shape);

/* ShapeElem */

int ShapeElem_is_void(ShapeElem elem);

int ShapeElem_add_center(ShapeElem elem, int vx, int vy);

int ShapeElem_add_radius(ShapeElem elem, int add);

int ShapeElem_copy(ShapeElem * dest, ShapeElem src);

SDL_Rect ShapeElem_average_center(ShapeElem elem);

int ShapeElem_save(FILE * ecr, ShapeElem elem);

int ShapeElem_load(FILE * lec, ShapeElem * elem);

int ShapeElem_version2_save(FILE * ecr, ShapeElem elem);

int ShapeElem_version2_load(FILE * lec, ShapeElem * elem);

/* ShapeSet */

int ShapeSet_is_void(ShapeSet shape);

int ShapeSet_set_center(ShapeSet shape, int x, int y);

int ShapeSet_add_center(ShapeSet shape, int vx, int vy);

int ShapeSet_add_center_elem(ShapeSet shape, int vx, int vy);

int ShapeSet_add_radius(ShapeSet shape, int add);

int ShapeSet_append_elem(ShapeSet shape, SDL_Color color, SDL_Color border_color, int * appened_id);

int ShapeSet_delete_elem(ShapeSet shape, int del_id);

int ShapeSet_append(ShapeSet shape, SDL_Color color, SDL_Color border_color, int * appened_id, Shape * appened);

int ShapeSet_delete(ShapeSet shape, SDL_Color color, SDL_Color border_color, int del_id, Shape del, int * destroyed);

int ShapeSet_clean(ShapeSet shape);

int ShapeSet_switch(ShapeSet shape, int first, int second);

int ShapeSet_copy(ShapeSet * dest, ShapeSet src);

int ShapeSet_join_shape(ShapeSet shape, SDL_Color color, SDL_Color border_color, Shape add);

int ShapeSet_join_elem(ShapeSet shape, ShapeElem add);

int ShapeSet_join(ShapeSet shape, ShapeSet add);

SDL_Rect ShapeSet_average_center(ShapeSet shape);

int ShapeSet_set_center_as_average(ShapeSet shape);

int ShapeSet_get_dimensions(ShapeSet shape, int * x_min, int * y_min, int * x_max, int * y_max);

void ShapeSet_scale(ShapeSet shape, double scale);

int ShapeSet_resize_in(ShapeSet shape, int x_min, int y_min, int x_max, int y_max);

int ShapeSet_create_for_elem(ShapeSet * shape, ShapeElem elem);

int ShapeSet_save(FILE * ecr, ShapeSet shape);

int ShapeSet_load(FILE * lec, ShapeSet * shape);

int ShapeSet_load_stats(FILE * lec, ShapeSet shape);

int ShapeSet_real_scale(ShapeSet shape, double factor);

void ShapeSet_reverse(ShapeSet shape);

void ShapeSet_simplify(ShapeSet shape);

#endif