#include "shape_editor.h"

#include <stdlib.h>
#include "draw.h"
#include "window.h"
#include "math.h"
#include "color.h"
#include "image.h"
#include "dialog.h"
#include "key_gest.h"

#include "data_debug_print.h"

static void Shape_edit_draw(ShapeEdit_Select select, Bone arma)
{
	#if 0
	Image image = NULL;
	#endif
	clear_screen();
	Edit_print_background(get_screen());
	draw_shape_armature(arma);
	draw_shape_set(select->unset_shape->shape, select->unset_shape->start.x, select->unset_shape->start.y);
	ShapeEdit_Select_draw(select);
	ShapeList_draw(select->shape_list);
	#if 0
	ShapeSet_make_image(&image, (*(select->arma))->shape);
	Image_print_on_surface(get_screen(), image);
	Image_free(&image);
	#endif
	refresh_screen();
}

static void Shape_edit_draw_generic(void * select)
{
	Shape_edit_draw((ShapeEdit_Select)select, *(((ShapeEdit_Select)select)->arma));
}

static int Mini_Menu_set_shapeedit_background(Mini_Menu menu)
{
	SDL_Surface * rectangle = NULL;
	SDL_Surface * rectangle_hover = NULL;
	int success = 1;
	if((rectangle = SDL_CreateRGBSurface(SDL_HWSURFACE, 125, 25, 32, 0, 0, 0, 0)) == NULL)
		return 0;
	if(! SDL_CopySurface(&rectangle_hover, rectangle))
		return 0;
	SDL_FillRect(rectangle, NULL, SDL_MapRGB(rectangle->format, 255, 216, 0));
	SDL_FillRect(rectangle_hover, NULL, SDL_MapRGB(rectangle_hover->format, 255, 236, 127));
	success &= Mini_Menu_set_button_background(menu, rectangle, rectangle_hover);
	SDL_FreeSurface(rectangle);
	SDL_FreeSurface(rectangle_hover);
	return success;
}

int ShapeEdit_create(ShapeEdit * data, Bone * arma)
{
	if(data == NULL)
		return 0;
	data->select = NULL;
	
	if(! ShapeEdit_Select_create(&(data->select), arma))
		goto free_zone;
	
	return 1;
	
	free_zone:;
	ShapeEdit_destroy(data);
	return 0;
}

void ShapeEdit_destroy(ShapeEdit * data)
{
	if(data == NULL)
		return;
	ShapeEdit_Select_free(&(data->select));
}

int Shape_edit(Personnage * perso, ShapeEdit * data, EditExit * exit_choice, File_Explorer file_exp, FileSaver * file_saver)
{
	ShapeEdit_Select select = NULL;
	int cont = 1;
	int need_refresh = 1;
	int last_refresh_time = SDL_GetTicks() - REFRESH_TIME - 1;
	int success = 1;
	SDL_Event event;
	ShapeEdit alternative;
	ShapeEdit_Select_Action move_mode = SHAPEEDIT_SELECT_ACTION_TRANSLATION;
	ShapeList_Action shape_list_action = SHAPELIST_ACTION_NULL;
	int (*menu_fct)(ShapeEdit_Select) = NULL;
	Bone * arma = NULL;
	Key_Gest key_gest = Key_Gest_create();
	
	if(perso == NULL || *perso == NULL)
	{
		fprintf(stderr, "Shape Editor needs a target caracter to run\n");
		return 0;
	}
	arma = &((*perso)->armature);
	if(data == NULL)
	{
		if(! ShapeEdit_create(&alternative, arma))
			return 0;
		data = &alternative;
	}
	select = data->select;
	select->arma = arma;
	select->file_exp = file_exp;
	ShapeEdit_Select_cancel_selection(select);
	
	while(cont && success)
	{
		if(need_refresh && last_refresh_time + REFRESH_TIME < SDL_GetTicks())
		{
			Shape_edit_draw(select, *arma);
			need_refresh = 0;
			last_refresh_time = SDL_GetTicks();
		}
		SDL_WaitEvent(&event);
		if(ShapeList_event(select->shape_list, &event, &shape_list_action))
		{
			need_refresh = 1;
			switch(shape_list_action)
			{
				case SHAPELIST_ACTION_APPEND:
				ShapeEdit_Select_add_selection_to_shape_list(select);
				break;
				
				case SHAPELIST_ACTION_GET:
				ShapeEdit_Select_add_shape_list_to_selection(select);
				break;
				
				default:
				break;
			}
			if(shape_list_action == SHAPELIST_ACTION_SCROLL)
				continue;
		}
		switch(event.type)
		{
			case SDL_QUIT:
			cont = 0;
			if(exit_choice != NULL)
				*exit_choice = EDIT_EXIT_QUIT;
			break;
			
			case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
				cont = 0;
				if(exit_choice != NULL)
					*exit_choice = EDIT_EXIT_QUIT;
				break;
				
				case SDLK_LEFT:
				cont = 0;
				if(exit_choice != NULL)
					*exit_choice = EDIT_EXIT_GOTO_ARMA_EDITOR;
				break;
				
				case SDLK_RIGHT:
				cont = 0;
				if(exit_choice != NULL)
					*exit_choice = EDIT_EXIT_GOTO_ACTION_EDITOR;
				break;
				
				case SDLK_r:
				if(move_mode <= SHAPEEDIT_SELECT_ACTION_TRANSLATION)
					move_mode = SHAPEEDIT_SELECT_ACTION_NULL;
				move_mode |= SHAPEEDIT_SELECT_ACTION_ROTATE;
				break;
				
				case SDLK_s:
				if(move_mode <= SHAPEEDIT_SELECT_ACTION_TRANSLATION)
					move_mode = SHAPEEDIT_SELECT_ACTION_NULL;
				move_mode |= SHAPEEDIT_SELECT_ACTION_SCALE;
				break;
				
				default:
				break;
			}
			break;
			
			case SDL_KEYUP:
			switch(event.key.keysym.sym)
			{
				case SDLK_a:
				need_refresh |= ShapeEdit_Select_add_selection_to_shape_list(select);
				break;
				
				case SDLK_e:
				need_refresh |= ShapeEdit_Select_append(select);
				break;
				
				case SDLK_d:
				need_refresh |= ShapeEdit_Select_duplicate(select);
				break;
				
				case SDLK_p:
				if(! select->fusion_mode)
					select->change_parent = 1;
				break;
				
				case SDLK_c:
				if(! ShapeEdit_Select_color_edit(select))
				{
					if(exit_choice != NULL)
						*exit_choice = EDIT_EXIT_GOTO_ARMA_EDITOR;
					cont = 0;
				}
				need_refresh = 1;
				break;
				
				case SDLK_DELETE:
				case SDLK_BACKSPACE:
				need_refresh |= ShapeEdit_Select_delete(select);
				break;
				
				case SDLK_UP:
				need_refresh |= ShapeEdit_Select_elem_goto_front(select);
				break;
				
				case SDLK_DOWN:
				need_refresh |= ShapeEdit_Select_elem_goto_back(select);
				break;
				
				case SDLK_r:
				move_mode &= ~SHAPEEDIT_SELECT_ACTION_ROTATE;
				if(! move_mode)
					move_mode = SHAPEEDIT_SELECT_ACTION_TRANSLATION;
				break;
				
				case SDLK_s:
				if(FileSaver_save(file_saver, file_exp, &key_gest, *perso, &FileSaver_Personnage_save))
					need_refresh = 1;
				move_mode &= ~SHAPEEDIT_SELECT_ACTION_SCALE;
				if(! move_mode)
					move_mode = SHAPEEDIT_SELECT_ACTION_TRANSLATION;
				break;
				
				case SDLK_l:
				if(FileSaver_load(file_saver, file_exp, &key_gest, perso, &FileSaver_Personnage_load))
				{
					ShapeEdit_Select_cancel_selection(select);
					arma = &((*perso)->armature);
					need_refresh = 1;
				}
				break;
				
				case SDLK_SPACE:
				menu_fct = NULL;
				if(ShapeList_in(select->shape_list, select->mouse.x, select->mouse.y))
				{
					Mini_Menu_set_origin_on_screen(select->shape_list_menu, select->mouse.x, select->mouse.y, get_screen());
					success &= Mini_Menu_gest(select->shape_list_menu, get_screen(), &menu_fct, NULL);
				}
				else if(ShapeEdit_Select_is_selection(select))
				{
					Mini_Menu_set_origin_on_screen(select->shape_menu, select->mouse.x, select->mouse.y, get_screen());
					success &= Mini_Menu_gest(select->shape_menu, get_screen(), &menu_fct, NULL);
				}
				else
					break;
				need_refresh = 1;
				if(menu_fct == NULL)
					break;
				(*menu_fct)(select);
				menu_fct = NULL;
				break;
				
				default:
				break;
			}
			break;
			
			case SDL_MOUSEBUTTONDOWN:
			switch(event.button.button)
			{
				case SDL_BUTTON_LEFT:
				need_refresh |= ShapeEdit_Select_start_mouse_move(select, event.button.x, event.button.y, move_mode);
				break;
				
				default:
				break;
			}
			break;
			
			case SDL_MOUSEBUTTONUP:
			switch(event.button.button)
			{
				case SDL_BUTTON_LEFT:
				need_refresh |= ShapeEdit_Select_end_mouse_move(select);
				break;
				
				case SDL_BUTTON_RIGHT:
				if(ShapeEdit_Select_is_mouse_move(select))
					break;
				need_refresh |= ShapeEdit_Select_make_selection(select, event.button.x, event.button.y, SHAPE_SELECTION_RANGE);
				break;
				
				case SDL_BUTTON_WHEELUP:
				need_refresh |= ShapeEdit_Select_wheel_selection(select, 2);
				break;
				
				case SDL_BUTTON_WHEELDOWN:
				need_refresh |= ShapeEdit_Select_wheel_selection(select, -2);
				break;
				
				default:
				break;
			}
			break;
			
			case SDL_MOUSEMOTION:
			ShapeEdit_Select_set_mouse_motion(select, event.motion.x, event.motion.y);
			if(ShapeEdit_Select_is_mouse_move(select))
				need_refresh = ShapeEdit_Select_mouse_move_selection(select, event.motion.x, event.motion.y);
			break;
			
			default:
			break;
		}
		Key_Gest_event(&key_gest, &event);
	}
	
	select->file_exp = NULL;
	if(data == &alternative)
		ShapeEdit_destroy(data);
	return success;
}

static int ShapeEdit_Select_start_scale(ShapeEdit_Select select)
{
	return ShapeEdit_Select_start_mouse_move(select, select->mouse.x, select->mouse.y, SHAPEEDIT_SELECT_ACTION_SCALE);
}

static int ShapeEdit_Select_start_rotate(ShapeEdit_Select select)
{
	return ShapeEdit_Select_start_mouse_move(select, select->mouse.x, select->mouse.y, SHAPEEDIT_SELECT_ACTION_ROTATE);
}

static int ShapeEdit_Select_start_rotate_scale(ShapeEdit_Select select)
{
	return ShapeEdit_Select_start_mouse_move(select, select->mouse.x, select->mouse.y, SHAPEEDIT_SELECT_ACTION_ROTATE | SHAPEEDIT_SELECT_ACTION_SCALE);
}

static int ShapeEdit_Select_start_parenty(ShapeEdit_Select select)
{
	if(! select->fusion_mode)
	{
		select->change_parent = 1;
		return 1;
	}
	return 0;
}

static int ShapeEdit_Select_select_set(ShapeEdit_Select select)
{
	if(select->select_type == SHAPEEDIT_SELECT_NULL)
		return 0;
	select->select_type = SHAPEEDIT_SELECT_BONE;
	return 1;
}

static int ShapeEdit_Select_select_elem(ShapeEdit_Select select)
{
	if(select->select_type <= SHAPEEDIT_SELECT_BONE)
		return 0;
	select->select_type = SHAPEEDIT_SELECT_ELEM;
	return 1;
}

static int ShapeEdit_Select_delete_ShapeList_elem(ShapeEdit_Select select)
{
	return ShapeList_delete(select->shape_list, ShapeList_get_id_by_coords(select->shape_list, select->mouse.x, select->mouse.y));
}

static int ShapeEdit_Select_clean_ShapeList(ShapeEdit_Select select)
{
	return ShapeList_clean(select->shape_list);
}

static int ShapeEdit_Select_save_ShapeList(ShapeEdit_Select select)
{
	char * file_name = NULL;
	File_Explorer_gest_event(select->file_exp, get_screen(), FILE_EXPLORER_MODE_SAVE, &file_name);
	if(file_name != NULL)
	{
		if(! ShapeList_save_in_file(file_name, select->shape_list))
		{
			Dialog_global_print(get_screen(), "Erreur de sauvegarde de la liste de formes %s", file_name);
			fprintf(stderr, "Error while saving \"%s\" as a shape list\n", file_name);
			return 0;
		}
		return 1;
	}
	return 0;
}

static int ShapeEdit_Select_load_ShapeList(ShapeEdit_Select select)
{
	char * file_name = NULL;
	File_Explorer_gest_event(select->file_exp, get_screen(), FILE_EXPLORER_MODE_LOAD, &file_name);
	if(file_name != NULL)
	{
		if(! ShapeList_load_from_file(file_name, select->shape_list))
		{
			Dialog_global_print(get_screen(), "Erreur de chargement de la liste de formes %s", file_name);
			fprintf(stderr, "Error while loading \"%s\" as a shape list\n", file_name);
			return 0;
		}
		return 1;
	}
	return 0;
}

int ShapeEdit_Select_create(ShapeEdit_Select * select, Bone * arma)
{
	ShapeEdit_Select res = NULL;
	if((res = (ShapeEdit_Select)malloc(sizeof(struct _ShapeEdit_Select))) == NULL)
		return 0;
	if(! Bone_create(&(res->unset_shape)))
	{
		free(res);
		return 0;
	}
	if(! ShapeList_create(&(res->shape_list)))
	{
		Bone_free(&(res->unset_shape));
		free(res);
		return 0;
	}
	
	res->arma = arma;
	res->selected_bone = NULL;
	res->selected_elem_id = -1;
	res->selected_shape = NULL;
	res->select_type = SHAPEEDIT_SELECT_NULL;
	res->action = SHAPEEDIT_SELECT_ACTION_NULL;
	res->last_color = SDL_Color_create(200, 200, 200);
	res->bone_copy = NULL;
	res->fusion_mode = 0;
	res->change_parent = 0;
	res->file_exp = NULL;
	
	ShapeList_set_origin(res->shape_list, get_screen()->w - SHAPE_LIST_WIDTH, 0);
	ShapeList_set_size(res->shape_list, SHAPE_LIST_WIDTH, SHAPE_LIST_WIDTH);
	ShapeList_set_background_sizedRGB(res->shape_list, 127, 127, 127);
	
	res->shape_list_menu = NULL;
	res->shape_menu = NULL;
	
	if(! Mini_Menu_create(&(res->shape_list_menu))
	|| ! Mini_Menu_create(&(res->shape_menu)))
	{
		fprintf(stderr, "In the Shape Editor: error in creation of mini_menu\n");
		goto free_zone;
	}
	if(! Mini_Menu_set_shapeedit_background(res->shape_list_menu)
	|| ! Mini_Menu_configure(res->shape_list_menu, "media/arial.ttf", 20, SDL_Color_create(63, 0, 0), 6,
				"Nouvelle", &ShapeEdit_Select_clean_ShapeList,
				"Save", &ShapeEdit_Select_save_ShapeList,
				"Load", &ShapeEdit_Select_load_ShapeList,
				"Ajouter", &ShapeEdit_Select_add_selection_to_shape_list,
				"Supprimer", &ShapeEdit_Select_delete_ShapeList_elem,
				"Annuler", NULL))
	{
		fprintf(stderr, "In the Shape Editor: error in configuration of shape_list mini_menu\n");
		goto free_zone;
	}
	
	if(! Mini_Menu_set_shapeedit_background(res->shape_menu)
	|| ! Mini_Menu_configure(res->shape_menu, "media/arial.ttf", 20, SDL_Color_create(63, 0, 0), 14,
				"Ajouter", &ShapeEdit_Select_append,
				"Supprimer", &ShapeEdit_Select_delete,
				"Couleur", &ShapeEdit_Select_color_edit,
				"Dupliquer", &ShapeEdit_Select_duplicate,
				"Agrandir", &ShapeEdit_Select_start_scale,
				"Pivoter", &ShapeEdit_Select_start_rotate,
				"Agr/Pivo", &ShapeEdit_Select_start_rotate_scale,
				"Reparenter", &ShapeEdit_Select_start_parenty,
				"Ajouter Liste", &ShapeEdit_Select_add_selection_to_shape_list,
				"Ensemble", &ShapeEdit_Select_select_set,
				"Element", &ShapeEdit_Select_select_elem,
				"Avancer", &ShapeEdit_Select_elem_goto_front,
				"Reculer", &ShapeEdit_Select_elem_goto_back,
				"Annuler", NULL))
	{
		fprintf(stderr, "In the Shape Editor: error in configuration of shape_list mini_menu\n");
		goto free_zone;
	}
	
	*select = res;
	return 1;
	
	free_zone:;
	
	Bone_free(&(res->unset_shape));
	ShapeEdit_Select_free(&res);
	
	return 0;
}

void ShapeEdit_Select_free(ShapeEdit_Select * select)
{
	if(select == NULL || *select == NULL)
		return;
	Mini_Menu_free(&((*select)->shape_list_menu));
	Mini_Menu_free(&((*select)->shape_menu));
	ShapeList_free(&((*select)->shape_list));
	Bone_free(&((*select)->unset_shape));
	free(*select);
	*select = NULL;
}

void ShapeEdit_Select_draw(ShapeEdit_Select select)
{
	switch(select->select_type)
	{
		case SHAPEEDIT_SELECT_BONE:
		draw_in_armature_color_with_alpha(*(select->arma), select->selected_bone, ARMADRAW_PART_BONE, ARMADRAW_PART_IGNORE, SDL_Color_create(255, 216, 0), SDL_Color_create(0, 0, 0), SDL_Color_create(0, 0, 0), 127);
		break;
		
		case SHAPEEDIT_SELECT_ELEM:
		draw_select_shape_elem(*(select->arma), select->selected_bone, (*(select->selected_bone->shape->elems + select->selected_elem_id))->shape, SDL_Color_invert((*(select->selected_bone->shape->elems + select->selected_elem_id))->color));
		break;
		
		case SHAPEEDIT_SELECT_SHAPE:
		draw_select_shape(*(select->arma), select->selected_bone, select->selected_shape, SDL_Color_invert((*(select->selected_bone->shape->elems + select->selected_elem_id))->color));
		break;
		
		default:
		break;
	}
}

static int Shape_make_selection(int * min_dist, Shape cur, int x, int y, int range, Shape * select)
{
	int distance;
	int retour = 0;
	if(cur == NULL)
		return 0;
	if((distance = distance_carre(x, y, cur->center.x, cur->center.y)) < carre(range + cur->radius))
	{
		if(*min_dist == -1 || distance < *min_dist)
		{
			*min_dist = distance;
			*select = cur;
			retour = 1;
		}
	}
	retour |= Shape_make_selection(min_dist, cur->child, x, y, range, select);
	retour |= Shape_make_selection(min_dist, cur->sister, x, y, range, select);
	return retour;
}

static int ShapeLink_make_selection(int * min_dist, Shape shape, int x, int y, int range)
{
	int distance;
	int retour = 0;
	Shape cur = NULL;
	if(shape == NULL)
		return 0;
	for(cur = shape->child; cur != NULL; cur = cur->sister)
	{
		if((distance = distance_carre_segment(x, y, shape->center.x, shape->center.y, cur->center.x, cur->center.y)) < carre(range + min_int(2, shape->radius, cur->radius)))
		{
			if(*min_dist == -1 || distance < *min_dist)
			{
				*min_dist = distance;
				retour = 1;
			}
		}
	}
	retour |= ShapeLink_make_selection(min_dist, shape->child, x, y, range);
	retour |= ShapeLink_make_selection(min_dist, shape->sister, x, y, range);
	return retour;
}

static void Bone_ShapeSet_make_selection(int * min_dist, Bone bone, int x, int y, int range, int b_x, int b_y, ShapeEdit_Select select, int id)
{
	Shape sel = NULL;
	if(select->select_type < SHAPEEDIT_SELECT_ELEM)
		return;
	if(! Shape_make_selection(min_dist, (*(bone->shape->elems + id))->shape, x - bone->start.x - bone->shape->center.x - b_x, y - bone->start.y - bone->shape->center.y - b_y, range, &sel))
		return;
	if(sel != NULL)
	{
		select->selected_bone = bone;
		select->selected_elem_id = id;
		select->selected_shape = sel;
		select->select_type = SHAPEEDIT_SELECT_SHAPE;
	}
}

static void Bone_ShapeElem_make_selection(int * min_dist, Bone bone, int x, int y, int range, int b_x, int b_y, ShapeEdit_Select select, ShapeEdit_Select_Type select_type)
{
	int i;
	int select_id = -1;
	Shape sel = NULL;
	for(i = 0; i < bone->shape->nb_elems; ++i)
	{
		if(Shape_make_selection(min_dist, (*(bone->shape->elems + i))->shape, x - bone->start.x - bone->shape->center.x - b_x, y - bone->start.y - bone->shape->center.y - b_y, range, &sel)
		|| ShapeLink_make_selection(min_dist, (*(bone->shape->elems + i))->shape, x - bone->start.x - bone->shape->center.x - b_x, y - bone->start.y - bone->shape->center.y - b_y, range))
			select_id = i;
	}
	if(select_id != -1)
	{
		select->selected_bone = bone;
		select->selected_elem_id = select_id;
		select->selected_shape = sel;
		select->select_type = select_type;
	}
}

static void ShapeEdit_Select_make_selection_aux(int * min_dist, ShapeEdit_Select select, Bone cur, int x, int y, int range, int b_x, int b_y, ShapeEdit_Select_Type current_type)
{
	int distance;
	if(cur == NULL)
		return;
	/*DP_Bone(&cur);*/
	switch(current_type)
	{
		case SHAPEEDIT_SELECT_NULL:
		if((distance = distance_carre_segment(x, y, cur->start.x + b_x, cur->start.y + b_y, cur->end.x + b_x, cur->end.y + b_y)) < carre(range))
		{
			if(*min_dist == -1 || distance < *min_dist)
			{
				*min_dist = distance;
				select->selected_bone = cur;
				select->select_type = SHAPEEDIT_SELECT_BONE;
			}
		}
		break;
		
		case SHAPEEDIT_SELECT_ELEM:
		if(select->selected_bone != cur)
			break;
		Bone_ShapeSet_make_selection(min_dist, cur, x, y, range, b_x, b_y, select, select->selected_elem_id);
		break;
		
		case SHAPEEDIT_SELECT_BONE:
		if(select->selected_bone != cur)
			break;
		Bone_ShapeElem_make_selection(min_dist, cur, x, y, range, b_x, b_y, select, SHAPEEDIT_SELECT_ELEM);
		break;
		
		default:
		return;
	}
	ShapeEdit_Select_make_selection_aux(min_dist, select, cur->child, x, y, range, b_x + cur->end.x, b_y + cur->end.y, current_type);
	ShapeEdit_Select_make_selection_aux(min_dist, select, cur->brother, x, y, range, b_x, b_y, current_type);
}

static void ShapeEdit_Select_full_shape_make_selection_aux(int * min_dist, ShapeEdit_Select select, Bone cur, int x, int y, int range, int b_x, int b_y)
{
	if(cur == NULL)
		return;
	Bone_ShapeElem_make_selection(min_dist, cur, x, y, range, b_x, b_y, select, SHAPEEDIT_SELECT_ELEM);
	/*Bone_ShapeSet_make_selection(min_dist, cur, x, y, range, b_x, b_y, select, select->selected_elem_id);*/
	
	ShapeEdit_Select_full_shape_make_selection_aux(min_dist, select, cur->child, x, y, range, b_x + cur->end.x, b_y + cur->end.y);
	ShapeEdit_Select_full_shape_make_selection_aux(min_dist, select, cur->brother, x, y, range, b_x, b_y);
}

static void ShapeEdit_Select_full_shape_make_selection(int * min_dist, ShapeEdit_Select select, int x, int y, int range)
{
	ShapeEdit_Select_full_shape_make_selection_aux(min_dist, select, *(select->arma), x, y, range, 0, 0);
}

int ShapeEdit_Select_make_selection(ShapeEdit_Select select, int x, int y, int range)
{
	int min_dist = -1;
	int type;
	if(select == NULL)
		return 0;
	if(select->change_parent)
		ShapeEdit_Select_save_selection(select);
	for(type = select->select_type; min_dist == -1 && type >= SHAPEEDIT_SELECT_NULL; --type)
		ShapeEdit_Select_make_selection_aux(&min_dist, select, *(select->arma), x, y, range, 0, 0, type);
	if(min_dist == -1)
		ShapeEdit_Select_full_shape_make_selection(&min_dist, select, x, y, range);
	if(min_dist == -1)
	{
		return ShapeEdit_Select_cancel_selection(select);
	}
	else if(select->select_type >= SHAPEEDIT_SELECT_BONE)
	{
		if(select->fusion_mode)
			ShapeEdit_Select_fusion_to_selected(select);
		else if(select->change_parent)
			ShapeEdit_Select_change_parent(select);
	}
	
	return 1;
}

int ShapeEdit_Select_cancel_selection(ShapeEdit_Select select)
{
	if(select == NULL)
		return 0;
	if(select->select_type == SHAPEEDIT_SELECT_NULL)
		return 0;
	if(select->selected_bone == select->unset_shape)
	{
		ShapeSet_clean(select->unset_shape->shape);
	}
	select->select_type = SHAPEEDIT_SELECT_NULL;
	select->selected_bone = NULL;
	select->selected_elem_id = -1;
	select->selected_shape = NULL;
	select->fusion_mode = 0;
	select->change_parent = 0;
	return 1;
}

int ShapeEdit_Select_is_selection(ShapeEdit_Select select)
{
	if(select == NULL)
		return 0;
	return select->select_type > SHAPEEDIT_SELECT_NULL;
}

void ShapeEdit_Select_fusion_to_selected(ShapeEdit_Select select)
{
	int vx, vy;
	int sx, sy;
	if(! select->fusion_mode || select->change_parent)
		return;
	ShapeSet_absolute_coords(select->selected_bone, *(select->arma), &vx, &vy);
	ShapeSet_absolute_coords(select->unset_shape, select->unset_shape, &sx, &sy);
	ShapeSet_add_center_elem(select->unset_shape->shape, sx - vx, sy - vy);
	ShapeSet_join(select->selected_bone->shape, select->unset_shape->shape);
	ShapeSet_clean(select->unset_shape->shape);
	select->fusion_mode = 0;
}

void ShapeEdit_Select_change_parent(ShapeEdit_Select select)
{
	int sx, sy;
	if(! select->change_parent)
		return;
	ShapeSet_absolute_coords(select->saved_selected_bone, *(select->arma), &sx, &sy);
	switch(select->saved_select_type)
	{
		case SHAPEEDIT_SELECT_BONE:
		ShapeSet_join(select->unset_shape->shape, select->saved_selected_bone->shape);
		ShapeSet_clean(select->saved_selected_bone->shape);
		break;
		
		case SHAPEEDIT_SELECT_ELEM:
		ShapeSet_join_elem(select->unset_shape->shape, *(select->saved_selected_bone->shape->elems + select->saved_selected_elem_id));
		ShapeSet_delete_elem(select->saved_selected_bone->shape, select->saved_selected_elem_id);
		break;
		
		default:
		return;
	}
	ShapeSet_set_center(select->unset_shape->shape, sx, sy);
	ShapeEdit_Select_delete_saved_selection(select);
	select->change_parent = 0;
	select->fusion_mode = 1;
	ShapeEdit_Select_fusion_to_selected(select);
	select->fusion_mode = 0;
}

void ShapeEdit_Select_save_selection(ShapeEdit_Select select)
{
	if(select == NULL)
		return;
	select->saved_selected_bone = select->selected_bone;
	select->saved_selected_elem_id = select->selected_elem_id;
	select->saved_selected_shape = select->selected_shape;
	select->saved_select_type = select->select_type;
}

void ShapeEdit_Select_delete_saved_selection(ShapeEdit_Select select)
{
	if(select == NULL)
		return;
	select->saved_select_type = SHAPEEDIT_SELECT_NULL;
}

int ShapeEdit_Select_add_selection_to_shape_list(ShapeEdit_Select select)
{
	if(select == NULL)
		return 0;
	switch(select->select_type)
	{
		case SHAPEEDIT_SELECT_BONE:
		return ShapeList_append(select->shape_list, select->selected_bone->shape);
		
		case SHAPEEDIT_SELECT_ELEM:
		{
			ShapeSet tmp = NULL;
			int retour;
			
			if(! ShapeSet_create_for_elem(&tmp, *(select->selected_bone->shape->elems + select->selected_elem_id)))
				return 0;
			retour = ShapeList_append(select->shape_list, tmp);
			ShapeSet_free(&tmp);
			return retour;
		}
		break;
		
		default:
		break;
	}
	return 0;
}

int ShapeEdit_Select_add_shape_list_to_selection(ShapeEdit_Select select)
{
	if(select == NULL)
		return 0;
	ShapeSet_clean(select->unset_shape->shape);
	if(! ShapeList_get_join(select->shape_list, select->unset_shape->shape))
		return 0;
	ShapeSet_set_center(select->unset_shape->shape, select->mouse.x, select->mouse.y);
	select->select_type = SHAPEEDIT_SELECT_BONE;
	select->selected_bone = select->unset_shape;
	ShapeEdit_Select_start_mouse_move(select, select->mouse.x, select->mouse.y, SHAPEEDIT_SELECT_ACTION_TRANSLATION);
	select->fusion_mode = 1;
	select->shape_list->visible = 0;
	return 1;
}

/* Edition */

int ShapeEdit_Select_append(ShapeEdit_Select select)
{
	if(! ShapeEdit_Select_is_selection(select))
		return 0;
	switch(select->select_type)
	{
		case SHAPEEDIT_SELECT_BONE:
		if(! ShapeSet_append(select->selected_bone->shape, select->last_color, SDL_Color_less(select->last_color, 127), &(select->selected_elem_id), &(select->selected_shape)))
			return 0;
		select->select_type = SHAPEEDIT_SELECT_SHAPE;
		ShapeEdit_Select_start_mouse_move(select, select->mouse.x, select->mouse.y, SHAPEEDIT_SELECT_ACTION_TRANSLATION);
		break;
		
		case SHAPEEDIT_SELECT_ELEM:
		{
			if(! ShapeEdit_Select_make_selection(select, select->mouse.x, select->mouse.y, SHAPE_SELECTION_RANGE))
				return 0;
			if(select->select_type != SHAPEEDIT_SELECT_SHAPE)
				break;
		}
		case SHAPEEDIT_SELECT_SHAPE:
		{
			Shape * select_adr = NULL;
			if((select_adr = Shape_adress(&((*(select->selected_bone->shape->elems + select->selected_elem_id))->shape), select->selected_shape)) == NULL)
				return 0;
			if(! Shape_append(select_adr, &(select->selected_shape)))
				return 0;
			select->select_type = SHAPEEDIT_SELECT_SHAPE;
			ShapeEdit_Select_start_mouse_move(select, select->mouse.x, select->mouse.y, SHAPEEDIT_SELECT_ACTION_TRANSLATION);
		}
		break;
		
		default:
		return 0;
	}
	return 1;
}

int ShapeEdit_Select_duplicate(ShapeEdit_Select select)
{
	Shape to_app = NULL;
	int app_id = -1;
	if(! ShapeEdit_Select_is_selection(select))
		return 0;
	switch(select->select_type)
	{
		case SHAPEEDIT_SELECT_BONE:
		{
			int x, y;
			if(select->selected_bone->shape->nb_elems <= 0)
				return 0;
			if(! ShapeSet_join(select->unset_shape->shape, select->selected_bone->shape))
				return 0;
			ShapeSet_absolute_coords(select->selected_bone, *(select->arma), &x, &y);
			ShapeSet_set_center(select->unset_shape->shape, x, y);
			select->selected_bone = select->unset_shape;
			select->fusion_mode = 1;
			
			ShapeEdit_Select_start_mouse_move(select, select->mouse.x, select->mouse.y, SHAPEEDIT_SELECT_ACTION_TRANSLATION);
			return 1;
		}
		break;
		
		case SHAPEEDIT_SELECT_ELEM:
		if(! Shape_copy(&to_app, (*(select->selected_bone->shape->elems + select->selected_elem_id))->shape))
			return 0;
		break;
		
		case SHAPEEDIT_SELECT_SHAPE:
		if(! Shape_field_copy(&to_app, select->selected_shape))
			return 0;
		break;
		
		default:
		return 0;
	}
	if(to_app == NULL)
		return 0;
	if(! ShapeSet_append_elem(select->selected_bone->shape, (*(select->selected_bone->shape->elems + select->selected_elem_id))->color, (*(select->selected_bone->shape->elems + select->selected_elem_id))->border_color, &app_id))
		return 0;
	if(app_id == -1)
		return 0;
	(*(select->selected_bone->shape->elems + app_id))->shape = to_app;
	select->selected_elem_id = app_id;
	select->selected_shape = to_app;
	ShapeEdit_Select_start_mouse_move(select, select->mouse.x, select->mouse.y, SHAPEEDIT_SELECT_ACTION_TRANSLATION);
	return 1;
}

int ShapeEdit_Select_delete(ShapeEdit_Select select)
{
	if(! ShapeEdit_Select_is_selection(select))
		return 0;
	switch(select->select_type)
	{
		case SHAPEEDIT_SELECT_BONE:
		return 0;
		
		case SHAPEEDIT_SELECT_ELEM:
		{
			if(! ShapeSet_delete_elem(select->selected_bone->shape, select->selected_elem_id))
				return 0;
			select->selected_shape = NULL;
			select->selected_bone = NULL;
			select->select_type = SHAPEEDIT_SELECT_NULL;
		}
		break;
		
		case SHAPEEDIT_SELECT_SHAPE:
		{
			if(! Shape_delete(&((*(select->selected_bone->shape->elems + select->selected_elem_id))->shape), select->selected_shape))
				return 0;
			select->selected_shape = NULL;
			select->selected_bone = NULL;
			select->select_type = SHAPEEDIT_SELECT_NULL;
		}
		break;
		
		default:
		return 0;
	}
	return 1;
}

int ShapeEdit_Select_color_edit(ShapeEdit_Select select)
{
	if(select->select_type < SHAPEEDIT_SELECT_ELEM)
		return 1;
	clear_screen();
	if(! SDL_Color_window_gest(&((*(select->selected_bone->shape->elems + select->selected_elem_id))->color), get_screen(), get_screen()->w - COLOR_EDIT_SIZE - 50, get_screen()->h - COLOR_EDIT_SIZE - 25, COLOR_EDIT_SIZE, &Shape_edit_draw_generic, select))
		return 0;
	select->last_color = (*(select->selected_bone->shape->elems + select->selected_elem_id))->color;
	return 1;
}

static int ShapeEdit_Select_elem_goto_target(ShapeEdit_Select select, int target)
{
	if(select->select_type < SHAPEEDIT_SELECT_ELEM)
		return 0;
	if(! ShapeSet_switch(select->selected_bone->shape, select->selected_elem_id, target))
		return 0;
	select->selected_elem_id = target;
	return 1;
}

int ShapeEdit_Select_elem_goto_back(ShapeEdit_Select select)
{
	if(select == NULL)
		return 0;
	return ShapeEdit_Select_elem_goto_target(select, select->selected_elem_id - 1);
}

int ShapeEdit_Select_elem_goto_front(ShapeEdit_Select select)
{
	if(select == NULL)
		return 0;
	return ShapeEdit_Select_elem_goto_target(select, select->selected_elem_id + 1);
}

int ShapeEdit_Select_copy_selection(ShapeEdit_Select select)
{
	if(select == NULL)
		return 0;
	switch(select->select_type)
	{
		case SHAPEEDIT_SELECT_BONE:
		if(select->bone_copy != NULL)
			ShapeEdit_Select_free_copy_selection(select);
		return ShapeSet_copy(&(select->bone_copy), select->selected_bone->shape);
		
		case SHAPEEDIT_SELECT_ELEM:
		if(select->elem_copy != NULL)
			ShapeEdit_Select_free_copy_selection(select);
		return ShapeElem_copy(&(select->elem_copy), *(select->selected_bone->shape->elems + select->selected_elem_id));
		
		case SHAPEEDIT_SELECT_SHAPE:
		if(select->shape_copy != NULL)
			ShapeEdit_Select_free_copy_selection(select);
		return Shape_field_copy(&(select->shape_copy), select->selected_shape);
		
		default:
		break;
	}
	return 0;
}

void ShapeEdit_Select_free_copy_selection(ShapeEdit_Select select)
{
	if(select == NULL)
		return;
	switch(select->select_type)
	{
		case SHAPEEDIT_SELECT_BONE:
		ShapeSet_free(&(select->bone_copy));
		break;
		
		case SHAPEEDIT_SELECT_ELEM:
		ShapeElem_free(&(select->elem_copy));
		break;
		
		case SHAPEEDIT_SELECT_SHAPE:
		Shape_free(&(select->shape_copy));
		break;
		
		default:
		break;
	}
}

int ShapeSet_absolute_coords(Bone bone, Bone arma, int * x, int * y)
{
	if(! Bone_getXY_of_parent(bone, arma, x, y))
		return 0;
	if(x != NULL)
		(*x) += bone->start.x + bone->shape->center.x;
	if(y != NULL)
		(*y) += bone->start.y + bone->shape->center.y;
	return 1;
}

/* MouseMove */

int ShapeEdit_Select_start_mouse_move(ShapeEdit_Select select, int x, int y, ShapeEdit_Select_Action move_mode)
{
	if(select == NULL)
		return 0;
	if(ShapeEdit_Select_is_mouse_move(select))
		return 0;
	if(! ShapeEdit_Select_is_selection(select))
		return 0;
	if(ShapeSet_is_void(select->selected_bone->shape))
		return 0;
	if(select->fusion_mode)
		move_mode = SHAPEEDIT_SELECT_ACTION_TRANSLATION;
	select->action = move_mode;
	select->last_mouse.x = x;
	select->last_mouse.y = y;
	select->mouse_start.x = x;
	select->mouse_start.y = y;
	ShapeEdit_Select_set_mouse_motion(select, x, y);
	
	if(move_mode & (SHAPEEDIT_SELECT_ACTION_ROTATE | SHAPEEDIT_SELECT_ACTION_SCALE))
	{
		int vx, vy;
		vx = vy = 0;
		
		ShapeSet_absolute_coords(select->selected_bone, *(select->arma), &vx, &vy);
		select->action_center = ShapeEdit_Select_average_center(select);
		select->start_angle = angle_segment(select->action_center.x, select->action_center.y, x - vx, y - vy);
		
		if((select->start_distance = sqrt((double)distance_carre(select->action_center.x, select->action_center.y, x - vx, y - vy))) == 0)
		{
			select->start_distance = 1;
			if(move_mode & SHAPEEDIT_SELECT_ACTION_SCALE)
			{
				select->action = SHAPEEDIT_SELECT_ACTION_NULL;
				return 0;
			}
		}
		
		if(! ShapeEdit_Select_copy_selection(select))
			return 0;
	}
	
	return 1;
}

int ShapeEdit_Select_end_mouse_move(ShapeEdit_Select select)
{
	if(select == NULL)
		return 0;
	if(! ShapeEdit_Select_is_mouse_move(select))
		return 0;
	select->action = SHAPEEDIT_SELECT_ACTION_NULL;
	ShapeEdit_Select_free_copy_selection(select);
	return 1;
}

int ShapeEdit_Select_is_mouse_move(ShapeEdit_Select select)
{
	if(select == NULL)
		return 0;
	return select->action;
}

int ShapeEdit_Select_set_mouse_motion(ShapeEdit_Select select, int x, int y)
{
	if(select == NULL)
		return 0;
	select->mouse.x = x;
	select->mouse.y = y;
	return 1;
}

static void Shape_modif_scale_rotate(Shape shape, Shape copy, double factor, double angle, SDL_Rect center)
{
	shape->center.x = (((copy->center.x - center.x) * cos(angle)) - ((copy->center.y - center.y) * sin(angle))) * factor + center.x;
	shape->center.y = (((copy->center.y - center.y) * cos(angle)) + ((copy->center.x - center.x) * sin(angle))) * factor + center.y;
	shape->radius = copy->radius * factor;
}

static void Shape_modif_rotate(Shape shape, Shape copy, double angle, SDL_Rect center)
{
	Shape_modif_scale_rotate(shape, copy, 1, angle, center);
	/*shape->center.x = ((copy->center.x - center.x) * cos(angle)) - ((copy->center.y - center.y) * sin(angle)) + center.x;
	shape->center.y = ((copy->center.y - center.y) * cos(angle)) + ((copy->center.x - center.x) * sin(angle)) + center.y;*/
}

static void Shape_modif_scale(Shape shape, Shape copy, double factor, SDL_Rect center)
{
	Shape_modif_scale_rotate(shape, copy, factor, 0, center);
	/*shape->center.x = ((copy->center.x - center.x) * factor) + center.x;
	shape->center.y = ((copy->center.y - center.y) * factor) + center.y;
	shape->radius = copy->radius * factor;*/
}

static void Shape_apply_modif(Shape shape, Shape copy, double data, SDL_Rect center, void (*modif)(Shape, Shape, double, SDL_Rect))
{
	if(shape == NULL || copy == NULL)
		return;
	(*modif)(shape, copy, data, center);
	Shape_apply_modif(shape->child, copy->child, data, center, modif);
	Shape_apply_modif(shape->sister, copy->sister, data, center, modif);
}

static void ShapeSet_apply_modif(ShapeSet shape, ShapeSet copy, double data, SDL_Rect center, void (*modif)(Shape, Shape, double, SDL_Rect))
{
	int i;
	if(shape == NULL || copy == NULL)
		return;
	for(i = 0; i < shape->nb_elems && i < copy->nb_elems; ++i)
		Shape_apply_modif((*(shape->elems + i))->shape, (*(copy->elems + i))->shape, data, center, modif);
}

static void ShapeEdit_Select_apply_modif(ShapeEdit_Select select, double data, SDL_Rect center, void (*modif)(Shape, Shape, double, SDL_Rect))
{
	switch(select->select_type)
	{
		case SHAPEEDIT_SELECT_BONE:
		ShapeSet_apply_modif(select->selected_bone->shape, select->bone_copy, data, center, modif);
		break;
		
		case SHAPEEDIT_SELECT_ELEM:
		Shape_apply_modif((*(select->selected_bone->shape->elems + select->selected_elem_id))->shape, select->elem_copy->shape, data, center, modif);
		break;
		
		default:
		break;
	}
}

static void Shape_apply_double_modif(Shape shape, Shape copy, double first, double second, SDL_Rect center, void (*modif)(Shape, Shape, double, double, SDL_Rect))
{
	if(shape == NULL || copy == NULL)
		return;
	(*modif)(shape, copy, first, second, center);
	Shape_apply_double_modif(shape->child, copy->child, first, second, center, modif);
	Shape_apply_double_modif(shape->sister, copy->sister, first, second, center, modif);
}

static void ShapeSet_apply_double_modif(ShapeSet shape, ShapeSet copy, double first, double second, SDL_Rect center, void (*modif)(Shape, Shape, double, double, SDL_Rect))
{
	int i;
	if(shape == NULL || copy == NULL)
		return;
	for(i = 0; i < shape->nb_elems && i < copy->nb_elems; ++i)
		Shape_apply_double_modif((*(shape->elems + i))->shape, (*(copy->elems + i))->shape, first, second, center, modif);
}

static void ShapeEdit_Select_apply_modif_double(ShapeEdit_Select select, double first, double second, SDL_Rect center, void (*modif)(Shape, Shape, double, double, SDL_Rect))
{
	switch(select->select_type)
	{
		case SHAPEEDIT_SELECT_BONE:
		ShapeSet_apply_double_modif(select->selected_bone->shape, select->bone_copy, first, second, center, modif);
		break;
		
		case SHAPEEDIT_SELECT_ELEM:
		Shape_apply_double_modif((*(select->selected_bone->shape->elems + select->selected_elem_id))->shape, select->elem_copy->shape, first, second, center, modif);
		break;
		
		default:
		break;
	}
}

int ShapeEdit_Select_mouse_move_selection(ShapeEdit_Select select, int x, int y)
{
	int vx, vy;
	int retour = 0;
	double angle, scale;
	
	if(select == NULL)
		return 0;
	vx = x - select->last_mouse.x;
	vy = y - select->last_mouse.y;
	if(vx == 0 && vy == 0)
		return 0;
		
	if(select->action == SHAPEEDIT_SELECT_ACTION_NULL)
		goto end_move;
	if(select->action == SHAPEEDIT_SELECT_ACTION_TRANSLATION)
	{
		switch(select->select_type)
		{
			case SHAPEEDIT_SELECT_SHAPE:
			retour = Shape_add_center(select->selected_shape, vx, vy);
			break;
		
			case SHAPEEDIT_SELECT_ELEM:
			retour = ShapeElem_add_center(*(select->selected_bone->shape->elems + select->selected_elem_id), vx, vy);
			break;
		
			case SHAPEEDIT_SELECT_BONE:
			retour = ShapeSet_add_center(select->selected_bone->shape, vx, vy);
			break;
		
			default:
			break;
		}
		goto end_move;
	}
	ShapeSet_absolute_coords(select->selected_bone, *(select->arma), &vx, &vy);
	angle = correct_rad_angle(angle_segment(select->action_center.x, select->action_center.y, x - vx, y - vy) - select->start_angle);
	scale = sqrt((double)distance_carre(select->action_center.x, select->action_center.y, x - vx, y - vy)) / ((double)(select->start_distance));
	if(select->action & SHAPEEDIT_SELECT_ACTION_ROTATE)
		ShapeEdit_Select_apply_modif(select, angle, select->action_center, Shape_modif_rotate);
	if(select->action & SHAPEEDIT_SELECT_ACTION_SCALE)
	{
		if(select->action & SHAPEEDIT_SELECT_ACTION_ROTATE)
			ShapeEdit_Select_apply_modif_double(select, scale, angle, select->action_center, Shape_modif_scale_rotate);
		else
			ShapeEdit_Select_apply_modif(select, scale, select->action_center, Shape_modif_scale);
	}
	retour = 1;
	
	end_move:;
	select->last_mouse.x = x;
	select->last_mouse.y = y;
	return retour;
}

int ShapeEdit_Select_wheel_selection(ShapeEdit_Select select, int add)
{
	if(select == NULL)
		return 0;
	if(select->action > SHAPEEDIT_SELECT_ACTION_TRANSLATION)
		return 0;
	switch(select->select_type)
	{
		case SHAPEEDIT_SELECT_SHAPE:
		return Shape_add_radius(select->selected_shape, add);
		
		case SHAPEEDIT_SELECT_ELEM:
		return ShapeElem_add_radius(*(select->selected_bone->shape->elems + select->selected_elem_id), add);
		
		case SHAPEEDIT_SELECT_BONE:
		return ShapeSet_add_radius(select->selected_bone->shape, add);
		
		default:
		break;
	}
	return 0;
}

SDL_Rect ShapeEdit_Select_average_center(ShapeEdit_Select select)
{
	SDL_Rect res;
	res.x = res.y = 0;
	if(select == NULL)
		return res;
	switch(select->select_type)
	{
		case SHAPEEDIT_SELECT_BONE:
		return ShapeSet_average_center(select->selected_bone->shape);
		
		case SHAPEEDIT_SELECT_ELEM:
		return ShapeElem_average_center(*(select->selected_bone->shape->elems + select->selected_elem_id));
		
		case SHAPEEDIT_SELECT_SHAPE:
		res.x = select->selected_bone->start.x + select->selected_bone->shape->center.x + select->selected_shape->center.x;
		res.y = select->selected_bone->start.y + select->selected_bone->shape->center.y + select->selected_shape->center.y;
		return res;
		
		default:
		break;
	}
	return res;
}