#ifndef DEF_HEADER_SHAPE_EDITOR
#define DEF_HEADER_SHAPE_EDITOR

#include "armature.h"
#include "shape.h"
#include "edit.h"
#include "color.h"
#include "shape_list.h"
#include "file_explorer.h"
#include "mini_menu.h"
#include "personnage.h"
#include "file_saver.h"

#define SHAPE_SELECTION_RANGE 25
#define COLOR_EDIT_SIZE 250
#define SHAPE_LIST_WIDTH 100

#define LAST_SHAPE_LIST "media/last_shape_list.shape_list"

typedef enum
{
	SHAPEEDIT_SELECT_NULL,
	SHAPEEDIT_SELECT_BONE,
	SHAPEEDIT_SELECT_ELEM,
	SHAPEEDIT_SELECT_SHAPE,
	NB_SHAPEEDIT_SELECT
} ShapeEdit_Select_Type;

typedef enum
{
	SHAPEEDIT_SELECT_ACTION_NULL = 0,
	SHAPEEDIT_SELECT_ACTION_TRANSLATION = 0x1,
	SHAPEEDIT_SELECT_ACTION_ROTATE = 0x2,
	SHAPEEDIT_SELECT_ACTION_SCALE = 0x4,
	NB_SHAPEEDIT_SELECT_ACTION
	/* Possible to combine rotate with scale */
} ShapeEdit_Select_Action;

typedef struct _ShapeEdit_Select * ShapeEdit_Select;
struct _ShapeEdit_Select
{
	Bone * arma;
	Bone selected_bone;
	int selected_elem_id;
	Shape selected_shape;
	ShapeEdit_Select_Type select_type;
	
	SDL_Color last_color;

	ShapeEdit_Select_Action action;
	SDL_Rect last_mouse;
	SDL_Rect mouse;
	SDL_Rect mouse_start;
	
	SDL_Rect action_center;
	union
	{
		ShapeSet bone_copy;
		ShapeElem elem_copy;
		Shape shape_copy;
	};
	double start_angle;
	double start_distance;
	int fusion_mode;
	int change_parent;
	
	Bone unset_shape;/* shape set without a defined bone, can be a duplicated shape or an imported shape */
	
	Bone saved_selected_bone;
	int saved_selected_elem_id;
	Shape saved_selected_shape;
	ShapeEdit_Select_Type saved_select_type;
	
	ShapeList shape_list;
	
	Mini_Menu shape_list_menu;
	Mini_Menu shape_menu;
	File_Explorer file_exp;
};

typedef struct _ShapeEdit ShapeEdit;
struct _ShapeEdit
{
	ShapeEdit_Select select;
};

int Shape_edit(Personnage * perso, ShapeEdit * data, EditExit * exit_choice, File_Explorer file_exp, FileSaver * file_saver);

/* Global data */

int ShapeEdit_create(ShapeEdit * data, Bone * arma);

void ShapeEdit_destroy(ShapeEdit * data);

/* Selection */

int ShapeEdit_Select_create(ShapeEdit_Select * select, Bone * arma);

void ShapeEdit_Select_free(ShapeEdit_Select * select);

void ShapeEdit_Select_draw(ShapeEdit_Select select);

int ShapeEdit_Select_make_selection(ShapeEdit_Select select, int x, int y, int range);

int ShapeEdit_Select_cancel_selection(ShapeEdit_Select select);

int ShapeEdit_Select_is_selection(ShapeEdit_Select select);

/* Edition */

int ShapeEdit_Select_append(ShapeEdit_Select select);

int ShapeEdit_Select_duplicate(ShapeEdit_Select select);

int ShapeEdit_Select_delete(ShapeEdit_Select select);

int ShapeEdit_Select_color_edit(ShapeEdit_Select select);

int ShapeEdit_Select_elem_goto_back(ShapeEdit_Select select);

int ShapeEdit_Select_elem_goto_front(ShapeEdit_Select select);

int ShapeEdit_Select_copy_selection(ShapeEdit_Select select);

void ShapeEdit_Select_free_copy_selection(ShapeEdit_Select select);

int ShapeSet_absolute_coords(Bone bone, Bone arma, int * x, int * y);

void ShapeEdit_Select_fusion_to_selected(ShapeEdit_Select select);

void ShapeEdit_Select_change_parent(ShapeEdit_Select select);

void ShapeEdit_Select_save_selection(ShapeEdit_Select select);

void ShapeEdit_Select_delete_saved_selection(ShapeEdit_Select select);

int ShapeEdit_Select_add_selection_to_shape_list(ShapeEdit_Select select);

int ShapeEdit_Select_add_shape_list_to_selection(ShapeEdit_Select select);

/* MouseMove */

int ShapeEdit_Select_start_mouse_move(ShapeEdit_Select select, int x, int y, ShapeEdit_Select_Action move_mode);

int ShapeEdit_Select_end_mouse_move(ShapeEdit_Select select);

int ShapeEdit_Select_is_mouse_move(ShapeEdit_Select select);

int ShapeEdit_Select_set_mouse_motion(ShapeEdit_Select select, int x, int y);

int ShapeEdit_Select_mouse_move_selection(ShapeEdit_Select select, int x, int y);

int ShapeEdit_Select_wheel_selection(ShapeEdit_Select select, int add);

SDL_Rect ShapeEdit_Select_average_center(ShapeEdit_Select select);

#endif