#include "shape_list.h"
#include "draw.h"
#include "image.h"

int ShapeList_create(ShapeList * list)
{
	ShapeList res = NULL;
	if((res = (ShapeList)malloc(sizeof(struct _ShapeList))) == NULL)
		return 0;
	res->tab = NULL;
	res->nb_elems = 0;
	res->maxi_elems = 0;
	res->origin.x = res->origin.y = 0;
	res->size.w = res->size.h = 0;
	res->visible = 0;
	res->in_add_box = 0;
	res->background = NULL;
	res->select_shape = -1;
	res->scroll = 0;
	
	*list = res;
	return 1;
}

void ShapeList_free(ShapeList * list)
{
	int i;
	if(list == NULL || *list == NULL)
		return;
	if((*list)->tab != NULL)
	{
		for(i = 0; i < (*list)->nb_elems; ++i)
			ShapeSet_free((*list)->tab + i);
		free((*list)->tab);
	}
	if((*list)->background != NULL)
		SDL_FreeSurface((*list)->background);
	free(*list);
	*list = NULL;
}

int ShapeList_set_origin(ShapeList list, int x, int y)
{
	if(list == NULL)
		return 0;
	if(list->origin.x == x && list->origin.y == y)
		return 0;
	list->origin.x = x;
	list->origin.y = y;
	return 1;
}

int ShapeList_set_size(ShapeList list, int w, int h)
{
	if(list == NULL)
		return 0;
	list->size.w = w;
	list->size.h = h;
	return 1;
}

int ShapeList_set_background(ShapeList list, SDL_Surface * background)
{
	if(list == NULL)
		return 0;
	if(background == NULL)
		return 0;
	if(list->background != NULL)
		SDL_FreeSurface(list->background);
	list->background = background;
	return 1;
}

int ShapeList_set_background_sizedRGB(ShapeList list, int r, int g, int b)
{
	SDL_Surface * surface = NULL;
	if(list == NULL)
		return 0;
	if(! SDL_GetRectangleRGB(&surface, list->size.w, list->size.h, r, g, b))
		return 0;
	return ShapeList_set_background(list, surface);
}

static int ShapeList_set_new_max(ShapeList list, int new_size)
{
	int new_maxi = -1;
	ShapeSet * tmp = NULL;
	if(list == NULL)
		return 0;
		
	if(new_size >= list->maxi_elems)
		new_maxi = (list->maxi_elems * 2) + 1;
	if(new_size < (list->maxi_elems / 4) - 1)
		new_maxi = (list->maxi_elems / 2);
	if(new_maxi == -1)
		return 1;
	if((tmp = (ShapeSet *)realloc(list->tab, new_maxi * sizeof(ShapeSet))) == NULL)
		return 0;
	list->tab = tmp;
	list->maxi_elems = new_maxi;
	return 1;
}

int ShapeList_append(ShapeList list, ShapeSet shape)
{
	if(list == NULL)
		return 0;
	if(ShapeSet_is_void(shape))
		return 0;
	if(! ShapeList_set_new_max(list, list->nb_elems + 1))
		return 0;
	if(! ShapeSet_copy(list->tab + list->nb_elems, shape))
		return 0;
	ShapeSet_resize_in(*(list->tab + list->nb_elems), 0, 0, list->size.w, list->size.h);
	ShapeSet_set_center_as_average(*(list->tab + list->nb_elems));
	++(list->nb_elems);
	if((list->nb_elems * list->size.h) >= get_screen()->h)
		list->scroll = (list->nb_elems * list->size.h) + (list->size.h / 2) - get_screen()->h;
	list->visible = 1;
	return 1;
}

int ShapeList_delete(ShapeList list, int id)
{
	if(list == NULL)
		return 0;
	if(id < 0 || id >= list->nb_elems)
		return 0;
	ShapeSet_free(list->tab + id);
	for(; id < list->nb_elems - 1; ++id)
		*(list->tab + id) = *(list->tab + id + 1);
	--(list->nb_elems);
	if(! ShapeList_set_new_max(list, list->nb_elems))
		return 0;
	return 1;
}

int ShapeList_clean(ShapeList list)
{
	int i;
	if(list == NULL)
		return 0;
	for(i = 0; i < list->nb_elems; ++i)
		ShapeSet_free(list->tab + i);
	free(list->tab);
	list->tab = NULL;
	list->nb_elems = 0;
	list->maxi_elems = 0;
	return 1;
}

int ShapeList_get_id_by_coords(ShapeList list, int x, int y)
{
	if(list == NULL)
		return -1;
	if(! list->visible)
		return -1;
	if(! ShapeList_in(list, x, y))
		return -1;
	return (y + list->scroll - list->origin.y - (list->size.h / 2)) / (list->size.h);
}

int ShapeList_in(ShapeList list, int x, int y)
{
	if(! list->visible)
		return list->in_add_box;
	return in_area(x, y, list->origin.x, list->origin.y + (list->size.h / 2), list->origin.x + list->size.w, list->origin.y + ((list->nb_elems) * list->size.h) + (list->size.h / 2));
}

int ShapeList_draw(ShapeList list)
{
	int x, y;
	int x_min, x_max, y_min, y_max;
	int i;
	SDL_Rect back_pos;
	if(! list->visible)
	{
		back_pos = list->origin;
		back_pos.y -= (list->size.h / 2) * ! (list->in_add_box);
		if(list->background != NULL)
			SDL_BlitSurface(list->background, NULL, get_screen(), &back_pos);
		return 1;
	}
	for(i = 0; i < list->nb_elems; ++i)
	{
		if(! ShapeSet_get_dimensions(*(list->tab + i), &x_min, &y_min, &x_max, &y_max))
			return 0;
		back_pos = list->origin;
		back_pos.y += (list->size.h * i) + (list->size.h / 2) - list->scroll;
		x = -(*(list->tab + i))->center.x - ((x_max + x_min) / 2) + (list->size.w / 2) + back_pos.x;
		y = -(*(list->tab + i))->center.y - ((y_max + y_min) / 2) + (list->size.h / 2) + back_pos.y;
		if(list->background != NULL)
			SDL_BlitSurface(list->background, NULL, get_screen(), &back_pos);
		draw_shape_set(*(list->tab + i), x, y);
	}
	back_pos = list->origin;
	back_pos.y -= (list->size.h / 2);
	if(list->background != NULL)
		SDL_BlitSurface(list->background, NULL, get_screen(), &back_pos);
	return 1;
}

int ShapeList_event(ShapeList list, SDL_Event * event, ShapeList_Action * get_action)
{
	int need_refresh = 0;
	if(get_action != NULL)
		*get_action = SHAPELIST_ACTION_NULL;
	switch(event->type)
	{
		case SDL_MOUSEBUTTONDOWN:
		switch(event->button.button)
		{
			case SDL_BUTTON_WHEELDOWN:
			if(! in_area(event->button.x, event->button.y, list->origin.x, list->origin.y + (list->size.h / 2), list->origin.x + list->size.w, get_screen()->h))
				break;
			if(! list->visible)
				break;
			if(get_action != NULL)
				*get_action = SHAPELIST_ACTION_SCROLL;
			(list->scroll) -= SL_SCROLL_ADD;
			if(list->scroll < 0)
				list->scroll = 0;
			need_refresh = 1;
			break;
			
			case SDL_BUTTON_WHEELUP:
			if(! in_area(event->button.x, event->button.y, list->origin.x, list->origin.y + (list->size.h / 2), list->origin.x + list->size.w, get_screen()->h))
				break;
			if(! list->visible)
				break;
			if(get_action != NULL)
				*get_action = SHAPELIST_ACTION_SCROLL;
			if((list->nb_elems * list->size.h) < get_screen()->h)
				break;
			(list->scroll) += SL_SCROLL_ADD;
			if((list->nb_elems * list->size.h) - list->scroll + (list->size.h / 2) < get_screen()->h)
				list->scroll = (list->nb_elems * list->size.h) + (list->size.h / 2) - get_screen()->h;
			need_refresh = 1;
			break;
			
			default:
			break;
		}
		break;
		
		case SDL_MOUSEBUTTONUP:
		switch(event->button.button)
		{
			case SDL_BUTTON_LEFT:
			if(in_area(event->button.x, event->button.y, list->origin.x, list->origin.y, list->origin.x + list->size.w, list->origin.y + list->size.h))
			{
				if(! list->visible)
				{
					if(get_action != NULL)
						*get_action = SHAPELIST_ACTION_APPEND;
					need_refresh = 1;
				}
				if(list->nb_elems > 0 || list->visible)
				{
					list->visible = 1 - list->visible;
					need_refresh = 1;
				}
			}
			else if(ShapeList_in(list, event->button.x, event->button.y))
			{
				list->select_shape = ShapeList_get_id_by_coords(list, event->button.x, event->button.y);
				need_refresh = 1;
				if(get_action != NULL)
					*get_action = SHAPELIST_ACTION_GET;
			}
			break;
		}
		break;
		
		case SDL_MOUSEMOTION:
		if(in_area(event->motion.x, event->motion.y, list->origin.x, list->origin.y, list->origin.x + list->size.w, list->origin.y + list->size.h))
		{
			if(! list->in_add_box)
			{
				list->in_add_box = 1;
				need_refresh = 1;
			}
		}
		else
		{
			if(list->in_add_box)
			{
				list->in_add_box = 0;
				need_refresh = 1;
			}
		}
		break;
		
		default:
		break;
	}
	return need_refresh;
}

int ShapeList_get_join(ShapeList list, ShapeSet dest)
{
	if(list == NULL)
		return 0;
	if(list->select_shape < 0 || list->select_shape >= list->nb_elems)
		return 0;
	if(! ShapeSet_join(dest, *(list->tab + list->select_shape)))
		return 0;
	list->select_shape = -1;
	return 1;
}

int ShapeList_save(FILE * ecr, ShapeList list)
{
	unsigned char type = 'l';
	int i;
	if(list == NULL)
		return 0;
	if(! fwrite(&type, sizeof(unsigned char), 1, ecr)
	|| ! fwrite(&(list->origin), sizeof(SDL_Rect), 1, ecr)
	|| ! fwrite(&(list->size), sizeof(SDL_Rect), 1, ecr)
	|| ! fwrite(&(list->nb_elems), sizeof(int), 1, ecr)
	|| ! fwrite(&(list->maxi_elems), sizeof(int), 1, ecr))
		return 0;
	for(i = 0; i < list->nb_elems; ++i)
	{
		if(! ShapeSet_save(ecr, *(list->tab + i)))
			return 0;
	}
	return 1;
}

int ShapeList_load(FILE * lec, ShapeList list)
{
	unsigned char type;
	int i = 0;
	if(list == NULL)
		return 0;
	if(! fread(&type, sizeof(unsigned char), 1, lec))
		return 0;
	if(type != 'l')
		return 0;
	if(! fread(&(list->origin), sizeof(SDL_Rect), 1, lec)
	|| ! fread(&(list->size), sizeof(SDL_Rect), 1, lec)
	|| ! fread(&(list->nb_elems), sizeof(int), 1, lec)
	|| ! fread(&(list->maxi_elems), sizeof(int), 1, lec))
		return 0;
	if(list->tab != NULL)
	{
		free(list->tab);
		list->tab = NULL;
	}
	if(list->maxi_elems > 0 && (list->tab = (ShapeSet *)calloc(list->maxi_elems, sizeof(ShapeSet))) == NULL)
		goto free_zone;
	for(i = 0; i < list->nb_elems; ++i)
	{
		if(! ShapeSet_load(lec, list->tab + i))
			goto free_zone;
	}
	return 1;
	
	free_zone:;
	
	if(list->tab != NULL)
	{
		for(--i; i >= 0; --i)
			ShapeSet_free(list->tab + i);
		free(list->tab);
		list->tab = NULL;
	}
	list->maxi_elems = 0;
	list->nb_elems = 0;
	
	return 0;
}

int ShapeList_save_in_file(const char * path, ShapeList list)
{
	FILE * ecr = NULL;
	int retour;
	if((ecr = fopen(path, "w+")) == NULL)
		return 0;
	retour = ShapeList_save(ecr, list);
	fclose(ecr);
	return retour;
}

int ShapeList_load_from_file(const char * path, ShapeList list)
{
	FILE * lec = NULL;
	int retour;
	if((lec = fopen(path, "r")) == NULL)
		return 0;
	retour = ShapeList_load(lec, list);
	fclose(lec);
	return retour;
}