#include "text_input.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "color.h"
#include "image.h"

SDL_Surface * cursor = NULL;

static int Text_Input_verif_alloc(Text_Input input)
{
	int new_max;
	char * tmp = NULL;
	if(input == NULL)
		return 0;
	if(input->size >= input->max_size || input->size < input->max_size - (2 * TEXT_ALLOC_BLOC))
	{
		if(input->size >= input->max_size)
			new_max = input->max_size + TEXT_ALLOC_BLOC;
		else
			new_max = input->max_size - TEXT_ALLOC_BLOC;
		if((tmp = (char *)realloc(input->text, new_max * sizeof(char))) == NULL)
			return 0;
		input->text = tmp;
		if(input->max_size == 0)
		{
			*(input->text) = '\0';
			input->size = 1;
		}
		input->max_size = new_max;
	}
	return 1;
}

int Text_Input_create(Text_Input * input)
{
	Text_Input res = NULL;
	
	if((res = (Text_Input)malloc(sizeof(struct _Text_Input))) == NULL)
		return 0;
	
	res->text = NULL;
	res->max_size = 0;
	res->size = 0;
	res->cursor = 0;
	
	if(! Text_Input_verif_alloc(res))
	{
		free(res);
		return 0;
	}
	
	*input = res;
	
	return 1;
}

void Text_Input_free(Text_Input * input)
{
	if(input == NULL || *input == NULL)
		return;
	if((*input)->text != NULL)
		free((*input)->text);
	free(*input);
	input = NULL;
}

int Text_Input_add_char(Text_Input input, char car)
{
	int i;
	if(car == '\0')
		return 0;
	if(! Text_Input_verif_alloc(input))
		return 0;
	for(i = input->size; i > input->cursor; --i)
		*(input->text + i) = *(input->text + (i - 1));
	*(input->text + input->cursor) = car;
	++(input->size);
	++(input->cursor);
	return 1;
}

int Text_Input_del_char_after(Text_Input input)
{
	int i;
	if(input == NULL)
		return 0;
	if(! Text_Input_verif_alloc(input))
		return 0;
	if(input->cursor >= input->size - 1)
		return 0;
	for(i = input->cursor; i < input->size - 1; ++i)
		*(input->text + i) = *(input->text + i + 1);
	--(input->size);
	return 1;
}

int Text_Input_del_char_before(Text_Input input)
{
	int i;
	if(input == NULL)
		return 0;
	if(! Text_Input_verif_alloc(input))
		return 0;
	if(input->cursor <= 0)
		return 0;
	for(i = input->cursor - 1; i < input->size - 1; ++i)
		*(input->text + i) = *(input->text + i + 1);
	--(input->size);
	--(input->cursor);
	return 1;
}

int Text_Input_set_cursor_at_end(Text_Input input)
{
	if(input == NULL)
		return 0;
	(input->cursor) = input->size - 1;
	return 1;
}

int Text_Input_set_cursor_at_start(Text_Input input)
{
	if(input == NULL)
		return 0;
	(input->cursor) = 0;
	return 1;
}

int Text_Input_set_cursor(Text_Input input, int new_cursor)
{
	if(input == NULL)
		return 0;
	if(new_cursor < 0)
		new_cursor = 0;
	if(new_cursor >= input->size)
		new_cursor = input->size - 1;
	(input->cursor) = new_cursor;
	return 1;
}

int Text_Input_move_cursor(Text_Input input, int move)
{
	if(input == NULL)
		return 0;
	return Text_Input_set_cursor(input, input->cursor + move);
}

int Text_Input_redefine(Text_Input input, const char * text)
{
	char * copy_alloc = NULL;
	int taille;
	int max_taille;
	if(input == NULL)
		return 0;
	#ifdef DEBUG
	printf("redef: \"%s\"\n", text);
	#endif
	taille = strlen(text) + 1;
	max_taille = ((taille / TEXT_ALLOC_BLOC) + 1) * TEXT_ALLOC_BLOC;
	if((copy_alloc = (char *)malloc(max_taille * sizeof(char))) == NULL)
		return 0;
	strcpy(copy_alloc, text);
	free(input->text);
	input->text = copy_alloc;
	input->size = taille;
	input->max_size = max_taille;
	Text_Input_set_cursor_at_end(input);
	return 1;
}

int Text_Input_concat_text(Text_Input input, const char * text)
{
	char * copy_alloc = NULL;
	int taille;
	int max_taille;
	if(input == NULL)
		return 0;
	taille = input->size + strlen(text);
	max_taille = ((taille / TEXT_ALLOC_BLOC) + 1) * TEXT_ALLOC_BLOC;
	if((copy_alloc = (char *)malloc(max_taille * sizeof(char))) == NULL)
		return 0;
	strcpy(copy_alloc, input->text);
	strcat(copy_alloc, text);
	free(input->text);
	input->text = copy_alloc;
	input->size = taille;
	input->max_size = max_taille;
	Text_Input_set_cursor_at_end(input);
	return 1;
}

int Text_Input_event(Text_Input input, SDL_Event * event, int * exit_input)
{
	int char_get;
	if(input == NULL || event == NULL)
		return 0;
	switch(event->type)
	{
		case SDL_KEYDOWN:
		switch(event->key.keysym.sym)
		{
			case SDLK_LEFT:
			return Text_Input_move_cursor(input, -1);
			
			case SDLK_RIGHT:
			return Text_Input_move_cursor(input, 1);
			
			case SDLK_RETURN:
			if(exit_input != NULL)
				*exit_input = 1;
			return 1;
			
			case SDLK_DELETE:
			return Text_Input_del_char_after(input);
			
			case SDLK_BACKSPACE:
			return Text_Input_del_char_before(input);
			
			case SDLK_ESCAPE:
			if(exit_input != NULL)
				*exit_input = 1;
			return 0;
			
			default:
			break;
		}
		
		char_get = event->key.keysym.unicode;
		if(char_get <= 0xFF)
			return Text_Input_add_char(input, char_get);
		break;
	}
	return 0;
}

int Text_Input_get_text(Text_Input input, char ** text)
{
	if(input == NULL || text == NULL)
		return 0;
	if(input->text == NULL)
		return 0;
	*text = input->text;
	return 1;
}

int Text_Input_get_allocated_text(Text_Input input, char ** text)
{
	char * res = NULL;
	if(input == NULL || text == NULL)
		return 0;
	if(input->text == NULL)
		return 0;
	if((res = (char *)malloc((strlen(input->text) + 1) * sizeof(char))) == NULL)
		return 0;
	strcpy(res, input->text);
	*text = res;
	return 1;
}

int Text_Input_test()
{
	Text_Input input = NULL;
	char * chaine = NULL;
	int exit_input = 0;
	int i;
	SDL_Event event;
	if(! Text_Input_create(&input))
		return 0;
	while(! exit_input)
	{
		SDL_WaitEvent(&event);
		if(Text_Input_event(input, &event, &exit_input))
		{
			Text_Input_get_text(input, &chaine);
			printf("%s\n", chaine);
			for(i = 0; i < input->cursor; ++i)
				putchar(' ');
			printf("|\n\n");
		}
	}
	Text_Input_free(&input);
	return 1;
}

/* Input graphic */

int Input_surface_calcul(Input input)
{
	char * chaine = NULL;
	SDL_Surface * new_surface = NULL;
	
	if(input == NULL)
		return 0;
	if(input->font == NULL)
		return 0;
	if(! Text_Input_get_text(input->text, &chaine))
		return 0;
	if(strlen(chaine) == 0)
		new_surface = NULL;
	else
	{
		if((new_surface = TTF_RenderUTF8_Blended(input->font, chaine, input->color)) == NULL)
			return 0;
	}
	if(input->surface != NULL)
		SDL_FreeSurface(input->surface);
	input->surface = new_surface;
	if(TTF_SizeText(input->font, chaine + input->text->cursor, &(input->cursor_pos), NULL))
		return 0;
	input->cursor_pos = -input->cursor_pos;
	if(input->surface != NULL)
		(input->cursor_pos) += input->surface->w;
	return 1;
}

int Input_create(Input * input)
{
	Input res = NULL;
	
	if((res = (Input)malloc(sizeof(struct _Input))) == NULL)
		return 0;
	res->text = NULL;
	res->origin.x = res->origin.y = 0;
	res->surface = NULL;
	res->box_background = NULL;
	res->is_select = 0;
	res->font = NULL;
	res->color = SDL_Color_create(0, 0, 0);
	res->cursor_pos = 0;
	
	if(! Text_Input_create(&(res->text)))
	{
		Input_free(&res);
		return 0;
	}
	
	if(cursor == NULL)
	{
		if(! SDL_GetRectangleRGB(&cursor, 1, 5, 216, 216, 216))
			return 0;
	}
	
	*input = res;
	
	return 1;
}

void Input_free(Input * input)
{
	if(input == NULL || *input == NULL)
		return;
	Text_Input_free(&((*input)->text));
	if((*input)->surface != NULL)
		SDL_FreeSurface((*input)->surface);
	if((*input)->box_background != NULL)
		SDL_FreeSurface((*input)->box_background);
	if((*input)->font != NULL)
		TTF_CloseFont((*input)->font);
	free(*input);
	*input = NULL;
	
	if(cursor != NULL)
	{
		SDL_FreeSurface(cursor);
		cursor = NULL;
	}	
}

int Input_set_select(Input input, int is_select)
{
	if(input == NULL)
		return 0;
	if(! (input->is_select) == ! is_select)
		return 0;
	input->is_select = is_select;
	return 1;
}

int Input_set_origin(Input input, int x, int y)
{
	if(input == NULL)
		return 0;
	if(input->origin.x == x && input->origin.y == y)
		return 0;
	input->origin.x = x;
	input->origin.y = y;
	return 1;
}

int Input_set_font(Input input, const char * path, int size, SDL_Color color)
{
	TTF_Font * font = NULL;
	if(input == NULL)
		return 0;
	if(path == NULL || size <= 0)
		return 0;
	if((font = TTF_OpenFont(path, size)) == NULL)
		return 0;
	input->color = color;
	if(input->font != NULL)
		TTF_CloseFont(input->font);
	input->font = font;
	return 1;
}

int Input_set_box(Input input, SDL_Surface * background)
{
	SDL_Surface * new_back = NULL;
	if(input == NULL)
		return 0;
	if(! SDL_CopySurface(&new_back, background))
		return 0;
	if(input->box_background != NULL)
		SDL_FreeSurface(input->box_background);
	input->box_background = new_back;
	return 1;
}

int Input_redefine(Input input, const char * text)
{
	if(input == NULL)
		return 0;
	if(! Text_Input_redefine(input->text, text))
		return 0;
	if(! Input_surface_calcul(input))
		return 0;
	return 1;
}

int Input_concat_text(Input input, const char * text)
{
	if(input == NULL)
		return 0;
	if(! Text_Input_concat_text(input->text, text))
		return 0;
	if(! Input_surface_calcul(input))
		return 0;
	return 1;
}

int Input_event(Input input, SDL_Event * event, int * exit_input)
{
	int exit_input_local = 0;
	if(exit_input == NULL)
		exit_input = &exit_input_local;
	if(input == NULL)
		return 0;
	switch(event->type)
	{
		case SDL_MOUSEBUTTONUP:
		switch(event->button.button)
		{
			case SDL_BUTTON_LEFT:
			if(input->box_background == NULL)
				break;
			if(input->is_select)
			{
				if(! in_surface_area(event->button.x, event->button.y, input->origin, input->box_background))
				{
					if(exit_input != NULL)
						*exit_input = 1;
					return Input_set_select(input, INPUT_UNSELECT);
				}
			}
			else
			{
				if(in_surface_area(event->button.x, event->button.y, input->origin, input->box_background))
					return Input_set_select(input, INPUT_SELECT);
			}
			break;
		}
		break;
	}
	if(! input->is_select)
		return 0;
	if(! Text_Input_event(input->text, event, exit_input))
		return 0;
	if(*exit_input)
		input->is_select = 0;
	if(! Input_surface_calcul(input))
		return 0;
	
	return 1;
}

int Input_get_text(Input input, char ** text)
{
	if(input == NULL)
		return 0;
	return Text_Input_get_text(input->text, text);
}

int Input_get_allocated_text(Input input, char ** text)
{
	if(input == NULL)
		return 0;
	return Text_Input_get_allocated_text(input->text, text);
}

void Input_draw(Input input, SDL_Surface * screen)
{
	if(input == NULL || screen == NULL)
		return;
	if(input->box_background != NULL)
		SDL_BlitSurface(input->box_background, NULL, screen, &(input->origin));
	if(input->surface != NULL)
		SDL_BlitSurface(input->surface, NULL, screen, &(input->origin));
	if(cursor != NULL && input->is_select)
	{
		SDL_Rect cur_pos;
		cur_pos.x = input->origin.x + input->cursor_pos;
		cur_pos.y = input->origin.y - (cursor->h / 2);
		if(input->surface != NULL)
			cur_pos.y += input->surface->h;
		SDL_BlitSurface(cursor, NULL, screen, &cur_pos);
	}
}

int Input_test(SDL_Surface * screen)
{
	Input input = NULL;
	int exit_input = 0;
	int need_refresh = 1;
	SDL_Event event;
	
	if(! Input_create(&input))
		return 0;
	Input_set_select(input, INPUT_SELECT);
	if(! Input_set_origin(input, 100, 25)
	|| ! Input_set_font(input, "media/arial.ttf", 25, SDL_Color_create(216, 216, 216)))
	{
		Input_free(&input);
		return 0;
	}
	while(! exit_input)
	{
		if(need_refresh)
		{
			SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 0, 0, 0));
			Input_draw(input, screen);
			SDL_Flip(screen);
			need_refresh = 0;
		}
		SDL_WaitEvent(&event);
		need_refresh |= Input_event(input, &event, &exit_input);
	}
	Input_free(&input);
	return 1;
}