#ifndef DEF_HEADER_TEXT_INPUT
#define DEF_HEADER_TEXT_INPUT

#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

#define TEXT_ALLOC_BLOC 10

typedef struct _Text_Input * Text_Input;
struct _Text_Input
{
	char * text;
	int max_size;/* allocated memory to text */
	int size;/* to not make a strlen each time we want to know text length */
	int cursor;/* id of the current editing caracter */
};

int Text_Input_create(Text_Input * input);

void Text_Input_free(Text_Input * input);

int Text_Input_add_char(Text_Input input, char car);

int Text_Input_del_char_after(Text_Input input);

int Text_Input_del_char_before(Text_Input input);

int Text_Input_set_cursor_at_end(Text_Input input);

int Text_Input_set_cursor_at_start(Text_Input input);

int Text_Input_set_cursor(Text_Input input, int new_cursor);

int Text_Input_move_cursor(Text_Input input, int move);

int Text_Input_redefine(Text_Input input, const char * text);

int Text_Input_concat_text(Text_Input input, const char * text);

int Text_Input_event(Text_Input input, SDL_Event * event, int * exit_input);

int Text_Input_get_text(Text_Input input, char ** text);

int Text_Input_get_allocated_text(Text_Input input, char ** text);

int Text_Input_test();

/* Input graphic */

enum
{
	INPUT_UNSELECT = 0,
	INPUT_SELECT = 1
};

typedef struct _Input * Input;
struct _Input
{
	Text_Input text;
	SDL_Rect origin;
	SDL_Surface * surface;
	SDL_Surface * box_background;
	int is_select;
	int cursor_pos;
	TTF_Font * font;
	SDL_Color color;
};

int Input_create(Input * input);

void Input_free(Input * input);

int Input_set_select(Input input, int is_select);

int Input_set_origin(Input input, int x, int y);

int Input_set_font(Input input, const char * path, int size, SDL_Color color);

int Input_set_box(Input input, SDL_Surface * background);

int Input_redefine(Input input, const char * text);

int Input_concat_text(Input input, const char * text);

int Input_event(Input input, SDL_Event * event, int * exit_input);

int Input_get_text(Input input, char ** text);

int Input_get_allocated_text(Input input, char ** text);

void Input_draw(Input input, SDL_Surface * screen);

int Input_test(SDL_Surface * screen);

int Input_surface_calcul(Input input);

#endif