#include "window.h"

#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

Window window = {NULL, 0, 0};

int Window_Init(int width, int height, const char * caption)
{
	if(get_screen() != NULL)
		Window_Quit();
	if(SDL_Init(SDL_INIT_VIDEO) != 0)
	{
		fprintf(stderr, "SDL error when window was initialized: %s\n", SDL_GetError());
		return 0;
	}
	if(TTF_Init() == -1)
	{
		fprintf(stderr, "SDL_ttf error when window was intialized: %s\n", TTF_GetError());
		return 0;
	}
	if((window.screen = SDL_SetVideoMode(window.width = width, window.height = height, 32, SDL_HWSURFACE)) == NULL)
	{
		fprintf(stderr, "Error when screen was initialized\n");
		return 0;
	}
	if(caption != NULL)
		SDL_WM_SetCaption(caption, NULL);
	return 1;
}

void Window_Quit()
{
	if(get_screen() == NULL)
		return;
	SDL_FreeSurface(get_screen());
	window.screen = NULL;
	TTF_Quit();
	SDL_Quit();
}

SDL_Surface * get_screen()
{
	return window.screen;
}

void refresh_screen()
{
	SDL_Flip(get_screen());
}

void clear_screen()
{
	clear_screenRGB(0, 0, 0);
}

void clear_screenRGB(int r, int g, int b)
{
	SDL_FillRect(get_screen(), NULL, SDL_MapRGB(get_screen()->format, r, g, b));
}

int wait_any_key()
{
	int cont = 1;
	SDL_Event event;
	while(cont)
	{
		SDL_WaitEvent(&event);
		switch(event.type)
		{
			case SDL_QUIT:
			return 0;
		
			case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
				return 0;
				
				default:
				cont = 0;
			}
			break;
		}
	}
	return 1;
}